import re

import matplotlib.pyplot as plt
import networkx as nx

from .BFS import BFS
from .BipartiteFlow import BipartiteFlow
from .DFS import DFS
from .Dijkstra import Dijkstra
from .EdmondsKarp import EdmondsKarp
from .FordFulkerson import FordFulkerson
from .Hall import Hall
from .InteractiveGraph import InteractiveGraph
from .Kruskal import Kruskal
from .LabelPropagation import LabelPropagation
from .MaximumFlow import MaximumFlow
from .game import guess_adj_list, guess_adj_matrix
from .RandomWalk import RandomWalk
from .representation import draw_adj_list, draw_adj_matrix

# Städte im deutschen Sprachhrraum
dach_cities = nx.Graph()
dach_cities.add_edge('Hannover', 'Frankfurt', d=350)
dach_cities.add_edge('Hannover', 'Hamburg', d=150)
dach_cities.add_edge('Hannover', 'Berlin', d=290)
dach_cities.add_edge('Frankfurt', 'Zürich', d=400)
dach_cities.add_edge('Berlin', 'Wien', d=680)
dach_cities.add_edge('Hamburg', 'Mannheim', d=570)
dach_cities.add_edge('Hamburg', 'Stuttgart', d=650)
dach_cities.add_edge('Stuttgart', 'Dresden', d=510)
dach_cities.add_edge('Stuttgart', 'München', d=230)
dach_cities.add_edge('Zürich', 'München', d=310)
dach_cities.add_edge('Wien', 'München', d=400)

# Städte im Thüringer Wald + weitläufige Umgebung
tw = nx.Graph()
tw.add_edge('Eisenach', 'Ruhla', weight=20)
tw.add_edge('Eisenach', 'Waltershausen', weight=25)
tw.add_edge('Eisenach', 'Gehlberg', weight=25)
tw.add_edge('Eisenach', 'Friedrichroda', weight=30)
tw.add_edge('Ruhla', 'TambachDietharz', weight=25)
tw.add_edge('Ruhla', 'Waltershausen', weight=20)
tw.add_edge('Gehlberg', 'TambachDietharz', weight=15)
tw.add_edge('Gehlberg', 'Friedrichroda', weight=25)
tw.add_edge('Friedrichroda', 'Gehren', weight=25)
tw.add_edge('Ilmenau', 'Oberhof', weight=25)
tw.add_edge('Ilmenau', 'TambachDietharz', weight=30)
tw.add_edge('Ilmenau', 'ZellaMehlis', weight=25)
tw.add_edge('Oberhof', 'Suhl', weight=15)
tw.add_edge('Oberhof', 'Lauscha', weight=35)
tw.add_edge('TambachDietharz', 'Gotha', weight=55)
tw.add_edge('Suhl', 'ZellaMehlis', weight=10)
tw.add_edge('Suhl', 'Meiningen', weight=40)
tw.add_edge('Meiningen', 'Schleusingen', weight=30)
tw.add_edge('Schleusingen', 'Gotha', weight=50)
tw.add_edge('Schleusingen', 'NeuhausAR', weight=20)
tw.add_edge('Schleusingen', 'Hildburghausen', weight=15)
tw.add_edge('Masserberg', 'NeuhausAR', weight=20)
tw.add_edge('NeuhausAR', 'Lauscha', weight=35)
tw.add_edge('NeuhausAR', 'Oberhof', weight=55)
tw.add_edge('Lauscha', 'Sonneberg', weight=15)
tw.add_edge('Lauscha', 'Steinach', weight=15)
tw.add_edge('Sonneberg', 'Steinach', weight=15)
tw.add_edge('Gotha', 'Hildburghausen', weight=75)
tw.add_edge('Ilmenau', 'Lauscha', weight=40)
tw.add_edge('Ilmenau', 'Ruhla', weight=60)
tw.add_edge('TambachDietharz', 'Suhl', weight=40)
tw.add_edge('TambachDietharz', 'Meiningen', weight=45)

tw_pos = {'Eisenach': [1159.7633, -93.43375],
          'Ruhla': [986.3215, -16.088493],
          'Waltershausen': [1164.3373, 56.28163],
          'Gehlberg': [1021.95105, -175.97806],
          'Friedrichroda': [1267.1681, -230.15562],
          'Ilmenau': [450.01358, 38.451418],
          'Oberhof': [100.38327, 59.167953],
          'TambachDietharz': [724.0116, -88.65561],
          'Suhl': [7.873535, -75.329346],
          'ZellaMehlis': [204.26312, -11.835753],
          'Meiningen': [107.97316, -141.75964],
          'Schleusingen': [-53.44452, -239.20679],
          'Gotha': [404.65103, -196.93806],
          'Gehren': [1468.7135, -305.78067],
          'Masserberg': [-302.45526, -53.65928],
          'NeuhausAR': [-170.47804, 25.456665],
          'Lauscha': [-88.82424, 221.96655],
          'Sonneberg': [149.33255, 280.05417],
          'Steinach': [-239.35443, 320.05417],
          'Hildburghausen': [115.84885, -334.47046]}


def draw_tw(G: nx.Graph, pos=tw_pos):
    nx.draw(G, pos=pos, node_size=100, node_color='#EEE2DE')
    nx.draw_networkx_labels(G, pos, font_size=7)

    edge_labels = {(u, v): d['weight'] for u, v, d in G.edges(data=True)}
    nx.draw_networkx_edge_labels(G, pos=pos, edge_labels=edge_labels, font_size=6)

    plt.show()
    plt.close()


# Maximum-Flow Beispiele
mf1 = nx.DiGraph()
mf1.add_edge('s', 'a', capacity=3)
mf1.add_edge('s', 'b', capacity=1)
mf1.add_edge('a', 't', capacity=1)
mf1.add_edge('b', 't', capacity=4)
mf1.add_edge('a', 'b', capacity=2)

mf2 = nx.DiGraph()
mf2.add_edge('s', 'a', capacity=101)
mf2.add_edge('s', 'b', capacity=100)
mf2.add_edge('a', 't', capacity=100)
mf2.add_edge('b', 't', capacity=101)
mf2.add_edge('a', 'b', capacity=1)

# Bipartiter Graph Beispiele
small_bi = nx.Graph()
small_bi.add_edge('A', '1')
small_bi.add_edge('B', '1')
small_bi.add_edge('B', '2')
small_bi.add_edge('C', '2')
small_bi.add_edge('B', '2')
small_bi.add_edge('C', '2')
small_bi.add_edge('C', '3')
small_bi.add_edge('C', '4')
small_bi.add_edge('D', '4')

flow_bi = nx.DiGraph()
flow_bi.add_edge('s', 'A', capacity=1)
flow_bi.add_edge('s', 'B', capacity=1)
flow_bi.add_edge('s', 'C', capacity=1)
flow_bi.add_edge('s', 'D', capacity=1)
flow_bi.add_edge('A', '1', capacity=1)
flow_bi.add_edge('B', '1', capacity=1)
flow_bi.add_edge('B', '2', capacity=1)
flow_bi.add_edge('C', '2', capacity=1)
flow_bi.add_edge('B', '2', capacity=1)
flow_bi.add_edge('C', '2', capacity=1)
flow_bi.add_edge('C', '3', capacity=1)
flow_bi.add_edge('C', '4', capacity=1)
flow_bi.add_edge('D', '4', capacity=1)
flow_bi.add_edge('1', 't', capacity=1)
flow_bi.add_edge('2', 't', capacity=1)
flow_bi.add_edge('3', 't', capacity=1)
flow_bi.add_edge('4', 't', capacity=1)


def draw_bi(G):
    nodes = [node for node in G if re.fullmatch(r'[A-Za-z]+', node)]

    pos = nx.bipartite_layout(G, nodes)
    nx.draw(G, pos, with_labels=True, font_color='whitesmoke')


# Übungsaufgabe Examenstermine
exams = nx.DiGraph()
exams.add_edge('G1', 'Mo 09:30', weight=0)
exams.add_edge('G1', 'Mo 10:00', weight=0)
exams.add_edge('G1', 'Mo 10:30', weight=0)
exams.add_edge('G1', 'Di 12:00', weight=0)
exams.add_edge('G2', 'Do 11:30', weight=1)
exams.add_edge('G2', 'Do 12:30', weight=1)
exams.add_edge('G2', 'Di 11:30', weight=0)
exams.add_edge('G2', 'Mi 11:00', weight=0)
exams.add_edge('G2', 'Do 12:00', weight=1)
exams.add_edge('G2', 'Di 12:30', weight=0)
exams.add_edge('G2', 'Mi 10:30', weight=0)
exams.add_edge('G2', 'Di 12:00', weight=0)
exams.add_edge('G3', 'Di 09:30', weight=0)
exams.add_edge('G3', 'Di 10:00', weight=0)
exams.add_edge('G3', 'Do 11:30', weight=0)
exams.add_edge('G3', 'Di 09:00', weight=0)
exams.add_edge('G3', 'Do 12:00', weight=0)
exams.add_edge('G3', 'Do 11:00', weight=0)
exams.add_edge('G3', 'Mi 09:00', weight=1)
exams.add_edge('G3', 'Mi 09:30', weight=1)
exams.add_edge('G4', 'Do 12:30', weight=0)
exams.add_edge('G4', 'Do 11:30', weight=0)
exams.add_edge('G4', 'Do 12:00', weight=0)
exams.add_edge('G4', 'Do 11:00', weight=0)
exams.add_edge('G5', 'Mi 10:00', weight=0)
exams.add_edge('G5', 'Mi 09:00', weight=0)
exams.add_edge('G5', 'Mi 10:30', weight=0)
exams.add_edge('G5', 'Mi 09:30', weight=0)
exams.add_edge('G6', 'Di 09:30', weight=0)
exams.add_edge('G6', 'Di 11:00', weight=0)
exams.add_edge('G6', 'Di 09:00', weight=0)
exams.add_edge('G6', 'Di 11:30', weight=0)
exams.add_edge('G6', 'Di 10:30', weight=0)
exams.add_edge('G6', 'Di 10:00', weight=0)
exams.add_edge('G7', 'Do 12:30', weight=0)
exams.add_edge('G7', 'Mi 11:00', weight=0)
exams.add_edge('G8', 'Mo 09:00', weight=0)
exams.add_edge('G8', 'Mo 09:30', weight=0)
exams.add_edge('G8', 'Mo 10:00', weight=0)
exams.add_edge('G8', 'Di 09:00', weight=0)
exams.add_edge('G8', 'Mi 09:00', weight=1)
exams.add_edge('G8', 'Mo 10:30', weight=0)
exams.add_edge('G8', 'Di 09:30', weight=0)
exams.add_edge('G8', 'Mi 10:00', weight=1)
exams.add_edge('G8', 'Mi 11:00', weight=1)
exams.add_edge('G8', 'Di 10:00', weight=0)
exams.add_edge('G8', 'Mi 10:30', weight=1)
exams.add_edge('G8', 'Mi 09:30', weight=1)
exams.add_edge('G9', 'Mi 10:00', weight=0)
exams.add_edge('G9', 'Mo 09:00', weight=0)
exams.add_edge('G9', 'Mo 09:30', weight=0)
exams.add_edge('G9', 'Mo 10:00', weight=0)
exams.add_edge('G9', 'Mi 09:00', weight=0)
exams.add_edge('G9', 'Mi 09:30', weight=0)
exams.add_edge('G9', 'Mo 10:30', weight=0)
exams.add_edge('G10', 'Mi 10:00', weight=0)
exams.add_edge('G10', 'Mi 09:00', weight=0)
exams.add_edge('G10', 'Mi 09:30', weight=0)
exams.add_edge('G10', 'Di 12:30', weight=1)
exams.add_edge('G11', 'Di 09:30', weight=0)
exams.add_edge('G11', 'Di 10:00', weight=0)
exams.add_edge('G11', 'Mo 09:00', weight=0)
exams.add_edge('G11', 'Mo 09:30', weight=0)
exams.add_edge('G11', 'Mo 10:00', weight=0)
exams.add_edge('G11', 'Di 09:00', weight=0)
exams.add_edge('G12', 'Di 11:00', weight=0)
exams.add_edge('G12', 'Di 09:00', weight=0)
exams.add_edge('G12', 'Di 11:30', weight=0)
exams.add_edge('G12', 'Di 12:00', weight=0)
exams.add_edge('G12', 'Di 09:30', weight=0)
exams.add_edge('G12', 'Di 12:30', weight=0)
exams.add_edge('G12', 'Di 10:30', weight=0)
exams.add_edge('G12', 'Di 10:00', weight=0)
exams.add_edge('G13', 'Di 11:00', weight=0)
exams.add_edge('G13', 'Di 12:30', weight=0)
exams.add_edge('G13', 'Di 11:30', weight=0)
exams.add_edge('G13', 'Di 12:00', weight=0)
exams.add_edge('G14', 'Di 09:30', weight=0)
exams.add_edge('G14', 'Mo 09:00', weight=1)
exams.add_edge('G14', 'Mo 09:30', weight=1)
exams.add_edge('G14', 'Mo 10:00', weight=1)
exams.add_edge('G14', 'Do 11:30', weight=0)
exams.add_edge('G14', 'Di 09:00', weight=0)
exams.add_edge('G14', 'Do 11:00', weight=0)
exams.add_edge('G14', 'Di 10:30', weight=0)
exams.add_edge('G14', 'Di 10:00', weight=0)
exams.add_edge('G14', 'Mo 10:30', weight=1)
exams.add_edge('G14', 'Di 11:00', weight=1)
exams.add_edge('G14', 'Do 12:30', weight=1)
exams.add_edge('G14', 'Di 11:30', weight=1)
exams.add_edge('G14', 'Do 12:00', weight=1)
exams.add_edge('G14', 'Mi 10:30', weight=1)
exams.add_edge('G15', 'Mi 10:00', weight=0)
exams.add_edge('G15', 'Mi 09:00', weight=0)
exams.add_edge('G15', 'Mi 09:30', weight=0)
exams.add_edge('G16', 'Di 09:30', weight=0)
exams.add_edge('G16', 'Mi 10:00', weight=0)
exams.add_edge('G16', 'Mo 09:00', weight=1)
exams.add_edge('G16', 'Mo 09:30', weight=1)
exams.add_edge('G16', 'Mo 10:00', weight=1)
exams.add_edge('G16', 'Di 09:00', weight=0)
exams.add_edge('G16', 'Mi 09:00', weight=0)
exams.add_edge('G16', 'Di 10:00', weight=0)
exams.add_edge('G16', 'Mi 09:30', weight=0)
exams.add_edge('G16', 'Mo 10:30', weight=1)
exams.add_edge('G17', 'Do 11:30', weight=0)


def draw_exams(graph: nx.DiGraph, source: str = 's', target: str = 't'):
    def _filter_exams_nodes(graph: nx.DiGraph):
        for node in graph.nodes:
            if node.startswith('G'):
                yield node

    def _sort_exams_nodes(node):
        if node.startswith('G'):
            return f'G{int(node[1:]):03}'
        if node.startswith('Mo '):
            return f'W1 {node[3:]}'
        if node.startswith('Di '):
            return f'W2 {node[3:]}'
        if node.startswith('Mi '):
            return f'W3 {node[3:]}'
        if node.startswith('Do '):
            return f'W4 {node[3:]}'
        else:
            return node

    st = source, target
    left = set(_filter_exams_nodes(graph))

    # initialize figure
    plt.figure(figsize=(8, 11))

    # layout without source and target
    copy = graph.copy()
    copy.remove_nodes_from(st)
    pos = nx.bipartite_layout(copy, left)

    # add positions for source and target
    xs = [x for x, _ in pos.values()]
    ys = [y for _, y in pos.values()]

    min_x, max_x = min(xs), max(xs)
    min_y, max_y = min(ys), max(ys)
    dx, dy = max_x - min_x, max_y - min_y

    pos[source] = [min_x - dx / 3, max_y - dy / 2]
    pos[target] = [max_x + dx / 3, max_y - dy / 2]

    # sort other nodes
    for a in sorted(pos.keys(), key=_sort_exams_nodes):
        for b in sorted(pos.keys(), key=_sort_exams_nodes):
            if a in left and b not in left or a not in left and b in left or a in st or b in st:
                continue

            if pos[a][1] > pos[b][1]:
                pos[a], pos[b] = pos[b], pos[a]

    # draw graph
    nx.draw(graph, pos, with_labels=True, font_size=6, font_color='whitesmoke', node_size=1000)
    nx.draw_networkx_edge_labels(graph, pos, nx.get_edge_attributes(graph, 'weight'), font_size=6)

    # display image
    plt.show()
    plt.close()


# Wikipedia
wiki = nx.Graph()
wiki.add_edge('0.0.0.0', 'Socket (Software)')
wiki.add_edge('100BASE-TX', 'Cable-Sharing')
wiki.add_edge('10BASE-T', 'Cable-Sharing')
wiki.add_edge('10BASE-T1L', 'Advanced Physical Layer')
wiki.add_edge('10BASE2', '10BASE-T')
wiki.add_edge('10BASE2', 'EAD-Kabel')
wiki.add_edge('10BASE2', 'Jabber (Ethernet)')
wiki.add_edge('10BASE2', 'JetDirect')
wiki.add_edge('10BASE2', 'T-Stück')
wiki.add_edge('10BASE5', '10BASE2')
wiki.add_edge('10BASE5', 'Attachment Unit Interface')
wiki.add_edge('10BASE5', 'T-Stück')
wiki.add_edge('3964R', 'RK512')
wiki.add_edge('4+1 Sichtenmodell', 'C4 model')
wiki.add_edge('4+1 Sichtenmodell', 'Rational Rose')
wiki.add_edge('6LoWPAN', 'Constrained Application Protocol')
wiki.add_edge('6LoWPAN', 'IP500 Alliance')
wiki.add_edge('6LoWPAN', 'Routing Protocol for Low power and Lossy Networks')
wiki.add_edge('6LoWPAN', 'Thread (Netzwerkprotokoll)')
wiki.add_edge('6in4', 'Liste von IPv6-Tunnelbrokern')
wiki.add_edge('6to4', '6rd')
wiki.add_edge('6to4', 'DirectAccess')
wiki.add_edge('6to4', 'IP-Spoofing')
wiki.add_edge('6to4', 'ISATAP')
wiki.add_edge('6to4', 'Teredo')
wiki.add_edge('6to4', 'Tunnel (Rechnernetz)')
wiki.add_edge('ADO.NET', 'Entity Framework')
wiki.add_edge('ADO.NET', 'Remote Data Objects')
wiki.add_edge('ADOdb', 'ADOdb Lite')
wiki.add_edge('ADOdb', 'Data Source Name')
wiki.add_edge('ADOdb', 'Datenbankabstraktionsschicht')
wiki.add_edge('ADOdb', 'MySQLi')
wiki.add_edge('ADOdb', 'PHP Data Objects')
wiki.add_edge('ALOHA', 'Berkeley Media Access Control')
wiki.add_edge('ARP-Cache', 'ARP-Spoofing')
wiki.add_edge('ARQ-Protokoll', 'SACK')
wiki.add_edge('ARQ-Protokoll', 'X.75')
wiki.add_edge('AS1', 'ESTRI Gateway')
wiki.add_edge('AS2', 'AS1')
wiki.add_edge('AS2', 'AS4 (Schnittstelle)')
wiki.add_edge('AS2', 'ESTRI Gateway')
wiki.add_edge('AS3', 'AS1')
wiki.add_edge('AS3', 'AS2')
wiki.add_edge('AS3', 'ESTRI Gateway')
wiki.add_edge('ATA over Ethernet', 'Fibre Channel over IP')
wiki.add_edge('ATA over Ethernet', 'HyperSCSI')
wiki.add_edge('ATA over Ethernet', 'ISCSI')
wiki.add_edge('AYIYA', 'Liste von IPv6-Tunnelbrokern')
wiki.add_edge('Abhängigkeitsbeziehung', 'Zusicherung (Softwaretechnik)')
wiki.add_edge('Ablaufdiagramm', 'Funktionsbaum')
wiki.add_edge('Ablaufdiagramm', 'Jackson-Diagramm')
wiki.add_edge('Ablaufdiagramm', 'Microsoft Visio')
wiki.add_edge('Ablaufdiagramm', 'Nassi-Shneiderman-Diagramm')
wiki.add_edge('Ablaufdiagramm', 'YEd')
wiki.add_edge('Abstract Window Toolkit', 'Beobachter (Entwurfsmuster)')
wiki.add_edge('Abstract Window Toolkit', 'JavaFX')
wiki.add_edge('Abstrakte Fabrik', 'Asynchronous module definition')
wiki.add_edge('Abstrakte Fabrik', 'Jakarta Connectors')
wiki.add_edge('Abstrakte Fabrik', 'Java API for XML Processing')
wiki.add_edge('Active Record', 'Geschäftslogik')
wiki.add_edge('Active Record', 'SQLAlchemy')
wiki.add_edge('Active Record', 'Sicht (Datenbank)')
wiki.add_edge('ActiveX', 'ActiveX Data Objects')
wiki.add_edge('ActiveX', 'Component Object Model')
wiki.add_edge('ActiveX', 'Java Plug-in')
wiki.add_edge('ActiveX', 'Killbit')
wiki.add_edge('ActiveX', 'Komponente (Software)')
wiki.add_edge('ActiveX Data Objects', 'ADO.NET')
wiki.add_edge('ActiveX Data Objects', 'ADOdb')
wiki.add_edge('ActiveX Data Objects', 'Data Access Objects')
wiki.add_edge('ActiveX Data Objects', 'Datenbankschnittstelle')
wiki.add_edge('ActiveX Data Objects', 'Embedded SQL')
wiki.add_edge('ActiveX Data Objects', 'OLE DB')
wiki.add_edge('ActiveX Data Objects', 'Perl-DBI')
wiki.add_edge('ActiveX Data Objects', 'Remote Data Objects')
wiki.add_edge('Ad-hoc On-demand Distance Vector', '6LoWPAN')
wiki.add_edge('Ad-hoc On-demand Distance Vector', 'Babel (Protokoll)')
wiki.add_edge('Ad-hoc On-demand Distance Vector', 'Delay Tolerant Networking')
wiki.add_edge('Adapter (Entwurfsmuster)', 'Anwendungsserver')
wiki.add_edge('Adapter (Entwurfsmuster)', 'Delegation (Softwareentwicklung)')
wiki.add_edge('Adapter (Entwurfsmuster)', 'Java Management Extensions')
wiki.add_edge('Adapter (Entwurfsmuster)', 'Open Platform Communications')
wiki.add_edge('Adapter (Entwurfsmuster)', 'Phonon (KDE)')
wiki.add_edge('Adapter (Entwurfsmuster)', 'Wrapper-Klasse')
wiki.add_edge('Address Resolution Protocol', 'ARP-Cache')
wiki.add_edge('Address Resolution Protocol', 'ARP-Spoofing')
wiki.add_edge('Address Resolution Protocol', 'Bootstrap Protocol')
wiki.add_edge('Address Resolution Protocol', 'Classless Inter-Domain Routing')
wiki.add_edge('Address Resolution Protocol', 'Hot Standby Router Protocol')
wiki.add_edge('Address Resolution Protocol', 'ICMPv6')
wiki.add_edge('Address Resolution Protocol', 'Internet Control Message Protocol')
wiki.add_edge('Address Resolution Protocol', 'Mobile IP')
wiki.add_edge('Address Resolution Protocol', 'Neighbor Discovery Protocol')
wiki.add_edge('Address Resolution Protocol', 'Netzwerkadressübersetzung')
wiki.add_edge('Address Resolution Protocol', 'Profinet')
wiki.add_edge('Address Resolution Protocol', 'Programmablaufplan')
wiki.add_edge('Address Resolution Protocol', 'Protokollstapel')
wiki.add_edge('Address Resolution Protocol', 'Reverse Address Resolution Protocol')
wiki.add_edge('Address Resolution Protocol', 'Virtual Router Redundancy Protocol')
wiki.add_edge('Address Resolution Protocol', 'Wired Equivalent Privacy')
wiki.add_edge('Advanced Message Queuing Protocol', 'Production Performance Management Protocol')
wiki.add_edge('Advanced Peer-to-Peer Networking', 'VTAM')
wiki.add_edge('Aggregation (Informatik)', 'Besucher (Entwurfsmuster)')
wiki.add_edge('Aggregation (Informatik)', 'Delegation (Softwareentwicklung)')
wiki.add_edge('Aggregation (Informatik)', 'GRASP')
wiki.add_edge('Aggregation (Informatik)', 'Sicht (Datenbank)')
wiki.add_edge('Akteur (UML)', 'Classifier (UML)')
wiki.add_edge('Akteur (UML)', 'Generalisierung (UML)')
wiki.add_edge('Akteur (UML)', 'Modell (UML)')
wiki.add_edge('Aktion (UML)', 'Aktivitätsparameterknoten')
wiki.add_edge('Aktion (UML)', 'Ausprägungsspezifikation (UML)')
wiki.add_edge('Aktion (UML)', 'Ereignis (UML)')
wiki.add_edge('Aktion (UML)', 'Interaktion (UML)')
wiki.add_edge('Aktion (UML)', 'Kontrollfluss (UML)')
wiki.add_edge('Aktion (UML)', 'Merkmal (UML)')
wiki.add_edge('Aktion (UML)', 'Objektfluss (UML)')
wiki.add_edge('Aktion (UML)', 'Objektknoten (UML)')
wiki.add_edge('Aktion (UML)', 'Operation (UML)')
wiki.add_edge('Aktion (UML)', 'Pufferknoten (UML)')
wiki.add_edge('Aktion (UML)', 'Signal (UML)')
wiki.add_edge('Aktion (UML)', 'Verhalten (UML)')
wiki.add_edge('Aktivität (UML)', 'Aktion (UML)')
wiki.add_edge('Aktivität (UML)', 'Aktivitätsparameterknoten')
wiki.add_edge('Aktivität (UML)', 'Classifier (UML)')
wiki.add_edge('Aktivität (UML)', 'Datenflussdiagramm')
wiki.add_edge('Aktivität (UML)', 'Interaktionsübersichtsdiagramm')
wiki.add_edge('Aktivität (UML)', 'Kontrollfluss (UML)')
wiki.add_edge('Aktivität (UML)', 'Kontrollknoten (UML)')
wiki.add_edge('Aktivität (UML)', 'Objektfluss (UML)')
wiki.add_edge('Aktivität (UML)', 'Objektknoten (UML)')
wiki.add_edge('Aktivität (UML)', 'Operation (UML)')
wiki.add_edge('Aktivität (UML)', 'Pin (UML)')
wiki.add_edge('Aktivität (UML)', 'Pufferknoten (UML)')
wiki.add_edge('Aktivität (UML)', 'Verhalten (UML)')
wiki.add_edge('Aktivität (UML)', 'Zustandsdiagramm (UML)')
wiki.add_edge('Aktivitätsdiagramm', '4+1 Sichtenmodell')
wiki.add_edge('Aktivitätsdiagramm', 'Aktion (UML)')
wiki.add_edge('Aktivitätsdiagramm', 'Aktivität (UML)')
wiki.add_edge('Aktivitätsdiagramm', 'Aktivitätsparameterknoten')
wiki.add_edge('Aktivitätsdiagramm', 'Anwendungsfall (UML)')
wiki.add_edge('Aktivitätsdiagramm', 'Datenfluss')
wiki.add_edge('Aktivitätsdiagramm', 'Datenflussdiagramm')
wiki.add_edge('Aktivitätsdiagramm', 'Fujaba')
wiki.add_edge('Aktivitätsdiagramm', 'Gaphor')
wiki.add_edge('Aktivitätsdiagramm', 'Interaktionsübersichtsdiagramm')
wiki.add_edge('Aktivitätsdiagramm', 'Kontrollfluss (UML)')
wiki.add_edge('Aktivitätsdiagramm', 'Kontrollknoten (UML)')
wiki.add_edge('Aktivitätsdiagramm', 'Nassi-Shneiderman-Diagramm')
wiki.add_edge('Aktivitätsdiagramm', 'Objektfluss (UML)')
wiki.add_edge('Aktivitätsdiagramm', 'Pin (UML)')
wiki.add_edge('Aktivitätsdiagramm', 'Programmablaufplan')
wiki.add_edge('Aktivitätsdiagramm', 'Saga (Entwurfsmuster)')
wiki.add_edge('Aktivitätsdiagramm', 'Sequenzdiagramm')
wiki.add_edge('Aktivitätsdiagramm', 'Signal (UML)')
wiki.add_edge('Aktivitätsdiagramm', 'UMLet')
wiki.add_edge('Aktivitätsdiagramm', 'Violet (UML-Werkzeug)')
wiki.add_edge('Analysemuster', 'Accountability')
wiki.add_edge('Analysemuster', 'Accounting Transaction')
wiki.add_edge('Analysemuster', 'Anti-Pattern')
wiki.add_edge('Analysemuster', 'Anwendungsfassade')
wiki.add_edge('Analysemuster', 'Event (Analysemuster)')
wiki.add_edge('Analysemuster', 'Event Sourcing')
wiki.add_edge('Analysemuster', 'Fixed Property')
wiki.add_edge('Analysemuster', 'Item-Item Description')
wiki.add_edge('Analysemuster', 'Koordinator (Analysemuster)')
wiki.add_edge('Analysemuster', 'Measurement (Analysemuster)')
wiki.add_edge('Analysemuster', 'Money (Analysemuster)')
wiki.add_edge('Analysemuster', 'Observation (Analysemuster)')
wiki.add_edge('Analysemuster', 'Organization Hierarchy')
wiki.add_edge('Analysemuster', 'Organization Structure')
wiki.add_edge('Analysemuster', 'Quantität (Analysemuster)')
wiki.add_edge('Analysemuster', 'Range (Analysemuster)')
wiki.add_edge('Anomalie (Informatik)', 'Denormalisierung')
wiki.add_edge('Anomalie (Informatik)', 'Sternschema')
wiki.add_edge('Anti-Pattern', 'Big Ball of Mud')
wiki.add_edge('Anti-Pattern', 'Code-Smell')
wiki.add_edge('Anti-Pattern', 'Dark Pattern')
wiki.add_edge('Anti-Pattern', 'Doppelt überprüfte Sperrung')
wiki.add_edge('Anti-Pattern', 'Gottobjekt')
wiki.add_edge('Anti-Pattern', 'Kopplung (Softwareentwicklung)')
wiki.add_edge('Anti-Pattern', 'Liste von Singleton-Implementierungen')
wiki.add_edge('Anti-Pattern', 'Quelltextklon')
wiki.add_edge('Anti-Pattern', 'Refactoring')
wiki.add_edge('Anti-Pattern', 'Service-Locator')
wiki.add_edge('Anti-Pattern', 'Singleton (Entwurfsmuster)')
wiki.add_edge('Anti-Pattern', 'Smart UI')
wiki.add_edge('Anti-Pattern', 'Softwareerosion')
wiki.add_edge('Anwendungsfall (UML)', 'Abhängigkeitsbeziehung')
wiki.add_edge('Anwendungsfall (UML)', 'Akteur (UML)')
wiki.add_edge('Anwendungsfall (UML)', 'Generalisierung (UML)')
wiki.add_edge('Anwendungsfall (UML)', 'Komponente (UML)')
wiki.add_edge('Anwendungsfall (UML)', 'Verhalten (UML)')
wiki.add_edge('Anwendungsfalldiagramm', '4+1 Sichtenmodell')
wiki.add_edge('Anwendungsfalldiagramm', 'Akteur (UML)')
wiki.add_edge('Anwendungsfalldiagramm', 'Aktivitätsdiagramm')
wiki.add_edge('Anwendungsfalldiagramm', 'Anwendungsfall (UML)')
wiki.add_edge('Anwendungsfalldiagramm', 'Assoziation (UML)')
wiki.add_edge('Anwendungsfalldiagramm', 'Funktionsbaum')
wiki.add_edge('Anwendungsfalldiagramm', 'Gaphor')
wiki.add_edge('Anwendungsfalldiagramm', 'Kommunikationsdiagramm (UML)')
wiki.add_edge('Anwendungsfalldiagramm', 'Sequenzdiagramm')
wiki.add_edge('Anwendungsfalldiagramm', 'UMLet')
wiki.add_edge('Anwendungsfalldiagramm', 'Violet (UML-Werkzeug)')
wiki.add_edge('Anwendungsserver', 'Apache OpenJPA')
wiki.add_edge('Anwendungsserver', 'Hibernate (Framework)')
wiki.add_edge('Anwendungsserver', 'Jakarta Server Faces')
wiki.add_edge('Anwendungsserver', 'Jakarta Transactions API')
wiki.add_edge('Anwendungsserver', 'Lookeen')
wiki.add_edge('Anwendungsserver', 'Message Oriented Middleware')
wiki.add_edge('Anwendungsserver', 'TopLink')
wiki.add_edge('Apache Axis', 'Jakarta XML RPC')
wiki.add_edge('Apache Axis', 'SOAP Message Transmission Optimization Mechanism')
wiki.add_edge('Apache Axis', 'Streaming API for XML')
wiki.add_edge('Apache Axis', 'WS-Addressing')
wiki.add_edge('Apache Axis', 'WS-AtomicTransaction')
wiki.add_edge('Apache Axis', 'WS-Policy')
wiki.add_edge('Apache Axis', 'WS-Reliable Messaging')
wiki.add_edge('Apache Axis', 'Web Services Interoperability Technology')
wiki.add_edge('Apache CXF', 'Apache Axis')
wiki.add_edge('Apache CXF', 'Apache ServiceMix')
wiki.add_edge('Apache CXF', 'Jakarta RESTful Web Services')
wiki.add_edge('Apache CXF', 'Jakarta XML Web Services')
wiki.add_edge('Apache CXF', 'Java Management Extensions')
wiki.add_edge('Apache CXF', 'SOAP Message Transmission Optimization Mechanism')
wiki.add_edge('Apache CXF', 'Service Data Objects')
wiki.add_edge('Apache CXF', 'WS-Addressing')
wiki.add_edge('Apache CXF', 'WS-Policy')
wiki.add_edge('Apache CXF', 'WS-Reliable Messaging')
wiki.add_edge('Apache CXF', 'WS-SecureConversation')
wiki.add_edge('Apache CXF', 'WS-Trust')
wiki.add_edge('Apache CXF', 'Web Services Interoperability Technology')
wiki.add_edge('Apache CXF', 'XFire (Framework)')
wiki.add_edge('Apache OpenJPA', 'Apache Cayenne')
wiki.add_edge('Apple Filing Protocol', 'AppleShare')
wiki.add_edge('Apple Filing Protocol', 'Boot Service Discovery Protocol')
wiki.add_edge('Apple Filing Protocol', 'Datagram Delivery Protocol')
wiki.add_edge('Apple Filing Protocol', 'EtherTalk Link Access Protocol')
wiki.add_edge('Apple Filing Protocol', 'Gnome Virtual File System')
wiki.add_edge('Apple Filing Protocol', 'Liste der standardisierten Ports')
wiki.add_edge('Apple Filing Protocol', 'LocalTalk')
wiki.add_edge('Apple Filing Protocol', 'LocalTalk Link Access Protocol')
wiki.add_edge('Apple Filing Protocol', 'Network File System')
wiki.add_edge('Apple Filing Protocol', 'Portable Operating System Interface')
wiki.add_edge('Apple Filing Protocol', 'Routing Table Maintenance Protocol')
wiki.add_edge('Apple Filing Protocol', 'Server Message Block')
wiki.add_edge('Apple Filing Protocol', 'TokenTalk Link Access Protocol')
wiki.add_edge('Apple Filing Protocol', 'Zone Information Protocol')
wiki.add_edge('AppleTalk', 'Apple Filing Protocol')
wiki.add_edge('AppleTalk', 'AppleShare')
wiki.add_edge('AppleTalk', 'Appletalk')
wiki.add_edge('AppleTalk', 'Datagram Delivery Protocol')
wiki.add_edge('AppleTalk', 'EtherTalk Link Access Protocol')
wiki.add_edge('AppleTalk', 'Fast Ethernet')
wiki.add_edge('AppleTalk', 'Generic Routing Encapsulation')
wiki.add_edge('AppleTalk', 'Internet Printing Protocol')
wiki.add_edge('AppleTalk', 'JetDirect')
wiki.add_edge('AppleTalk', 'Kinetics Internet Protocol')
wiki.add_edge('AppleTalk', 'Liste der standardisierten Ports')
wiki.add_edge('AppleTalk', 'LocalTalk')
wiki.add_edge('AppleTalk', 'LocalTalk Link Access Protocol')
wiki.add_edge('AppleTalk', 'NetBEUI')
wiki.add_edge('AppleTalk', 'Routing Table Maintenance Protocol')
wiki.add_edge('AppleTalk', 'Socket (Software)')
wiki.add_edge('AppleTalk', 'TokenTalk Link Access Protocol')
wiki.add_edge('AppleTalk', 'Transmission Control Protocol/Internet Protocol')
wiki.add_edge('AppleTalk', 'Zone Information Protocol')
wiki.add_edge('Appletalk', 'Enhanced Interior Gateway Routing Protocol')
wiki.add_edge('Architekturmuster', 'Anti-Pattern')
wiki.add_edge('Architekturmuster', 'Common Object Request Broker Architecture')
wiki.add_edge('Architekturmuster', 'Data Context Interaction')
wiki.add_edge('Architekturmuster', 'Front Controller')
wiki.add_edge('Architekturmuster', 'GRASP')
wiki.add_edge('Architekturmuster', 'Geschäftslogik')
wiki.add_edge('Architekturmuster', 'Meta-Architektur-Dokument')
wiki.add_edge('Architekturmuster', 'Microservices')
wiki.add_edge('Architekturmuster', 'Model 2')
wiki.add_edge('Architekturmuster', 'Model View Controller')
wiki.add_edge('Architekturmuster', 'Model View Presenter')
wiki.add_edge('Architekturmuster', 'Pipes und Filter')
wiki.add_edge('Architekturmuster', 'Presentation-Abstraction-Control')
wiki.add_edge('Architekturmuster', 'Remote Presentation Model')
wiki.add_edge('Architekturmuster', 'Self-contained Systems')
wiki.add_edge('Architekturmuster', 'Service-Locator')
wiki.add_edge('Architekturmuster', 'Serviceorientierte Architektur')
wiki.add_edge('Architekturmuster', 'Smart UI')
wiki.add_edge('ArgoUML', 'Poseidon for UML')
wiki.add_edge('Artefakt (UML)', 'Abhängigkeitsbeziehung')
wiki.add_edge('Artefakt (UML)', 'Fachkomponente')
wiki.add_edge('Artefakt (UML)', 'Innovator (Software)')
wiki.add_edge('Artefakt (UML)', 'Knoten (UML)')
wiki.add_edge('Artefakt (UML)', 'Stereotyp (UML)')
wiki.add_edge('Asia-Pacific Network Information Centre', 'Regional Internet Registry')
wiki.add_edge('Asia-Pacific Network Information Centre', 'WHOIS')
wiki.add_edge('Assoziation (UML)', 'Abhängigkeitsbeziehung')
wiki.add_edge('Assoziation (UML)', 'Aggregation (Informatik)')
wiki.add_edge('Assoziation (UML)', 'Aktion (UML)')
wiki.add_edge('Assoziation (UML)', 'Anwendungsfall (UML)')
wiki.add_edge('Assoziation (UML)', 'Assoziationsklasse')
wiki.add_edge('Assoziation (UML)', 'Classifier (UML)')
wiki.add_edge('Assoziation (UML)', 'Entity-Relationship-Modell')
wiki.add_edge('Assoziation (UML)', 'Fixed Property')
wiki.add_edge('Assoziation (UML)', 'Generalisierung (UML)')
wiki.add_edge('Assoziation (UML)', 'Informationseinheit (UML)')
wiki.add_edge('Assoziation (UML)', 'Item-Item Description')
wiki.add_edge('Assoziation (UML)', 'Klassendiagramm')
wiki.add_edge('Assoziation (UML)', 'Knoten (UML)')
wiki.add_edge('Assoziation (UML)', 'Komponente (UML)')
wiki.add_edge('Assoziation (UML)', 'Kompositionsstrukturdiagramm')
wiki.add_edge('Assoziation (UML)', 'Konnektor (UML)')
wiki.add_edge('Assoziation (UML)', 'Koordinator (Analysemuster)')
wiki.add_edge('Assoziation (UML)', 'Multiplizität (UML)')
wiki.add_edge('Assoziation (UML)', 'Object-relational impedance mismatch')
wiki.add_edge('Assoziation (UML)', 'Schnittstelle (UML)')
wiki.add_edge('Assoziation (UML)', 'Stereotyp (UML)')
wiki.add_edge('Assoziation (UML)', 'Zusicherung (Softwaretechnik)')
wiki.add_edge('Assoziationsklasse', 'Entity-Relationship-Modell')
wiki.add_edge('Attachment Unit Interface', '10BASE-T')
wiki.add_edge('Attachment Unit Interface', '10BASE2')
wiki.add_edge('Attribut (UML)', 'Entität (Informatik)')
wiki.add_edge('Attribut (UML)', 'Generalisierung (UML)')
wiki.add_edge('Attribut (UML)', 'Klassendiagramm')
wiki.add_edge('Attribut (UML)', 'Komponente (UML)')
wiki.add_edge('Attribut (UML)', 'Merkmal (UML)')
wiki.add_edge('Attribut (UML)', 'Normalisierung (Datenbank)')
wiki.add_edge('Attribut (UML)', 'Operation (UML)')
wiki.add_edge('Attribut (UML)', 'Profil (UML)')
wiki.add_edge('Attribut (UML)', 'Stereotyp (UML)')
wiki.add_edge('Attribut (UML)', 'Zusicherung (Softwaretechnik)')
wiki.add_edge('Audio Stream Input/Output', 'Core Audio')
wiki.add_edge('Audio Unit', 'Audio Random Access')
wiki.add_edge('Audio Unit', 'Core Audio')
wiki.add_edge('Ausprägungsspezifikation (UML)', 'Entität (Informatik)')
wiki.add_edge('Ausprägungsspezifikation (UML)', 'Knoten (UML)')
wiki.add_edge('Ausprägungsspezifikation (UML)', 'Kompositionsstrukturdiagramm')
wiki.add_edge('Ausprägungsspezifikation (UML)', 'Objektdiagramm')
wiki.add_edge('Authenticated Post Office Protocol', 'POP3S')
wiki.add_edge('Autonegotiation', '10BASE-T')
wiki.add_edge('B.A.T.M.A.N.', 'Babel (Protokoll)')
wiki.add_edge('B.A.T.M.A.N.', 'Briar (Instant Messenger)')
wiki.add_edge('BACnet', 'IP500 Alliance')
wiki.add_edge('BACnet', 'Liste der standardisierten Ports')
wiki.add_edge('BACnet', 'Open Platform Communications')
wiki.add_edge('BACnet', 'ZigBee')
wiki.add_edge('BNC-Steckverbinder', '10BASE2')
wiki.add_edge('BNC-Steckverbinder', 'EAD-Kabel')
wiki.add_edge('Beobachter (Entwurfsmuster)', 'MQTT')
wiki.add_edge('Beobachter (Entwurfsmuster)', 'Message Oriented Middleware')
wiki.add_edge('Beobachter (Entwurfsmuster)', 'Model View ViewModel')
wiki.add_edge('Berkeley Printing System', 'Druckprotokoll')
wiki.add_edge('Bewertung von Softwarearchitekturen', 'Modularität')
wiki.add_edge('Binary Large Object', 'Microservices')
wiki.add_edge('Binärschnittstelle', 'ActiveX')
wiki.add_edge('Binärschnittstelle', 'Binärstandard')
wiki.add_edge('Binärschnittstelle', 'Common Object Request Broker Architecture')
wiki.add_edge('Binärschnittstelle', 'Component Object Model')
wiki.add_edge('Binärschnittstelle', 'Dienst (Informatik)')
wiki.add_edge('Binärschnittstelle', 'DirectX')
wiki.add_edge('Binärschnittstelle', 'Object Linking and Embedding')
wiki.add_edge('BitTorrent', 'EDonkey2000')
wiki.add_edge('BitTorrent', 'I2P')
wiki.add_edge('BitTorrent', 'InterPlanetary File System')
wiki.add_edge('BitTorrent', 'Internet Cache Protocol')
wiki.add_edge('BitTorrent', 'Liste der standardisierten Ports')
wiki.add_edge('BitTorrent', 'Tron (Kryptowährung)')
wiki.add_edge('BitTorrent', 'ZeroNet')
wiki.add_edge('Bitcoin', 'Bildschirmschoner')
wiki.add_edge('Bitcoin', 'BitTorrent')
wiki.add_edge('Bitcoin', 'Bitcoin Cash')
wiki.add_edge('Bitcoin', 'Bitcoin SV')
wiki.add_edge('Bitcoin', 'Bitstamp')
wiki.add_edge('Bitcoin', 'CyberGhost VPN')
wiki.add_edge('Bitcoin', 'Dash (Kryptowährung)')
wiki.add_edge('Bitcoin', 'Diem (Internetwährung)')
wiki.add_edge('Bitcoin', 'Dogecoin')
wiki.add_edge('Bitcoin', 'EOS (Kryptowährung)')
wiki.add_edge('Bitcoin', 'Ethereum')
wiki.add_edge('Bitcoin', 'Facebook')
wiki.add_edge('Bitcoin', 'InterPlanetary File System')
wiki.add_edge('Bitcoin', 'Iota (Kommunikationsprotokoll)')
wiki.add_edge('Bitcoin', 'Kryptowährung')
wiki.add_edge('Bitcoin', 'Lightning-Netzwerk')
wiki.add_edge('Bitcoin', 'Liste der standardisierten Ports')
wiki.add_edge('Bitcoin', 'Liste von Kryptowährungen')
wiki.add_edge('Bitcoin', 'Litecoin')
wiki.add_edge('Bitcoin', 'Monero')
wiki.add_edge('Bitcoin', 'Netzwerkprotokoll')
wiki.add_edge('Bitcoin', 'Noise Protocol Framework')
wiki.add_edge('Bitcoin', 'Nxt (Kryptowährung)')
wiki.add_edge('Bitcoin', 'OSI-Modell')
wiki.add_edge('Bitcoin', 'Particl (Kryptowährung)')
wiki.add_edge('Bitcoin', 'Petro (Kryptowährung)')
wiki.add_edge('Bitcoin', 'Privacy Coin')
wiki.add_edge('Bitcoin', 'Proof of Work')
wiki.add_edge('Bitcoin', 'Ripple (Geldsystem)')
wiki.add_edge('Bitcoin', 'SegWit')
wiki.add_edge('Bitcoin', 'Steem')
wiki.add_edge('Bitcoin', 'Surespot')
wiki.add_edge('Bitcoin', 'Telegram')
wiki.add_edge('Bitcoin', 'Tether')
wiki.add_edge('Bitcoin', 'Threema')
wiki.add_edge('Bitcoin', 'Tron (Kryptowährung)')
wiki.add_edge('Bitcoin', 'Waves (Kryptowährung)')
wiki.add_edge('Bitcoin', 'Webservice')
wiki.add_edge('Bitcoin', 'Zcash')
wiki.add_edge('Bitcoin', 'ZeroNet')
wiki.add_edge('Bitcoin Cash', 'Bitcoin SV')
wiki.add_edge('Bitcoin Cash', 'Bitstamp')
wiki.add_edge('Bitcoin Cash', 'Liste von Kryptowährungen')
wiki.add_edge('Bitcoin SV', 'Liste von Kryptowährungen')
wiki.add_edge('Bitstamp', 'Ethereum')
wiki.add_edge('Bitstamp', 'Ripple (Geldsystem)')
wiki.add_edge('Bonjour (Apple)', 'AirPrint')
wiki.add_edge('Bonjour (Apple)', 'AppleTalk')
wiki.add_edge('Bonjour (Apple)', 'Digital Audio Access Protocol')
wiki.add_edge('Bonjour (Apple)', 'Java Native Interface')
wiki.add_edge('Bonjour (Apple)', 'JetDirect')
wiki.add_edge('Bonjour (Apple)', 'NAT Port Mapping Protocol')
wiki.add_edge('Bonjour (Apple)', 'Service Location Protocol')
wiki.add_edge('Bonjour (Apple)', 'Streaming-Protokoll')
wiki.add_edge('Bootstrap Protocol', '0.0.0.0')
wiki.add_edge('Bootstrap Protocol', 'Boot Service Discovery Protocol')
wiki.add_edge('Bootstrap Protocol', 'Liste der standardisierten Ports')
wiki.add_edge('Bootstrap Protocol', 'Network File System')
wiki.add_edge('Bootstrap Protocol', 'Parameter (Informatik)')
wiki.add_edge('Bootstrap Protocol', 'Preboot Execution Environment')
wiki.add_edge('Bootstrap Protocol', 'Reverse Address Resolution Protocol')
wiki.add_edge('Border Gateway Protocol', 'Aggregation (Informatik)')
wiki.add_edge('Border Gateway Protocol', 'Distanzvektoralgorithmus')
wiki.add_edge('Border Gateway Protocol', 'Enhanced Interior Gateway Routing Protocol')
wiki.add_edge('Border Gateway Protocol', 'Exterior-Gateway-Protokoll')
wiki.add_edge('Border Gateway Protocol', 'IS-IS')
wiki.add_edge('Border Gateway Protocol', 'Interior Gateway Protocol')
wiki.add_edge('Border Gateway Protocol', 'Interior Gateway Routing Protocol')
wiki.add_edge('Border Gateway Protocol', 'Liste von IPv6-Tunnelbrokern')
wiki.add_edge('Border Gateway Protocol', 'Locator/Identifier Separation Protocol')
wiki.add_edge('Border Gateway Protocol', 'Netflow')
wiki.add_edge('Border Gateway Protocol', 'Open Shortest Path First')
wiki.add_edge('Border Gateway Protocol', 'Pfadvektorprotokoll')
wiki.add_edge('Border Gateway Protocol', 'Whatsapp')
wiki.add_edge('Bouncy Castle', 'Time Stamp Protocol')
wiki.add_edge('BroadR-Reach', '100BASE-TX')
wiki.add_edge('Brücke (Entwurfsmuster)', 'Adapter (Entwurfsmuster)')
wiki.add_edge('Brücke (Entwurfsmuster)', 'Java Authentication and Authorization Service')
wiki.add_edge('Bulkhead', 'Durchsatzratenbegrenzung')
wiki.add_edge('CANaerospace', 'ARINC 825')
wiki.add_edge('CAcert', 'POP3S')
wiki.add_edge('CAcert', 'SMTPS')
wiki.add_edge('CRUD', 'ADO.NET')
wiki.add_edge('CRUD', 'Command-Query-Responsibility-Segregation')
wiki.add_edge('CRUD', 'Entity Framework')
wiki.add_edge('CRUD', 'Framework')
wiki.add_edge('CRUD', 'Java Database Connectivity')
wiki.add_edge('CRUD', 'Model View Controller')
wiki.add_edge('CRUD', 'Open Data Protocol')
wiki.add_edge('CRUD', 'SQLAlchemy')
wiki.add_edge('CSLIP', 'ROHC')
wiki.add_edge('CalDAV', 'CardDAV')
wiki.add_edge('CalDAV', 'GroupDAV')
wiki.add_edge('CalDAV', 'JSON Meta Application Protocol')
wiki.add_edge('Call Level Interface', 'Java Database Connectivity')
wiki.add_edge('Call Level Interface', 'Modul (Software)')
wiki.add_edge('Canvas (HTML-Element)', 'Web Audio API')
wiki.add_edge('Canvas (HTML-Element)', 'Web Worker')
wiki.add_edge('Canvas (HTML-Element)', 'WebVR')
wiki.add_edge('Carbon (Apple)', 'Macintosh-Baukasten')
wiki.add_edge('Carbon (Apple)', 'QuickDraw')
wiki.add_edge('CardDAV', 'Exchange ActiveSync')
wiki.add_edge('CardDAV', 'JSON Meta Application Protocol')
wiki.add_edge('Carrier-grade NAT', 'American Registry for Internet Numbers')
wiki.add_edge('Carrier-grade NAT', 'NAT64')
wiki.add_edge('Certificate Management Protocol', 'Bouncy Castle')
wiki.add_edge('Certificate Management Protocol', 'Certificate Request Message Format')
wiki.add_edge('Certificate Request Message Format', 'Bouncy Castle')
wiki.add_edge('Cisco Group Management Protocol', 'GMRP')
wiki.add_edge('Class of Service', 'Delay Tolerant Networking')
wiki.add_edge('Class of Service', 'Frame Relay')
wiki.add_edge('Class of Service', 'IEEE 802.1p')
wiki.add_edge('Class of Service', 'Service Advertising Protocol')
wiki.add_edge('Classifier (UML)', 'Kompositionsstrukturdiagramm')
wiki.add_edge('Classifier (UML)', 'Merkmal (UML)')
wiki.add_edge('Classless Inter-Domain Routing', '6to4')
wiki.add_edge('Classless Inter-Domain Routing', 'Aggregation (Informatik)')
wiki.add_edge('Classless Inter-Domain Routing', 'Border Gateway Protocol')
wiki.add_edge('Classless Inter-Domain Routing', 'Dotted decimal notation')
wiki.add_edge('Classless Inter-Domain Routing', 'Enhanced Interior Gateway Routing Protocol')
wiki.add_edge('Classless Inter-Domain Routing', 'IS-IS')
wiki.add_edge('Classless Inter-Domain Routing', 'Link-State')
wiki.add_edge('Classless Inter-Domain Routing', 'Open Shortest Path First')
wiki.add_edge('Classless Inter-Domain Routing', 'Private IP-Adresse')
wiki.add_edge('Classless Inter-Domain Routing', 'Provider Independent Address Space')
wiki.add_edge('Classless Inter-Domain Routing', 'RIPE Network Coordination Centre')
wiki.add_edge('Classless Inter-Domain Routing', 'Routing Information Protocol')
wiki.add_edge('Classless Inter-Domain Routing', 'Source-specific Multicast')
wiki.add_edge('Classless Inter-Domain Routing', 'Variable Length Subnet Mask')
wiki.add_edge('Client', '464XLAT')
wiki.add_edge('Client', 'ARINC 825')
wiki.add_edge('Client', 'Anwendungsserver')
wiki.add_edge('Client', 'Apache Axis')
wiki.add_edge('Client', 'Apache Thrift')
wiki.add_edge('Client', 'Apple Filing Protocol')
wiki.add_edge('Client', 'AppleShare')
wiki.add_edge('Client', 'BattleEye RCon Protocol')
wiki.add_edge('Client', 'BitTorrent')
wiki.add_edge('Client', 'Bitcoin')
wiki.add_edge('Client', 'Boot Service Discovery Protocol')
wiki.add_edge('Client', 'Bootstrap Protocol')
wiki.add_edge('Client', 'Business Delegate')
wiki.add_edge('Client', 'CANaerospace')
wiki.add_edge('Client', 'Common Open Policy Service')
wiki.add_edge('Client', 'CyberGhost VPN')
wiki.add_edge('Client', 'DNS over TLS')
wiki.add_edge('Client', 'Datagramm')
wiki.add_edge('Client', 'Diffusing Update Algorithm')
wiki.add_edge('Client', 'Dynamic Data Exchange')
wiki.add_edge('Client', 'Echo (Netzwerkdienst)')
wiki.add_edge('Client', 'Extension Interface')
wiki.add_edge('Client', 'External Data Representation')
wiki.add_edge('Client', 'FastCGI')
wiki.add_edge('Client', 'FastTrack')
wiki.add_edge('Client', 'File Service Protocol')
wiki.add_edge('Client', 'File Transfer Protocol')
wiki.add_edge('Client', 'File eXchange Protocol')
wiki.add_edge('Client', 'GSSAPI')
wiki.add_edge('Client', 'Geodienst')
wiki.add_edge('Client', 'Gopher')
wiki.add_edge('Client', 'Hypertext Transfer Protocol Secure')
wiki.add_edge('Client', 'Immediate Mode (Computergrafik)')
wiki.add_edge('Client', 'InterAsterisk eXchange')
wiki.add_edge('Client', 'Internet Message Access Protocol')
wiki.add_edge('Client', 'Internet Relay Chat')
wiki.add_edge('Client', 'JSON-RPC')
wiki.add_edge('Client', 'Jakarta Enterprise Beans')
wiki.add_edge('Client', 'Jakarta Messaging')
wiki.add_edge('Client', 'Kerberos (Protokoll)')
wiki.add_edge('Client', 'Kopplung (Softwareentwicklung)')
wiki.add_edge('Client', 'Litecoin')
wiki.add_edge('Client', 'LocalTalk')
wiki.add_edge('Client', 'Lookeen')
wiki.add_edge('Client', 'MMS-Protokoll')
wiki.add_edge('Client', 'Matrix (Kommunikationsprotokoll)')
wiki.add_edge('Client', 'Model 2')
wiki.add_edge('Client', 'Model View Controller')
wiki.add_edge('Client', 'Multi-purpose Business Security over IP')
wiki.add_edge('Client', 'Mumble')
wiki.add_edge('Client', 'NTLM')
wiki.add_edge('Client', 'Network File System')
wiki.add_edge('Client', 'Netzwerkadressübersetzung')
wiki.add_edge('Client', 'Netzwerkprotokoll')
wiki.add_edge('Client', 'OSCAR (Protokoll)')
wiki.add_edge('Client', 'Off-the-Record Messaging')
wiki.add_edge('Client', 'On-Demand Mail Relay')
wiki.add_edge('Client', 'Open Database Connectivity')
wiki.add_edge('Client', 'Port Based Network Access Control')
wiki.add_edge('Client', 'Post Office Protocol')
wiki.add_edge('Client', 'Preboot Execution Environment')
wiki.add_edge('Client', 'Production Performance Management Protocol')
wiki.add_edge('Client', 'Prospero (Protokoll)')
wiki.add_edge('Client', 'Protocol for SYnchronous Conferencing')
wiki.add_edge('Client', 'Real-Time Streaming Protocol')
wiki.add_edge('Client', 'Remote Authentication Dial-In User Service')
wiki.add_edge('Client', 'Remote Framebuffer Protocol')
wiki.add_edge('Client', 'Remote Method Invocation')
wiki.add_edge('Client', 'Retained Mode')
wiki.add_edge('Client', 'Reverse Connection')
wiki.add_edge('Client', 'Ripple (Geldsystem)')
wiki.add_edge('Client', 'Rsync')
wiki.add_edge('Client', 'SMTPS')
wiki.add_edge('Client', 'SOCKS')
wiki.add_edge('Client', 'SPDY')
wiki.add_edge('Client', 'SPICE (Protokoll)')
wiki.add_edge('Client', 'SSH File Transfer Protocol')
wiki.add_edge('Client', 'STARTTLS')
wiki.add_edge('Client', 'Secure Copy')
wiki.add_edge('Client', 'Serverseitige Anwendung')
wiki.add_edge('Client', 'Service Advertising Protocol')
wiki.add_edge('Client', 'Session Announcement Protocol')
wiki.add_edge('Client', 'Session Initiation Protocol')
wiki.add_edge('Client', 'Signal (Messenger)')
wiki.add_edge('Client', 'Simple Mail Access Protocol')
wiki.add_edge('Client', 'Socket (Software)')
wiki.add_edge('Client', 'Strukturmuster')
wiki.add_edge('Client', 'TCP Fast Open')
wiki.add_edge('Client', 'TOC (Protokoll)')
wiki.add_edge('Client', 'Transport Layer Security')
wiki.add_edge('Client', 'Tunnel (Rechnernetz)')
wiki.add_edge('Client', 'Typbibliothek')
wiki.add_edge('Client', 'Unified Diagnostic Services')
wiki.add_edge('Client', 'Virtual Network Computing')
wiki.add_edge('Client', 'Web Feature Service')
wiki.add_edge('Client', 'Web Map Service')
wiki.add_edge('Client', 'Web Processing Service')
wiki.add_edge('Client', 'Web Proxy Autodiscovery Protocol')
wiki.add_edge('Client', 'Webservice')
wiki.add_edge('Client', 'Whois')
wiki.add_edge('Client', 'Windows ML')
wiki.add_edge('Client', 'Windows Server Update Services')
wiki.add_edge('Client', 'X Window System')
wiki.add_edge('Client', 'XIMSS')
wiki.add_edge('Client', 'Xfire')
wiki.add_edge('Client', 'Zeroconf')
wiki.add_edge('Cocoa (API)', 'Apache Thrift')
wiki.add_edge('Cocoa (API)', 'Carbon (Apple)')
wiki.add_edge('Cocoa (API)', 'Core Animation')
wiki.add_edge('Cocoa (API)', 'Core Image')
wiki.add_edge('Cocoa (API)', 'Framework')
wiki.add_edge('Cocoa (API)', 'Grand Central Dispatch')
wiki.add_edge('Cocoa (API)', 'Macintosh-Baukasten')
wiki.add_edge('Cocoa (API)', 'Model View Controller')
wiki.add_edge('Cocoa (API)', 'Model View ViewModel')
wiki.add_edge('Cocoa (API)', 'Undo')
wiki.add_edge('Code-Smell', 'Gottobjekt')
wiki.add_edge('Code-Smell', 'Markierungsschnittstelle')
wiki.add_edge('Code-Smell', 'Quelltextklon')
wiki.add_edge('Code-Smell', 'ReSharper')
wiki.add_edge('Code-Smell', 'Softwareerosion')
wiki.add_edge('Command-Query-Responsibility-Segregation', 'Event Sourcing')
wiki.add_edge('Command-Query-Responsibility-Segregation', 'Serviceorientierte Architektur')
wiki.add_edge('Common Address Redundancy Protocol', 'Pfsync')
wiki.add_edge('Common Address Redundancy Protocol', 'Virtual Router Redundancy Protocol')
wiki.add_edge('Common Gateway Interface', 'FastCGI')
wiki.add_edge('Common Gateway Interface', 'Perl Web Server Gateway Interface')
wiki.add_edge('Common Gateway Interface', 'Simple Common Gateway Interface')
wiki.add_edge('Common ISDN Application Programming Interface', 'H.323')
wiki.add_edge('Common ISDN Application Programming Interface', 'Network Driver Interface Specification')
wiki.add_edge('Common ISDN Application Programming Interface', 'Telephony Application Programming Interface')
wiki.add_edge('Common Object Request Broker Architecture', 'Apache CXF')
wiki.add_edge('Common Object Request Broker Architecture', 'Bonobo (GNOME)')
wiki.add_edge('Common Object Request Broker Architecture', 'Brücke (Entwurfsmuster)')
wiki.add_edge('Common Object Request Broker Architecture', 'Cross Platform Component Object Model')
wiki.add_edge('Common Object Request Broker Architecture', 'Etch (Protokoll)')
wiki.add_edge('Common Object Request Broker Architecture', 'Internet Communications Engine')
wiki.add_edge('Common Object Request Broker Architecture', 'Jakarta Messaging')
wiki.add_edge('Common Object Request Broker Architecture', 'Java Naming and Directory Interface')
wiki.add_edge('Common Object Request Broker Architecture', 'Java Native Interface')
wiki.add_edge('Common Object Request Broker Architecture', 'Komponente (Software)')
wiki.add_edge('Common Object Request Broker Architecture', 'Komponentenbasierte Entwicklung')
wiki.add_edge('Common Object Request Broker Architecture', 'Kopplung (Softwareentwicklung)')
wiki.add_edge('Common Object Request Broker Architecture', 'Pull-Model')
wiki.add_edge('Common Object Request Broker Architecture', 'Remote Method Invocation')
wiki.add_edge('Common Object Request Broker Architecture', 'SIMON')
wiki.add_edge('Common Object Request Broker Architecture', 'XML-RPC')
wiki.add_edge('Component Object Model', 'Binärstandard')
wiki.add_edge('Component Object Model', 'Common Object Request Broker Architecture')
wiki.add_edge('Component Object Model', 'Cross Platform Component Object Model')
wiki.add_edge('Component Object Model', 'DirectX')
wiki.add_edge('Component Object Model', 'DirectX Media Objects')
wiki.add_edge('Component Object Model', 'Internet Communications Engine')
wiki.add_edge('Component Object Model', 'Killbit')
wiki.add_edge('Component Object Model', 'Komponente (Software)')
wiki.add_edge('Component Object Model', 'Kopplung (Softwareentwicklung)')
wiki.add_edge('Component Object Model', 'Messaging Application Programming Interface')
wiki.add_edge('Component Object Model', 'OLE DB')
wiki.add_edge('Component Object Model', 'Object Linking and Embedding')
wiki.add_edge('Component Object Model', 'Telephony Application Programming Interface')
wiki.add_edge('Component Object Model', 'Universally Unique Identifier')
wiki.add_edge('Connectionless Network Protocol', 'Datagramm')
wiki.add_edge('Connectionless Network Protocol', 'IP-Header')
wiki.add_edge('Connectionless Network Protocol', 'IS-IS')
wiki.add_edge('Connectionless Network Protocol', 'Streaming-Protokoll')
wiki.add_edge('Connectionless Network Protocol', 'Vermittlungsschicht')
wiki.add_edge('Constrained Application Protocol', 'Certificate Management Protocol')
wiki.add_edge('Container (Entwurfsmuster)', 'Bulkhead')
wiki.add_edge('Container (Entwurfsmuster)', 'Kompositum (Entwurfsmuster)')
wiki.add_edge('Controller Area Network', 'ARINC 825')
wiki.add_edge('Controller Area Network', 'CANaerospace')
wiki.add_edge('Controller Area Network', 'CANopen')
wiki.add_edge('Controller Area Network', 'DoIP')
wiki.add_edge('Controller Area Network', 'FlexRay')
wiki.add_edge('Controller Area Network', 'ISO 15765-2')
wiki.add_edge('Controller Area Network', 'ISOBUS')
wiki.add_edge('Controller Area Network', 'KWP2000')
wiki.add_edge('Controller Area Network', 'Open Platform Communications')
wiki.add_edge('Controller Area Network', 'SAE J1939')
wiki.add_edge('Controller Area Network', 'Teilnetzbetrieb')
wiki.add_edge('Controller Area Network', 'Unified Diagnostic Services')
wiki.add_edge('Controller Area Network', 'Universal Measurement and Calibration Protocol')
wiki.add_edge('Core Animation', 'Core Audio')
wiki.add_edge('Core Image', 'Core Animation')
wiki.add_edge('Core Image', 'Core Audio')
wiki.add_edge('DHCPv6', 'Liste der standardisierten Ports')
wiki.add_edge('DNP3', 'IEC 62351')
wiki.add_edge('DNS over HTTPS', 'DNSCurve')
wiki.add_edge('DNS over TLS', 'DNS over HTTPS')
wiki.add_edge('DNS over TLS', 'DNSCurve')
wiki.add_edge('DSSI', 'LV2')
wiki.add_edge('DSSI', 'Linux Audio Developer’s Simple Plugin API')
wiki.add_edge('Dash (Kryptowährung)', 'Cardano (Blockchainnetzwerk)')
wiki.add_edge('Dash (Kryptowährung)', 'Dezentralisierte Autonome Organisation')
wiki.add_edge('Dash (Kryptowährung)', 'Petro (Kryptowährung)')
wiki.add_edge('Data Access Object', 'Apache Tuscany')
wiki.add_edge('Data Access Object', 'MyBatis')
wiki.add_edge('Data Access Objects', 'Data Access Object')
wiki.add_edge('Data Access Objects', 'OLE DB')
wiki.add_edge('Data Access Objects', 'Remote Data Objects')
wiki.add_edge('Data-Dictionary', 'Datenflussdiagramm')
wiki.add_edge('Data-Dictionary', 'Innovator (Software)')
wiki.add_edge('Data-Dictionary', 'Semistrukturierte Daten')
wiki.add_edge('DataNucleus', 'Jakarta Persistence API')
wiki.add_edge('Datagram Congestion Control Protocol', 'Multipath TCP')
wiki.add_edge('Datagram Congestion Control Protocol', 'Protokoll (IP)')
wiki.add_edge('Datagram Congestion Control Protocol', 'Real-Time Transport Protocol')
wiki.add_edge('Datagram Congestion Control Protocol', 'Sequenznummer')
wiki.add_edge('Datagram Congestion Control Protocol', 'Streaming-Protokoll')
wiki.add_edge('Datagram Congestion Control Protocol', 'Transportschicht')
wiki.add_edge('Datagram Delivery Protocol', 'AppleShare')
wiki.add_edge('Datagram Delivery Protocol', 'EtherTalk Link Access Protocol')
wiki.add_edge('Datagram Delivery Protocol', 'LocalTalk')
wiki.add_edge('Datagram Delivery Protocol', 'LocalTalk Link Access Protocol')
wiki.add_edge('Datagram Delivery Protocol', 'Protokoll (IP)')
wiki.add_edge('Datagram Delivery Protocol', 'TokenTalk Link Access Protocol')
wiki.add_edge('Datagram Transport Layer Security', 'Certificate Management Protocol')
wiki.add_edge('Datagram Transport Layer Security', 'Ethernet')
wiki.add_edge('Datagram Transport Layer Security', 'Fiber Distributed Data Interface')
wiki.add_edge('Datagram Transport Layer Security', 'GnuTLS')
wiki.add_edge('Datagram Transport Layer Security', 'Heartbleed')
wiki.add_edge('Datagram Transport Layer Security', 'IPv4')
wiki.add_edge('Datagram Transport Layer Security', 'IPv6')
wiki.add_edge('Datagram Transport Layer Security', 'Internet Protocol')
wiki.add_edge('Datagram Transport Layer Security', 'OpenSSL')
wiki.add_edge('Datagram Transport Layer Security', 'Session Initiation Protocol')
wiki.add_edge('Datagram Transport Layer Security', 'Token Ring')
wiki.add_edge('Datagram Transport Layer Security', 'Transport Layer Security')
wiki.add_edge('Datagram Transport Layer Security', 'User Datagram Protocol')
wiki.add_edge('Datagram Transport Layer Security', 'Verschlüsselungsprotokoll')
wiki.add_edge('Datagram Transport Layer Security', 'WebRTC')
wiki.add_edge('Datagram Transport Layer Security', 'Wire (Messenger)')
wiki.add_edge('Datagramm', 'Internet Group Management Protocol')
wiki.add_edge('Datagramm', 'Netflow')
wiki.add_edge('Datasource', 'Java Transaction Service')
wiki.add_edge('Dataspaces', 'International Data Spaces')
wiki.add_edge('Dateikonverter', 'Anwendungsserver')
wiki.add_edge('Dateikonverter', 'Java Sound')
wiki.add_edge('Datenaustausch', 'AS4 (Schnittstelle)')
wiki.add_edge('Datenaustausch', 'Common Gateway Interface')
wiki.add_edge('Datenaustausch', 'Dateikonverter')
wiki.add_edge('Datenaustausch', 'Dynamic Data Exchange')
wiki.add_edge('Datenaustausch', 'ESTRI Gateway')
wiki.add_edge('Datenaustausch', 'Extensible Messaging and Presence Protocol')
wiki.add_edge('Datenaustausch', 'Hessian (Webprotokoll)')
wiki.add_edge('Datenaustausch', 'International Data Spaces')
wiki.add_edge('Datenaustausch', 'Noise Protocol Framework')
wiki.add_edge('Datenaustausch', 'Open Banking')
wiki.add_edge('Datenaustausch', 'Open Platform Communications')
wiki.add_edge('Datenaustausch', 'Positive Acknowledgement with Retransmission')
wiki.add_edge('Datenaustausch', 'SOAP')
wiki.add_edge('Datenaustausch', 'Umati')
wiki.add_edge('Datenbankabstraktionsschicht', 'ADOdb Lite')
wiki.add_edge('Datenbankabstraktionsschicht', 'Data Source Name')
wiki.add_edge('Datenbankmodell', 'Datenbankdesign')
wiki.add_edge('Datenbankmodell', 'Innovator (Software)')
wiki.add_edge('Datenbankmodell', 'Netzwerkdatenbankmodell')
wiki.add_edge('Datenbankmodell', 'Semistrukturierte Daten')
wiki.add_edge('Datenbankschnittstelle', 'ADO.NET')
wiki.add_edge('Datenbankschnittstelle', 'ADOdb Lite')
wiki.add_edge('Datenbankschnittstelle', 'Borland Database Engine')
wiki.add_edge('Datenbankschnittstelle', 'Call Level Interface')
wiki.add_edge('Datenbankschnittstelle', 'Creole (Framework)')
wiki.add_edge('Datenbankschnittstelle', 'Datenbankabstraktionsschicht')
wiki.add_edge('Datenbankschnittstelle', 'Java Database Connectivity')
wiki.add_edge('Datenbankschnittstelle', 'OpenDBX')
wiki.add_edge('Datenbankschnittstelle', 'Perl-DBI')
wiki.add_edge('Datenbankschnittstelle', 'Propel')
wiki.add_edge('Datenfluss', 'Datenflussdiagramm')
wiki.add_edge('Datenfluss', 'Future (Programmierung)')
wiki.add_edge('Datenfluss', 'Kopplung (Softwareentwicklung)')
wiki.add_edge('Datenfluss', 'Objektfluss (UML)')
wiki.add_edge('Datenfluss', 'WS-Flow Language')
wiki.add_edge('Datenflussdiagramm', 'Pufferknoten (UML)')
wiki.add_edge('Datenmodell', 'Data-Dictionary')
wiki.add_edge('Datenmodell', 'Datenbankmodell')
wiki.add_edge('Datenmodell', 'Denormalisierung')
wiki.add_edge('Datenmodell', 'Entität (Informatik)')
wiki.add_edge('Datenmodell', 'Schema (Informatik)')
wiki.add_edge('Datenmodell', 'Sternschema')
wiki.add_edge('Datenmodell', 'TOGAF')
wiki.add_edge('Datenmodell', 'Zachman Framework')
wiki.add_edge('Datenmodellierung', 'ANSI-SPARC-Architektur')
wiki.add_edge('Datenmodellierung', 'Data Vault')
wiki.add_edge('Datenmodellierung', 'Data-Dictionary')
wiki.add_edge('Datenmodellierung', 'Datenbankmodell')
wiki.add_edge('Datenmodellierung', 'Datenmodell')
wiki.add_edge('Datenmodellierung', 'Entität (Informatik)')
wiki.add_edge('Datenmodellierung', 'IDEF1X')
wiki.add_edge('Datenmodellierung', 'Innovator (Software)')
wiki.add_edge('Datenmodellierung', 'Kardinalität (Datenbankmodellierung)')
wiki.add_edge('Datenmodellierung', 'Liste von Datenmodellierungswerkzeugen')
wiki.add_edge('Datenmodellierung', 'Normalisierung (Datenbank)')
wiki.add_edge('Datenmodellierung', 'Schema (Informatik)')
wiki.add_edge('Datenmodellierung', 'Schwache Entität')
wiki.add_edge('Datenmodellierung', 'Softwaredesign')
wiki.add_edge('Datenmodellierung', 'Structured-Entity-Relationship-Modell')
wiki.add_edge('Datenmodellierung', 'Synthesealgorithmus')
wiki.add_edge('Daytime', 'Discard')
wiki.add_edge('Daytime', 'Echo (Netzwerkdienst)')
wiki.add_edge('Decorator', 'Delegation (Softwareentwicklung)')
wiki.add_edge('Decorator', 'Kompositum (Entwurfsmuster)')
wiki.add_edge('Decorator', 'Stellvertreter (Entwurfsmuster)')
wiki.add_edge('Decorator', 'Strategie (Entwurfsmuster)')
wiki.add_edge('Decorator', 'UMLet')
wiki.add_edge('Decorator', 'Zuständigkeitskette')
wiki.add_edge('Deep Packet Inspection', 'Durchsatzratenbegrenzung')
wiki.add_edge('Denormalisierung', 'ANSI-SPARC-Architektur')
wiki.add_edge('Denormalisierung', 'Apache ServiceMix')
wiki.add_edge('Denormalisierung', 'Binary Large Object')
wiki.add_edge('Denormalisierung', 'Datenbankdesign')
wiki.add_edge('Denormalisierung', 'Event Sourcing')
wiki.add_edge('Denormalisierung', 'Schneeflockenschema')
wiki.add_edge('Denormalisierung', 'Sternschema')
wiki.add_edge('Dia (Software)', 'Microsoft Visio')
wiki.add_edge('Dia (Software)', 'Umbrello')
wiki.add_edge('Diameter (Protokoll)', 'Extensible Authentication Protocol')
wiki.add_edge('Diem (Internetwährung)', 'Ethereum')
wiki.add_edge('Diem (Internetwährung)', 'Proof of Work')
wiki.add_edge('Diem (Internetwährung)', 'Stablecoin')
wiki.add_edge('Dienst (Informatik)', 'Delegation (Softwareentwicklung)')
wiki.add_edge('Dienst (Informatik)', 'Entität (Informatik)')
wiki.add_edge('Dienst (Informatik)', 'Microservices')
wiki.add_edge('Dienst (Informatik)', 'Model View ViewModel')
wiki.add_edge('Dienst (Informatik)', 'Open Cirrus')
wiki.add_edge('Dienst (Informatik)', 'Saga (Entwurfsmuster)')
wiki.add_edge('Dienst (Informatik)', 'Software-Ökosystem')
wiki.add_edge('Dienstekomposition', 'SoaML')
wiki.add_edge('DiffServ', 'Explicit Congestion Notification')
wiki.add_edge('DiffServ', 'IEEE 802.1p')
wiki.add_edge('DiffServ', 'IntServ')
wiki.add_edge('DiffServ', 'Resource Reservation Protocol')
wiki.add_edge('Diffusing Update Algorithm', 'Babel (Protokoll)')
wiki.add_edge('Digital Subscriber System No. 1', 'Common ISDN Application Programming Interface')
wiki.add_edge('Digital Subscriber System No. 1', 'Link Access Procedure for the D-Channel')
wiki.add_edge('Digital Subscriber System No. 1', 'Q.931')
wiki.add_edge('DirectX', 'AMD Mantle')
wiki.add_edge('DirectX', 'Canvas (HTML-Element)')
wiki.add_edge('DirectX', 'DirectX Media Objects')
wiki.add_edge('DirectX', 'DirectX Raytracing')
wiki.add_edge('DirectX', 'Gallium3D')
wiki.add_edge('DirectX', 'Graphics Device Interface')
wiki.add_edge('DirectX', 'Java 3D')
wiki.add_edge('DirectX', 'Mumble')
wiki.add_edge('DirectX', 'OpenAL')
wiki.add_edge('DirectX', 'OpenCL')
wiki.add_edge('DirectX', 'OpenGL')
wiki.add_edge('DirectX', 'OpenML')
wiki.add_edge('DirectX', 'Virtual Studio Technology')
wiki.add_edge('DirectX', 'Vulkan (API)')
wiki.add_edge('DirectX', 'WinG')
wiki.add_edge('DirectX', 'Winsock')
wiki.add_edge('DirectX', 'XNA (Microsoft)')
wiki.add_edge('DirectX Media Objects', 'Multimedia-Framework')
wiki.add_edge('Distance Vector Multicast Routing Protocol', 'MOSPF')
wiki.add_edge('Distanzvektoralgorithmus', '6LoWPAN')
wiki.add_edge('Distanzvektoralgorithmus', 'Balanced hybrid')
wiki.add_edge('Distanzvektoralgorithmus', 'Routing Protocol for Low power and Lossy Networks')
wiki.add_edge('Distanzvektoralgorithmus', 'Triggered Updates')
wiki.add_edge('Document Object Model', 'Apache Axis')
wiki.add_edge('Document Object Model', 'Cross Platform Component Object Model')
wiki.add_edge('Document Object Model', 'Hypertext Transfer Protocol')
wiki.add_edge('Document Object Model', 'Jakarta XML Binding')
wiki.add_edge('Document Object Model', 'Java API for XML Processing')
wiki.add_edge('Document Object Model', 'JavaFX')
wiki.add_edge('Document Object Model', 'MSXML')
wiki.add_edge('Document Object Model', 'Model View ViewModel')
wiki.add_edge('Document Object Model', 'Objektmodell')
wiki.add_edge('Document Object Model', 'Programmierschnittstelle')
wiki.add_edge('Document Object Model', 'QXML DOM')
wiki.add_edge('Document Object Model', 'SOAP')
wiki.add_edge('Document Object Model', 'Simple API for XML')
wiki.add_edge('Document Object Model', 'Streaming API for XML')
wiki.add_edge('Document Object Model', 'Web Storage')
wiki.add_edge('Document Object Model', 'Web Worker')
wiki.add_edge('Document Object Model', 'Webservice')
wiki.add_edge('Document Object Model', 'XFire (Framework)')
wiki.add_edge('Document Object Model', 'XMLHttpRequest')
wiki.add_edge('Dogecoin', 'Liste von Kryptowährungen')
wiki.add_edge('Dogecoin', 'Proof of Work')
wiki.add_edge('Dogecoin', 'Shiba Inu (Kryptowährung)')
wiki.add_edge('Domain Name System', '464XLAT')
wiki.add_edge('Domain Name System', 'Bitcoin')
wiki.add_edge('Domain Name System', 'Bonjour (Apple)')
wiki.add_edge('Domain Name System', 'CSLIP')
wiki.add_edge('Domain Name System', 'Client')
wiki.add_edge('Domain Name System', 'DHCPv6')
wiki.add_edge('Domain Name System', 'DNS over HTTPS')
wiki.add_edge('Domain Name System', 'DNS over TLS')
wiki.add_edge('Domain Name System', 'DNSCurve')
wiki.add_edge('Domain Name System', 'Datagram Transport Layer Security')
wiki.add_edge('Domain Name System', 'Datasource')
wiki.add_edge('Domain Name System', 'Diameter (Protokoll)')
wiki.add_edge('Domain Name System', 'Domain Name System Security Extensions')
wiki.add_edge('Domain Name System', 'Dynamic Host Configuration Protocol')
wiki.add_edge('Domain Name System', 'ETRN')
wiki.add_edge('Domain Name System', 'Explicit Congestion Notification')
wiki.add_edge('Domain Name System', 'I2P')
wiki.add_edge('Domain Name System', 'IP Address Management')
wiki.add_edge('Domain Name System', 'IP-Adresse')
wiki.add_edge('Domain Name System', 'IPv4')
wiki.add_edge('Domain Name System', 'IPv6')
wiki.add_edge('Domain Name System', 'InterPlanetary File System')
wiki.add_edge('Domain Name System', 'Internet Assigned Numbers Authority')
wiki.add_edge('Domain Name System', 'Internet Corporation for Assigned Names and Numbers')
wiki.add_edge('Domain Name System', 'Internet Protocol')
wiki.add_edge('Domain Name System', 'Internetprotokollfamilie')
wiki.add_edge('Domain Name System', 'Java Naming and Directory Interface')
wiki.add_edge('Domain Name System', 'Kerberos (Protokoll)')
wiki.add_edge('Domain Name System', 'Kryptowährung')
wiki.add_edge('Domain Name System', 'Lightweight Directory Access Protocol')
wiki.add_edge('Domain Name System', 'Link-local Multicast Name Resolution')
wiki.add_edge('Domain Name System', 'Liste der standardisierten Ports')
wiki.add_edge('Domain Name System', 'Liste von IPv6-Tunnelbrokern')
wiki.add_edge('Domain Name System', 'Liste von TCP/IP-basierten Netzwerkdiensten')
wiki.add_edge('Domain Name System', 'Mobile IP')
wiki.add_edge('Domain Name System', 'NAT64')
wiki.add_edge('Domain Name System', 'NetBIOS')
wiki.add_edge('Domain Name System', 'Network Information Center')
wiki.add_edge('Domain Name System', 'OpenNIC')
wiki.add_edge('Domain Name System', 'Peer Name Resolution Protocol')
wiki.add_edge('Domain Name System', 'Point-to-Point Protocol')
wiki.add_edge('Domain Name System', 'Port (Protokoll)')
wiki.add_edge('Domain Name System', 'Proxy Auto-Config')
wiki.add_edge('Domain Name System', 'RIPE Network Coordination Centre')
wiki.add_edge('Domain Name System', 'Resource Reservation Protocol')
wiki.add_edge('Domain Name System', 'Rogue DHCP')
wiki.add_edge('Domain Name System', 'STARTTLS')
wiki.add_edge('Domain Name System', 'Serial Line Internet Protocol')
wiki.add_edge('Domain Name System', 'Server Message Block')
wiki.add_edge('Domain Name System', 'Server Name Indication')
wiki.add_edge('Domain Name System', 'Simple Mail Transfer Protocol')
wiki.add_edge('Domain Name System', 'Subnetz')
wiki.add_edge('Domain Name System', 'Telephone Number Mapping')
wiki.add_edge('Domain Name System', 'Transmission Control Protocol')
wiki.add_edge('Domain Name System', 'Transport Layer Security')
wiki.add_edge('Domain Name System', 'Trivial File Transfer Protocol')
wiki.add_edge('Domain Name System', 'Universally Unique Identifier')
wiki.add_edge('Domain Name System', 'Unix to Unix Copy')
wiki.add_edge('Domain Name System', 'User Datagram Protocol')
wiki.add_edge('Domain Name System', 'Virtual Local Area Network')
wiki.add_edge('Domain Name System', 'Web Proxy Autodiscovery Protocol')
wiki.add_edge('Domain Name System', 'WhatsApp')
wiki.add_edge('Domain Name System', 'Whois')
wiki.add_edge('Domain Name System', 'X.25')
wiki.add_edge('Domain Name System', 'ZeroNet')
wiki.add_edge('Domain Name System', 'Zeroconf')
wiki.add_edge('Domain Name System Security Extensions', '464XLAT')
wiki.add_edge('Domain Name System Security Extensions', 'DNS over HTTPS')
wiki.add_edge('Domain Name System Security Extensions', 'DNS over TLS')
wiki.add_edge('Domain Name System Security Extensions', 'DNSCurve')
wiki.add_edge('Doppelt überprüfte Sperrung', 'Initialization-on-demand holder idiom')
wiki.add_edge('Doppelt überprüfte Sperrung', 'Verzögerte Initialisierung')
wiki.add_edge('Dynamic Data Exchange', 'Netzwerk-DDE')
wiki.add_edge('Dynamic Host Configuration Protocol', '0.0.0.0')
wiki.add_edge('Dynamic Host Configuration Protocol', 'Address Resolution Protocol')
wiki.add_edge('Dynamic Host Configuration Protocol', 'Boot Service Discovery Protocol')
wiki.add_edge('Dynamic Host Configuration Protocol', 'Bootstrap Protocol')
wiki.add_edge('Dynamic Host Configuration Protocol', 'Client')
wiki.add_edge('Dynamic Host Configuration Protocol', 'IEEE 802.1X')
wiki.add_edge('Dynamic Host Configuration Protocol', 'IP Address Management')
wiki.add_edge('Dynamic Host Configuration Protocol', 'Internet Assigned Numbers Authority')
wiki.add_edge('Dynamic Host Configuration Protocol', 'Internet Control Message Protocol')
wiki.add_edge('Dynamic Host Configuration Protocol', 'Internet Protocol Control Protocol')
wiki.add_edge('Dynamic Host Configuration Protocol', 'Liste der standardisierten Ports')
wiki.add_edge('Dynamic Host Configuration Protocol', 'Liste von TCP/IP-basierten Netzwerkdiensten')
wiki.add_edge('Dynamic Host Configuration Protocol', 'Mobile IP')
wiki.add_edge('Dynamic Host Configuration Protocol', 'Neighbor Discovery Protocol')
wiki.add_edge('Dynamic Host Configuration Protocol', 'Network Time Protocol')
wiki.add_edge('Dynamic Host Configuration Protocol', 'Netzmaske')
wiki.add_edge('Dynamic Host Configuration Protocol', 'OpenNIC')
wiki.add_edge('Dynamic Host Configuration Protocol', 'Peg DHCP')
wiki.add_edge('Dynamic Host Configuration Protocol', 'Preboot Execution Environment')
wiki.add_edge('Dynamic Host Configuration Protocol', 'Profinet')
wiki.add_edge('Dynamic Host Configuration Protocol', 'Remote Authentication Dial-In User Service')
wiki.add_edge('Dynamic Host Configuration Protocol', 'Reverse Address Resolution Protocol')
wiki.add_edge('Dynamic Host Configuration Protocol', 'Rogue DHCP')
wiki.add_edge('Dynamic Host Configuration Protocol', 'Service Location Protocol')
wiki.add_edge('Dynamic Host Configuration Protocol', 'Session Initiation Protocol')
wiki.add_edge('Dynamic Host Configuration Protocol', 'Trivial File Transfer Protocol')
wiki.add_edge('Dynamic Host Configuration Protocol', 'Web Proxy Autodiscovery Protocol')
wiki.add_edge('Dynamic Host Configuration Protocol', 'Zeroconf')
wiki.add_edge('Dynamic Trunking Protocol', 'IEEE 802.1q')
wiki.add_edge('EDonkey2000', 'FastTrack')
wiki.add_edge('EOS (Kryptowährung)', 'Steem')
wiki.add_edge('ETRN', 'On-Demand Mail Relay')
wiki.add_edge('Echo (Netzwerkdienst)', 'Discard')
wiki.add_edge('Echtzeit-Ethernet', 'Advanced Physical Layer')
wiki.add_edge('Echtzeit-Ethernet', 'EtherCAT')
wiki.add_edge('Echtzeit-Ethernet', 'EtherNet/IP')
wiki.add_edge('Echtzeit-Ethernet', 'Ethernet Powerlink')
wiki.add_edge('Echtzeit-Ethernet', 'Industrial Ethernet')
wiki.add_edge('Echtzeit-Ethernet', 'Parallel Redundancy Protocol')
wiki.add_edge('Echtzeit-Ethernet', 'Profinet')
wiki.add_edge('Echtzeit-Ethernet', 'SERCOS III')
wiki.add_edge('Echtzeit-Ethernet', 'SafetyNET p')
wiki.add_edge('Echtzeit-Ethernet', 'Time-Sensitive Networking')
wiki.add_edge('EclipseLink', 'Apache OpenJPA')
wiki.add_edge('EclipseLink', 'Datenbankabstraktionsschicht')
wiki.add_edge('EclipseLink', 'TopLink')
wiki.add_edge('Electronic Banking Internet Communication Standard', 'Banking Communication Standard')
wiki.add_edge('Electronic Banking Internet Communication Standard', 'File Transfer, Access and Management')
wiki.add_edge('Embedded SQL', 'ADO.NET')
wiki.add_edge('Embedded SQL', 'Object-relational impedance mismatch')
wiki.add_edge('Energy Efficient Ethernet', 'Dante (Netzwerkprotokoll)')
wiki.add_edge('Enhanced Interior Gateway Routing Protocol', '6LoWPAN')
wiki.add_edge('Enhanced Interior Gateway Routing Protocol', 'Babel (Protokoll)')
wiki.add_edge('Enhanced Interior Gateway Routing Protocol', 'Balanced hybrid')
wiki.add_edge('Enhanced Interior Gateway Routing Protocol', 'Diffusing Update Algorithm')
wiki.add_edge('Enhanced Interior Gateway Routing Protocol', 'Distanzvektoralgorithmus')
wiki.add_edge('Enhanced Interior Gateway Routing Protocol', 'Generic Routing Encapsulation')
wiki.add_edge('Enhanced Interior Gateway Routing Protocol', 'Interior Gateway Routing Protocol')
wiki.add_edge('Enhanced Interior Gateway Routing Protocol', 'Open Shortest Path First')
wiki.add_edge('Enhanced Interior Gateway Routing Protocol', 'Protokoll (IP)')
wiki.add_edge('Enterprise Application Integration', 'Jakarta Connectors')
wiki.add_edge('Enterprise Application Integration', 'Webintegration')
wiki.add_edge('Entity Framework', 'Hibernate (Framework)')
wiki.add_edge('Entity-Relationship-Modell', 'Aggregation (Informatik)')
wiki.add_edge('Entity-Relationship-Modell', 'ArchiMate')
wiki.add_edge('Entity-Relationship-Modell', 'C4 model')
wiki.add_edge('Entity-Relationship-Modell', 'Datenbankmodell')
wiki.add_edge('Entity-Relationship-Modell', 'Datenmodell')
wiki.add_edge('Entity-Relationship-Modell', 'Datenmodellierung')
wiki.add_edge('Entity-Relationship-Modell', 'Diagrams.net')
wiki.add_edge('Entity-Relationship-Modell', 'Entität (Informatik)')
wiki.add_edge('Entity-Relationship-Modell', 'IDEF1X')
wiki.add_edge('Entity-Relationship-Modell', 'Innovator (Software)')
wiki.add_edge('Entity-Relationship-Modell', 'Jackson-Diagramm')
wiki.add_edge('Entity-Relationship-Modell', 'Kardinalität (Datenbankmodellierung)')
wiki.add_edge('Entity-Relationship-Modell', 'Liste von Datenmodellierungswerkzeugen')
wiki.add_edge('Entity-Relationship-Modell', 'Martin-Notation')
wiki.add_edge('Entity-Relationship-Modell', 'Min-Max-Notation')
wiki.add_edge('Entity-Relationship-Modell', 'Normalisierung (Datenbank)')
wiki.add_edge('Entity-Relationship-Modell', 'Schema (Informatik)')
wiki.add_edge('Entity-Relationship-Modell', 'Schwache Entität')
wiki.add_edge('Entity-Relationship-Modell', 'Semantisches Datenmodell')
wiki.add_edge('Entity-Relationship-Modell', 'Semistrukturierte Daten')
wiki.add_edge('Entity-Relationship-Modell', 'Softwaredesign')
wiki.add_edge('Entity-Relationship-Modell', 'Structured-Entity-Relationship-Modell')
wiki.add_edge('Entity-Relationship-Modell', 'Temporale Datenhaltung')
wiki.add_edge('Entität (Informatik)', 'Data-Dictionary')
wiki.add_edge('Entität (Informatik)', 'Diameter (Protokoll)')
wiki.add_edge('Entität (Informatik)', 'Martin-Notation')
wiki.add_edge('Entität (Informatik)', 'Min-Max-Notation')
wiki.add_edge('Entwurfsmuster', 'Abstrakte Fabrik')
wiki.add_edge('Entwurfsmuster', 'Active Record')
wiki.add_edge('Entwurfsmuster', 'Adapter (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster', 'Analysemuster')
wiki.add_edge('Entwurfsmuster', 'Anti-Pattern')
wiki.add_edge('Entwurfsmuster', 'Anwendungsfassade')
wiki.add_edge('Entwurfsmuster', 'Architekturmuster')
wiki.add_edge('Entwurfsmuster', 'Asynchronous module definition')
wiki.add_edge('Entwurfsmuster', 'Beobachter (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster', 'Besucher (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster', 'Big Ball of Mud')
wiki.add_edge('Entwurfsmuster', 'Business Delegate')
wiki.add_edge('Entwurfsmuster', 'Code-Smell')
wiki.add_edge('Entwurfsmuster', 'Command-Query-Responsibility-Segregation')
wiki.add_edge('Entwurfsmuster', 'Container (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster', 'Cross-Cutting Concern')
wiki.add_edge('Entwurfsmuster', 'Data Access Object')
wiki.add_edge('Entwurfsmuster', 'Data Vault')
wiki.add_edge('Entwurfsmuster', 'Datenbank-Muster')
wiki.add_edge('Entwurfsmuster', 'Datenmodellierung')
wiki.add_edge('Entwurfsmuster', 'Decorator')
wiki.add_edge('Entwurfsmuster', 'Document Message')
wiki.add_edge('Entwurfsmuster', 'Enterprise Application Integration')
wiki.add_edge('Entwurfsmuster', 'Entwurfsmuster (Buch)')
wiki.add_edge('Entwurfsmuster', 'Erbauer (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster', 'Erzeugungsmuster')
wiki.add_edge('Entwurfsmuster', 'Event (Analysemuster)')
wiki.add_edge('Entwurfsmuster', 'Extension Interface')
wiki.add_edge('Entwurfsmuster', 'Fabrikmethode')
wiki.add_edge('Entwurfsmuster', 'Fassade (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster', 'Fliegengewicht (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster', 'Fluent Interface')
wiki.add_edge('Entwurfsmuster', 'Front Controller')
wiki.add_edge('Entwurfsmuster', 'GRASP')
wiki.add_edge('Entwurfsmuster', 'Idiom (Softwaretechnik)')
wiki.add_edge('Entwurfsmuster', 'Immediate Mode (Computergrafik)')
wiki.add_edge('Entwurfsmuster', 'Interaktionsdesignmuster')
wiki.add_edge('Entwurfsmuster', 'Interceptor (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster', 'Interpreter (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster', 'Inversion of Control')
wiki.add_edge('Entwurfsmuster', 'Iterator (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster', 'Jakarta Enterprise Beans')
wiki.add_edge('Entwurfsmuster', 'Kommando (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster', 'Kompositum (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster', 'Liste von Singleton-Implementierungen')
wiki.add_edge('Entwurfsmuster', 'Lose Kopplung')
wiki.add_edge('Entwurfsmuster', 'MapReduce')
wiki.add_edge('Entwurfsmuster', 'Mapper (Softwareentwicklung)')
wiki.add_edge('Entwurfsmuster', 'Markierungsschnittstelle')
wiki.add_edge('Entwurfsmuster', 'Memento (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster', 'Message (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster', 'Meta-Architektur-Dokument')
wiki.add_edge('Entwurfsmuster', 'Model 2')
wiki.add_edge('Entwurfsmuster', 'Model View Controller')
wiki.add_edge('Entwurfsmuster', 'Model View Presenter')
wiki.add_edge('Entwurfsmuster', 'Model View ViewModel')
wiki.add_edge('Entwurfsmuster', 'Modifizierbarkeit')
wiki.add_edge('Entwurfsmuster', 'Multiton')
wiki.add_edge('Entwurfsmuster', 'Mustersprache')
wiki.add_edge('Entwurfsmuster', 'Nachrichtenübermittlungsmuster')
wiki.add_edge('Entwurfsmuster', 'Nebenläufigkeit (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster', 'Nullobjekt (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster', 'Objektpool')
wiki.add_edge('Entwurfsmuster', 'Plugin (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster', 'Prinzipien objektorientierten Designs')
wiki.add_edge('Entwurfsmuster', 'Proactor')
wiki.add_edge('Entwurfsmuster', 'Prototyp (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster', 'Pull-Model')
wiki.add_edge('Entwurfsmuster', 'Push-Model')
wiki.add_edge('Entwurfsmuster', 'Refactoring')
wiki.add_edge('Entwurfsmuster', 'Referenzarchitektur')
wiki.add_edge('Entwurfsmuster', 'Registry (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster', 'Repository (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster', 'Retained Mode')
wiki.add_edge('Entwurfsmuster', 'SQLAlchemy')
wiki.add_edge('Entwurfsmuster', 'Saga (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster', 'Schablonenmethode')
wiki.add_edge('Entwurfsmuster', 'Serviceorientierte Architektur')
wiki.add_edge('Entwurfsmuster', 'Singleton (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster', 'Smart UI')
wiki.add_edge('Entwurfsmuster', 'Softwarearchitekt')
wiki.add_edge('Entwurfsmuster', 'Softwarearchitektur')
wiki.add_edge('Entwurfsmuster', 'Softwaredesign')
wiki.add_edge('Entwurfsmuster', 'Spaghetticode')
wiki.add_edge('Entwurfsmuster', 'Stellvertreter (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster', 'Strategie (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster', 'Strukturmuster')
wiki.add_edge('Entwurfsmuster', 'Together (Software)')
wiki.add_edge('Entwurfsmuster', 'Transferobjekt')
wiki.add_edge('Entwurfsmuster', 'Unit of Work')
wiki.add_edge('Entwurfsmuster', 'VIPER (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster', 'Value Object')
wiki.add_edge('Entwurfsmuster', 'Verhaltensmuster (Software)')
wiki.add_edge('Entwurfsmuster', 'Vermittler (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster', 'Verzögerte Initialisierung')
wiki.add_edge('Entwurfsmuster', 'Wrapper-Klasse')
wiki.add_edge('Entwurfsmuster', 'Zustand (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster', 'Zuständigkeitskette')
wiki.add_edge('Entwurfsmuster (Buch)', 'Abstrakte Fabrik')
wiki.add_edge('Entwurfsmuster (Buch)', 'Adapter (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster (Buch)', 'Aggregation (Informatik)')
wiki.add_edge('Entwurfsmuster (Buch)', 'Anti-Pattern')
wiki.add_edge('Entwurfsmuster (Buch)', 'Anwendungsfassade')
wiki.add_edge('Entwurfsmuster (Buch)', 'Assoziation (UML)')
wiki.add_edge('Entwurfsmuster (Buch)', 'Beobachter (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster (Buch)', 'Besucher (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster (Buch)', 'Brücke (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster (Buch)', 'Decorator')
wiki.add_edge('Entwurfsmuster (Buch)', 'Delegation (Softwareentwicklung)')
wiki.add_edge('Entwurfsmuster (Buch)', 'Erbauer (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster (Buch)', 'Erzeugungsmuster')
wiki.add_edge('Entwurfsmuster (Buch)', 'Fabrikmethode')
wiki.add_edge('Entwurfsmuster (Buch)', 'Fassade (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster (Buch)', 'Fliegengewicht (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster (Buch)', 'GRASP')
wiki.add_edge('Entwurfsmuster (Buch)', 'Interpreter (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster (Buch)', 'Iterator (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster (Buch)', 'Kommando (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster (Buch)', 'Kompositum (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster (Buch)', 'Memento (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster (Buch)', 'Prototyp (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster (Buch)', 'Schablonenmethode')
wiki.add_edge('Entwurfsmuster (Buch)', 'Singleton (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster (Buch)', 'Stellvertreter (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster (Buch)', 'Strategie (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster (Buch)', 'Vermittler (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster (Buch)', 'Zustand (Entwurfsmuster)')
wiki.add_edge('Entwurfsmuster (Buch)', 'Zuständigkeitskette')
wiki.add_edge('Erbauer (Entwurfsmuster)', 'Besucher (Entwurfsmuster)')
wiki.add_edge('Erbauer (Entwurfsmuster)', 'Fluent Interface')
wiki.add_edge('Erbauer (Entwurfsmuster)', 'Interpreter (Entwurfsmuster)')
wiki.add_edge('Erbauer (Entwurfsmuster)', 'Kompositum (Entwurfsmuster)')
wiki.add_edge('Erbauer (Entwurfsmuster)', 'Singleton (Entwurfsmuster)')
wiki.add_edge('Ereignis (UML)', 'Sequenzdiagramm')
wiki.add_edge('Ereignisgesteuerte Architektur', 'Distributed Application Runtime')
wiki.add_edge('Erzeugungsmuster', 'Abstrakte Fabrik')
wiki.add_edge('Erzeugungsmuster', 'Erbauer (Entwurfsmuster)')
wiki.add_edge('Erzeugungsmuster', 'Fabrikmethode')
wiki.add_edge('Erzeugungsmuster', 'Multiton')
wiki.add_edge('Erzeugungsmuster', 'Nebenläufigkeit (Entwurfsmuster)')
wiki.add_edge('Erzeugungsmuster', 'Objektpool')
wiki.add_edge('Erzeugungsmuster', 'Prototyp (Entwurfsmuster)')
wiki.add_edge('Erzeugungsmuster', 'Singleton (Entwurfsmuster)')
wiki.add_edge('Erzeugungsmuster', 'Strukturmuster')
wiki.add_edge('Erzeugungsmuster', 'Verhaltensmuster (Software)')
wiki.add_edge('EtherCAT', 'CANopen')
wiki.add_edge('EtherCAT', 'Controller Area Network')
wiki.add_edge('EtherCAT', 'Open Platform Communications')
wiki.add_edge('EtherCAT', 'Sercos III')
wiki.add_edge('EtherCAT', 'Transmission Control Protocol/Internet Protocol')
wiki.add_edge('EtherCAT', 'Tunnel (Rechnernetz)')
wiki.add_edge('EtherNet/IP', 'Advanced Physical Layer')
wiki.add_edge('EtherNet/IP', 'EchoRing')
wiki.add_edge('EtherNet/IP', 'Ethernet APL')
wiki.add_edge('EtherNet/IP', 'OpenSAFETY')
wiki.add_edge('EtherTalk Link Access Protocol', 'LocalTalk')
wiki.add_edge('EtherTalk Link Access Protocol', 'LocalTalk Link Access Protocol')
wiki.add_edge('EtherTalk Link Access Protocol', 'TokenTalk Link Access Protocol')
wiki.add_edge('EtherType', 'Generic Routing Encapsulation')
wiki.add_edge('EtherType', 'IEEE 802.1ah-2008')
wiki.add_edge('Etherchannel', 'PAgP')
wiki.add_edge('Etherchannel', 'Split Multi-Link Trunking')
wiki.add_edge('Ethereum', 'Cardano (Blockchainnetzwerk)')
wiki.add_edge('Ethereum', 'Chainlink')
wiki.add_edge('Ethereum', 'Dash (Kryptowährung)')
wiki.add_edge('Ethereum', 'Dezentralisierte Autonome Organisation')
wiki.add_edge('Ethereum', 'EOS (Kryptowährung)')
wiki.add_edge('Ethereum', 'Ethereum Classic')
wiki.add_edge('Ethereum', 'NEO (Kryptowährung)')
wiki.add_edge('Ethereum', 'Shiba Inu (Kryptowährung)')
wiki.add_edge('Ethereum', 'Solana (Blockchain)')
wiki.add_edge('Ethereum', 'Steem')
wiki.add_edge('Ethereum', 'Theta (Kryptowährung)')
wiki.add_edge('Ethereum', 'Tron (Kryptowährung)')
wiki.add_edge('Ethernet', '10BASE2')
wiki.add_edge('Ethernet', '10BASE5')
wiki.add_edge('Ethernet', '6in4')
wiki.add_edge('Ethernet', 'ALOHA')
wiki.add_edge('Ethernet', 'ARINC 825')
wiki.add_edge('Ethernet', 'ATA over Ethernet')
wiki.add_edge('Ethernet', 'Address Resolution Protocol')
wiki.add_edge('Ethernet', 'Advanced Physical Layer')
wiki.add_edge('Ethernet', 'AmiTCP')
wiki.add_edge('Ethernet', 'Apple Filing Protocol')
wiki.add_edge('Ethernet', 'AppleTalk')
wiki.add_edge('Ethernet', 'Attachment Unit Interface')
wiki.add_edge('Ethernet', 'Autonegotiation')
wiki.add_edge('Ethernet', 'BACnet')
wiki.add_edge('Ethernet', 'BNC-Steckverbinder')
wiki.add_edge('Ethernet', 'BroadR-Reach')
wiki.add_edge('Ethernet', 'CANaerospace')
wiki.add_edge('Ethernet', 'Cable-Sharing')
wiki.add_edge('Ethernet', 'Certificate Management Protocol')
wiki.add_edge('Ethernet', 'Cisco Discovery Protocol')
wiki.add_edge('Ethernet', 'Cisco Group Management Protocol')
wiki.add_edge('Ethernet', 'Class of Service')
wiki.add_edge('Ethernet', 'Data Link Control')
wiki.add_edge('Ethernet', 'Datagram Delivery Protocol')
wiki.add_edge('Ethernet', 'Datagramm')
wiki.add_edge('Ethernet', 'DoIP')
wiki.add_edge('Ethernet', 'Dynamic Trunking Protocol')
wiki.add_edge('Ethernet', 'EAD-Kabel')
wiki.add_edge('Ethernet', 'EAPOL')
wiki.add_edge('Ethernet', 'Echtzeit-Ethernet')
wiki.add_edge('Ethernet', 'Energy Efficient Ethernet')
wiki.add_edge('Ethernet', 'EtherCAT')
wiki.add_edge('Ethernet', 'EtherNet/IP')
wiki.add_edge('Ethernet', 'EtherTalk Link Access Protocol')
wiki.add_edge('Ethernet', 'EtherType')
wiki.add_edge('Ethernet', 'Etherchannel')
wiki.add_edge('Ethernet', 'Ethernet Powerlink')
wiki.add_edge('Ethernet', 'Ethernet-Anschlussdose')
wiki.add_edge('Ethernet', 'Fiber Distributed Data Interface')
wiki.add_edge('Ethernet', 'Fibre Channel over Ethernet')
wiki.add_edge('Ethernet', 'Full-duplex Ethernet')
wiki.add_edge('Ethernet', 'GMRP')
wiki.add_edge('Ethernet', 'Generic Object Oriented Substation Events')
wiki.add_edge('Ethernet', 'HDBaseT')
wiki.add_edge('Ethernet', 'HyperSCSI')
wiki.add_edge('Ethernet', 'ICMPv6')
wiki.add_edge('Ethernet', 'IEC 61850')
wiki.add_edge('Ethernet', 'IEEE 802')
wiki.add_edge('Ethernet', 'IEEE 802.12')
wiki.add_edge('Ethernet', 'IEEE 802.1Q')
wiki.add_edge('Ethernet', 'IEEE 802.1p')
wiki.add_edge('Ethernet', 'IEEE 802.3ae')
wiki.add_edge('Ethernet', 'IEEE 802.3ak')
wiki.add_edge('Ethernet', 'IEEE 802.3an')
wiki.add_edge('Ethernet', 'ISCSI')
wiki.add_edge('Ethernet', 'Industrial Ethernet')
wiki.add_edge('Ethernet', 'Inter-Switch Link Protocol')
wiki.add_edge('Ethernet', 'Internet Control Message Protocol')
wiki.add_edge('Ethernet', 'Internet Group Management Protocol')
wiki.add_edge('Ethernet', 'Internet Printing Protocol')
wiki.add_edge('Ethernet', 'Internet Protocol Control Protocol')
wiki.add_edge('Ethernet', 'Internet Protocol over Fibre Channel')
wiki.add_edge('Ethernet', 'Internetwork Packet Exchange')
wiki.add_edge('Ethernet', 'Internetwork Packet eXchange Fragmentation')
wiki.add_edge('Ethernet', 'Jabber (Ethernet)')
wiki.add_edge('Ethernet', 'JetDirect')
wiki.add_edge('Ethernet', 'Jumbo Frames')
wiki.add_edge('Ethernet', 'Kinetics Internet Protocol')
wiki.add_edge('Ethernet', 'LLDP')
wiki.add_edge('Ethernet', 'Link Aggregation')
wiki.add_edge('Ethernet', 'Liste der standardisierten Ports')
wiki.add_edge('Ethernet', 'LocalTalk')
wiki.add_edge('Ethernet', 'LocalTalk Link Access Protocol')
wiki.add_edge('Ethernet', 'Manufacturing Messaging Specification')
wiki.add_edge('Ethernet', 'Media Access Control')
wiki.add_edge('Ethernet', 'Media Redundancy Protocol')
wiki.add_edge('Ethernet', 'Neighbor Discovery Protocol')
wiki.add_edge('Ethernet', 'NetBIOS')
wiki.add_edge('Ethernet', 'NetBIOS over TCP/IP')
wiki.add_edge('Ethernet', 'NetWare Core Protocol')
wiki.add_edge('Ethernet', 'Network File System')
wiki.add_edge('Ethernet', 'PAgP')
wiki.add_edge('Ethernet', 'PROFINET')
wiki.add_edge('Ethernet', 'Pfsync')
wiki.add_edge('Ethernet', 'Port Based Network Access Control')
wiki.add_edge('Ethernet', 'Power over Ethernet')
wiki.add_edge('Ethernet', 'Preboot Execution Environment')
wiki.add_edge('Ethernet', 'Profinet')
wiki.add_edge('Ethernet', 'Protokollstapel')
wiki.add_edge('Ethernet', 'RJ-Steckverbindung')
wiki.add_edge('Ethernet', 'Resilient Packet Ring')
wiki.add_edge('Ethernet', 'Resource Reservation Protocol')
wiki.add_edge('Ethernet', 'Reverse Address Resolution Protocol')
wiki.add_edge('Ethernet', 'Routing Table Maintenance Protocol')
wiki.add_edge('Ethernet', 'SafetyNET p')
wiki.add_edge('Ethernet', 'Secure Socket Tunneling Protocol')
wiki.add_edge('Ethernet', 'Sequenced Packet Exchange')
wiki.add_edge('Ethernet', 'Sercos III')
wiki.add_edge('Ethernet', 'Server Message Block')
wiki.add_edge('Ethernet', 'Spanning Tree Protocol')
wiki.add_edge('Ethernet', 'T-Stück')
wiki.add_edge('Ethernet', 'TCP Receive Window')
wiki.add_edge('Ethernet', 'Time Token Rotation Protocol')
wiki.add_edge('Ethernet', 'Time-Sensitive Networking')
wiki.add_edge('Ethernet', 'Token Bus')
wiki.add_edge('Ethernet', 'Token Ring')
wiki.add_edge('Ethernet', 'TokenTalk Link Access Protocol')
wiki.add_edge('Ethernet', 'Universal Measurement and Calibration Protocol')
wiki.add_edge('Ethernet', 'Universal Plug and Play')
wiki.add_edge('Ethernet', 'Virtual Local Area Network')
wiki.add_edge('Ethernet', 'Virtual Router Redundancy Protocol')
wiki.add_edge('Ethernet', 'Weihenstephaner Standards')
wiki.add_edge('Ethernet', 'Zone Information Protocol')
wiki.add_edge('Ethernet APL', 'Advanced Physical Layer')
wiki.add_edge('Ethernet Powerlink', 'CANopen')
wiki.add_edge('Ethernet Powerlink', 'EchoRing')
wiki.add_edge('Ethernet Powerlink', 'Fast Ethernet')
wiki.add_edge('Ethernet Powerlink', 'Open Platform Communications')
wiki.add_edge('Ethernet Powerlink', 'OpenSAFETY')
wiki.add_edge('Ethernet Powerlink', 'Power over Ethernet')
wiki.add_edge('Ethernet-Anschlussdose', '1000BASE-T')
wiki.add_edge('Ethernet-Anschlussdose', '100BASE-TX')
wiki.add_edge('Ethernet-Anschlussdose', '10BASE2')
wiki.add_edge('Ethernet-Anschlussdose', 'EAD-Kabel')
wiki.add_edge('Event Sourcing', 'Event (Analysemuster)')
wiki.add_edge('Explicit Congestion Notification', 'Durchsatzratenbegrenzung')
wiki.add_edge('Explicit Congestion Notification', 'TCP Global Synchronization')
wiki.add_edge('Extensible Authentication Protocol', 'EAPOL')
wiki.add_edge('Extensible Authentication Protocol', 'Port Based Network Access Control')
wiki.add_edge('Extensible Authentication Protocol', 'Secure Socket Tunneling Protocol')
wiki.add_edge('Extensible Authentication Protocol', 'WPA2')
wiki.add_edge('Extensible Messaging and Presence Protocol', 'Gadu-Gadu')
wiki.add_edge('Extensible Messaging and Presence Protocol', 'I2P')
wiki.add_edge('Extensible Messaging and Presence Protocol', 'InterAsterisk eXchange')
wiki.add_edge('Extensible Messaging and Presence Protocol', 'Liste der standardisierten Ports')
wiki.add_edge('Extensible Messaging and Presence Protocol', 'Matrix (Kommunikationsprotokoll)')
wiki.add_edge('Extensible Messaging and Presence Protocol', 'OSCAR (Protokoll)')
wiki.add_edge('Extensible Messaging and Presence Protocol', 'Protocol for SYnchronous Conferencing')
wiki.add_edge('Extensible Messaging and Presence Protocol', 'Push-Model')
wiki.add_edge('Extensible Messaging and Presence Protocol', 'STARTTLS')
wiki.add_edge('Extensible Messaging and Presence Protocol', 'Session Initiation Protocol')
wiki.add_edge('Extensible Messaging and Presence Protocol', 'Signal-Protokoll')
wiki.add_edge('Extensible Messaging and Presence Protocol', 'Silent Circle Instant Messaging Protocol')
wiki.add_edge('Extensible Messaging and Presence Protocol', 'Simple Authentication and Security Layer')
wiki.add_edge('Extensible Messaging and Presence Protocol', 'Telepathy')
wiki.add_edge('Extensible Messaging and Presence Protocol', 'WhatsApp')
wiki.add_edge('Extensible Messaging and Presence Protocol', 'Windows Live Messenger')
wiki.add_edge('Extensible Messaging and Presence Protocol', 'Yahoo Messenger')
wiki.add_edge('Extensible Messaging and Presence Protocol', 'Zephyr (Protokoll)')
wiki.add_edge('Extensible Provisioning Protocol', 'Blocks Extensible Exchange Protocol')
wiki.add_edge('Extension Interface', 'Softwaredesign')
wiki.add_edge('Exterior-Gateway-Protokoll', 'Bidirectional Forwarding Detection')
wiki.add_edge('Exterior-Gateway-Protokoll', 'Interior Gateway Protocol')
wiki.add_edge('Exterior-Gateway-Protokoll', 'Open Shortest Path First')
wiki.add_edge('Exterior-Gateway-Protokoll', 'Protokoll (IP)')
wiki.add_edge('FTZ 1 TR 6', 'Digital Subscriber System No. 1')
wiki.add_edge('FTZ 1 TR 6', 'Q.931')
wiki.add_edge('Fabrikmethode', 'ADO.NET')
wiki.add_edge('Fabrikmethode', 'Abstrakte Fabrik')
wiki.add_edge('Fabrikmethode', 'Fliegengewicht (Entwurfsmuster)')
wiki.add_edge('Fabrikmethode', 'GRASP')
wiki.add_edge('Fabrikmethode', 'Inversion of Control')
wiki.add_edge('Fabrikmethode', 'Iterator (Entwurfsmuster)')
wiki.add_edge('Fabrikmethode', 'Klassendiagramm')
wiki.add_edge('Fabrikmethode', 'Liste von Singleton-Implementierungen')
wiki.add_edge('Fabrikmethode', 'Schablonenmethode')
wiki.add_edge('Fabrikmethode', 'Singleton (Entwurfsmuster)')
wiki.add_edge('Fabrikmethode', 'Verzögerte Initialisierung')
wiki.add_edge('Facebook', 'Apache Thrift')
wiki.add_edge('Facebook', 'Border Gateway Protocol')
wiki.add_edge('Facebook', 'Dark Pattern')
wiki.add_edge('Facebook', 'Diem (Internetwährung)')
wiki.add_edge('Facebook', 'Extensible Messaging and Presence Protocol')
wiki.add_edge('Facebook', 'Gadu-Gadu')
wiki.add_edge('Facebook', 'IFTTT')
wiki.add_edge('Facebook', 'KakaoTalk')
wiki.add_edge('Facebook', 'MQTT')
wiki.add_edge('Facebook', 'Messaging Layer Security')
wiki.add_edge('Facebook', 'OpenSocial')
wiki.add_edge('Facebook', 'Signal-Protokoll')
wiki.add_edge('Facebook', 'Signalling System 7')
wiki.add_edge('Facebook', 'Skype')
wiki.add_edge('Facebook', 'Tron (Kryptowährung)')
wiki.add_edge('Facebook', 'WeChat')
wiki.add_edge('Facebook', 'WebHooks')
wiki.add_edge('Facebook', 'WhatsApp')
wiki.add_edge('Facebook', 'Windows Live Messenger')
wiki.add_edge('Facebook', 'World IPv6 Day und World IPv6 Launch Day')
wiki.add_edge('Fassade (Entwurfsmuster)', 'Adapter (Entwurfsmuster)')
wiki.add_edge('Fassade (Entwurfsmuster)', 'Anwendungsfassade')
wiki.add_edge('Fassade (Entwurfsmuster)', 'Bulkhead')
wiki.add_edge('Fassade (Entwurfsmuster)', 'Business Delegate')
wiki.add_edge('Fassade (Entwurfsmuster)', 'Delegation (Softwareentwicklung)')
wiki.add_edge('Fassade (Entwurfsmuster)', 'GRASP')
wiki.add_edge('Fassade (Entwurfsmuster)', 'Lose Kopplung')
wiki.add_edge('Fassade (Entwurfsmuster)', 'Vermittler (Entwurfsmuster)')
wiki.add_edge('FastCGI', 'Anwendungsserver')
wiki.add_edge('FastCGI', 'Perl Web Server Gateway Interface')
wiki.add_edge('FastCGI', 'Simple Common Gateway Interface')
wiki.add_edge('FastTrack', 'Skype')
wiki.add_edge('Fiber Distributed Data Interface', 'Apple Filing Protocol')
wiki.add_edge('Fiber Distributed Data Interface', 'AppleTalk')
wiki.add_edge('Fiber Distributed Data Interface', 'Certificate Management Protocol')
wiki.add_edge('Fiber Distributed Data Interface', 'Cisco Discovery Protocol')
wiki.add_edge('Fiber Distributed Data Interface', 'Datagram Delivery Protocol')
wiki.add_edge('Fiber Distributed Data Interface', 'EtherTalk Link Access Protocol')
wiki.add_edge('Fiber Distributed Data Interface', 'ICMPv6')
wiki.add_edge('Fiber Distributed Data Interface', 'Internet Group Management Protocol')
wiki.add_edge('Fiber Distributed Data Interface', 'Internet Printing Protocol')
wiki.add_edge('Fiber Distributed Data Interface', 'Internetwork Packet Exchange')
wiki.add_edge('Fiber Distributed Data Interface', 'Internetwork Packet eXchange Fragmentation')
wiki.add_edge('Fiber Distributed Data Interface', 'LocalTalk')
wiki.add_edge('Fiber Distributed Data Interface', 'LocalTalk Link Access Protocol')
wiki.add_edge('Fiber Distributed Data Interface', 'NetBIOS over TCP/IP')
wiki.add_edge('Fiber Distributed Data Interface', 'NetWare Core Protocol')
wiki.add_edge('Fiber Distributed Data Interface', 'Network File System')
wiki.add_edge('Fiber Distributed Data Interface', 'Pfsync')
wiki.add_edge('Fiber Distributed Data Interface', 'Resource Reservation Protocol')
wiki.add_edge('Fiber Distributed Data Interface', 'Routing Table Maintenance Protocol')
wiki.add_edge('Fiber Distributed Data Interface', 'Secure Socket Tunneling Protocol')
wiki.add_edge('Fiber Distributed Data Interface', 'Sequenced Packet Exchange')
wiki.add_edge('Fiber Distributed Data Interface', 'Server Message Block')
wiki.add_edge('Fiber Distributed Data Interface', 'Time Token Rotation Protocol')
wiki.add_edge('Fiber Distributed Data Interface', 'TokenTalk Link Access Protocol')
wiki.add_edge('Fiber Distributed Data Interface', 'Virtual Router Redundancy Protocol')
wiki.add_edge('Fiber Distributed Data Interface', 'Zone Information Protocol')
wiki.add_edge('Fibre Channel over Ethernet', 'ATA over Ethernet')
wiki.add_edge('Fibre Channel over Ethernet', 'Fibre Channel over IP')
wiki.add_edge('Fibre Channel over Ethernet', 'HyperSCSI')
wiki.add_edge('Fibre Channel over Ethernet', 'ISCSI')
wiki.add_edge('Fibre Channel over Ethernet', 'Internet Protocol over Fibre Channel')
wiki.add_edge('Fibre Channel over Ethernet', 'Jumbo Frames')
wiki.add_edge('Fibre Channel over IP', 'HyperSCSI')
wiki.add_edge('Fibre Channel over IP', 'Internet Protocol over Fibre Channel')
wiki.add_edge('File Exchange Protocol', 'File eXchange Protocol')
wiki.add_edge('File Transfer Protocol', 'AS2')
wiki.add_edge('File Transfer Protocol', 'AS3')
wiki.add_edge('File Transfer Protocol', 'ASCII-Protokoll')
wiki.add_edge('File Transfer Protocol', 'Background File Transfer Program')
wiki.add_edge('File Transfer Protocol', 'Banking Communication Standard')
wiki.add_edge('File Transfer Protocol', 'BitTorrent')
wiki.add_edge('File Transfer Protocol', 'CSLIP')
wiki.add_edge('File Transfer Protocol', 'Certificate Management Protocol')
wiki.add_edge('File Transfer Protocol', 'Datenaustausch')
wiki.add_edge('File Transfer Protocol', 'Direct Client-to-Client')
wiki.add_edge('File Transfer Protocol', 'File Exchange Protocol')
wiki.add_edge('File Transfer Protocol', 'File Service Protocol')
wiki.add_edge('File Transfer Protocol', 'File Transfer, Access and Management')
wiki.add_edge('File Transfer Protocol', 'File eXchange Protocol')
wiki.add_edge('File Transfer Protocol', 'Gnome Virtual File System')
wiki.add_edge('File Transfer Protocol', 'Gopher')
wiki.add_edge('File Transfer Protocol', 'HFTP')
wiki.add_edge('File Transfer Protocol', 'Internet Content Adaptation Protocol')
wiki.add_edge('File Transfer Protocol', 'JetDirect')
wiki.add_edge('File Transfer Protocol', 'Liste der standardisierten Ports')
wiki.add_edge('File Transfer Protocol', 'Liste von TCP/IP-basierten Netzwerkdiensten')
wiki.add_edge('File Transfer Protocol', 'Multi-purpose Business Security over IP')
wiki.add_edge('File Transfer Protocol', 'Netfilter')
wiki.add_edge('File Transfer Protocol', 'Network File System')
wiki.add_edge('File Transfer Protocol', 'Netzwerkprotokoll')
wiki.add_edge('File Transfer Protocol', 'OSI-Modell')
wiki.add_edge('File Transfer Protocol', 'Picture Transfer Protocol over IP')
wiki.add_edge('File Transfer Protocol', 'Portweiterleitung')
wiki.add_edge('File Transfer Protocol', 'Resource Reservation Protocol')
wiki.add_edge('File Transfer Protocol', 'Reverse Connection')
wiki.add_edge('File Transfer Protocol', 'SOAP')
wiki.add_edge('File Transfer Protocol', 'SSH File Transfer Protocol')
wiki.add_edge('File Transfer Protocol', 'STARTTLS')
wiki.add_edge('File Transfer Protocol', 'Secure Copy')
wiki.add_edge('File Transfer Protocol', 'Secure Shell')
wiki.add_edge('File Transfer Protocol', 'Serial Line Internet Protocol')
wiki.add_edge('File Transfer Protocol', 'Serviceorientierte Architektur')
wiki.add_edge('File Transfer Protocol', 'Simple Asynchronous File Transfer')
wiki.add_edge('File Transfer Protocol', 'Simple File Transfer Protocol')
wiki.add_edge('File Transfer Protocol', 'Streaming-Protokoll')
wiki.add_edge('File Transfer Protocol', 'TR-143')
wiki.add_edge('File Transfer Protocol', 'Transmission Control Protocol/Internet Protocol')
wiki.add_edge('File Transfer Protocol', 'Trivial File Transfer Protocol')
wiki.add_edge('File Transfer Protocol', 'Tunnel (Rechnernetz)')
wiki.add_edge('File Transfer Protocol', 'WebDAV')
wiki.add_edge('File Transfer Protocol', 'Webservice')
wiki.add_edge('File Transfer Protocol', 'Zustandsbehaftung')
wiki.add_edge('File Transfer, Access and Management', 'AS2')
wiki.add_edge('File Transfer, Access and Management', 'Banking Communication Standard')
wiki.add_edge('File Transfer, Access and Management', 'Job Transfer and Manipulation')
wiki.add_edge('File eXchange Protocol', 'Virtual Studio Technology')
wiki.add_edge('Filter (Software)', 'Normierte Programmierung')
wiki.add_edge('Filter (Software)', 'Pipe (Informatik)')
wiki.add_edge('Filter (Software)', 'Pipes und Filter')
wiki.add_edge('Finger (Internetprotokoll)', 'Ident')
wiki.add_edge('FlexRay', 'Profinet')
wiki.add_edge('FlexRay', 'Teilnetzbetrieb')
wiki.add_edge('FlexRay', 'Universal Measurement and Calibration Protocol')
wiki.add_edge('Fliegengewicht (Entwurfsmuster)', 'Code-Smell')
wiki.add_edge('Fliegengewicht (Entwurfsmuster)', 'Interpreter (Entwurfsmuster)')
wiki.add_edge('Fliegengewicht (Entwurfsmuster)', 'Kompositum (Entwurfsmuster)')
wiki.add_edge('Fliegengewicht (Entwurfsmuster)', 'Stellvertreter (Entwurfsmuster)')
wiki.add_edge('Fliegengewicht (Entwurfsmuster)', 'Strategie (Entwurfsmuster)')
wiki.add_edge('Fliegengewicht (Entwurfsmuster)', 'Unified Modeling Language')
wiki.add_edge('Fliegengewicht (Entwurfsmuster)', 'Zustand (Entwurfsmuster)')
wiki.add_edge('Framework', 'Active Record')
wiki.add_edge('Framework', 'Advanced Linux Sound Architecture')
wiki.add_edge('Framework', 'Anti-Pattern')
wiki.add_edge('Framework', 'Apache CXF')
wiki.add_edge('Framework', 'Apache Cayenne')
wiki.add_edge('Framework', 'Apache MINA')
wiki.add_edge('Framework', 'Bonobo (GNOME)')
wiki.add_edge('Framework', 'Command-Query-Responsibility-Segregation')
wiki.add_edge('Framework', 'Creole (Framework)')
wiki.add_edge('Framework', 'Cross Platform Component Object Model')
wiki.add_edge('Framework', 'DataNucleus')
wiki.add_edge('Framework', 'Datenbankabstraktionsschicht')
wiki.add_edge('Framework', 'EclipseLink')
wiki.add_edge('Framework', 'Entity Framework')
wiki.add_edge('Framework', 'Entwurfsmuster')
wiki.add_edge('Framework', 'Fabrikmethode')
wiki.add_edge('Framework', 'Grand Central Dispatch')
wiki.add_edge('Framework', 'Hibernate (Framework)')
wiki.add_edge('Framework', 'Interaktionsdesignmuster')
wiki.add_edge('Framework', 'Interceptor (Entwurfsmuster)')
wiki.add_edge('Framework', 'Inversion of Control')
wiki.add_edge('Framework', 'Jakarta Server Faces')
wiki.add_edge('Framework', 'Jakarta Transactions API')
wiki.add_edge('Framework', 'Java Authentication and Authorization Service')
wiki.add_edge('Framework', 'Java Cryptography Extension')
wiki.add_edge('Framework', 'Java Data Objects')
wiki.add_edge('Framework', 'Java Foundation Classes')
wiki.add_edge('Framework', 'Java Media Framework')
wiki.add_edge('Framework', 'JavaFX')
wiki.add_edge('Framework', 'KParts')
wiki.add_edge('Framework', 'Keeping Emulation Environments Portable')
wiki.add_edge('Framework', 'Kontrollfluss')
wiki.add_edge('Framework', 'Model View Controller')
wiki.add_edge('Framework', 'Modeling and Analysis of Real Time and Embedded systems')
wiki.add_edge('Framework', 'Multimedia-Framework')
wiki.add_edge('Framework', 'MyBatis')
wiki.add_edge('Framework', 'Noise Protocol Framework')
wiki.add_edge('Framework', 'OpenAPI')
wiki.add_edge('Framework', 'Plugin (Entwurfsmuster)')
wiki.add_edge('Framework', 'Propel')
wiki.add_edge('Framework', 'SQLAlchemy')
wiki.add_edge('Framework', 'Security Assertion Markup Language')
wiki.add_edge('Framework', 'Service-Locator')
wiki.add_edge('Framework', 'Session Traversal Utilities for NAT')
wiki.add_edge('Framework', 'Simple Authentication and Security Layer')
wiki.add_edge('Framework', 'Softwarearchitektur')
wiki.add_edge('Framework', 'Standalone')
wiki.add_edge('Framework', 'Strukturmuster')
wiki.add_edge('Framework', 'Telepathy')
wiki.add_edge('Framework', 'Temporale Datenhaltung')
wiki.add_edge('Framework', 'TopLink')
wiki.add_edge('Framework', 'Universal Plug and Play')
wiki.add_edge('Framework', 'XNA (Microsoft)')
wiki.add_edge('Framework', 'ZigBee')
wiki.add_edge('Full-duplex Ethernet', '1000BASE-T')
wiki.add_edge('Full-duplex Ethernet', '100BASE-TX')
wiki.add_edge('Full-duplex Ethernet', '10BASE-T')
wiki.add_edge('Full-duplex Ethernet', 'Autonegotiation')
wiki.add_edge('Full-duplex Ethernet', 'BroadR-Reach')
wiki.add_edge('Full-duplex Ethernet', 'Inter-Switch Link Protocol')
wiki.add_edge('Fundamental Modeling Concepts', 'Softwaredesign')
wiki.add_edge('GSSAPI', 'Kerberos (Protokoll)')
wiki.add_edge('GSSAPI', 'NTLM')
wiki.add_edge('GSSAPI', 'Network File System')
wiki.add_edge('GSSAPI', 'Remote Authentication Dial-In User Service')
wiki.add_edge('GSSAPI', 'SOCKS')
wiki.add_edge('GSSAPI', 'Simple Authentication and Security Layer')
wiki.add_edge('Generalisierung (UML)', 'Abhängigkeitsbeziehung')
wiki.add_edge('Generalisierung (UML)', 'Aggregation (Informatik)')
wiki.add_edge('Generalisierung (UML)', 'Classifier (UML)')
wiki.add_edge('Generalisierung (UML)', 'Klassendiagramm')
wiki.add_edge('Generalisierung (UML)', 'Knoten (UML)')
wiki.add_edge('Generalisierung (UML)', 'Komponente (UML)')
wiki.add_edge('Generalisierung (UML)', 'Merkmal (UML)')
wiki.add_edge('Generalisierung (UML)', 'Profil (UML)')
wiki.add_edge('Generalisierung (UML)', 'Stereotyp (UML)')
wiki.add_edge('Generic Object Oriented Substation Events', 'Datagramm')
wiki.add_edge('Generic Object Oriented Substation Events', 'IEC 61850')
wiki.add_edge('Generic Routing Encapsulation', 'Crypto IP Encapsulation')
wiki.add_edge('Generic Routing Encapsulation', 'Netfilter')
wiki.add_edge('Geodienst', 'Web Feature Service')
wiki.add_edge('Geodienst', 'Web Map Tile Service')
wiki.add_edge('Geodienst', 'Web Processing Service')
wiki.add_edge('Geschäftslogik', 'Business Delegate')
wiki.add_edge('Geschäftslogik', 'Jakarta Enterprise Beans')
wiki.add_edge('Geschäftslogik', 'Model View ViewModel')
wiki.add_edge('Geschäftslogik', 'Organization Hierarchy')
wiki.add_edge('Geschäftslogik', 'VIPER (Entwurfsmuster)')
wiki.add_edge('Gesetz von Conway', 'Microservices')
wiki.add_edge('Global-as-View', 'Local-as-View')
wiki.add_edge('GnuTLS', 'Konsolenanwendung')
wiki.add_edge('GnuTLS', 'Mbed TLS')
wiki.add_edge('Gopher', 'Geminispace')
wiki.add_edge('Graphics Device Interface', 'GDI+')
wiki.add_edge('H.323', 'H.-Standards')
wiki.add_edge('H.323', 'H.225.0')
wiki.add_edge('H.323', 'InterAsterisk eXchange')
wiki.add_edge('H.323', 'Media Gateway Control Protocol (Protokoll)')
wiki.add_edge('H.323', 'Megaco')
wiki.add_edge('H.323', 'Q.931')
wiki.add_edge('H.323', 'Skinny Client Control Protocol')
wiki.add_edge('H.323', 'Streaming-Protokoll')
wiki.add_edge('H.323', 'ZRTP')
wiki.add_edge('HX DOS Extender', 'DOS Protected Mode Interface')
wiki.add_edge('Heartbleed', 'Extensible Authentication Protocol')
wiki.add_edge('Heartbleed', 'GnuTLS')
wiki.add_edge('Heartbleed', 'LibreSSL')
wiki.add_edge('Heartbleed', 'Online Certificate Status Protocol stapling')
wiki.add_edge('Heartbleed', 'Secure Shell')
wiki.add_edge('Heartbleed', 'Serverseitige Anwendung')
wiki.add_edge('Heartbleed', 'Stream Control Transmission Protocol')
wiki.add_edge('Heartbleed', 'Verschlüsselungsprotokoll')
wiki.add_edge('Hessian (Webprotokoll)', 'Apache Cayenne')
wiki.add_edge('Hessian (Webprotokoll)', 'Burlap')
wiki.add_edge('Hibernate (Framework)', 'Apache OpenJPA')
wiki.add_edge('Hibernate (Framework)', 'Datenbankabstraktionsschicht')
wiki.add_edge('Hibernate (Framework)', 'EclipseLink')
wiki.add_edge('Hibernate (Framework)', 'Jakarta Transactions API')
wiki.add_edge('Hibernate (Framework)', 'Model 2')
wiki.add_edge('Hibernate (Framework)', 'MyBatis')
wiki.add_edge('Hibernate (Framework)', 'SQLAlchemy')
wiki.add_edge('Hibernate (Framework)', 'TopLink')
wiki.add_edge('High-Level Data Link Control', 'Binary Synchronous Communication')
wiki.add_edge('High-Level Data Link Control', 'Data Link Control')
wiki.add_edge('High-Level Data Link Control', 'Digital Data Communications Message Protocol')
wiki.add_edge('High-Level Data Link Control', 'Frame Relay')
wiki.add_edge('High-Level Data Link Control', 'Link Access Procedure Balanced')
wiki.add_edge('High-Level Data Link Control', 'Link Access Procedure for the D-Channel')
wiki.add_edge('High-Level Data Link Control', 'Packet over SONET/SDH')
wiki.add_edge('High-Level Data Link Control', 'Positive Acknowledgement with Retransmission')
wiki.add_edge('High-Level Data Link Control', 'Signalling System 7')
wiki.add_edge('High-Level Data Link Control', 'Synchronous Data Link Control')
wiki.add_edge('High-Level Data Link Control', 'Vermittlungsschicht')
wiki.add_edge('High-Level Data Link Control', 'X.75')
wiki.add_edge('Highway Addressable Remote Transducer', 'Advanced Physical Layer')
wiki.add_edge('Highway Addressable Remote Transducer', 'WiTECK')
wiki.add_edge('Homebanking Computer Interface', 'Banking Communication Standard')
wiki.add_edge('Homebanking Computer Interface', 'Electronic Banking Internet Communication Standard')
wiki.add_edge('Hot Standby Router Protocol', 'Advanced Peer-to-Peer Networking')
wiki.add_edge('Hot Standby Router Protocol', 'Common Address Redundancy Protocol')
wiki.add_edge('Hot Standby Router Protocol', 'Virtual Router Redundancy Protocol')
wiki.add_edge('Humane Registry', 'Swagger (Software)')
wiki.add_edge('Hyper Text Coffee Pot Control Protocol', 'Pi Digit Generation Protocol')
wiki.add_edge('Hypertext Transfer Protocol', 'Advanced Physical Layer')
wiki.add_edge('Hypertext Transfer Protocol', 'Asynchronous module definition')
wiki.add_edge('Hypertext Transfer Protocol', 'BitTorrent')
wiki.add_edge('Hypertext Transfer Protocol', 'CRUD')
wiki.add_edge('Hypertext Transfer Protocol', 'CalDAV')
wiki.add_edge('Hypertext Transfer Protocol', 'Certificate Management Protocol')
wiki.add_edge('Hypertext Transfer Protocol', 'Client')
wiki.add_edge('Hypertext Transfer Protocol', 'Common Gateway Interface')
wiki.add_edge('Hypertext Transfer Protocol', 'Digital Audio Access Protocol')
wiki.add_edge('Hypertext Transfer Protocol', 'Domain Name System')
wiki.add_edge('Hypertext Transfer Protocol', 'Electronic Banking Internet Communication Standard')
wiki.add_edge('Hypertext Transfer Protocol', 'Facebook')
wiki.add_edge('Hypertext Transfer Protocol', 'File Service Protocol')
wiki.add_edge('Hypertext Transfer Protocol', 'File Transfer Protocol')
wiki.add_edge('Hypertext Transfer Protocol', 'File Transfer, Access and Management')
wiki.add_edge('Hypertext Transfer Protocol', 'Gopher')
wiki.add_edge('Hypertext Transfer Protocol', 'HFTP')
wiki.add_edge('Hypertext Transfer Protocol', 'HTTP-Authentifizierung')
wiki.add_edge('Hypertext Transfer Protocol', 'Hyper Text Coffee Pot Control Protocol')
wiki.add_edge('Hypertext Transfer Protocol', 'Hypertext Transfer Protocol Secure')
wiki.add_edge('Hypertext Transfer Protocol', 'I2P')
wiki.add_edge('Hypertext Transfer Protocol', 'IEEE 802.1Q')
wiki.add_edge('Hypertext Transfer Protocol', 'IP-Adresse')
wiki.add_edge('Hypertext Transfer Protocol', 'IPv6')
wiki.add_edge('Hypertext Transfer Protocol', 'InterPlanetary File System')
wiki.add_edge('Hypertext Transfer Protocol', 'Internet Content Adaptation Protocol')
wiki.add_edge('Hypertext Transfer Protocol', 'Internet Media Type')
wiki.add_edge('Hypertext Transfer Protocol', 'Internet Printing Protocol')
wiki.add_edge('Hypertext Transfer Protocol', 'Internetprotokollfamilie')
wiki.add_edge('Hypertext Transfer Protocol', 'Jakarta Server Faces')
wiki.add_edge('Hypertext Transfer Protocol', 'Java Management Extensions')
wiki.add_edge('Hypertext Transfer Protocol', 'JetDirect')
wiki.add_edge('Hypertext Transfer Protocol', 'Keepalive')
wiki.add_edge('Hypertext Transfer Protocol', 'Liste der standardisierten Ports')
wiki.add_edge('Hypertext Transfer Protocol', 'Liste von TCP/IP-basierten Netzwerkdiensten')
wiki.add_edge('Hypertext Transfer Protocol', 'MMS-Protokoll')
wiki.add_edge('Hypertext Transfer Protocol', 'Matrix (Kommunikationsprotokoll)')
wiki.add_edge('Hypertext Transfer Protocol', 'Message Oriented Middleware')
wiki.add_edge('Hypertext Transfer Protocol', 'NTLM')
wiki.add_edge('Hypertext Transfer Protocol', 'Netzwerkprotokoll')
wiki.add_edge('Hypertext Transfer Protocol', 'OSI-Modell')
wiki.add_edge('Hypertext Transfer Protocol', 'Oparl')
wiki.add_edge('Hypertext Transfer Protocol', 'Open Building Information Exchange')
wiki.add_edge('Hypertext Transfer Protocol', 'Open Data Protocol')
wiki.add_edge('Hypertext Transfer Protocol', 'Open Platform Communications')
wiki.add_edge('Hypertext Transfer Protocol', 'Port (Protokoll)')
wiki.add_edge('Hypertext Transfer Protocol', 'Production Performance Management Protocol')
wiki.add_edge('Hypertext Transfer Protocol', 'Protocol for SYnchronous Conferencing')
wiki.add_edge('Hypertext Transfer Protocol', 'Push-Model')
wiki.add_edge('Hypertext Transfer Protocol', 'QUIC')
wiki.add_edge('Hypertext Transfer Protocol', 'Real Time Messaging Protocol')
wiki.add_edge('Hypertext Transfer Protocol', 'Real-Time Streaming Protocol')
wiki.add_edge('Hypertext Transfer Protocol', 'Remote Procedure Call')
wiki.add_edge('Hypertext Transfer Protocol', 'Representational State Transfer')
wiki.add_edge('Hypertext Transfer Protocol', 'S-HTTP')
wiki.add_edge('Hypertext Transfer Protocol', 'SOAP')
wiki.add_edge('Hypertext Transfer Protocol', 'SPDY')
wiki.add_edge('Hypertext Transfer Protocol', 'STARTTLS')
wiki.add_edge('Hypertext Transfer Protocol', 'Schichtenarchitektur')
wiki.add_edge('Hypertext Transfer Protocol', 'Serverseitige Anwendung')
wiki.add_edge('Hypertext Transfer Protocol', 'Session Initiation Protocol')
wiki.add_edge('Hypertext Transfer Protocol', 'Simple Common Gateway Interface')
wiki.add_edge('Hypertext Transfer Protocol', 'Simple Service Discovery Protocol')
wiki.add_edge('Hypertext Transfer Protocol', 'Streaming-Protokoll')
wiki.add_edge('Hypertext Transfer Protocol', 'TOC (Protokoll)')
wiki.add_edge('Hypertext Transfer Protocol', 'TR-069')
wiki.add_edge('Hypertext Transfer Protocol', 'TR-143')
wiki.add_edge('Hypertext Transfer Protocol', 'Transmission Control Protocol')
wiki.add_edge('Hypertext Transfer Protocol', 'Transport Layer Security')
wiki.add_edge('Hypertext Transfer Protocol', 'Tunnel (Rechnernetz)')
wiki.add_edge('Hypertext Transfer Protocol', 'Universal Plug and Play')
wiki.add_edge('Hypertext Transfer Protocol', 'User Datagram Protocol')
wiki.add_edge('Hypertext Transfer Protocol', 'Virtual Local Area Network')
wiki.add_edge('Hypertext Transfer Protocol', 'WPA2')
wiki.add_edge('Hypertext Transfer Protocol', 'Web Feature Service')
wiki.add_edge('Hypertext Transfer Protocol', 'WebDAV')
wiki.add_edge('Hypertext Transfer Protocol', 'WebHooks')
wiki.add_edge('Hypertext Transfer Protocol', 'WebSocket')
wiki.add_edge('Hypertext Transfer Protocol', 'Webservice')
wiki.add_edge('Hypertext Transfer Protocol', 'Whois')
wiki.add_edge('Hypertext Transfer Protocol', 'XML-RPC')
wiki.add_edge('Hypertext Transfer Protocol', 'XMLHttpRequest')
wiki.add_edge('Hypertext Transfer Protocol', 'Zustandslosigkeit')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'ARP-Spoofing')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'CAcert')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'Certificate Management Protocol')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'DNS over HTTPS')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'Exchange ActiveSync')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'Geminispace')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'HTTP-Authentifizierung')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'HTTPS Everywhere')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'Heartbleed')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'Hoccer')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'Homebanking Computer Interface')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'I2P')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'Internet Content Adaptation Protocol')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'JSON Meta Application Protocol')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'Liste der standardisierten Ports')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'Liste von TCP/IP-basierten Netzwerkdiensten')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'OpenSSL')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'Production Performance Management Protocol')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'Real Time Messaging Protocol')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'Representational State Transfer')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'S-HTTP')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'SOAP')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'STARTTLS')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'Secure Socket Tunneling Protocol')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'Server Name Indication')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'Service Worker')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'Skype')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'Tunnel (Rechnernetz)')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'Virtual Network Computing')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'WS-Security')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'Webservice')
wiki.add_edge('Hypertext Transfer Protocol Secure', 'Wireless Transport Layer Security')
wiki.add_edge('I2P', 'CyberGhost VPN')
wiki.add_edge('I2P', 'RetroShare')
wiki.add_edge('I2P', 'Tox (Protokoll)')
wiki.add_edge('I2P', 'Verge (Kryptowährung)')
wiki.add_edge('I2P', 'ZeroNet')
wiki.add_edge('ICMPv6', 'Any-source Multicast')
wiki.add_edge('ICMPv6', 'ICMP-Tunnel')
wiki.add_edge('ICMPv6', 'Internet Group Management Protocol')
wiki.add_edge('ICMPv6', 'Protokoll (IP)')
wiki.add_edge('IDEF', 'Datenmodellierung')
wiki.add_edge('IDEF', 'IDEF1X')
wiki.add_edge('IDEF', 'Softwaredesign')
wiki.add_edge('IDEF1X', 'Innovator (Software)')
wiki.add_edge('IEC 61850', 'IEC 60870')
wiki.add_edge('IEC 61850', 'IEC 61970')
wiki.add_edge('IEC 61850', 'IEC 62351')
wiki.add_edge('IEC 61850', 'Manufacturing Messaging Specification')
wiki.add_edge('IEEE 1471', 'ArchiMate')
wiki.add_edge('IEEE 802', '10-Gigabit-Ethernet')
wiki.add_edge('IEEE 802', '10BASE-T')
wiki.add_edge('IEEE 802', '10BASE2')
wiki.add_edge('IEEE 802', 'Distributed Queue Dual Bus')
wiki.add_edge('IEEE 802', 'Energy Efficient Ethernet')
wiki.add_edge('IEEE 802', 'Fast Ethernet')
wiki.add_edge('IEEE 802', 'Gigabit Ethernet')
wiki.add_edge('IEEE 802', 'IEEE 802.12')
wiki.add_edge('IEEE 802', 'IEEE 802.1X')
wiki.add_edge('IEEE 802', 'IEEE 802.1ah-2008')
wiki.add_edge('IEEE 802', 'IEEE 802.1aq')
wiki.add_edge('IEEE 802', 'IEEE 802.1p')
wiki.add_edge('IEEE 802', 'IEEE 802.3ae')
wiki.add_edge('IEEE 802', 'IEEE 802.3ak')
wiki.add_edge('IEEE 802', 'IEEE 802.3an')
wiki.add_edge('IEEE 802', 'Jumbo Frames')
wiki.add_edge('IEEE 802', 'LLDP')
wiki.add_edge('IEEE 802', 'Link Aggregation')
wiki.add_edge('IEEE 802', 'Power over Ethernet')
wiki.add_edge('IEEE 802', 'Rapid Spanning Tree Protocol')
wiki.add_edge('IEEE 802', 'Resilient Packet Ring')
wiki.add_edge('IEEE 802', 'Split Multi-Link Trunking')
wiki.add_edge('IEEE 802', 'Time-Sensitive Networking')
wiki.add_edge('IEEE 802', 'Virtual Local Area Network')
wiki.add_edge('IEEE 802', 'ZigBee')
wiki.add_edge('IEEE 802.1Q', 'Generic Object Oriented Substation Events')
wiki.add_edge('IEEE 802.1Q', 'IEEE 802')
wiki.add_edge('IEEE 802.1Q', 'IEEE 802.1ah-2008')
wiki.add_edge('IEEE 802.1Q', 'IEEE 802.1aq')
wiki.add_edge('IEEE 802.1Q', 'IEEE 802.1p')
wiki.add_edge('IEEE 802.1Q', 'IEEE 802.1q')
wiki.add_edge('IEEE 802.1Q', 'Inter-Switch Link Protocol')
wiki.add_edge('IEEE 802.1Q', 'Jumbo Frames')
wiki.add_edge('IEEE 802.1Q', 'Time-Sensitive Networking')
wiki.add_edge('IEEE 802.1Q', 'Virtual Local Area Network')
wiki.add_edge('IEEE 802.1X', 'Extensible Authentication Protocol')
wiki.add_edge('IEEE 802.1X', 'Port Based Network Access Control')
wiki.add_edge('IEEE 802.1X', 'Remote Authentication Dial-In User Service')
wiki.add_edge('IEEE 802.1aq', 'Distance Vector Multicast Routing Protocol')
wiki.add_edge('IEEE 802.1aq', 'IEEE 802.1ah-2008')
wiki.add_edge('IEEE 802.1aq', 'Link Aggregation')
wiki.add_edge('IEEE 802.1aq', 'MOSPF')
wiki.add_edge('IEEE 802.1aq', 'Multi-Path Routing')
wiki.add_edge('IEEE 802.1aq', 'PAgP')
wiki.add_edge('IEEE 802.1p', 'GMRP')
wiki.add_edge('IEEE 802.3ae', '10-Gigabit-Ethernet')
wiki.add_edge('IEEE 802.3ae', 'IEEE 802.3ak')
wiki.add_edge('IGD-Protokoll', 'NAT Port Mapping Protocol')
wiki.add_edge('IP-Adresse', '6over4')
wiki.add_edge('IP-Adresse', '6to4')
wiki.add_edge('IP-Adresse', 'ARP-Cache')
wiki.add_edge('IP-Adresse', 'ARP-Spoofing')
wiki.add_edge('IP-Adresse', 'Ad-hoc On-demand Distance Vector')
wiki.add_edge('IP-Adresse', 'Address Resolution Protocol')
wiki.add_edge('IP-Adresse', 'American Registry for Internet Numbers')
wiki.add_edge('IP-Adresse', 'Asia-Pacific Network Information Centre')
wiki.add_edge('IP-Adresse', 'B.A.T.M.A.N.')
wiki.add_edge('IP-Adresse', 'BitTorrent')
wiki.add_edge('IP-Adresse', 'Boot Service Discovery Protocol')
wiki.add_edge('IP-Adresse', 'Bootstrap Protocol')
wiki.add_edge('IP-Adresse', 'Carrier-grade NAT')
wiki.add_edge('IP-Adresse', 'Classless Inter-Domain Routing')
wiki.add_edge('IP-Adresse', 'Client')
wiki.add_edge('IP-Adresse', 'Common Address Redundancy Protocol')
wiki.add_edge('IP-Adresse', 'Connectionless Network Protocol')
wiki.add_edge('IP-Adresse', 'DNS over HTTPS')
wiki.add_edge('IP-Adresse', 'DNS over TLS')
wiki.add_edge('IP-Adresse', 'Datasource')
wiki.add_edge('IP-Adresse', 'Dotted decimal notation')
wiki.add_edge('IP-Adresse', 'Dynamic Host Configuration Protocol')
wiki.add_edge('IP-Adresse', 'EDonkey2000')
wiki.add_edge('IP-Adresse', 'Ethernet')
wiki.add_edge('IP-Adresse', 'FastTrack')
wiki.add_edge('IP-Adresse', 'File Transfer Protocol')
wiki.add_edge('IP-Adresse', 'File eXchange Protocol')
wiki.add_edge('IP-Adresse', 'Forwarding Equivalence Class')
wiki.add_edge('IP-Adresse', 'Host Identity Protocol')
wiki.add_edge('IP-Adresse', 'Hot Standby Router Protocol')
wiki.add_edge('IP-Adresse', 'Hypertext Transfer Protocol Secure')
wiki.add_edge('IP-Adresse', 'IGD-Protokoll')
wiki.add_edge('IP-Adresse', 'IP Address Management')
wiki.add_edge('IP-Adresse', 'IP-Paket')
wiki.add_edge('IP-Adresse', 'IP-Spoofing')
wiki.add_edge('IP-Adresse', 'IPv4')
wiki.add_edge('IP-Adresse', 'IPv6')
wiki.add_edge('IP-Adresse', 'IS-IS')
wiki.add_edge('IP-Adresse', 'Internet Assigned Numbers Authority')
wiki.add_edge('IP-Adresse', 'Internet Control Message Protocol')
wiki.add_edge('IP-Adresse', 'Internet Corporation for Assigned Names and Numbers')
wiki.add_edge('IP-Adresse', 'Internet Group Management Protocol')
wiki.add_edge('IP-Adresse', 'Internet Protocol Control Protocol')
wiki.add_edge('IP-Adresse', 'Link Layer Topology Discovery')
wiki.add_edge('IP-Adresse', 'Liste von TCP/IP-basierten Netzwerkdiensten')
wiki.add_edge('IP-Adresse', 'Local Internet Registry')
wiki.add_edge('IP-Adresse', 'Locator/Identifier Separation Protocol')
wiki.add_edge('IP-Adresse', 'MMS-Protokoll')
wiki.add_edge('IP-Adresse', 'Mobile IP')
wiki.add_edge('IP-Adresse', 'Multicast Backbone')
wiki.add_edge('IP-Adresse', 'Multiprotocol over ATM')
wiki.add_edge('IP-Adresse', 'NAT Port Mapping Protocol')
wiki.add_edge('IP-Adresse', 'NetBEUI')
wiki.add_edge('IP-Adresse', 'Netfilter')
wiki.add_edge('IP-Adresse', 'Network File System')
wiki.add_edge('IP-Adresse', 'Netzklasse')
wiki.add_edge('IP-Adresse', 'Netzmaske')
wiki.add_edge('IP-Adresse', 'Netzwerkadressübersetzung')
wiki.add_edge('IP-Adresse', 'OSI-Modell')
wiki.add_edge('IP-Adresse', 'On-Demand Mail Relay')
wiki.add_edge('IP-Adresse', 'OpenNIC')
wiki.add_edge('IP-Adresse', 'Peg DHCP')
wiki.add_edge('IP-Adresse', 'Point-to-Point Protocol')
wiki.add_edge('IP-Adresse', 'Port (Protokoll)')
wiki.add_edge('IP-Adresse', 'Port Address Translation')
wiki.add_edge('IP-Adresse', 'Portweiterleitung')
wiki.add_edge('IP-Adresse', 'Preboot Execution Environment')
wiki.add_edge('IP-Adresse', 'Private IP-Adresse')
wiki.add_edge('IP-Adresse', 'Profinet')
wiki.add_edge('IP-Adresse', 'Protokollstapel')
wiki.add_edge('IP-Adresse', 'Provider Aggregatable Address Space')
wiki.add_edge('IP-Adresse', 'Provider Independent Address Space')
wiki.add_edge('IP-Adresse', 'RIPE Network Coordination Centre')
wiki.add_edge('IP-Adresse', 'Rail Safe Transport Application')
wiki.add_edge('IP-Adresse', 'Regional Internet Registry')
wiki.add_edge('IP-Adresse', 'Registration Data Access Protocol')
wiki.add_edge('IP-Adresse', 'Remote Shell')
wiki.add_edge('IP-Adresse', 'Representational State Transfer')
wiki.add_edge('IP-Adresse', 'RetroShare')
wiki.add_edge('IP-Adresse', 'Reverse Address Resolution Protocol')
wiki.add_edge('IP-Adresse', 'SYN-Cookies')
wiki.add_edge('IP-Adresse', 'Secure Shell')
wiki.add_edge('IP-Adresse', 'Serial Line Internet Protocol')
wiki.add_edge('IP-Adresse', 'Server Name Indication')
wiki.add_edge('IP-Adresse', 'Service Location Protocol')
wiki.add_edge('IP-Adresse', 'Session Announcement Protocol')
wiki.add_edge('IP-Adresse', 'Session Initiation Protocol')
wiki.add_edge('IP-Adresse', 'Session Traversal Utilities for NAT')
wiki.add_edge('IP-Adresse', 'Skype')
wiki.add_edge('IP-Adresse', 'Socket (Software)')
wiki.add_edge('IP-Adresse', 'Stream Control Transmission Protocol')
wiki.add_edge('IP-Adresse', 'Subnetz')
wiki.add_edge('IP-Adresse', 'TP/IX')
wiki.add_edge('IP-Adresse', 'Teredo')
wiki.add_edge('IP-Adresse', 'Transmission Control Protocol')
wiki.add_edge('IP-Adresse', 'Transmission Control Protocol/Internet Protocol')
wiki.add_edge('IP-Adresse', 'Transport Layer Security')
wiki.add_edge('IP-Adresse', 'Trivial File Transfer Protocol')
wiki.add_edge('IP-Adresse', 'Tunnel (Rechnernetz)')
wiki.add_edge('IP-Adresse', 'Universal Plug and Play')
wiki.add_edge('IP-Adresse', 'Variable Length Subnet Mask')
wiki.add_edge('IP-Adresse', 'Verge (Kryptowährung)')
wiki.add_edge('IP-Adresse', 'Virtual Local Area Network')
wiki.add_edge('IP-Adresse', 'Virtual Router Redundancy Protocol')
wiki.add_edge('IP-Adresse', 'W3C Geolocation API')
wiki.add_edge('IP-Adresse', 'WhatsApp')
wiki.add_edge('IP-Adresse', 'Whois')
wiki.add_edge('IP-Adresse', 'XDMCP')
wiki.add_edge('IP-Adresse', 'Zephyr (Protokoll)')
wiki.add_edge('IP-Adresse', 'ZeroNet')
wiki.add_edge('IP-Adresse', 'Zeroconf')
wiki.add_edge('IP-Fragmentierung', '464XLAT')
wiki.add_edge('IP-Fragmentierung', '6LoWPAN')
wiki.add_edge('IP-Fragmentierung', '6in4')
wiki.add_edge('IP-Fragmentierung', 'BattleEye RCon Protocol')
wiki.add_edge('IP-Fragmentierung', 'Connectionless Network Protocol')
wiki.add_edge('IP-Fragmentierung', 'Datagramm')
wiki.add_edge('IP-Fragmentierung', 'ICMPv6')
wiki.add_edge('IP-Fragmentierung', 'IP-Header')
wiki.add_edge('IP-Fragmentierung', 'Jumbo Frames')
wiki.add_edge('IP-Fragmentierung', 'Protokoll (IP)')
wiki.add_edge('IP-Fragmentierung', 'TCP Receive Window')
wiki.add_edge('IP-Fragmentierung', 'Time-Sensitive Networking')
wiki.add_edge('IP-Header', '6in4')
wiki.add_edge('IP-Header', 'Explicit Congestion Notification')
wiki.add_edge('IP-Header', 'ILMI')
wiki.add_edge('IP-Header', 'Jumbo Frames')
wiki.add_edge('IP-Header', 'Protokoll (IP)')
wiki.add_edge('IP-Multicast', 'Train Real Time Data Protocol')
wiki.add_edge('IP-Paket', '464XLAT')
wiki.add_edge('IP-Paket', '4in6')
wiki.add_edge('IP-Paket', '6in4')
wiki.add_edge('IP-Paket', '6over4')
wiki.add_edge('IP-Paket', 'Address Resolution Protocol')
wiki.add_edge('IP-Paket', 'Datagramm')
wiki.add_edge('IP-Paket', 'DiffServ')
wiki.add_edge('IP-Paket', 'Explicit Congestion Notification')
wiki.add_edge('IP-Paket', 'Generic Object Oriented Substation Events')
wiki.add_edge('IP-Paket', 'IP-Fragmentierung')
wiki.add_edge('IP-Paket', 'IP-Spoofing')
wiki.add_edge('IP-Paket', 'IPsec')
wiki.add_edge('IP-Paket', 'ISATAP')
wiki.add_edge('IP-Paket', 'Internet Group Management Protocol')
wiki.add_edge('IP-Paket', 'Jumbo Frames')
wiki.add_edge('IP-Paket', 'Protokoll (IP)')
wiki.add_edge('IP-Spoofing', 'Sequenznummer')
wiki.add_edge('IP-Spoofing', 'TCP Sequence Prediction')
wiki.add_edge('IPSec', 'CyberGhost VPN')
wiki.add_edge('IPsec', 'Crypto IP Encapsulation')
wiki.add_edge('IPsec', 'Diameter (Protokoll)')
wiki.add_edge('IPsec', 'DirectAccess')
wiki.add_edge('IPsec', 'Generic Routing Encapsulation')
wiki.add_edge('IPsec', 'IEEE 802.1X')
wiki.add_edge('IPsec', 'IP-Spoofing')
wiki.add_edge('IPsec', 'IPSec')
wiki.add_edge('IPsec', 'Internet Security Association and Key Management Protocol')
wiki.add_edge('IPsec', 'KAME-Projekt')
wiki.add_edge('IPsec', 'Liste der standardisierten Ports')
wiki.add_edge('IPsec', 'Netfilter')
wiki.add_edge('IPsec', 'Protokoll (IP)')
wiki.add_edge('IPsec', 'Rail Safe Transport Application')
wiki.add_edge('IPsec', 'Secure Socket Tunneling Protocol')
wiki.add_edge('IPsec', 'Tunnel (Rechnernetz)')
wiki.add_edge('IPsec', 'Verschlüsselungsprotokoll')
wiki.add_edge('IPsec', 'Virtual Local Area Network')
wiki.add_edge('IPv4', '0.0.0.0')
wiki.add_edge('IPv4', '464XLAT')
wiki.add_edge('IPv4', '4in6')
wiki.add_edge('IPv4', '6in4')
wiki.add_edge('IPv4', '6over4')
wiki.add_edge('IPv4', '6to4')
wiki.add_edge('IPv4', 'Address Resolution Protocol')
wiki.add_edge('IPv4', 'AppleTalk')
wiki.add_edge('IPv4', 'Asia-Pacific Network Information Centre')
wiki.add_edge('IPv4', 'B.A.T.M.A.N.')
wiki.add_edge('IPv4', 'BACnet')
wiki.add_edge('IPv4', 'Border Gateway Protocol')
wiki.add_edge('IPv4', 'Carrier-grade NAT')
wiki.add_edge('IPv4', 'Certificate Management Protocol')
wiki.add_edge('IPv4', 'Classless Inter-Domain Routing')
wiki.add_edge('IPv4', 'Common Address Redundancy Protocol')
wiki.add_edge('IPv4', 'DNS over HTTPS')
wiki.add_edge('IPv4', 'DNS over TLS')
wiki.add_edge('IPv4', 'Datagramm')
wiki.add_edge('IPv4', 'DiffServ')
wiki.add_edge('IPv4', 'DirectAccess')
wiki.add_edge('IPv4', 'Distanzvektoralgorithmus')
wiki.add_edge('IPv4', 'Dotted decimal notation')
wiki.add_edge('IPv4', 'Dynamic Host Configuration Protocol')
wiki.add_edge('IPv4', 'Ethernet')
wiki.add_edge('IPv4', 'Explicit Congestion Notification')
wiki.add_edge('IPv4', 'Generic Routing Encapsulation')
wiki.add_edge('IPv4', 'GigE Vision')
wiki.add_edge('IPv4', 'I2P')
wiki.add_edge('IPv4', 'ICMPv6')
wiki.add_edge('IPv4', 'IP Address Management')
wiki.add_edge('IPv4', 'IP-Fragmentierung')
wiki.add_edge('IPv4', 'IP-Header')
wiki.add_edge('IPv4', 'IP-Paket')
wiki.add_edge('IPv4', 'IPv6')
wiki.add_edge('IPv4', 'ISATAP')
wiki.add_edge('IPv4', 'IntServ')
wiki.add_edge('IPv4', 'Internet Assigned Numbers Authority')
wiki.add_edge('IPv4', 'Internet Control Message Protocol')
wiki.add_edge('IPv4', 'Internet Corporation for Assigned Names and Numbers')
wiki.add_edge('IPv4', 'Internet Group Management Protocol')
wiki.add_edge('IPv4', 'Internet Printing Protocol')
wiki.add_edge('IPv4', 'Internetwork Packet Exchange')
wiki.add_edge('IPv4', 'Link Layer Topology Discovery')
wiki.add_edge('IPv4', 'Link-local Multicast Name Resolution')
wiki.add_edge('IPv4', 'Locator/Identifier Separation Protocol')
wiki.add_edge('IPv4', 'Multicast Backbone')
wiki.add_edge('IPv4', 'NAT64')
wiki.add_edge('IPv4', 'Neighbor Discovery Protocol')
wiki.add_edge('IPv4', 'NetBIOS')
wiki.add_edge('IPv4', 'Network File System')
wiki.add_edge('IPv4', 'Netzklasse')
wiki.add_edge('IPv4', 'Netzmaske')
wiki.add_edge('IPv4', 'Netzwerkadressübersetzung')
wiki.add_edge('IPv4', 'OSI-Modell')
wiki.add_edge('IPv4', 'Pfsync')
wiki.add_edge('IPv4', 'Point-to-Point Protocol')
wiki.add_edge('IPv4', 'Private IP-Adresse')
wiki.add_edge('IPv4', 'Protokoll (IP)')
wiki.add_edge('IPv4', 'Protokollstapel')
wiki.add_edge('IPv4', 'RIPE Network Coordination Centre')
wiki.add_edge('IPv4', 'ROHC')
wiki.add_edge('IPv4', 'Regional Internet Registry')
wiki.add_edge('IPv4', 'Resource Reservation Protocol')
wiki.add_edge('IPv4', 'Routing Protocol for Low power and Lossy Networks')
wiki.add_edge('IPv4', 'SOCKS')
wiki.add_edge('IPv4', 'Secure Socket Tunneling Protocol')
wiki.add_edge('IPv4', 'Serial Line Internet Protocol')
wiki.add_edge('IPv4', 'Session Announcement Protocol')
wiki.add_edge('IPv4', 'Simple Service Discovery Protocol')
wiki.add_edge('IPv4', 'Source-specific Multicast')
wiki.add_edge('IPv4', 'Subnetz')
wiki.add_edge('IPv4', 'TP/IX')
wiki.add_edge('IPv4', 'Teredo')
wiki.add_edge('IPv4', 'Transmission Control Protocol')
wiki.add_edge('IPv4', 'Transmission Control Protocol/Internet Protocol')
wiki.add_edge('IPv4', 'Transport Layer Security')
wiki.add_edge('IPv4', 'User Datagram Protocol')
wiki.add_edge('IPv4', 'Virtual Local Area Network')
wiki.add_edge('IPv4', 'Whois')
wiki.add_edge('IPv4', 'World IPv6 Day und World IPv6 Launch Day')
wiki.add_edge('IPv4', 'Zeroconf')
wiki.add_edge('IPv6', '0.0.0.0')
wiki.add_edge('IPv6', '464XLAT')
wiki.add_edge('IPv6', '4in6')
wiki.add_edge('IPv6', '6Bone')
wiki.add_edge('IPv6', '6LoWPAN')
wiki.add_edge('IPv6', '6in4')
wiki.add_edge('IPv6', '6over4')
wiki.add_edge('IPv6', '6rd')
wiki.add_edge('IPv6', '6to4')
wiki.add_edge('IPv6', 'AYIYA')
wiki.add_edge('IPv6', 'Address Resolution Protocol')
wiki.add_edge('IPv6', 'American Registry for Internet Numbers')
wiki.add_edge('IPv6', 'Any-source Multicast')
wiki.add_edge('IPv6', 'Asia-Pacific Network Information Centre')
wiki.add_edge('IPv6', 'B.A.T.M.A.N.')
wiki.add_edge('IPv6', 'BACnet')
wiki.add_edge('IPv6', 'Border Gateway Protocol')
wiki.add_edge('IPv6', 'Carrier-grade NAT')
wiki.add_edge('IPv6', 'Certificate Management Protocol')
wiki.add_edge('IPv6', 'Classless Inter-Domain Routing')
wiki.add_edge('IPv6', 'Common Address Redundancy Protocol')
wiki.add_edge('IPv6', 'DHCPv6')
wiki.add_edge('IPv6', 'DNS over HTTPS')
wiki.add_edge('IPv6', 'DNS over TLS')
wiki.add_edge('IPv6', 'DiffServ')
wiki.add_edge('IPv6', 'DirectAccess')
wiki.add_edge('IPv6', 'Distanzvektoralgorithmus')
wiki.add_edge('IPv6', 'Dynamic Host Configuration Protocol')
wiki.add_edge('IPv6', 'Enhanced Interior Gateway Routing Protocol')
wiki.add_edge('IPv6', 'Ethernet')
wiki.add_edge('IPv6', 'Explicit Congestion Notification')
wiki.add_edge('IPv6', 'File Transfer Protocol')
wiki.add_edge('IPv6', 'Generic Routing Encapsulation')
wiki.add_edge('IPv6', 'I2P')
wiki.add_edge('IPv6', 'ICMPv6')
wiki.add_edge('IPv6', 'IP Address Management')
wiki.add_edge('IPv6', 'IP-Fragmentierung')
wiki.add_edge('IPv6', 'IP-Paket')
wiki.add_edge('IPv6', 'IP500 Alliance')
wiki.add_edge('IPv6', 'IPsec')
wiki.add_edge('IPv6', 'IS-IS')
wiki.add_edge('IPv6', 'ISATAP')
wiki.add_edge('IPv6', 'Interior Gateway Protocol')
wiki.add_edge('IPv6', 'Internet Assigned Numbers Authority')
wiki.add_edge('IPv6', 'Internet Control Message Protocol')
wiki.add_edge('IPv6', 'Internet Corporation for Assigned Names and Numbers')
wiki.add_edge('IPv6', 'Internet Group Management Protocol')
wiki.add_edge('IPv6', 'Internet Printing Protocol')
wiki.add_edge('IPv6', 'Internet Protocol over Avian Carriers')
wiki.add_edge('IPv6', 'KAME-Projekt')
wiki.add_edge('IPv6', 'Label Distribution Protocol')
wiki.add_edge('IPv6', 'Layer 2 Tunneling Protocol')
wiki.add_edge('IPv6', 'Link Layer Topology Discovery')
wiki.add_edge('IPv6', 'Link-local Multicast Name Resolution')
wiki.add_edge('IPv6', 'Liste von IPv6-Tunnelbrokern')
wiki.add_edge('IPv6', 'Mobile IP')
wiki.add_edge('IPv6', 'NAT64')
wiki.add_edge('IPv6', 'Neighbor Discovery Protocol')
wiki.add_edge('IPv6', 'Netflow')
wiki.add_edge('IPv6', 'Network File System')
wiki.add_edge('IPv6', 'Network Time Protocol')
wiki.add_edge('IPv6', 'Netzmaske')
wiki.add_edge('IPv6', 'Netzwerkprotokoll')
wiki.add_edge('IPv6', 'OSI-Modell')
wiki.add_edge('IPv6', 'Open Shortest Path First')
wiki.add_edge('IPv6', 'Peer Name Resolution Protocol')
wiki.add_edge('IPv6', 'Pfsync')
wiki.add_edge('IPv6', 'Point-to-Point Protocol')
wiki.add_edge('IPv6', 'Private IP-Adresse')
wiki.add_edge('IPv6', 'Profinet')
wiki.add_edge('IPv6', 'Protocol Independent Multicast')
wiki.add_edge('IPv6', 'Protokoll (IP)')
wiki.add_edge('IPv6', 'Protokollstapel')
wiki.add_edge('IPv6', 'RIPE Network Coordination Centre')
wiki.add_edge('IPv6', 'ROHC')
wiki.add_edge('IPv6', 'Regional Internet Registry')
wiki.add_edge('IPv6', 'Resource Reservation Protocol')
wiki.add_edge('IPv6', 'Routing Information Protocol')
wiki.add_edge('IPv6', 'Routing Protocol for Low power and Lossy Networks')
wiki.add_edge('IPv6', 'SOCKS')
wiki.add_edge('IPv6', 'Schichtenarchitektur')
wiki.add_edge('IPv6', 'Secure Socket Tunneling Protocol')
wiki.add_edge('IPv6', 'Session Announcement Protocol')
wiki.add_edge('IPv6', 'Session Initiation Protocol')
wiki.add_edge('IPv6', 'Simple Service Discovery Protocol')
wiki.add_edge('IPv6', 'Socket (Software)')
wiki.add_edge('IPv6', 'Source-specific Multicast')
wiki.add_edge('IPv6', 'Subnetz')
wiki.add_edge('IPv6', 'TCP Receive Window')
wiki.add_edge('IPv6', 'TP/IX')
wiki.add_edge('IPv6', 'Teredo')
wiki.add_edge('IPv6', 'Thread (Netzwerkprotokoll)')
wiki.add_edge('IPv6', 'Transmission Control Protocol')
wiki.add_edge('IPv6', 'Transmission Control Protocol/Internet Protocol')
wiki.add_edge('IPv6', 'Transport Layer Security')
wiki.add_edge('IPv6', 'Tunnel (Rechnernetz)')
wiki.add_edge('IPv6', 'User Datagram Protocol')
wiki.add_edge('IPv6', 'Whois')
wiki.add_edge('IPv6', 'World IPv6 Day und World IPv6 Launch Day')
wiki.add_edge('IPv6', 'Zeroconf')
wiki.add_edge('IPv6', 'ZigBee')
wiki.add_edge('IPv6', 'Zustandsbehaftung')
wiki.add_edge('IPv6', 'Zustandslosigkeit')
wiki.add_edge('IS-IS', 'Aggregation (Informatik)')
wiki.add_edge('IS-IS', 'Bidirectional Forwarding Detection')
wiki.add_edge('IS-IS', 'IEEE 802.1aq')
wiki.add_edge('IS-IS', 'Interior Gateway Routing Protocol')
wiki.add_edge('IS-IS', 'Open Shortest Path First')
wiki.add_edge('IS-IS', 'Protocol Independent Multicast')
wiki.add_edge('IS-IS', 'Protokoll (IP)')
wiki.add_edge('IS-IS', 'Routing Protocol for Low power and Lossy Networks')
wiki.add_edge('ISCSI', 'Fibre Channel over IP')
wiki.add_edge('ISCSI', 'HyperSCSI')
wiki.add_edge('ISO 15765-2', 'FlexRay')
wiki.add_edge('ISO 15765-2', 'KWP2000')
wiki.add_edge('ISO 15765-2', 'Unified Diagnostic Services')
wiki.add_edge('ISOBUS', 'SAE J1939')
wiki.add_edge('Idiom (Softwaretechnik)', 'Architekturmuster')
wiki.add_edge('Image and Scanner Interface Specification', 'Scanner Access Now Easy')
wiki.add_edge('Immediate Mode (Computergrafik)', 'Immediate mode (Computergrafik)')
wiki.add_edge('Immediate Mode (Computergrafik)', 'Retained Mode')
wiki.add_edge('Independent Computing Architecture', 'Druckprotokoll')
wiki.add_edge('Indexed Database API', 'Service Worker')
wiki.add_edge('Industrial Ethernet', 'Advanced Physical Layer')
wiki.add_edge('Industrial Ethernet', 'EchoRing')
wiki.add_edge('Industrial Ethernet', 'EtherCAT')
wiki.add_edge('Industrial Ethernet', 'EtherNet/IP')
wiki.add_edge('Industrial Ethernet', 'Ethernet Powerlink')
wiki.add_edge('Industrial Ethernet', 'Liste der standardisierten Ports')
wiki.add_edge('Industrial Ethernet', 'OpenSAFETY')
wiki.add_edge('Industrial Ethernet', 'Profinet')
wiki.add_edge('Industrial Ethernet', 'Rapid Spanning Tree Protocol')
wiki.add_edge('Industrial Ethernet', 'SERCOS III')
wiki.add_edge('Industrial Ethernet', 'SafetyNET p')
wiki.add_edge('Informationseinheit (UML)', 'Generalisierung (UML)')
wiki.add_edge('Informationseinheit (UML)', 'Informationsfluss (UML)')
wiki.add_edge('Innovator (Software)', 'ArchiMate')
wiki.add_edge('Innovator (Software)', 'Data Vault')
wiki.add_edge('Inter-Switch Link Protocol', 'Dynamic Trunking Protocol')
wiki.add_edge('Interactive Connectivity Establishment', 'Session Traversal Utilities for NAT')
wiki.add_edge('Interactive Connectivity Establishment', 'Threema')
wiki.add_edge('Interaktion (UML)', 'Classifier (UML)')
wiki.add_edge('Interaktion (UML)', 'Ereignis (UML)')
wiki.add_edge('Interaktion (UML)', 'Interaktionsübersichtsdiagramm')
wiki.add_edge('Interaktion (UML)', 'Knoten (UML)')
wiki.add_edge('Interaktion (UML)', 'Kommunikationsdiagramm (UML)')
wiki.add_edge('Interaktion (UML)', 'Operation (UML)')
wiki.add_edge('Interaktion (UML)', 'Sequenzdiagramm')
wiki.add_edge('Interaktion (UML)', 'Signal (UML)')
wiki.add_edge('Interaktion (UML)', 'Zeitverlaufsdiagramm')
wiki.add_edge('Interaktionsübersichtsdiagramm', 'StarUML')
wiki.add_edge('Interior Gateway Protocol', 'Bidirectional Forwarding Detection')
wiki.add_edge('Interior Gateway Protocol', 'Enhanced Interior Gateway Routing Protocol')
wiki.add_edge('Interior Gateway Protocol', 'IS-IS')
wiki.add_edge('Interior Gateway Protocol', 'Open Shortest Path First')
wiki.add_edge('Interior Gateway Protocol', 'Protokoll (IP)')
wiki.add_edge('Interior Gateway Routing Protocol', '6LoWPAN')
wiki.add_edge('Interior Gateway Routing Protocol', 'Distanzvektoralgorithmus')
wiki.add_edge('Interior Gateway Routing Protocol', 'Open Shortest Path First')
wiki.add_edge('Interior Gateway Routing Protocol', 'Triggered Updates')
wiki.add_edge('Internet Assigned Numbers Authority', 'Address Resolution Protocol')
wiki.add_edge('Internet Assigned Numbers Authority', 'Bonjour (Apple)')
wiki.add_edge('Internet Assigned Numbers Authority', 'Classless Inter-Domain Routing')
wiki.add_edge('Internet Assigned Numbers Authority', 'Domain Name System Security Extensions')
wiki.add_edge('Internet Assigned Numbers Authority', 'IP-Paket')
wiki.add_edge('Internet Assigned Numbers Authority', 'Internet Corporation for Assigned Names and Numbers')
wiki.add_edge('Internet Assigned Numbers Authority', 'Internet Media Type')
wiki.add_edge('Internet Assigned Numbers Authority', 'Liste der standardisierten Ports')
wiki.add_edge('Internet Assigned Numbers Authority', 'Local Internet Registry')
wiki.add_edge('Internet Assigned Numbers Authority', 'MQTT')
wiki.add_edge('Internet Assigned Numbers Authority', 'Network Information Center')
wiki.add_edge('Internet Assigned Numbers Authority', 'Network News Transfer Protocol')
wiki.add_edge('Internet Assigned Numbers Authority', 'Netzwerkprotokoll')
wiki.add_edge('Internet Assigned Numbers Authority', 'Private IP-Adresse')
wiki.add_edge('Internet Assigned Numbers Authority', 'Protokoll (IP)')
wiki.add_edge('Internet Assigned Numbers Authority', 'Provider Aggregatable Address Space')
wiki.add_edge('Internet Assigned Numbers Authority', 'Provider Independent Address Space')
wiki.add_edge('Internet Assigned Numbers Authority', 'Regional Internet Registry')
wiki.add_edge('Internet Assigned Numbers Authority', 'SMTPS')
wiki.add_edge('Internet Assigned Numbers Authority', 'SOCKS')
wiki.add_edge('Internet Assigned Numbers Authority', 'STARTTLS')
wiki.add_edge('Internet Assigned Numbers Authority', 'Secure Shell')
wiki.add_edge('Internet Assigned Numbers Authority', 'Simple Asynchronous File Transfer')
wiki.add_edge('Internet Assigned Numbers Authority', 'Simple Network Management Protocol')
wiki.add_edge('Internet Assigned Numbers Authority', 'Simple Service Discovery Protocol')
wiki.add_edge('Internet Assigned Numbers Authority', 'Virtual Extensible LAN')
wiki.add_edge('Internet Assigned Numbers Authority', 'Whois')
wiki.add_edge('Internet Assigned Numbers Authority', 'Zeroconf')
wiki.add_edge('Internet Cache Protocol', 'Hyper Text Caching Protocol')
wiki.add_edge('Internet Communications Engine', 'Etch (Protokoll)')
wiki.add_edge('Internet Control Message Protocol', '464XLAT')
wiki.add_edge('Internet Control Message Protocol', 'Fiber Distributed Data Interface')
wiki.add_edge('Internet Control Message Protocol', 'ICMP-Tunnel')
wiki.add_edge('Internet Control Message Protocol', 'ICMPv6')
wiki.add_edge('Internet Control Message Protocol', 'IP-Fragmentierung')
wiki.add_edge('Internet Control Message Protocol', 'Internet Group Management Protocol')
wiki.add_edge('Internet Control Message Protocol', 'Internet Protocol over Avian Carriers')
wiki.add_edge('Internet Control Message Protocol', 'Mobile IP')
wiki.add_edge('Internet Control Message Protocol', 'Netfilter')
wiki.add_edge('Internet Control Message Protocol', 'Protokoll (IP)')
wiki.add_edge('Internet Control Message Protocol', 'Token Bus')
wiki.add_edge('Internet Control Message Protocol', 'Token Ring')
wiki.add_edge('Internet Control Message Protocol', 'Transmission Control Protocol/Internet Protocol')
wiki.add_edge('Internet Control Message Protocol', 'Tunnel (Rechnernetz)')
wiki.add_edge('Internet Corporation for Assigned Names and Numbers', 'Domain Name System Security Extensions')
wiki.add_edge('Internet Corporation for Assigned Names and Numbers', 'Local Internet Registry')
wiki.add_edge('Internet Corporation for Assigned Names and Numbers', 'Network Information Center')
wiki.add_edge('Internet Corporation for Assigned Names and Numbers', 'Netzwerkprotokoll')
wiki.add_edge('Internet Corporation for Assigned Names and Numbers', 'OpenNIC')
wiki.add_edge('Internet Corporation for Assigned Names and Numbers', 'RIPE Network Coordination Centre')
wiki.add_edge('Internet Corporation for Assigned Names and Numbers', 'Regional Internet Registry')
wiki.add_edge('Internet Corporation for Assigned Names and Numbers', 'Whois')
wiki.add_edge('Internet Group Management Protocol', 'Any-source Multicast')
wiki.add_edge('Internet Group Management Protocol', 'Cisco Group Management Protocol')
wiki.add_edge('Internet Group Management Protocol', 'Distance Vector Multicast Routing Protocol')
wiki.add_edge('Internet Group Management Protocol', 'MOSPF')
wiki.add_edge('Internet Group Management Protocol', 'Protocol Independent Multicast')
wiki.add_edge('Internet Group Management Protocol', 'Protokoll (IP)')
wiki.add_edge('Internet Group Management Protocol', 'Source-specific Multicast')
wiki.add_edge('Internet Group Management Protocol', 'Streaming-Protokoll')
wiki.add_edge('Internet Media Type', 'Geminispace')
wiki.add_edge('Internet Media Type', 'Push-Model')
wiki.add_edge('Internet Message Access Protocol', 'CRAM-MD5')
wiki.add_edge('Internet Message Access Protocol', 'GSSAPI')
wiki.add_edge('Internet Message Access Protocol', 'JSON Meta Application Protocol')
wiki.add_edge('Internet Message Access Protocol', 'Jakarta Mail')
wiki.add_edge('Internet Message Access Protocol', 'Liste der standardisierten Ports')
wiki.add_edge('Internet Message Access Protocol', 'Liste von TCP/IP-basierten Netzwerkdiensten')
wiki.add_edge('Internet Message Access Protocol', 'Push-Model')
wiki.add_edge('Internet Message Access Protocol', 'STARTTLS')
wiki.add_edge('Internet Message Access Protocol', 'Simple Authentication and Security Layer')
wiki.add_edge('Internet Message Access Protocol', 'Simple Mail Access Protocol')
wiki.add_edge('Internet Printing Protocol', 'AirPrint')
wiki.add_edge('Internet Printing Protocol', 'Berkeley Printing System')
wiki.add_edge('Internet Printing Protocol', 'Druckprotokoll')
wiki.add_edge('Internet Protocol', '6LoWPAN')
wiki.add_edge('Internet Protocol', 'ATA over Ethernet')
wiki.add_edge('Internet Protocol', 'Address Resolution Protocol')
wiki.add_edge('Internet Protocol', 'Advanced Physical Layer')
wiki.add_edge('Internet Protocol', 'AppleTalk')
wiki.add_edge('Internet Protocol', 'B.A.T.M.A.N.')
wiki.add_edge('Internet Protocol', 'BattleEye RCon Protocol')
wiki.add_edge('Internet Protocol', 'Bidirectional Forwarding Detection')
wiki.add_edge('Internet Protocol', 'Bonjour (Apple)')
wiki.add_edge('Internet Protocol', 'Border Gateway Protocol')
wiki.add_edge('Internet Protocol', 'CSLIP')
wiki.add_edge('Internet Protocol', 'Cellular-IP')
wiki.add_edge('Internet Protocol', 'Class of Service')
wiki.add_edge('Internet Protocol', 'Common Address Redundancy Protocol')
wiki.add_edge('Internet Protocol', 'Common ISDN Application Programming Interface')
wiki.add_edge('Internet Protocol', 'Connectionless Network Protocol')
wiki.add_edge('Internet Protocol', 'CorNet')
wiki.add_edge('Internet Protocol', 'Crypto IP Encapsulation')
wiki.add_edge('Internet Protocol', 'Dante (Netzwerkprotokoll)')
wiki.add_edge('Internet Protocol', 'Datagram Congestion Control Protocol')
wiki.add_edge('Internet Protocol', 'Datagram Delivery Protocol')
wiki.add_edge('Internet Protocol', 'DiffServ')
wiki.add_edge('Internet Protocol', 'Enhanced Interior Gateway Routing Protocol')
wiki.add_edge('Internet Protocol', 'Ethernet Powerlink')
wiki.add_edge('Internet Protocol', 'Explicit Congestion Notification')
wiki.add_edge('Internet Protocol', 'Extensible Authentication Protocol')
wiki.add_edge('Internet Protocol', 'File Transfer Protocol')
wiki.add_edge('Internet Protocol', 'Finger (Internetprotokoll)')
wiki.add_edge('Internet Protocol', 'Frame Relay')
wiki.add_edge('Internet Protocol', 'GPRS Tunneling Protocol')
wiki.add_edge('Internet Protocol', 'Generic Routing Encapsulation')
wiki.add_edge('Internet Protocol', 'H.-Standards')
wiki.add_edge('Internet Protocol', 'Host Identity Protocol')
wiki.add_edge('Internet Protocol', 'HyperSCSI')
wiki.add_edge('Internet Protocol', 'I2P')
wiki.add_edge('Internet Protocol', 'IP-Adresse')
wiki.add_edge('Internet Protocol', 'IP-Fragmentierung')
wiki.add_edge('Internet Protocol', 'IP-Header')
wiki.add_edge('Internet Protocol', 'IP-Paket')
wiki.add_edge('Internet Protocol', 'IPsec')
wiki.add_edge('Internet Protocol', 'IPv4')
wiki.add_edge('Internet Protocol', 'IPv6')
wiki.add_edge('Internet Protocol', 'IS-IS')
wiki.add_edge('Internet Protocol', 'ISCSI')
wiki.add_edge('Internet Protocol', 'ISDN over IP')
wiki.add_edge('Internet Protocol', 'IntServ')
wiki.add_edge('Internet Protocol', 'Internet Assigned Numbers Authority')
wiki.add_edge('Internet Protocol', 'Internet Control Message Protocol')
wiki.add_edge('Internet Protocol', 'Internet Corporation for Assigned Names and Numbers')
wiki.add_edge('Internet Protocol', 'Internet Group Management Protocol')
wiki.add_edge('Internet Protocol', 'Internet Protocol over Fibre Channel')
wiki.add_edge('Internet Protocol', 'Internet Relay Chat')
wiki.add_edge('Internet Protocol', 'Internetwork Datagram Protocol')
wiki.add_edge('Internet Protocol', 'Internetwork Packet Exchange')
wiki.add_edge('Internet Protocol', 'Kinetics Internet Protocol')
wiki.add_edge('Internet Protocol', 'Layer 2 Tunneling Protocol')
wiki.add_edge('Internet Protocol', 'Lightweight Directory Access Protocol')
wiki.add_edge('Internet Protocol', 'Local Internet Registry')
wiki.add_edge('Internet Protocol', 'Locator/Identifier Separation Protocol')
wiki.add_edge('Internet Protocol', 'MQTT')
wiki.add_edge('Internet Protocol', 'Mobile IP')
wiki.add_edge('Internet Protocol', 'Multicast Backbone')
wiki.add_edge('Internet Protocol', 'Multiprotocol over ATM')
wiki.add_edge('Internet Protocol', 'NetBEUI')
wiki.add_edge('Internet Protocol', 'NetBIOS over TCP/IP')
wiki.add_edge('Internet Protocol', 'Netflow')
wiki.add_edge('Internet Protocol', 'Network File System')
wiki.add_edge('Internet Protocol', 'Netzklasse')
wiki.add_edge('Internet Protocol', 'Netzwerkadressübersetzung')
wiki.add_edge('Internet Protocol', 'Netzwerkprotokoll')
wiki.add_edge('Internet Protocol', 'OSI-Modell')
wiki.add_edge('Internet Protocol', 'Open Shortest Path First')
wiki.add_edge('Internet Protocol', 'Packet Data Convergence Protocol')
wiki.add_edge('Internet Protocol', 'Point-to-Point Protocol')
wiki.add_edge('Internet Protocol', 'Point-to-Point Tunneling Protocol')
wiki.add_edge('Internet Protocol', 'Private IP-Adresse')
wiki.add_edge('Internet Protocol', 'Profinet')
wiki.add_edge('Internet Protocol', 'Protokoll (IP)')
wiki.add_edge('Internet Protocol', 'Protokollstapel')
wiki.add_edge('Internet Protocol', 'ROHC')
wiki.add_edge('Internet Protocol', 'Rail Safe Transport Application')
wiki.add_edge('Internet Protocol', 'Real-Time Streaming Protocol')
wiki.add_edge('Internet Protocol', 'Real-Time Transport Protocol')
wiki.add_edge('Internet Protocol', 'Regional Internet Registry')
wiki.add_edge('Internet Protocol', 'Remote Authentication Dial-In User Service')
wiki.add_edge('Internet Protocol', 'Resource Reservation Protocol')
wiki.add_edge('Internet Protocol', 'Routing Information Protocol')
wiki.add_edge('Internet Protocol', 'SafetyNET p')
wiki.add_edge('Internet Protocol', 'Secure Socket Tunneling Protocol')
wiki.add_edge('Internet Protocol', 'Sequenced Packet Exchange')
wiki.add_edge('Internet Protocol', 'Serial Line Internet Protocol')
wiki.add_edge('Internet Protocol', 'Server Message Block')
wiki.add_edge('Internet Protocol', 'Signalling Connection Control Part')
wiki.add_edge('Internet Protocol', 'Signalling System 7')
wiki.add_edge('Internet Protocol', 'Skinny Client Control Protocol')
wiki.add_edge('Internet Protocol', 'Stream Control Transmission Protocol')
wiki.add_edge('Internet Protocol', 'Streaming-Protokoll')
wiki.add_edge('Internet Protocol', 'Subnetz')
wiki.add_edge('Internet Protocol', 'Syslog')
wiki.add_edge('Internet Protocol', 'TCP Sequence Prediction')
wiki.add_edge('Internet Protocol', 'TP/IX')
wiki.add_edge('Internet Protocol', 'Thread (Netzwerkprotokoll)')
wiki.add_edge('Internet Protocol', 'Transmission Control Protocol')
wiki.add_edge('Internet Protocol', 'Transmission Control Protocol/Internet Protocol')
wiki.add_edge('Internet Protocol', 'Transport Layer Security')
wiki.add_edge('Internet Protocol', 'Tunnel (Rechnernetz)')
wiki.add_edge('Internet Protocol', 'Universal Measurement and Calibration Protocol')
wiki.add_edge('Internet Protocol', 'Universal Plug and Play')
wiki.add_edge('Internet Protocol', 'User Datagram Protocol')
wiki.add_edge('Internet Protocol', 'VTun')
wiki.add_edge('Internet Protocol', 'Virtual Router Redundancy Protocol')
wiki.add_edge('Internet Protocol', 'Web Application Description Language')
wiki.add_edge('Internet Protocol', 'Weihenstephaner Standards')
wiki.add_edge('Internet Protocol', 'Whois')
wiki.add_edge('Internet Protocol', 'Winsock')
wiki.add_edge('Internet Protocol', 'World IPv6 Day und World IPv6 Launch Day')
wiki.add_edge('Internet Protocol', 'Zeroconf')
wiki.add_edge('Internet Protocol Flow Information Export', 'Netflow')
wiki.add_edge('Internet Protocol over Avian Carriers', 'Pi Digit Generation Protocol')
wiki.add_edge('Internet Relay Chat', 'ASCII-Protokoll')
wiki.add_edge('Internet Relay Chat', 'Direct Client-to-Client')
wiki.add_edge('Internet Relay Chat', 'Extensible Messaging and Presence Protocol')
wiki.add_edge('Internet Relay Chat', 'I2P')
wiki.add_edge('Internet Relay Chat', 'Ident')
wiki.add_edge('Internet Relay Chat', 'Liste der standardisierten Ports')
wiki.add_edge('Internet Relay Chat', 'Matrix (Kommunikationsprotokoll)')
wiki.add_edge('Internet Relay Chat', 'OSCAR (Protokoll)')
wiki.add_edge('Internet Relay Chat', 'Off-the-Record Messaging')
wiki.add_edge('Internet Relay Chat', 'Pichat')
wiki.add_edge('Internet Relay Chat', 'Protocol for SYnchronous Conferencing')
wiki.add_edge('Internet Relay Chat', 'RetroShare')
wiki.add_edge('Internet Relay Chat', 'SILC')
wiki.add_edge('Internet Relay Chat', 'Telepathy')
wiki.add_edge('Internet Relay Chat', 'Whois')
wiki.add_edge('Internet Security Association and Key Management Protocol', 'Encapsulating Security Payload')
wiki.add_edge('Internetprotokollfamilie', 'Address Resolution Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Agent Extensibility Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Boot Service Discovery Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Bootstrap Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Border Gateway Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Common Address Redundancy Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Common Open Policy Service')
wiki.add_edge('Internetprotokollfamilie', 'Datagram Transport Layer Security')
wiki.add_edge('Internetprotokollfamilie', 'Discard')
wiki.add_edge('Internetprotokollfamilie', 'Distanzvektoralgorithmus')
wiki.add_edge('Internetprotokollfamilie', 'DoIP')
wiki.add_edge('Internetprotokollfamilie', 'Dynamic Host Configuration Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Echo (Netzwerkdienst)')
wiki.add_edge('Internetprotokollfamilie', 'Enhanced Interior Gateway Routing Protocol')
wiki.add_edge('Internetprotokollfamilie', 'EtherNet/IP')
wiki.add_edge('Internetprotokollfamilie', 'EtherType')
wiki.add_edge('Internetprotokollfamilie', 'Ethernet')
wiki.add_edge('Internetprotokollfamilie', 'Extensible Messaging and Presence Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Fiber Distributed Data Interface')
wiki.add_edge('Internetprotokollfamilie', 'File Transfer Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Finger (Internetprotokoll)')
wiki.add_edge('Internetprotokollfamilie', 'GMRP')
wiki.add_edge('Internetprotokollfamilie', 'Gopher')
wiki.add_edge('Internetprotokollfamilie', 'Hot Standby Router Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Hypertext Transfer Protocol Secure')
wiki.add_edge('Internetprotokollfamilie', 'ICMPv6')
wiki.add_edge('Internetprotokollfamilie', 'IEC 60870')
wiki.add_edge('Internetprotokollfamilie', 'IEEE 802.1Q')
wiki.add_edge('Internetprotokollfamilie', 'IP-Adresse')
wiki.add_edge('Internetprotokollfamilie', 'IP-Header')
wiki.add_edge('Internetprotokollfamilie', 'IP-Paket')
wiki.add_edge('Internetprotokollfamilie', 'IPsec')
wiki.add_edge('Internetprotokollfamilie', 'IPv4')
wiki.add_edge('Internetprotokollfamilie', 'IPv6')
wiki.add_edge('Internetprotokollfamilie', 'ISCSI')
wiki.add_edge('Internetprotokollfamilie', 'Interior Gateway Routing Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Internet Control Message Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Internet Group Management Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Internet Message Access Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Internet Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Internet Protocol Flow Information Export')
wiki.add_edge('Internetprotokollfamilie', 'Internet Protocol over Avian Carriers')
wiki.add_edge('Internetprotokollfamilie', 'Internet Relay Chat')
wiki.add_edge('Internetprotokollfamilie', 'JSON Meta Application Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Kerberos (Protokoll)')
wiki.add_edge('Internetprotokollfamilie', 'Layer 2 Tunneling Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Link-local Multicast Name Resolution')
wiki.add_edge('Internetprotokollfamilie', 'Liste der standardisierten Ports')
wiki.add_edge('Internetprotokollfamilie', 'Matrix (Kommunikationsprotokoll)')
wiki.add_edge('Internetprotokollfamilie', 'Multi-purpose Business Security over IP')
wiki.add_edge('Internetprotokollfamilie', 'Multipath TCP')
wiki.add_edge('Internetprotokollfamilie', 'Neighbor Discovery Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Network News Transfer Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Network Time Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Netzwerkprotokoll')
wiki.add_edge('Internetprotokollfamilie', 'OPC Unified Architecture')
wiki.add_edge('Internetprotokollfamilie', 'OSI-Modell')
wiki.add_edge('Internetprotokollfamilie', 'On-Demand Mail Relay')
wiki.add_edge('Internetprotokollfamilie', 'Open Shortest Path First')
wiki.add_edge('Internetprotokollfamilie', 'Pfadvektorprotokoll')
wiki.add_edge('Internetprotokollfamilie', 'Point-to-Point Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Post Office Protocol')
wiki.add_edge('Internetprotokollfamilie', 'QUIC')
wiki.add_edge('Internetprotokollfamilie', 'Real-Time Transport Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Remote Desktop Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Remote Framebuffer Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Resource Reservation Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Reverse Address Resolution Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Routing Information Protocol')
wiki.add_edge('Internetprotokollfamilie', 'S-HTTP')
wiki.add_edge('Internetprotokollfamilie', 'SOAP')
wiki.add_edge('Internetprotokollfamilie', 'SOCKS')
wiki.add_edge('Internetprotokollfamilie', 'SPDY')
wiki.add_edge('Internetprotokollfamilie', 'SSH File Transfer Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Secure Copy')
wiki.add_edge('Internetprotokollfamilie', 'Secure Shell')
wiki.add_edge('Internetprotokollfamilie', 'Secure Socket Tunneling Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Session Description Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Session Initiation Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Simple Mail Transfer Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Simple Network Management Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Socket (Software)')
wiki.add_edge('Internetprotokollfamilie', 'Stream Control Transmission Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Telnet')
wiki.add_edge('Internetprotokollfamilie', 'Token Bus')
wiki.add_edge('Internetprotokollfamilie', 'Token Ring')
wiki.add_edge('Internetprotokollfamilie', 'Train Real Time Data Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Transmission Control Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Transmission Control Protocol/Internet Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Transport Layer Security')
wiki.add_edge('Internetprotokollfamilie', 'Unified Diagnostic Services')
wiki.add_edge('Internetprotokollfamilie', 'User Datagram Protocol')
wiki.add_edge('Internetprotokollfamilie', 'Virtual Router Redundancy Protocol')
wiki.add_edge('Internetprotokollfamilie', 'WebSocket')
wiki.add_edge('Internetprotokollfamilie', 'Whois')
wiki.add_edge('Internetprotokollfamilie', 'Winsock')
wiki.add_edge('Internetprotokollfamilie', 'Z39.50')
wiki.add_edge('Internetwork Datagram Protocol', 'Routing Information Protocol')
wiki.add_edge('Internetwork Packet Exchange', 'Generic Routing Encapsulation')
wiki.add_edge('Internetwork Packet Exchange', 'Internet Protocol Control Protocol')
wiki.add_edge('Internetwork Packet Exchange', 'Internetwork Datagram Protocol')
wiki.add_edge('Internetwork Packet Exchange', 'Internetwork Packet eXchange Fragmentation')
wiki.add_edge('Internetwork Packet Exchange', 'JetDirect')
wiki.add_edge('Internetwork Packet Exchange', 'Link-State')
wiki.add_edge('Internetwork Packet Exchange', 'Liste der standardisierten Ports')
wiki.add_edge('Internetwork Packet Exchange', 'NetBEUI')
wiki.add_edge('Internetwork Packet Exchange', 'NetBIOS')
wiki.add_edge('Internetwork Packet Exchange', 'NetWare Core Protocol')
wiki.add_edge('Internetwork Packet Exchange', 'NetWare Link-Services Protocol')
wiki.add_edge('Internetwork Packet Exchange', 'Open Datalink Interface')
wiki.add_edge('Internetwork Packet Exchange', 'Protokoll (IP)')
wiki.add_edge('Internetwork Packet Exchange', 'Routing Information Protocol')
wiki.add_edge('Internetwork Packet Exchange', 'Sequenced Packet Exchange')
wiki.add_edge('Internetwork Packet Exchange', 'Service Advertising Protocol')
wiki.add_edge('Internetwork Packet Exchange', 'Streaming-Protokoll')
wiki.add_edge('Internetwork Packet Exchange', 'Virtual Local Area Network')
wiki.add_edge('Internetwork Packet Exchange', 'Winsock')
wiki.add_edge('Internetwork Packet eXchange Fragmentation', 'Socket (Software)')
wiki.add_edge('Interpreter (Entwurfsmuster)', 'Besucher (Entwurfsmuster)')
wiki.add_edge('Interpreter (Entwurfsmuster)', 'Iterator (Entwurfsmuster)')
wiki.add_edge('Inversion of Control', 'Asynchronous module definition')
wiki.add_edge('Inversion of Control', 'Beobachter (Entwurfsmuster)')
wiki.add_edge('Inversion of Control', 'Kontrollfluss')
wiki.add_edge('Iterator (Entwurfsmuster)', 'Aggregation (Informatik)')
wiki.add_edge('Iterator (Entwurfsmuster)', 'Besucher (Entwurfsmuster)')
wiki.add_edge('Iterator (Entwurfsmuster)', 'Strategie (Entwurfsmuster)')
wiki.add_edge('Jakarta Enterprise Beans', 'Jakarta Connectors')
wiki.add_edge('Jakarta Enterprise Beans', 'Singleton (Entwurfsmuster)')
wiki.add_edge('Jakarta Persistence API', 'Apache OpenJPA')
wiki.add_edge('Jakarta Persistence API', 'EclipseLink')
wiki.add_edge('Jakarta Persistence API', 'Hibernate (Framework)')
wiki.add_edge('Jakarta Persistence API', 'MyBatis')
wiki.add_edge('Jakarta Persistence API', 'TopLink')
wiki.add_edge('Jakarta XML Binding', 'Jakarta XML Web Services')
wiki.add_edge('Jakarta XML RPC', 'Jakarta RESTful Web Services')
wiki.add_edge('Jakarta XML RPC', 'Jakarta XML Web Services')
wiki.add_edge('Jakarta XML Web Services', 'WS-Coordination')
wiki.add_edge('Jakarta XML Web Services', 'WS-SecureConversation')
wiki.add_edge('Jakarta XML Web Services', 'WS-SecurityPolicy')
wiki.add_edge('Jakarta XML Web Services', 'WS-Trust')
wiki.add_edge('Java 3D', 'Java Foundation Classes')
wiki.add_edge('Java Business Integration', 'Apache ServiceMix')
wiki.add_edge('Java Community Process', 'Jakarta Messaging')
wiki.add_edge('Java Community Process', 'Jakarta Persistence API')
wiki.add_edge('Java Community Process', 'Jakarta RESTful Web Services')
wiki.add_edge('Java Community Process', 'Jakarta Server Faces')
wiki.add_edge('Java Community Process', 'Jakarta Transactions API')
wiki.add_edge('Java Community Process', 'Jakarta XML Binding')
wiki.add_edge('Java Community Process', 'Jakarta XML RPC')
wiki.add_edge('Java Community Process', 'Java APIs for Integrated Networks')
wiki.add_edge('Java Community Process', 'Java Business Integration')
wiki.add_edge('Java Community Process', 'Java Management Extensions')
wiki.add_edge('Java Community Process', 'Java Native Interface')
wiki.add_edge('Java Community Process', 'Java Telephony API')
wiki.add_edge('Java Community Process', 'JavaHelp System')
wiki.add_edge('Java Community Process', 'Message Oriented Middleware')
wiki.add_edge('Java Community Process', 'Real-Time Java')
wiki.add_edge('Java Community Process', 'Swing (Java)')
wiki.add_edge('Java Cryptography Extension', 'Bouncy Castle')
wiki.add_edge('Java Data Objects', 'Apache OpenJPA')
wiki.add_edge('Java Data Objects', 'DataNucleus')
wiki.add_edge('Java Data Objects', 'Hibernate (Framework)')
wiki.add_edge('Java Data Objects', 'Jakarta Persistence API')
wiki.add_edge('Java Data Objects', 'MyBatis')
wiki.add_edge('Java Database Connectivity', 'Binary Large Object')
wiki.add_edge('Java Database Connectivity', 'Hibernate (Framework)')
wiki.add_edge('Java Database Connectivity', 'Jakarta Transactions API')
wiki.add_edge('Java Database Connectivity', 'Java Community Process')
wiki.add_edge('Java Database Connectivity', 'Model 2')
wiki.add_edge('Java Database Connectivity', 'Perl-DBI')
wiki.add_edge('Java Database Connectivity', 'SoapUI')
wiki.add_edge('Java Foundation Classes', 'Abstract Window Toolkit')
wiki.add_edge('Java Foundation Classes', 'Java Accessibility API')
wiki.add_edge('Java Foundation Classes', 'Java Media Framework')
wiki.add_edge('Java Foundation Classes', 'Java Sound')
wiki.add_edge('Java Foundation Classes', 'JavaHelp System')
wiki.add_edge('Java Management Extensions', 'Apache MINA')
wiki.add_edge('Java Media Framework', 'Java Sound')
wiki.add_edge('Java Media Framework', 'Multimedia-Framework')
wiki.add_edge('Java Naming and Directory Interface', 'Jakarta Messaging')
wiki.add_edge('Java Naming and Directory Interface', 'SIMON')
wiki.add_edge('Java Native Interface', 'Java Plug-in')
wiki.add_edge('Java Transaction Service', 'Jakarta Transactions API')
wiki.add_edge('JavaFX', 'Java Media Framework')
wiki.add_edge('JavaFX', 'Mashup (Internet)')
wiki.add_edge('JavaFX', 'Model View ViewModel')
wiki.add_edge('JetDirect', 'Fast Ethernet')
wiki.add_edge('Jumbo Frames', 'Ethernet Powerlink')
wiki.add_edge('Jumbo Frames', 'Gigabit Ethernet')
wiki.add_edge('Jumbo Frames', 'ISCSI')
wiki.add_edge('Jumbo Frames', 'Jabber (Ethernet)')
wiki.add_edge('KAME-Projekt', 'Internet Security Association and Key Management Protocol')
wiki.add_edge('KParts', 'Bonobo (GNOME)')
wiki.add_edge('KakaoTalk', 'Line (Instant Messaging)')
wiki.add_edge('KakaoTalk', 'WeChat')
wiki.add_edge('Kardinalität (Datenbankmodellierung)', 'Hibernate (Framework)')
wiki.add_edge('Kardinalität (Datenbankmodellierung)', 'IDEF1X')
wiki.add_edge('Kardinalität (Datenbankmodellierung)', 'Martin-Notation')
wiki.add_edge('Kardinalität (Datenbankmodellierung)', 'Min-Max-Notation')
wiki.add_edge('Kardinalität (Datenbankmodellierung)', 'Schema (Informatik)')
wiki.add_edge('Keepalive', 'AppleTalk')
wiki.add_edge('Keepalive', 'BattleEye RCon Protocol')
wiki.add_edge('Keepalive', 'Bidirectional Forwarding Detection')
wiki.add_edge('Keepalive', 'IPsec')
wiki.add_edge('Keepalive', 'OSCAR (Protokoll)')
wiki.add_edge('Keepalive', 'Open Shortest Path First')
wiki.add_edge('Kerberos (Protokoll)', 'NTLM')
wiki.add_edge('Kernel-Modul', 'B.A.T.M.A.N.')
wiki.add_edge('Kernel-Modul', 'TIPC')
wiki.add_edge('Klasse (UML)', 'Abhängigkeitsbeziehung')
wiki.add_edge('Klasse (UML)', 'Accounting Transaction')
wiki.add_edge('Klasse (UML)', 'Artefakt (UML)')
wiki.add_edge('Klasse (UML)', 'Assoziation (UML)')
wiki.add_edge('Klasse (UML)', 'Attribut (UML)')
wiki.add_edge('Klasse (UML)', 'Classifier (UML)')
wiki.add_edge('Klasse (UML)', 'Generalisierung (UML)')
wiki.add_edge('Klasse (UML)', 'Informationseinheit (UML)')
wiki.add_edge('Klasse (UML)', 'Klassendiagramm')
wiki.add_edge('Klasse (UML)', 'Komponente (UML)')
wiki.add_edge('Klasse (UML)', 'Komponentendiagramm')
wiki.add_edge('Klasse (UML)', 'Konnektor (UML)')
wiki.add_edge('Klasse (UML)', 'Koordinator (Analysemuster)')
wiki.add_edge('Klasse (UML)', 'Merkmal (UML)')
wiki.add_edge('Klasse (UML)', 'Namensraum (UML)')
wiki.add_edge('Klasse (UML)', 'Operation (UML)')
wiki.add_edge('Klasse (UML)', 'Port (UML)')
wiki.add_edge('Klasse (UML)', 'Schnittstelle (UML)')
wiki.add_edge('Klasse (UML)', 'Signalempfänger (UML)')
wiki.add_edge('Klasse (UML)', 'Stereotyp (UML)')
wiki.add_edge('Klasse (UML)', 'Verhalten (UML)')
wiki.add_edge('Klasse (UML)', 'Zusicherung (Softwaretechnik)')
wiki.add_edge('Klassendiagramm', '4+1 Sichtenmodell')
wiki.add_edge('Klassendiagramm', 'Beobachter (Entwurfsmuster)')
wiki.add_edge('Klassendiagramm', 'Datenmodell')
wiki.add_edge('Klassendiagramm', 'Fujaba')
wiki.add_edge('Klassendiagramm', 'Gaphor')
wiki.add_edge('Klassendiagramm', 'MOF QVT')
wiki.add_edge('Klassendiagramm', 'Merkmal (UML)')
wiki.add_edge('Klassendiagramm', 'Objektdiagramm')
wiki.add_edge('Klassendiagramm', 'Operation (UML)')
wiki.add_edge('Klassendiagramm', 'Softwaredesign')
wiki.add_edge('Klassendiagramm', 'StarUML')
wiki.add_edge('Klassendiagramm', 'Stereotyp (UML)')
wiki.add_edge('Klassendiagramm', 'UMLet')
wiki.add_edge('Klassendiagramm', 'Umbrello')
wiki.add_edge('Klassendiagramm', 'Violet (UML-Werkzeug)')
wiki.add_edge('Knoten (UML)', 'Abhängigkeitsbeziehung')
wiki.add_edge('Knoten (UML)', 'Signal (UML)')
wiki.add_edge('Kommando (Entwurfsmuster)', 'Besucher (Entwurfsmuster)')
wiki.add_edge('Kommando (Entwurfsmuster)', 'Model View ViewModel')
wiki.add_edge('Kommando (Entwurfsmuster)', 'Money (Analysemuster)')
wiki.add_edge('Kommentar (UML)', 'Stereotyp (UML)')
wiki.add_edge('Kommunikationsdiagramm (UML)', '4+1 Sichtenmodell')
wiki.add_edge('Kommunikationsdiagramm (UML)', 'Interaktionsübersichtsdiagramm')
wiki.add_edge('Kommunikationsdiagramm (UML)', 'Klassendiagramm')
wiki.add_edge('Komponente (Software)', 'Bonobo (GNOME)')
wiki.add_edge('Komponente (Software)', 'Dateikonverter')
wiki.add_edge('Komponente (Software)', 'Fachkomponente')
wiki.add_edge('Komponente (Software)', 'International Data Spaces')
wiki.add_edge('Komponente (Software)', 'Jakarta Enterprise Beans')
wiki.add_edge('Komponente (Software)', 'Jakarta Server Faces')
wiki.add_edge('Komponente (Software)', 'KParts')
wiki.add_edge('Komponente (Software)', 'Komponentenbasierte Entwicklung')
wiki.add_edge('Komponente (Software)', 'Lose Kopplung')
wiki.add_edge('Komponente (Software)', 'Mapper (Softwareentwicklung)')
wiki.add_edge('Komponente (Software)', 'Modul (Software)')
wiki.add_edge('Komponente (Software)', 'Modularität')
wiki.add_edge('Komponente (Software)', 'Pipes und Filter')
wiki.add_edge('Komponente (UML)', 'Classifier (UML)')
wiki.add_edge('Komponente (UML)', 'Knoten (UML)')
wiki.add_edge('Komponente (UML)', 'Komponentendiagramm')
wiki.add_edge('Komponente (UML)', 'Kompositionsstrukturdiagramm')
wiki.add_edge('Komponente (UML)', 'Konnektor (UML)')
wiki.add_edge('Komponente (UML)', 'Merkmal (UML)')
wiki.add_edge('Komponente (UML)', 'Operation (UML)')
wiki.add_edge('Komponentenbasierte Entwicklung', 'C4 model')
wiki.add_edge('Komponentenbasierte Entwicklung', 'Modularität')
wiki.add_edge('Komponentenbasierte Entwicklung', 'Szenariobasierte Architekturbewertung')
wiki.add_edge('Komponentendiagramm', '4+1 Sichtenmodell')
wiki.add_edge('Komponentendiagramm', 'Artefakt (UML)')
wiki.add_edge('Komponentendiagramm', 'Gaphor')
wiki.add_edge('Komponentendiagramm', 'Kompositionsstrukturdiagramm')
wiki.add_edge('Kompositionsstrukturdiagramm', 'Innovator (Software)')
wiki.add_edge('Kompositionsstrukturdiagramm', 'Merkmal (UML)')
wiki.add_edge('Kompositum (Entwurfsmuster)', 'Besucher (Entwurfsmuster)')
wiki.add_edge('Kompositum (Entwurfsmuster)', 'Interpreter (Entwurfsmuster)')
wiki.add_edge('Kompositum (Entwurfsmuster)', 'Iterator (Entwurfsmuster)')
wiki.add_edge('Kompositum (Entwurfsmuster)', 'Kommando (Entwurfsmuster)')
wiki.add_edge('Kompositum (Entwurfsmuster)', 'Model View Controller')
wiki.add_edge('Kompositum (Entwurfsmuster)', 'Zuständigkeitskette')
wiki.add_edge('Konnektor (UML)', 'Java Management Extensions')
wiki.add_edge('Konnektor (UML)', 'Komponentendiagramm')
wiki.add_edge('Konnektor (UML)', 'Kompositionsstrukturdiagramm')
wiki.add_edge('Kontrollfluss', 'Datenflussdiagramm')
wiki.add_edge('Kontrollfluss', 'Kontrollfluss (UML)')
wiki.add_edge('Kontrollfluss', 'Kontrollflussgraph')
wiki.add_edge('Kontrollfluss', 'Programmablaufplan')
wiki.add_edge('Kontrollfluss', 'WS-Flow Language')
wiki.add_edge('Kontrollfluss (UML)', 'Aktivitätsparameterknoten')
wiki.add_edge('Kontrollknoten (UML)', 'Kontrollfluss (UML)')
wiki.add_edge('Kontrollknoten (UML)', 'Objektfluss (UML)')
wiki.add_edge('Kopplung (Softwareentwicklung)', 'Bewertung von Softwarearchitekturen')
wiki.add_edge('Kopplung (Softwareentwicklung)', 'GRASP')
wiki.add_edge('Kopplung (Softwareentwicklung)', 'Jakarta Enterprise Beans')
wiki.add_edge('Kopplung (Softwareentwicklung)', 'Microservices')
wiki.add_edge('Kopplung (Softwareentwicklung)', 'Remote Function Call')
wiki.add_edge('Kopplung (Softwareentwicklung)', 'Singleton (Entwurfsmuster)')
wiki.add_edge('Kryptowährung', 'Bildschirmschoner')
wiki.add_edge('Kryptowährung', 'Bitcoin Cash')
wiki.add_edge('Kryptowährung', 'Bitconnect')
wiki.add_edge('Kryptowährung', 'Cardano (Blockchainnetzwerk)')
wiki.add_edge('Kryptowährung', 'Chainlink')
wiki.add_edge('Kryptowährung', 'Chia (Kryptowährung)')
wiki.add_edge('Kryptowährung', 'Dash (Kryptowährung)')
wiki.add_edge('Kryptowährung', 'Dezentralisierte Autonome Organisation')
wiki.add_edge('Kryptowährung', 'Diem (Internetwährung)')
wiki.add_edge('Kryptowährung', 'Dogecoin')
wiki.add_edge('Kryptowährung', 'EOS (Kryptowährung)')
wiki.add_edge('Kryptowährung', 'Ethereum')
wiki.add_edge('Kryptowährung', 'GNU Taler')
wiki.add_edge('Kryptowährung', 'Iota (Kommunikationsprotokoll)')
wiki.add_edge('Kryptowährung', 'Lightning-Netzwerk')
wiki.add_edge('Kryptowährung', 'Liste von Kryptowährungen')
wiki.add_edge('Kryptowährung', 'Litecoin')
wiki.add_edge('Kryptowährung', 'MobileCoin')
wiki.add_edge('Kryptowährung', 'Monero')
wiki.add_edge('Kryptowährung', 'NEM (Kryptowährung)')
wiki.add_edge('Kryptowährung', 'NEO (Kryptowährung)')
wiki.add_edge('Kryptowährung', 'Nano (Kryptowährung)')
wiki.add_edge('Kryptowährung', 'Nxt (Kryptowährung)')
wiki.add_edge('Kryptowährung', 'OneCoin')
wiki.add_edge('Kryptowährung', 'Particl (Kryptowährung)')
wiki.add_edge('Kryptowährung', 'Petro (Kryptowährung)')
wiki.add_edge('Kryptowährung', 'Privacy Coin')
wiki.add_edge('Kryptowährung', 'Proof of Work')
wiki.add_edge('Kryptowährung', 'Ripple (Geldsystem)')
wiki.add_edge('Kryptowährung', 'SegWit')
wiki.add_edge('Kryptowährung', 'Shiba Inu (Kryptowährung)')
wiki.add_edge('Kryptowährung', 'Solana (Blockchain)')
wiki.add_edge('Kryptowährung', 'Stablecoin')
wiki.add_edge('Kryptowährung', 'Steem')
wiki.add_edge('Kryptowährung', 'Stellar (Bezahlsystem)')
wiki.add_edge('Kryptowährung', 'Telegram')
wiki.add_edge('Kryptowährung', 'Tether')
wiki.add_edge('Kryptowährung', 'Theta (Kryptowährung)')
wiki.add_edge('Kryptowährung', 'Tron (Kryptowährung)')
wiki.add_edge('Kryptowährung', 'Verge (Kryptowährung)')
wiki.add_edge('Kryptowährung', 'Waves (Kryptowährung)')
wiki.add_edge('Kryptowährung', 'Zcash')
wiki.add_edge('LLDP', 'Cisco Discovery Protocol')
wiki.add_edge('LLDP', 'Profinet')
wiki.add_edge('LLDP', 'Time-Sensitive Networking')
wiki.add_edge('Label Distribution Protocol', 'Forwarding Equivalence Class')
wiki.add_edge('Layer 2 Tunneling Protocol', 'Crypto IP Encapsulation')
wiki.add_edge('Layer 2 Tunneling Protocol', 'Frame Relay')
wiki.add_edge('Layer 2 Tunneling Protocol', 'IPSec')
wiki.add_edge('Layer 2 Tunneling Protocol', 'Layer 2 Forwarding')
wiki.add_edge('Layer 2 Tunneling Protocol', 'Liste von TCP/IP-basierten Netzwerkdiensten')
wiki.add_edge('Layer 2 Tunneling Protocol', 'Protokoll (IP)')
wiki.add_edge('Layer 2 Tunneling Protocol', 'Q.931')
wiki.add_edge('Layer 2 Tunneling Protocol', 'Remote Authentication Dial-In User Service')
wiki.add_edge('Layer 2 Tunneling Protocol', 'Secure Socket Tunneling Protocol')
wiki.add_edge('Lightning-Netzwerk', 'Bitcoin Cash')
wiki.add_edge('Lightning-Netzwerk', 'Bitcoin SV')
wiki.add_edge('Lightning-Netzwerk', 'Liste von Kryptowährungen')
wiki.add_edge('Lightning-Netzwerk', 'Noise Protocol Framework')
wiki.add_edge('Lightning-Netzwerk', 'Particl (Kryptowährung)')
wiki.add_edge('Lightweight Directory Access Protocol', 'ADOdb')
wiki.add_edge('Lightweight Directory Access Protocol', 'Datenmodell')
wiki.add_edge('Lightweight Directory Access Protocol', 'Filter (Software)')
wiki.add_edge('Lightweight Directory Access Protocol', 'Java Naming and Directory Interface')
wiki.add_edge('Lightweight Directory Access Protocol', 'Kerberos (Protokoll)')
wiki.add_edge('Lightweight Directory Access Protocol', 'Liste der standardisierten Ports')
wiki.add_edge('Lightweight Directory Access Protocol', 'Lose Kopplung')
wiki.add_edge('Lightweight Directory Access Protocol', 'Netzwerkprotokoll')
wiki.add_edge('Lightweight Directory Access Protocol', 'Normalisierung (Datenbank)')
wiki.add_edge('Lightweight Directory Access Protocol', 'OSI-Modell')
wiki.add_edge('Lightweight Directory Access Protocol', 'Pluggable Authentication Modules')
wiki.add_edge('Lightweight Directory Access Protocol', 'Portable Operating System Interface')
wiki.add_edge('Lightweight Directory Access Protocol', 'Post Office Protocol')
wiki.add_edge('Lightweight Directory Access Protocol', 'STARTTLS')
wiki.add_edge('Lightweight Directory Access Protocol', 'Schema (Informatik)')
wiki.add_edge('Lightweight Directory Access Protocol', 'Simple Authentication and Security Layer')
wiki.add_edge('Lightweight Directory Access Protocol', 'Whois')
wiki.add_edge('Lightweight Directory Access Protocol', 'X.500')
wiki.add_edge('Link Access Procedure Balanced', 'Data Link Control')
wiki.add_edge('Link Access Procedure for the D-Channel', 'Link Access Procedure Balanced')
wiki.add_edge('Link Aggregation', 'Etherchannel')
wiki.add_edge('Link Aggregation', 'Link Aggregation Control Protocol')
wiki.add_edge('Link Aggregation', 'PAgP')
wiki.add_edge('Link Aggregation', 'Split Multi-Link Trunking')
wiki.add_edge('Link Aggregation Control Protocol', 'Split Multi-Link Trunking')
wiki.add_edge('Link-State', 'Balanced hybrid')
wiki.add_edge('Link-State', 'Diffusing Update Algorithm')
wiki.add_edge('Link-State', 'Distanzvektoralgorithmus')
wiki.add_edge('Link-State', 'Enhanced Interior Gateway Routing Protocol')
wiki.add_edge('Link-State', 'IEEE 802.1aq')
wiki.add_edge('Link-State', 'IS-IS')
wiki.add_edge('Link-State', 'MOSPF')
wiki.add_edge('Link-State', 'NetWare Link-Services Protocol')
wiki.add_edge('Link-State', 'Open Shortest Path First')
wiki.add_edge('Link-State', 'Optimized Link State Routing')
wiki.add_edge('Linux Audio Developer’s Simple Plugin API', 'LV2')
wiki.add_edge('Liste der standardisierten Ports', 'Background File Transfer Program')
wiki.add_edge('Liste der standardisierten Ports', 'Blocks Extensible Exchange Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'Border Gateway Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'DNP3')
wiki.add_edge('Liste der standardisierten Ports', 'DNS over TLS')
wiki.add_edge('Liste der standardisierten Ports', 'Daytime')
wiki.add_edge('Liste der standardisierten Ports', 'Diameter (Protokoll)')
wiki.add_edge('Liste der standardisierten Ports', 'Digital Audio Access Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'Discard')
wiki.add_edge('Liste der standardisierten Ports', 'Echo (Netzwerkdienst)')
wiki.add_edge('Liste der standardisierten Ports', 'EtherNet/IP')
wiki.add_edge('Liste der standardisierten Ports', 'Extensible Provisioning Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'Fibre Channel over IP')
wiki.add_edge('Liste der standardisierten Ports', 'Finger (Internetprotokoll)')
wiki.add_edge('Liste der standardisierten Ports', 'GPRS Tunneling Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'Gadu-Gadu')
wiki.add_edge('Liste der standardisierten Ports', 'Gopher')
wiki.add_edge('Liste der standardisierten Ports', 'H.323')
wiki.add_edge('Liste der standardisierten Ports', 'Hot Standby Router Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'IGD-Protokoll')
wiki.add_edge('Liste der standardisierten Ports', 'Independent Computing Architecture')
wiki.add_edge('Liste der standardisierten Ports', 'Internet Group Management Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'Internet Printing Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'Internet Security Association and Key Management Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'JSON-RPC')
wiki.add_edge('Liste der standardisierten Ports', 'Label Distribution Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'Layer 2 Tunneling Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'Liste von TCP/IP-basierten Netzwerkdiensten')
wiki.add_edge('Liste der standardisierten Ports', 'Local Mail Transfer Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'MQTT')
wiki.add_edge('Liste der standardisierten Ports', 'Media Gateway Control Protocol (Protokoll)')
wiki.add_edge('Liste der standardisierten Ports', 'Megaco')
wiki.add_edge('Liste der standardisierten Ports', 'Mumble')
wiki.add_edge('Liste der standardisierten Ports', 'NAT Port Mapping Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'NetBIOS')
wiki.add_edge('Liste der standardisierten Ports', 'NetWare Core Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'Network File System')
wiki.add_edge('Liste der standardisierten Ports', 'Network Information Center')
wiki.add_edge('Liste der standardisierten Ports', 'Network News Transfer Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'Network Time Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'OPC Unified Architecture')
wiki.add_edge('Liste der standardisierten Ports', 'Optimized Link State Routing')
wiki.add_edge('Liste der standardisierten Ports', 'Peer Name Resolution Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'Pichat')
wiki.add_edge('Liste der standardisierten Ports', 'Real Time Messaging Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'Real-Time Streaming Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'Remote Desktop Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'Remote Method Invocation')
wiki.add_edge('Liste der standardisierten Ports', 'Routing Information Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'Rsync')
wiki.add_edge('Liste der standardisierten Ports', 'SIMON')
wiki.add_edge('Liste der standardisierten Ports', 'SOCKS')
wiki.add_edge('Liste der standardisierten Ports', 'SafetyNET p')
wiki.add_edge('Liste der standardisierten Ports', 'Secure Copy')
wiki.add_edge('Liste der standardisierten Ports', 'Server Message Block')
wiki.add_edge('Liste der standardisierten Ports', 'Service Location Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'Session Announcement Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'Session Initiation Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'Simple File Transfer Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'Simple Network Management Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'Simple Service Discovery Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'Skinny Client Control Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'Skype')
wiki.add_edge('Liste der standardisierten Ports', 'Source-specific Multicast')
wiki.add_edge('Liste der standardisierten Ports', 'Syslog')
wiki.add_edge('Liste der standardisierten Ports', 'TCP Port Service Multiplexer')
wiki.add_edge('Liste der standardisierten Ports', 'TR-069')
wiki.add_edge('Liste der standardisierten Ports', 'TeamSpeak')
wiki.add_edge('Liste der standardisierten Ports', 'Teredo')
wiki.add_edge('Liste der standardisierten Ports', 'Time Stamp Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'Tunnel (Rechnernetz)')
wiki.add_edge('Liste der standardisierten Ports', 'Universal Measurement and Calibration Protocol')
wiki.add_edge('Liste der standardisierten Ports', 'Universal Plug and Play')
wiki.add_edge('Liste der standardisierten Ports', 'VTun')
wiki.add_edge('Liste der standardisierten Ports', 'Virtual Extensible LAN')
wiki.add_edge('Liste der standardisierten Ports', 'Virtual Network Computing')
wiki.add_edge('Liste der standardisierten Ports', 'Windows Live Messenger')
wiki.add_edge('Liste der standardisierten Ports', 'Windows Server Update Services')
wiki.add_edge('Liste der standardisierten Ports', 'XDMCP')
wiki.add_edge('Liste der standardisierten Ports', 'Xfire')
wiki.add_edge('Liste der standardisierten Ports', 'Yahoo Messenger')
wiki.add_edge('Liste der standardisierten Ports', 'Z39.50')
wiki.add_edge('Liste von Datenmodellierungswerkzeugen', 'ArgoUML')
wiki.add_edge('Liste von Datenmodellierungswerkzeugen', 'Datenbankdesign')
wiki.add_edge('Liste von Datenmodellierungswerkzeugen', 'Dia (Software)')
wiki.add_edge('Liste von Datenmodellierungswerkzeugen', 'Enterprise Architect')
wiki.add_edge('Liste von Datenmodellierungswerkzeugen', 'Entity Framework')
wiki.add_edge('Liste von Datenmodellierungswerkzeugen', 'Fujaba')
wiki.add_edge('Liste von Datenmodellierungswerkzeugen', 'Innovator (Software)')
wiki.add_edge('Liste von Datenmodellierungswerkzeugen', 'Microsoft Visio')
wiki.add_edge('Liste von Datenmodellierungswerkzeugen', 'Poseidon for UML')
wiki.add_edge('Liste von Datenmodellierungswerkzeugen', 'PowerDesigner')
wiki.add_edge('Liste von Datenmodellierungswerkzeugen', 'Umbrello')
wiki.add_edge('Liste von Datenmodellierungswerkzeugen', 'Visual Paradigm')
wiki.add_edge('Liste von IPv6-Tunnelbrokern', '4in6')
wiki.add_edge('Liste von IPv6-Tunnelbrokern', 'DirectAccess')
wiki.add_edge('Liste von Kryptowährungen', 'Bitconnect')
wiki.add_edge('Liste von Kryptowährungen', 'Cardano (Blockchainnetzwerk)')
wiki.add_edge('Liste von Kryptowährungen', 'Chainlink')
wiki.add_edge('Liste von Kryptowährungen', 'Chia (Kryptowährung)')
wiki.add_edge('Liste von Kryptowährungen', 'Dash (Kryptowährung)')
wiki.add_edge('Liste von Kryptowährungen', 'EOS (Kryptowährung)')
wiki.add_edge('Liste von Kryptowährungen', 'Ethereum')
wiki.add_edge('Liste von Kryptowährungen', 'Ethereum Classic')
wiki.add_edge('Liste von Kryptowährungen', 'Iota (Kommunikationsprotokoll)')
wiki.add_edge('Liste von Kryptowährungen', 'MobileCoin')
wiki.add_edge('Liste von Kryptowährungen', 'Monero')
wiki.add_edge('Liste von Kryptowährungen', 'NEM (Kryptowährung)')
wiki.add_edge('Liste von Kryptowährungen', 'NEO (Kryptowährung)')
wiki.add_edge('Liste von Kryptowährungen', 'Nano (Kryptowährung)')
wiki.add_edge('Liste von Kryptowährungen', 'Nxt (Kryptowährung)')
wiki.add_edge('Liste von Kryptowährungen', 'OneCoin')
wiki.add_edge('Liste von Kryptowährungen', 'Particl (Kryptowährung)')
wiki.add_edge('Liste von Kryptowährungen', 'Petro (Kryptowährung)')
wiki.add_edge('Liste von Kryptowährungen', 'Ripple (Geldsystem)')
wiki.add_edge('Liste von Kryptowährungen', 'Shiba Inu (Kryptowährung)')
wiki.add_edge('Liste von Kryptowährungen', 'Solana (Blockchain)')
wiki.add_edge('Liste von Kryptowährungen', 'Stablecoin')
wiki.add_edge('Liste von Kryptowährungen', 'Steem')
wiki.add_edge('Liste von Kryptowährungen', 'Stellar (Bezahlsystem)')
wiki.add_edge('Liste von Kryptowährungen', 'Tether')
wiki.add_edge('Liste von Kryptowährungen', 'Theta (Kryptowährung)')
wiki.add_edge('Liste von Kryptowährungen', 'Tron (Kryptowährung)')
wiki.add_edge('Liste von Kryptowährungen', 'Verge (Kryptowährung)')
wiki.add_edge('Liste von Kryptowährungen', 'Waves (Kryptowährung)')
wiki.add_edge('Liste von Singleton-Implementierungen', 'Doppelt überprüfte Sperrung')
wiki.add_edge('Liste von TCP/IP-basierten Netzwerkdiensten', 'Remote login')
wiki.add_edge('Liste von TCP/IP-basierten Netzwerkdiensten', 'SMTPS')
wiki.add_edge('Litecoin', 'Bitstamp')
wiki.add_edge('Litecoin', 'Dash (Kryptowährung)')
wiki.add_edge('Litecoin', 'Dogecoin')
wiki.add_edge('Litecoin', 'Liste von Kryptowährungen')
wiki.add_edge('Litecoin', 'Monero')
wiki.add_edge('Litecoin', 'Tether')
wiki.add_edge('Litecoin', 'Waves (Kryptowährung)')
wiki.add_edge('Local Internet Registry', 'Provider Aggregatable Address Space')
wiki.add_edge('Local-as-View', 'Sicht (Datenbank)')
wiki.add_edge('LocalTalk', 'JetDirect')
wiki.add_edge('LocalTalk', 'Kinetics Internet Protocol')
wiki.add_edge('LocalTalk', 'LocalTalk Link Access Protocol')
wiki.add_edge('LocalTalk', 'TokenTalk Link Access Protocol')
wiki.add_edge('LocalTalk Link Access Protocol', 'TokenTalk Link Access Protocol')
wiki.add_edge('Logical Link Control', 'Cisco Discovery Protocol')
wiki.add_edge('Logical Link Control', 'High-Level Data Link Control')
wiki.add_edge('Logical Link Control', 'IEEE 802')
wiki.add_edge('Logical Link Control', 'Media Access Control')
wiki.add_edge('Logical Link Control', 'NetBEUI')
wiki.add_edge('Lose Kopplung', 'Ereignisgesteuerte Architektur')
wiki.add_edge('Lose Kopplung', 'Kopplung (Softwareentwicklung)')
wiki.add_edge('Lose Kopplung', 'Message Oriented Middleware')
wiki.add_edge('Lose Kopplung', 'Model View ViewModel')
wiki.add_edge('Lose Kopplung', 'Singleton (Entwurfsmuster)')
wiki.add_edge('Lose Kopplung', 'Softwaredesign')
wiki.add_edge('MIDI', 'Audio Random Access')
wiki.add_edge('MIDI', 'Audio Unit')
wiki.add_edge('MIDI', 'Core Audio')
wiki.add_edge('MIDI', 'DirectX')
wiki.add_edge('MIDI', 'Java Sound')
wiki.add_edge('MOF QVT', 'Entity-Relationship-Modell')
wiki.add_edge('MOF QVT', 'Modellgetriebene Architektur')
wiki.add_edge('MOF QVT', 'Tripel-Graph-Grammatik')
wiki.add_edge('MQTT', 'Production Performance Management Protocol')
wiki.add_edge('MQTT', 'TR-369')
wiki.add_edge('Macintosh-Baukasten', 'QuickDraw')
wiki.add_edge('Manufacturing Messaging Specification', 'IEC 60870')
wiki.add_edge('Manufacturing Messaging Specification', 'TASE.2')
wiki.add_edge('Measurement (Analysemuster)', 'Observation (Analysemuster)')
wiki.add_edge('Measurement (Analysemuster)', 'Range (Analysemuster)')
wiki.add_edge('Media Access Control', '6LoWPAN')
wiki.add_edge('Media Access Control', 'ALOHA')
wiki.add_edge('Media Access Control', 'Attachment Unit Interface')
wiki.add_edge('Media Access Control', 'Berkeley Media Access Control')
wiki.add_edge('Media Access Control', 'Data Link Control')
wiki.add_edge('Media Access Control', 'Fiber Distributed Data Interface')
wiki.add_edge('Media Access Control', 'IEEE 802')
wiki.add_edge('Media Access Control', 'Network Driver Interface Specification')
wiki.add_edge('Media Access Control', 'Radio Link Control')
wiki.add_edge('Media Access Control', 'Sensor Media Access Control')
wiki.add_edge('Media Access Control', 'Split Multi-Link Trunking')
wiki.add_edge('Media Access Control', 'Time Token Rotation Protocol')
wiki.add_edge('Media Access Control', 'Time-Sensitive Networking')
wiki.add_edge('Media Access Control', 'Token Ring')
wiki.add_edge('Media Access Control', 'Virtual Extensible LAN')
wiki.add_edge('Media Access Control', 'Zebra Media Access Control')
wiki.add_edge('Media Gateway Control Protocol (Protokoll)', 'Skinny Client Control Protocol')
wiki.add_edge('Media Redundancy Protocol', 'Parallel Redundancy Protocol')
wiki.add_edge('Megaco', 'Media Gateway Control Protocol (Protokoll)')
wiki.add_edge('Memento (Entwurfsmuster)', 'Undo')
wiki.add_edge('Memento (Entwurfsmuster)', 'Zustand (Entwurfsmuster)')
wiki.add_edge('Merkmal (UML)', 'Signalempfänger (UML)')
wiki.add_edge('Message (Entwurfsmuster)', 'Document Message')
wiki.add_edge('Message Oriented Middleware', 'Advanced Message Queuing Protocol')
wiki.add_edge('Message Oriented Middleware', 'Jakarta Messaging')
wiki.add_edge('Message Oriented Middleware', 'Java Business Integration')
wiki.add_edge('Message Oriented Middleware', 'WebHooks')
wiki.add_edge('Messaging Application Programming Interface', 'GroupDAV')
wiki.add_edge('Messaging Application Programming Interface', 'Vendor Independent Messaging')
wiki.add_edge('Metal (API)', 'MoltenVK')
wiki.add_edge('Microservices', 'Cloud-native Computing')
wiki.add_edge('Microservices', 'Distributed Application Runtime')
wiki.add_edge('Microservices', 'Humane Registry')
wiki.add_edge('Microservices', 'Operations-Datenbank')
wiki.add_edge('Microservices', 'Saga (Entwurfsmuster)')
wiki.add_edge('Microservices', 'Self-contained Systems')
wiki.add_edge('Microservices', 'Swagger (Software)')
wiki.add_edge('Microsoft Visio', 'Diagrams.net')
wiki.add_edge('Microsoft Visio', 'YEd')
wiki.add_edge('Mobile IP', 'Generic Routing Encapsulation')
wiki.add_edge('Mobile IP', 'Socket (Software)')
wiki.add_edge('Mobile IP', 'Transportschicht')
wiki.add_edge('Mobile IP', 'Tunnel (Rechnernetz)')
wiki.add_edge('MobileCoin', 'Particl (Kryptowährung)')
wiki.add_edge('MobileCoin', 'Stellar (Bezahlsystem)')
wiki.add_edge('Model View Controller', 'Beobachter (Entwurfsmuster)')
wiki.add_edge('Model View Controller', 'Data Context Interaction')
wiki.add_edge('Model View Controller', 'Front Controller')
wiki.add_edge('Model View Controller', 'GRASP')
wiki.add_edge('Model View Controller', 'Geschäftslogik')
wiki.add_edge('Model View Controller', 'Jakarta Server Faces')
wiki.add_edge('Model View Controller', 'Model 2')
wiki.add_edge('Model View Controller', 'Model View Presenter')
wiki.add_edge('Model View Controller', 'Model View ViewModel')
wiki.add_edge('Model View Controller', 'Presentation-Abstraction-Control')
wiki.add_edge('Model View Controller', 'Remote Presentation Model')
wiki.add_edge('Model View Controller', 'Smart UI')
wiki.add_edge('Model View Controller', 'Swing (Java)')
wiki.add_edge('Model View Controller', 'VIPER (Entwurfsmuster)')
wiki.add_edge('Model View Controller', 'WebSocket')
wiki.add_edge('Model View ViewModel', 'Model View Presenter')
wiki.add_edge('Model View ViewModel', 'Remote Presentation Model')
wiki.add_edge('Model View ViewModel', 'VIPER (Entwurfsmuster)')
wiki.add_edge('Modeling and Analysis of Real Time and Embedded systems', 'Enterprise Architect')
wiki.add_edge('Modeling and Analysis of Real Time and Embedded systems', 'Modelio')
wiki.add_edge('Modeling and Analysis of Real Time and Embedded systems', 'Papyrus (UML)')
wiki.add_edge('Modeling and Analysis of Real Time and Embedded systems', 'Rational Rhapsody')
wiki.add_edge('Modelio', 'TOGAF')
wiki.add_edge('Modell (UML)', 'Profil (UML)')
wiki.add_edge('Modellgetriebene Architektur', 'Innovator (Software)')
wiki.add_edge('Modul (Software)', 'Asynchronous module definition')
wiki.add_edge('Modul (Software)', 'Cross-Cutting Concern')
wiki.add_edge('Modul (Software)', 'DirectX Media Objects')
wiki.add_edge('Modul (Software)', 'Kernel-Modul')
wiki.add_edge('Modul (Software)', 'Kopplung (Softwareentwicklung)')
wiki.add_edge('Modul (Software)', 'Modularität')
wiki.add_edge('Modul (Software)', 'Programmpaket')
wiki.add_edge('Modul (Software)', 'Quelltextklon')
wiki.add_edge('Modul (Software)', 'Top-Down- und Bottom-Up-Design')
wiki.add_edge('Modularität', 'Gesetz von Conway')
wiki.add_edge('Modularität', 'Kernel-Modul')
wiki.add_edge('Modularität', 'Microservices')
wiki.add_edge('Monero', 'Dash (Kryptowährung)')
wiki.add_edge('Monero', 'MobileCoin')
wiki.add_edge('Monero', 'Particl (Kryptowährung)')
wiki.add_edge('Monero', 'Privacy Coin')
wiki.add_edge('Multicast Backbone', '6Bone')
wiki.add_edge('Multicast Backbone', 'Distance Vector Multicast Routing Protocol')
wiki.add_edge('Multicast Backbone', 'IP-Multicast')
wiki.add_edge('Multicast Backbone', 'Internet Group Management Protocol')
wiki.add_edge('Multicast Backbone', 'MOSPF')
wiki.add_edge('Multicast Backbone', 'Tunnel (Rechnernetz)')
wiki.add_edge('Multipath TCP', 'Multi-Path Routing')
wiki.add_edge('Multiplizität (UML)', 'Assoziationsklasse')
wiki.add_edge('Multiplizität (UML)', 'Attribut (UML)')
wiki.add_edge('Multiplizität (UML)', 'Klassendiagramm')
wiki.add_edge('Multiplizität (UML)', 'Kompositionsstrukturdiagramm')
wiki.add_edge('Multiplizität (UML)', 'Operation (UML)')
wiki.add_edge('Multiplizität (UML)', 'Pin (UML)')
wiki.add_edge('Multiton', 'Initialization-on-demand holder idiom')
wiki.add_edge('Multiton', 'Singleton (Entwurfsmuster)')
wiki.add_edge('Multiton', 'Verzögerte Initialisierung')
wiki.add_edge('Mumble', 'Mbed TLS')
wiki.add_edge('Mustersprache', 'Interaktionsdesignmuster')
wiki.add_edge('MySQLi', 'ADOdb Lite')
wiki.add_edge('NAT Port Mapping Protocol', 'RetroShare')
wiki.add_edge('NAT64', '464XLAT')
wiki.add_edge('NAT64', 'DirectAccess')
wiki.add_edge('NEM (Kryptowährung)', 'Petro (Kryptowährung)')
wiki.add_edge('NTLM', 'HTTP-Authentifizierung')
wiki.add_edge('Nachrichtenübermittlungsmuster', 'Message (Entwurfsmuster)')
wiki.add_edge('Namensraum (UML)', 'Generalisierung (UML)')
wiki.add_edge('Namensraum (UML)', 'Paket (UML)')
wiki.add_edge('Nassi-Shneiderman-Diagramm', 'Innovator (Software)')
wiki.add_edge('Nassi-Shneiderman-Diagramm', 'Jackson-Diagramm')
wiki.add_edge('Nassi-Shneiderman-Diagramm', 'Top-Down- und Bottom-Up-Design')
wiki.add_edge('Nebenläufigkeit (Entwurfsmuster)', 'Doppelt überprüfte Sperrung')
wiki.add_edge('Neighbor Discovery Protocol', 'ICMPv6')
wiki.add_edge('NetBEUI', 'NetBIOS')
wiki.add_edge('NetBEUI', 'Peer Name Resolution Protocol')
wiki.add_edge('NetBEUI', 'Server Message Block')
wiki.add_edge('NetBIOS', 'Data Link Switching')
wiki.add_edge('NetBIOS', 'Link-local Multicast Name Resolution')
wiki.add_edge('NetBIOS', 'NetBIOS over TCP/IP')
wiki.add_edge('NetBIOS', 'Netzwerk-DDE')
wiki.add_edge('NetWare Core Protocol', 'Internetwork Packet eXchange Fragmentation')
wiki.add_edge('NetWare Link-Services Protocol', 'Open Shortest Path First')
wiki.add_edge('Netfilter', 'Common Address Redundancy Protocol')
wiki.add_edge('Netflow', 'Sequenznummer')
wiki.add_edge('Network Driver Interface Specification', 'Open Datalink Interface')
wiki.add_edge('Network File System', 'AmiTCP')
wiki.add_edge('Network File System', 'Boot Service Discovery Protocol')
wiki.add_edge('Network File System', 'External Data Representation')
wiki.add_edge('Network File System', 'Gnome Virtual File System')
wiki.add_edge('Network Information Center', 'Extensible Provisioning Protocol')
wiki.add_edge('Network News Transfer Protocol', 'Jakarta Mail')
wiki.add_edge('Network News Transfer Protocol', 'Liste von TCP/IP-basierten Netzwerkdiensten')
wiki.add_edge('Network Time Protocol', 'Daytime')
wiki.add_edge('Network Time Protocol', 'Kerberos (Protokoll)')
wiki.add_edge('Network Time Protocol', 'Liste von TCP/IP-basierten Netzwerkdiensten')
wiki.add_edge('Network Time Protocol', 'Microservices')
wiki.add_edge('Network Time Protocol', 'Session Announcement Protocol')
wiki.add_edge('Netzklasse', 'Classless Inter-Domain Routing')
wiki.add_edge('Netzklasse', 'Netzmaske')
wiki.add_edge('Netzklasse', 'Point-to-Point Protocol')
wiki.add_edge('Netzklasse', 'Private IP-Adresse')
wiki.add_edge('Netzklasse', 'RIPE Network Coordination Centre')
wiki.add_edge('Netzklasse', 'Regional Internet Registry')
wiki.add_edge('Netzklasse', 'Routing Information Protocol')
wiki.add_edge('Netzklasse', 'Variable Length Subnet Mask')
wiki.add_edge('Netzmaske', 'Classless Inter-Domain Routing')
wiki.add_edge('Netzmaske', 'Peg DHCP')
wiki.add_edge('Netzmaske', 'Preboot Execution Environment')
wiki.add_edge('Netzmaske', 'Variable Length Subnet Mask')
wiki.add_edge('Netzwerkadressübersetzung', 'IP-Header')
wiki.add_edge('Netzwerkadressübersetzung', 'IPsec')
wiki.add_edge('Netzwerkadressübersetzung', 'NAT64')
wiki.add_edge('Netzwerkadressübersetzung', 'Netfilter')
wiki.add_edge('Netzwerkadressübersetzung', 'Port Address Translation')
wiki.add_edge('Netzwerkadressübersetzung', 'Portweiterleitung')
wiki.add_edge('Netzwerkadressübersetzung', 'Private IP-Adresse')
wiki.add_edge('Netzwerkadressübersetzung', 'SOCKS')
wiki.add_edge('Netzwerkadressübersetzung', 'Telnet')
wiki.add_edge('Netzwerkadressübersetzung', 'Universal Plug and Play')
wiki.add_edge('Netzwerkdatenbankmodell', 'I2P')
wiki.add_edge('Netzwerkprotokoll', '3964R')
wiki.add_edge('Netzwerkprotokoll', 'ALOHA')
wiki.add_edge('Netzwerkprotokoll', 'ARINC 825')
wiki.add_edge('Netzwerkprotokoll', 'ARQ-Protokoll')
wiki.add_edge('Netzwerkprotokoll', 'ASCII-Protokoll')
wiki.add_edge('Netzwerkprotokoll', 'Address Resolution Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Advanced Message Queuing Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Advanced Program to Program Communications')
wiki.add_edge('Netzwerkprotokoll', 'Agent Extensibility Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Aggregation (Informatik)')
wiki.add_edge('Netzwerkprotokoll', 'Apache Thrift')
wiki.add_edge('Netzwerkprotokoll', 'Apple Filing Protocol')
wiki.add_edge('Netzwerkprotokoll', 'AppleTalk')
wiki.add_edge('Netzwerkprotokoll', 'BACnet')
wiki.add_edge('Netzwerkprotokoll', 'Background File Transfer Program')
wiki.add_edge('Netzwerkprotokoll', 'Berkeley Media Access Control')
wiki.add_edge('Netzwerkprotokoll', 'Berkeley Printing System')
wiki.add_edge('Netzwerkprotokoll', 'Bidirectional Forwarding Detection')
wiki.add_edge('Netzwerkprotokoll', 'Binary Synchronous Communication')
wiki.add_edge('Netzwerkprotokoll', 'Binärschnittstelle')
wiki.add_edge('Netzwerkprotokoll', 'BitTorrent')
wiki.add_edge('Netzwerkprotokoll', 'Blocks Extensible Exchange Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Border Gateway Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Burlap')
wiki.add_edge('Netzwerkprotokoll', 'CEBus')
wiki.add_edge('Netzwerkprotokoll', 'CalDAV')
wiki.add_edge('Netzwerkprotokoll', 'Cellular-IP')
wiki.add_edge('Netzwerkprotokoll', 'Certificate Management Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Cisco Discovery Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Cisco Group Management Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Common Address Redundancy Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Common Object Request Broker Architecture')
wiki.add_edge('Netzwerkprotokoll', 'Connectionless Network Protocol')
wiki.add_edge('Netzwerkprotokoll', 'CorNet')
wiki.add_edge('Netzwerkprotokoll', 'Crypto IP Encapsulation')
wiki.add_edge('Netzwerkprotokoll', 'DICT (Protokoll)')
wiki.add_edge('Netzwerkprotokoll', 'DNP3')
wiki.add_edge('Netzwerkprotokoll', 'Dante (Netzwerkprotokoll)')
wiki.add_edge('Netzwerkprotokoll', 'Data Link Switching')
wiki.add_edge('Netzwerkprotokoll', 'Datagram Congestion Control Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Daytime')
wiki.add_edge('Netzwerkprotokoll', 'Deep Packet Inspection')
wiki.add_edge('Netzwerkprotokoll', 'Digital Audio Access Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Digital Data Communications Message Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Discard')
wiki.add_edge('Netzwerkprotokoll', 'Distributed Queue Dual Bus')
wiki.add_edge('Netzwerkprotokoll', 'Dynamic Information and Action Gateway')
wiki.add_edge('Netzwerkprotokoll', 'Dynamic Trunking Protocol')
wiki.add_edge('Netzwerkprotokoll', 'ETRN')
wiki.add_edge('Netzwerkprotokoll', 'Echo (Netzwerkdienst)')
wiki.add_edge('Netzwerkprotokoll', 'Encryption Control Protocol')
wiki.add_edge('Netzwerkprotokoll', 'EtherCAT')
wiki.add_edge('Netzwerkprotokoll', 'EtherTalk Link Access Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Ethernet')
wiki.add_edge('Netzwerkprotokoll', 'Explicit Congestion Notification')
wiki.add_edge('Netzwerkprotokoll', 'Extensible Provisioning Protocol')
wiki.add_edge('Netzwerkprotokoll', 'FISH (Protokoll)')
wiki.add_edge('Netzwerkprotokoll', 'Fast and Secure Protocol')
wiki.add_edge('Netzwerkprotokoll', 'FastCGI')
wiki.add_edge('Netzwerkprotokoll', 'FastTrack')
wiki.add_edge('Netzwerkprotokoll', 'File Service Protocol')
wiki.add_edge('Netzwerkprotokoll', 'File Transfer, Access and Management')
wiki.add_edge('Netzwerkprotokoll', 'GNU Taler')
wiki.add_edge('Netzwerkprotokoll', 'Geminispace')
wiki.add_edge('Netzwerkprotokoll', 'Generic Object Oriented Substation Events')
wiki.add_edge('Netzwerkprotokoll', 'Gnome Virtual File System')
wiki.add_edge('Netzwerkprotokoll', 'Gopher')
wiki.add_edge('Netzwerkprotokoll', 'GroupDAV')
wiki.add_edge('Netzwerkprotokoll', 'Hessian (Webprotokoll)')
wiki.add_edge('Netzwerkprotokoll', 'High-Level Data Link Control')
wiki.add_edge('Netzwerkprotokoll', 'Highway Addressable Remote Transducer')
wiki.add_edge('Netzwerkprotokoll', 'Host Identity Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Hyper Text Coffee Pot Control Protocol')
wiki.add_edge('Netzwerkprotokoll', 'HyperSCSI')
wiki.add_edge('Netzwerkprotokoll', 'ILMI')
wiki.add_edge('Netzwerkprotokoll', 'IP-Fragmentierung')
wiki.add_edge('Netzwerkprotokoll', 'IP-Paket')
wiki.add_edge('Netzwerkprotokoll', 'Ident')
wiki.add_edge('Netzwerkprotokoll', 'Industrial Ethernet')
wiki.add_edge('Netzwerkprotokoll', 'Inter-Switch Link Protocol')
wiki.add_edge('Netzwerkprotokoll', 'InterAsterisk eXchange')
wiki.add_edge('Netzwerkprotokoll', 'Interior Gateway Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Internet Group Management Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Internet Message Access Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Internet Protocol Control Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Internet Protocol over Avian Carriers')
wiki.add_edge('Netzwerkprotokoll', 'Internet Relay Chat')
wiki.add_edge('Netzwerkprotokoll', 'Internet Security Association and Key Management Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Internetwork Datagram Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Internetwork Packet Exchange')
wiki.add_edge('Netzwerkprotokoll', 'Internetwork Packet eXchange Fragmentation')
wiki.add_edge('Netzwerkprotokoll', 'JSON Meta Application Protocol')
wiki.add_edge('Netzwerkprotokoll', 'JetDirect')
wiki.add_edge('Netzwerkprotokoll', 'Job Transfer and Manipulation')
wiki.add_edge('Netzwerkprotokoll', 'Jumbo Frames')
wiki.add_edge('Netzwerkprotokoll', 'Keepalive')
wiki.add_edge('Netzwerkprotokoll', 'Kerberos (Protokoll)')
wiki.add_edge('Netzwerkprotokoll', 'Kinetics Internet Protocol')
wiki.add_edge('Netzwerkprotokoll', 'LLDP')
wiki.add_edge('Netzwerkprotokoll', 'Layer 2 Forwarding')
wiki.add_edge('Netzwerkprotokoll', 'Layer 2 Tunneling Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Link Layer Topology Discovery')
wiki.add_edge('Netzwerkprotokoll', 'Link-State')
wiki.add_edge('Netzwerkprotokoll', 'Link-local Multicast Name Resolution')
wiki.add_edge('Netzwerkprotokoll', 'Liste der standardisierten Ports')
wiki.add_edge('Netzwerkprotokoll', 'Liste von TCP/IP-basierten Netzwerkdiensten')
wiki.add_edge('Netzwerkprotokoll', 'Local Mail Transfer Protocol')
wiki.add_edge('Netzwerkprotokoll', 'LocalTalk Link Access Protocol')
wiki.add_edge('Netzwerkprotokoll', 'MMS-Protokoll')
wiki.add_edge('Netzwerkprotokoll', 'MQTT')
wiki.add_edge('Netzwerkprotokoll', 'Media Access Control')
wiki.add_edge('Netzwerkprotokoll', 'Media Gateway Control Protocol (Protokoll)')
wiki.add_edge('Netzwerkprotokoll', 'Megaco')
wiki.add_edge('Netzwerkprotokoll', 'Message Oriented Middleware')
wiki.add_edge('Netzwerkprotokoll', 'Microsoft Point-To-Point Encryption Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Multi-purpose Business Security over IP')
wiki.add_edge('Netzwerkprotokoll', 'Nagle-Algorithmus')
wiki.add_edge('Netzwerkprotokoll', 'NetBEUI')
wiki.add_edge('Netzwerkprotokoll', 'NetBIOS over TCP/IP')
wiki.add_edge('Netzwerkprotokoll', 'NetWare Core Protocol')
wiki.add_edge('Netzwerkprotokoll', 'NetWare Link-Services Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Network Driver Interface Specification')
wiki.add_edge('Netzwerkprotokoll', 'Network File System')
wiki.add_edge('Netzwerkprotokoll', 'Network Time Protocol')
wiki.add_edge('Netzwerkprotokoll', 'OSCAR (Protokoll)')
wiki.add_edge('Netzwerkprotokoll', 'OSI-Modell')
wiki.add_edge('Netzwerkprotokoll', 'Off-the-Record Messaging')
wiki.add_edge('Netzwerkprotokoll', 'OpenSSL')
wiki.add_edge('Netzwerkprotokoll', 'PAgP')
wiki.add_edge('Netzwerkprotokoll', 'POP3S')
wiki.add_edge('Netzwerkprotokoll', 'Packet over SONET/SDH')
wiki.add_edge('Netzwerkprotokoll', 'Peer Name Resolution Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Pfadvektorprotokoll')
wiki.add_edge('Netzwerkprotokoll', 'Pfsync')
wiki.add_edge('Netzwerkprotokoll', 'Pi Digit Generation Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Pichat')
wiki.add_edge('Netzwerkprotokoll', 'Picture Transfer Protocol over IP')
wiki.add_edge('Netzwerkprotokoll', 'Point-to-Point Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Point-to-Point Tunneling Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Post Office Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Programmpaket')
wiki.add_edge('Netzwerkprotokoll', 'Prospero (Protokoll)')
wiki.add_edge('Netzwerkprotokoll', 'Protocol for SYnchronous Conferencing')
wiki.add_edge('Netzwerkprotokoll', 'Protokoll (IP)')
wiki.add_edge('Netzwerkprotokoll', 'Protokollstapel')
wiki.add_edge('Netzwerkprotokoll', 'Q.931')
wiki.add_edge('Netzwerkprotokoll', 'QUIC')
wiki.add_edge('Netzwerkprotokoll', 'Rail Safe Transport Application')
wiki.add_edge('Netzwerkprotokoll', 'Real Time Messaging Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Real-Time Streaming Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Real-Time Transport Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Registration Data Access Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Remote Desktop Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Remote Framebuffer Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Resilient Packet Ring')
wiki.add_edge('Netzwerkprotokoll', 'Reverse Address Resolution Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Ripple (Geldsystem)')
wiki.add_edge('Netzwerkprotokoll', 'Robots Exclusion Standard')
wiki.add_edge('Netzwerkprotokoll', 'Routing Information Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Routing Table Maintenance Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Rsync')
wiki.add_edge('Netzwerkprotokoll', 'S-HTTP')
wiki.add_edge('Netzwerkprotokoll', 'SAE J1939')
wiki.add_edge('Netzwerkprotokoll', 'SILC')
wiki.add_edge('Netzwerkprotokoll', 'SOAP')
wiki.add_edge('Netzwerkprotokoll', 'SOCKS')
wiki.add_edge('Netzwerkprotokoll', 'SPDY')
wiki.add_edge('Netzwerkprotokoll', 'STARTTLS')
wiki.add_edge('Netzwerkprotokoll', 'Schichtenarchitektur')
wiki.add_edge('Netzwerkprotokoll', 'Secure Copy')
wiki.add_edge('Netzwerkprotokoll', 'Secure Shell')
wiki.add_edge('Netzwerkprotokoll', 'Sensor Media Access Control')
wiki.add_edge('Netzwerkprotokoll', 'Sequenced Packet Exchange')
wiki.add_edge('Netzwerkprotokoll', 'Sequenznummer')
wiki.add_edge('Netzwerkprotokoll', 'Serial Line Internet Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Server Message Block')
wiki.add_edge('Netzwerkprotokoll', 'Service Data Unit')
wiki.add_edge('Netzwerkprotokoll', 'Service Location Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Serviceorientierte Architektur')
wiki.add_edge('Netzwerkprotokoll', 'Session Announcement Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Session Initiation Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Session Traversal Utilities for NAT')
wiki.add_edge('Netzwerkprotokoll', 'Short Message Peer to Peer')
wiki.add_edge('Netzwerkprotokoll', 'Signalling System 7')
wiki.add_edge('Netzwerkprotokoll', 'Simple Asynchronous File Transfer')
wiki.add_edge('Netzwerkprotokoll', 'Simple Authentication and Security Layer')
wiki.add_edge('Netzwerkprotokoll', 'Simple Common Gateway Interface')
wiki.add_edge('Netzwerkprotokoll', 'Simple Mail Access Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Simple Network Management Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Simple Service Discovery Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Skype')
wiki.add_edge('Netzwerkprotokoll', 'Socket (Software)')
wiki.add_edge('Netzwerkprotokoll', 'Sockets Direct Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Stream Control Transmission Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Streaming-Protokoll')
wiki.add_edge('Netzwerkprotokoll', 'Synchronous Data Link Control')
wiki.add_edge('Netzwerkprotokoll', 'Syslog')
wiki.add_edge('Netzwerkprotokoll', 'T.120')
wiki.add_edge('Netzwerkprotokoll', 'TCP Port Service Multiplexer')
wiki.add_edge('Netzwerkprotokoll', 'TCP Receive Window')
wiki.add_edge('Netzwerkprotokoll', 'TIPC')
wiki.add_edge('Netzwerkprotokoll', 'Telnet')
wiki.add_edge('Netzwerkprotokoll', 'Threema')
wiki.add_edge('Netzwerkprotokoll', 'Timeout Media Access Control')
wiki.add_edge('Netzwerkprotokoll', 'TokenTalk Link Access Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Tox (Protokoll)')
wiki.add_edge('Netzwerkprotokoll', 'Transmission Control Protocol/Internet Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Transparent Network Substrate')
wiki.add_edge('Netzwerkprotokoll', 'Trivial File Transfer Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Tunnel (Rechnernetz)')
wiki.add_edge('Netzwerkprotokoll', 'Universal Measurement and Calibration Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Universal Plug and Play')
wiki.add_edge('Netzwerkprotokoll', 'VTAM')
wiki.add_edge('Netzwerkprotokoll', 'Verschlüsselungsprotokoll')
wiki.add_edge('Netzwerkprotokoll', 'WebDAV')
wiki.add_edge('Netzwerkprotokoll', 'WebSocket')
wiki.add_edge('Netzwerkprotokoll', 'Webservice')
wiki.add_edge('Netzwerkprotokoll', 'Whois')
wiki.add_edge('Netzwerkprotokoll', 'X Window System')
wiki.add_edge('Netzwerkprotokoll', 'X.500')
wiki.add_edge('Netzwerkprotokoll', 'X.75')
wiki.add_edge('Netzwerkprotokoll', 'X10 (Protokoll)')
wiki.add_edge('Netzwerkprotokoll', 'XDMCP')
wiki.add_edge('Netzwerkprotokoll', 'Z39.50')
wiki.add_edge('Netzwerkprotokoll', 'Zebra Media Access Control')
wiki.add_edge('Netzwerkprotokoll', 'Zephyr (Protokoll)')
wiki.add_edge('Netzwerkprotokoll', 'Zone Information Protocol')
wiki.add_edge('Netzwerkprotokoll', 'Zustandslosigkeit')
wiki.add_edge('Normalisierung (Datenbank)', 'ANSI-SPARC-Architektur')
wiki.add_edge('Normalisierung (Datenbank)', 'Anomalie (Informatik)')
wiki.add_edge('Normalisierung (Datenbank)', 'Denormalisierung')
wiki.add_edge('Normalisierung (Datenbank)', 'Item-Item Description')
wiki.add_edge('Normalisierung (Datenbank)', 'Object-relational impedance mismatch')
wiki.add_edge('Normalisierung (Datenbank)', 'Schneeflockenschema')
wiki.add_edge('Normalisierung (Datenbank)', 'Sicht (Datenbank)')
wiki.add_edge('Normalisierung (Datenbank)', 'Sternschema')
wiki.add_edge('Normalisierung (Datenbank)', 'Synthesealgorithmus')
wiki.add_edge('Normierte Programmierung', 'Spaghetticode')
wiki.add_edge('Nullobjekt (Entwurfsmuster)', 'Singleton (Entwurfsmuster)')
wiki.add_edge('Nxt (Kryptowährung)', 'NEM (Kryptowährung)')
wiki.add_edge('Nxt (Kryptowährung)', 'Waves (Kryptowährung)')
wiki.add_edge('OLE DB', 'ADO.NET')
wiki.add_edge('OLE DB', 'Remote Data Objects')
wiki.add_edge('OPC Unified Architecture', 'Advanced Physical Layer')
wiki.add_edge('OPC Unified Architecture', 'Open Platform Communications')
wiki.add_edge('OPC Unified Architecture', 'Profinet')
wiki.add_edge('OPC Unified Architecture', 'Umati')
wiki.add_edge('OPC Unified Architecture', 'Weihenstephaner Standards')
wiki.add_edge('OPENSTEP', 'Carbon (Apple)')
wiki.add_edge('OPENSTEP', 'Cocoa (API)')
wiki.add_edge('OPENSTEP', 'OpenStep')
wiki.add_edge('OSCAR (Protokoll)', 'SecureIM')
wiki.add_edge('OSCAR (Protokoll)', 'TOC (Protokoll)')
wiki.add_edge('OSI-Modell', '3964R')
wiki.add_edge('OSI-Modell', '4in6')
wiki.add_edge('OSI-Modell', '6LoWPAN')
wiki.add_edge('OSI-Modell', '6over4')
wiki.add_edge('OSI-Modell', 'Address Resolution Protocol')
wiki.add_edge('OSI-Modell', 'Apple Filing Protocol')
wiki.add_edge('OSI-Modell', 'AppleTalk')
wiki.add_edge('OSI-Modell', 'Autonegotiation')
wiki.add_edge('OSI-Modell', 'BACnet')
wiki.add_edge('OSI-Modell', 'BitTorrent')
wiki.add_edge('OSI-Modell', 'CANopen')
wiki.add_edge('OSI-Modell', 'Cisco Discovery Protocol')
wiki.add_edge('OSI-Modell', 'Common ISDN Application Programming Interface')
wiki.add_edge('OSI-Modell', 'Connectionless Network Protocol')
wiki.add_edge('OSI-Modell', 'Controller Area Network')
wiki.add_edge('OSI-Modell', 'Dante (Netzwerkprotokoll)')
wiki.add_edge('OSI-Modell', 'Data Link Control')
wiki.add_edge('OSI-Modell', 'Datagram Congestion Control Protocol')
wiki.add_edge('OSI-Modell', 'Datagram Delivery Protocol')
wiki.add_edge('OSI-Modell', 'Datagramm')
wiki.add_edge('OSI-Modell', 'Digital Data Communications Message Protocol')
wiki.add_edge('OSI-Modell', 'Digital Subscriber System No. 1')
wiki.add_edge('OSI-Modell', 'DirectX')
wiki.add_edge('OSI-Modell', 'Druckprotokoll')
wiki.add_edge('OSI-Modell', 'Durchsatzratenbegrenzung')
wiki.add_edge('OSI-Modell', 'Dynamic Information and Action Gateway')
wiki.add_edge('OSI-Modell', 'Echtzeit-Ethernet')
wiki.add_edge('OSI-Modell', 'Enhanced Interior Gateway Routing Protocol')
wiki.add_edge('OSI-Modell', 'EtherTalk Link Access Protocol')
wiki.add_edge('OSI-Modell', 'EtherType')
wiki.add_edge('OSI-Modell', 'Ethernet')
wiki.add_edge('OSI-Modell', 'Ethernet Powerlink')
wiki.add_edge('OSI-Modell', 'Explicit Congestion Notification')
wiki.add_edge('OSI-Modell', 'External Data Representation')
wiki.add_edge('OSI-Modell', 'FTZ 1 TR 6')
wiki.add_edge('OSI-Modell', 'Fiber Distributed Data Interface')
wiki.add_edge('OSI-Modell', 'Fibre Channel over Ethernet')
wiki.add_edge('OSI-Modell', 'File Transfer, Access and Management')
wiki.add_edge('OSI-Modell', 'High-Level Data Link Control')
wiki.add_edge('OSI-Modell', 'Highway Addressable Remote Transducer')
wiki.add_edge('OSI-Modell', 'HyperSCSI')
wiki.add_edge('OSI-Modell', 'IEEE 802')
wiki.add_edge('OSI-Modell', 'IEEE 802.1Q')
wiki.add_edge('OSI-Modell', 'IP-Fragmentierung')
wiki.add_edge('OSI-Modell', 'IP-Header')
wiki.add_edge('OSI-Modell', 'IP-Paket')
wiki.add_edge('OSI-Modell', 'IPsec')
wiki.add_edge('OSI-Modell', 'ISATAP')
wiki.add_edge('OSI-Modell', 'ISO 15765-2')
wiki.add_edge('OSI-Modell', 'Interface Control Information')
wiki.add_edge('OSI-Modell', 'Internet Control Message Protocol')
wiki.add_edge('OSI-Modell', 'Internet Group Management Protocol')
wiki.add_edge('OSI-Modell', 'Internetwork Packet Exchange')
wiki.add_edge('OSI-Modell', 'Internetwork Packet eXchange Fragmentation')
wiki.add_edge('OSI-Modell', 'KWP2000')
wiki.add_edge('OSI-Modell', 'LLDP')
wiki.add_edge('OSI-Modell', 'Layer 2 Tunneling Protocol')
wiki.add_edge('OSI-Modell', 'Link Access Procedure Balanced')
wiki.add_edge('OSI-Modell', 'Link Access Procedure for the D-Channel')
wiki.add_edge('OSI-Modell', 'LocalTalk')
wiki.add_edge('OSI-Modell', 'LocalTalk Link Access Protocol')
wiki.add_edge('OSI-Modell', 'Logical Link Control')
wiki.add_edge('OSI-Modell', 'Media Access Control')
wiki.add_edge('OSI-Modell', 'Mobile IP')
wiki.add_edge('OSI-Modell', 'Multiprotocol over ATM')
wiki.add_edge('OSI-Modell', 'NetBEUI')
wiki.add_edge('OSI-Modell', 'NetBIOS')
wiki.add_edge('OSI-Modell', 'NetWare Core Protocol')
wiki.add_edge('OSI-Modell', 'Netzwerkadressübersetzung')
wiki.add_edge('OSI-Modell', 'Open Datalink Interface')
wiki.add_edge('OSI-Modell', 'Packet Data Convergence Protocol')
wiki.add_edge('OSI-Modell', 'Packet over SONET/SDH')
wiki.add_edge('OSI-Modell', 'Point-to-Point Protocol')
wiki.add_edge('OSI-Modell', 'Profinet')
wiki.add_edge('OSI-Modell', 'Protokollstapel')
wiki.add_edge('OSI-Modell', 'RK512')
wiki.add_edge('OSI-Modell', 'RMON')
wiki.add_edge('OSI-Modell', 'Radio Link Control')
wiki.add_edge('OSI-Modell', 'Remote Desktop Protocol')
wiki.add_edge('OSI-Modell', 'Remote Procedure Call')
wiki.add_edge('OSI-Modell', 'Resilient Packet Ring')
wiki.add_edge('OSI-Modell', 'Reverse Address Resolution Protocol')
wiki.add_edge('OSI-Modell', 'Routing Table Maintenance Protocol')
wiki.add_edge('OSI-Modell', 'SafetyNET p')
wiki.add_edge('OSI-Modell', 'Schichtenarchitektur')
wiki.add_edge('OSI-Modell', 'Sequenced Packet Exchange')
wiki.add_edge('OSI-Modell', 'Service Data Unit')
wiki.add_edge('OSI-Modell', 'Signalling Connection Control Part')
wiki.add_edge('OSI-Modell', 'Softwarearchitektur')
wiki.add_edge('OSI-Modell', 'Spanning Tree Protocol')
wiki.add_edge('OSI-Modell', 'Stream Control Transmission Protocol')
wiki.add_edge('OSI-Modell', 'Synchronous Data Link Control')
wiki.add_edge('OSI-Modell', 'TR-143')
wiki.add_edge('OSI-Modell', 'Token Bus')
wiki.add_edge('OSI-Modell', 'Token Ring')
wiki.add_edge('OSI-Modell', 'TokenTalk Link Access Protocol')
wiki.add_edge('OSI-Modell', 'Transparent Network Substrate')
wiki.add_edge('OSI-Modell', 'Unified Diagnostic Services')
wiki.add_edge('OSI-Modell', 'Virtual Extensible LAN')
wiki.add_edge('OSI-Modell', 'Virtual Terminal')
wiki.add_edge('OSI-Modell', 'WS-SecurityPolicy')
wiki.add_edge('OSI-Modell', 'X.500')
wiki.add_edge('OSI-Modell', 'Z39.50')
wiki.add_edge('OSI-Modell', 'Zone Information Protocol')
wiki.add_edge('Object Constraint Language', 'Innovator (Software)')
wiki.add_edge('Object Constraint Language', 'Klassendiagramm')
wiki.add_edge('Object Constraint Language', 'MOF QVT')
wiki.add_edge('Object Constraint Language', 'Sequenzdiagramm')
wiki.add_edge('Object Constraint Language', 'Softwarearchitekt')
wiki.add_edge('Object Constraint Language', 'Zusicherung (Softwaretechnik)')
wiki.add_edge('Object Linking and Embedding', 'Bonobo (GNOME)')
wiki.add_edge('Object Linking and Embedding', 'Dynamic Data Exchange')
wiki.add_edge('Object Linking and Embedding', 'Komponente (Software)')
wiki.add_edge('Object Linking and Embedding', 'Open Platform Communications')
wiki.add_edge('Object-relational impedance mismatch', 'Jakarta Persistence API')
wiki.add_edge('Object-relational impedance mismatch', 'SQLAlchemy')
wiki.add_edge('Objektdiagramm', 'Fujaba')
wiki.add_edge('Objektdiagramm', 'Kompositionsstrukturdiagramm')
wiki.add_edge('Objektdiagramm', 'StarUML')
wiki.add_edge('Objektdiagramm', 'Tripel-Graph-Grammatik')
wiki.add_edge('Objektdiagramm', 'Violet (UML-Werkzeug)')
wiki.add_edge('Objektfluss (UML)', 'Aktivitätsparameterknoten')
wiki.add_edge('Objektfluss (UML)', 'Interaktion (UML)')
wiki.add_edge('Objektfluss (UML)', 'Kontrollfluss (UML)')
wiki.add_edge('Objektknoten (UML)', 'Aktivitätsparameterknoten')
wiki.add_edge('Objektknoten (UML)', 'Kontrollfluss (UML)')
wiki.add_edge('Objektknoten (UML)', 'Kontrollknoten (UML)')
wiki.add_edge('Objektknoten (UML)', 'Objektfluss (UML)')
wiki.add_edge('Objektknoten (UML)', 'Pufferknoten (UML)')
wiki.add_edge('Objektmodell', 'Immediate Mode (Computergrafik)')
wiki.add_edge('Objektmodell', 'Modularität')
wiki.add_edge('Objektmodell', 'Retained Mode')
wiki.add_edge('Objektmodell', 'Unified Modeling Language')
wiki.add_edge('Objektmodell', 'Zachman Framework')
wiki.add_edge('Observation (Analysemuster)', 'Anwendungsfassade')
wiki.add_edge('Observation (Analysemuster)', 'Range (Analysemuster)')
wiki.add_edge('Off-the-Record Messaging', 'Extensible Messaging and Presence Protocol')
wiki.add_edge('Off-the-Record Messaging', 'OSCAR (Protokoll)')
wiki.add_edge('Off-the-Record Messaging', 'SecureIM')
wiki.add_edge('Off-the-Record Messaging', 'Signal-Protokoll')
wiki.add_edge('Off-the-Record Messaging', 'Silent Circle Instant Messaging Protocol')
wiki.add_edge('Off-the-Record Messaging', 'Windows Live Messenger')
wiki.add_edge('Off-the-Record Messaging', 'Yahoo Messenger')
wiki.add_edge('Open Database Connectivity', 'ADOdb')
wiki.add_edge('Open Database Connectivity', 'Borland Database Engine')
wiki.add_edge('Open Database Connectivity', 'Call Level Interface')
wiki.add_edge('Open Database Connectivity', 'Data Access Objects')
wiki.add_edge('Open Database Connectivity', 'Datenbankschnittstelle')
wiki.add_edge('Open Database Connectivity', 'Embedded SQL')
wiki.add_edge('Open Database Connectivity', 'Java Database Connectivity')
wiki.add_edge('Open Database Connectivity', 'OLE DB')
wiki.add_edge('Open Database Connectivity', 'Remote Data Objects')
wiki.add_edge('Open Graphics Library for Embedded Systems', 'Gallium3D')
wiki.add_edge('Open Graphics Library for Embedded Systems', 'WebGL')
wiki.add_edge('Open Platform Communications', 'International Data Spaces')
wiki.add_edge('Open Shortest Path First', '6LoWPAN')
wiki.add_edge('Open Shortest Path First', 'Aggregation (Informatik)')
wiki.add_edge('Open Shortest Path First', 'Bidirectional Forwarding Detection')
wiki.add_edge('Open Shortest Path First', 'Protokoll (IP)')
wiki.add_edge('OpenAL', 'Core Audio')
wiki.add_edge('OpenAL', 'OpenCL')
wiki.add_edge('OpenAPI', 'Innovator (Software)')
wiki.add_edge('OpenAPI', 'Swagger (Software)')
wiki.add_edge('OpenCL', 'Grand Central Dispatch')
wiki.add_edge('OpenCL', 'Metal (API)')
wiki.add_edge('OpenCL', 'WebCL')
wiki.add_edge('OpenDBX', 'Datenbankabstraktionsschicht')
wiki.add_edge('OpenGL', 'AMD Mantle')
wiki.add_edge('OpenGL', 'Bildschirmschoner')
wiki.add_edge('OpenGL', 'Canvas (HTML-Element)')
wiki.add_edge('OpenGL', 'EGL (Programmierschnittstelle)')
wiki.add_edge('OpenGL', 'Gallium3D')
wiki.add_edge('OpenGL', 'Graphics Device Interface')
wiki.add_edge('OpenGL', 'IRIS GL')
wiki.add_edge('OpenGL', 'Immediate Mode (Computergrafik)')
wiki.add_edge('OpenGL', 'Java 3D')
wiki.add_edge('OpenGL', 'JavaFX')
wiki.add_edge('OpenGL', 'Metal (API)')
wiki.add_edge('OpenGL', 'MoltenVK')
wiki.add_edge('OpenGL', 'Mumble')
wiki.add_edge('OpenGL', 'Open Graphics Library for Embedded Systems')
wiki.add_edge('OpenGL', 'OpenAL')
wiki.add_edge('OpenGL', 'OpenCL')
wiki.add_edge('OpenGL', 'OpenGL Utility Toolkit')
wiki.add_edge('OpenGL', 'OpenML')
wiki.add_edge('OpenGL', 'Retained Mode')
wiki.add_edge('OpenGL', 'Vulkan (API)')
wiki.add_edge('OpenGL', 'WebGL')
wiki.add_edge('OpenGL', 'XNA (Microsoft)')
wiki.add_edge('OpenNIC', 'Gopher')
wiki.add_edge('OpenNIC', 'Network Information Center')
wiki.add_edge('OpenSSL', 'Certificate Management Protocol')
wiki.add_edge('OpenSSL', 'Domain Name System Security Extensions')
wiki.add_edge('OpenSSL', 'FREAK')
wiki.add_edge('OpenSSL', 'GnuTLS')
wiki.add_edge('OpenSSL', 'Heartbleed')
wiki.add_edge('OpenSSL', 'Kerberos (Protokoll)')
wiki.add_edge('OpenSSL', 'LibreSSL')
wiki.add_edge('OpenSSL', 'Mbed TLS')
wiki.add_edge('OpenSSL', 'Online Certificate Status Protocol stapling')
wiki.add_edge('OpenSSL', 'RetroShare')
wiki.add_edge('OpenSSL', 'Secure Reliable Transport')
wiki.add_edge('OpenSSL', 'Secure Shell')
wiki.add_edge('OpenSSL', 'Time Stamp Protocol')
wiki.add_edge('OpenStep', 'Carbon (Apple)')
wiki.add_edge('OpenStep', 'Cocoa (API)')
wiki.add_edge('OpenStep', 'Macintosh-Baukasten')
wiki.add_edge('Operation (UML)', 'Aktivitätsparameterknoten')
wiki.add_edge('Operation (UML)', 'Merkmal (UML)')
wiki.add_edge('Operation (UML)', 'Quantität (Analysemuster)')
wiki.add_edge('Operation (UML)', 'Zusicherung (Softwaretechnik)')
wiki.add_edge('Optimized Link State Routing', 'Ad-hoc On-demand Distance Vector')
wiki.add_edge('Optimized Link State Routing', 'B.A.T.M.A.N.')
wiki.add_edge('Optimized Link State Routing', 'Babel (Protokoll)')
wiki.add_edge('Organization Hierarchy', 'Accountability')
wiki.add_edge('Organization Hierarchy', 'Organization Structure')
wiki.add_edge('Organization Structure', 'Accountability')
wiki.add_edge('PAgP', 'Dynamic Trunking Protocol')
wiki.add_edge('PHP Data Objects', 'ADOdb Lite')
wiki.add_edge('PHP Data Objects', 'MySQLi')
wiki.add_edge('POP3S', 'Liste von TCP/IP-basierten Netzwerkdiensten')
wiki.add_edge('PROFINET', 'EchoRing')
wiki.add_edge('PROFINET', 'Profinet')
wiki.add_edge('Packet Data Convergence Protocol', 'ROHC')
wiki.add_edge('Paket (UML)', 'Generalisierung (UML)')
wiki.add_edge('Paket (UML)', 'JavaFX')
wiki.add_edge('Paket (UML)', 'Modell (UML)')
wiki.add_edge('Paket (UML)', 'Paketdiagramm')
wiki.add_edge('Paket (UML)', 'Paketimport (UML)')
wiki.add_edge('Paket (UML)', 'Paketverschmelzung')
wiki.add_edge('Paket (UML)', 'Profil (UML)')
wiki.add_edge('Paket (UML)', 'Stereotyp (UML)')
wiki.add_edge('Paket (UML)', 'Tripel-Graph-Grammatik')
wiki.add_edge('Paketdiagramm', '4+1 Sichtenmodell')
wiki.add_edge('Paketdiagramm', 'Modell (UML)')
wiki.add_edge('Paketdiagramm', 'Profil (UML)')
wiki.add_edge('Paketdiagramm', 'StarUML')
wiki.add_edge('Paketimport (UML)', 'Paketdiagramm')
wiki.add_edge('Paketimport (UML)', 'Paketverschmelzung')
wiki.add_edge('Paketimport (UML)', 'Profil (UML)')
wiki.add_edge('Paketverschmelzung', 'Generalisierung (UML)')
wiki.add_edge('Paketverschmelzung', 'Paketdiagramm')
wiki.add_edge('Papyrus (UML)', 'TOPCASED')
wiki.add_edge('Parameter (Informatik)', 'Code-Smell')
wiki.add_edge('Parameter (Informatik)', 'Datenmodellierung')
wiki.add_edge('Parameter (Informatik)', 'JavaFX')
wiki.add_edge('Parameter (Informatik)', 'Normierte Programmierung')
wiki.add_edge('Parameter (Informatik)', 'OPC Unified Architecture')
wiki.add_edge('Parameter (Informatik)', 'Verzögerte Initialisierung')
wiki.add_edge('Parlay', 'Java APIs for Integrated Networks')
wiki.add_edge('Peg DHCP', 'Internet Protocol over Avian Carriers')
wiki.add_edge('Peg DHCP', 'Pi Digit Generation Protocol')
wiki.add_edge('Perl-DBI', 'Datenbankabstraktionsschicht')
wiki.add_edge('Pfadvektorprotokoll', 'Distanzvektoralgorithmus')
wiki.add_edge('Pin (UML)', 'Aktion (UML)')
wiki.add_edge('Pin (UML)', 'Kontrollfluss (UML)')
wiki.add_edge('Pin (UML)', 'Objektfluss (UML)')
wiki.add_edge('Pin (UML)', 'Objektknoten (UML)')
wiki.add_edge('Pin (UML)', 'Pufferknoten (UML)')
wiki.add_edge('Pipe (Informatik)', 'Blocks Extensible Exchange Protocol')
wiki.add_edge('Pipe (Informatik)', 'Konsolenanwendung')
wiki.add_edge('Pipe (Informatik)', 'Pipes und Filter')
wiki.add_edge('Pluggable Authentication Modules', 'Java Authentication and Authorization Service')
wiki.add_edge('Pluggable Authentication Modules', 'Modul (Software)')
wiki.add_edge('Point-to-Point Protocol', '0.0.0.0')
wiki.add_edge('Point-to-Point Protocol', 'AmiTCP')
wiki.add_edge('Point-to-Point Protocol', 'AppleTalk')
wiki.add_edge('Point-to-Point Protocol', 'Appletalk')
wiki.add_edge('Point-to-Point Protocol', 'CSLIP')
wiki.add_edge('Point-to-Point Protocol', 'Data Link Control')
wiki.add_edge('Point-to-Point Protocol', 'EAPOL')
wiki.add_edge('Point-to-Point Protocol', 'Encryption Control Protocol')
wiki.add_edge('Point-to-Point Protocol', 'Extensible Authentication Protocol')
wiki.add_edge('Point-to-Point Protocol', 'High-Level Data Link Control')
wiki.add_edge('Point-to-Point Protocol', 'IEEE 802.1X')
wiki.add_edge('Point-to-Point Protocol', 'Internet Protocol Control Protocol')
wiki.add_edge('Point-to-Point Protocol', 'Internetwork Packet Exchange')
wiki.add_edge('Point-to-Point Protocol', 'Layer 2 Forwarding')
wiki.add_edge('Point-to-Point Protocol', 'Layer 2 Tunneling Protocol')
wiki.add_edge('Point-to-Point Protocol', 'Microsoft Point-To-Point Encryption Protocol')
wiki.add_edge('Point-to-Point Protocol', 'Packet over SONET/SDH')
wiki.add_edge('Point-to-Point Protocol', 'Point-to-Point Tunneling Protocol')
wiki.add_edge('Point-to-Point Protocol', 'Remote Authentication Dial-In User Service')
wiki.add_edge('Point-to-Point Protocol', 'Secure Socket Tunneling Protocol')
wiki.add_edge('Point-to-Point Protocol', 'Serial Line Internet Protocol')
wiki.add_edge('Point-to-Point Protocol', 'VTun')
wiki.add_edge('Point-to-Point Protocol', 'Vermittlungsschicht')
wiki.add_edge('Point-to-Point Protocol', 'X.75')
wiki.add_edge('Point-to-Point Tunneling Protocol', 'Crypto IP Encapsulation')
wiki.add_edge('Point-to-Point Tunneling Protocol', 'Generic Routing Encapsulation')
wiki.add_edge('Point-to-Point Tunneling Protocol', 'Layer 2 Tunneling Protocol')
wiki.add_edge('Point-to-Point Tunneling Protocol', 'Liste der standardisierten Ports')
wiki.add_edge('Point-to-Point Tunneling Protocol', 'Microsoft Point-To-Point Encryption Protocol')
wiki.add_edge('Point-to-Point Tunneling Protocol', 'Secure Socket Tunneling Protocol')
wiki.add_edge('Point-to-Point Tunneling Protocol', 'Tunnel (Rechnernetz)')
wiki.add_edge('Port (Protokoll)', 'Apple Filing Protocol')
wiki.add_edge('Port (Protokoll)', 'AppleTalk')
wiki.add_edge('Port (Protokoll)', 'BattleEye RCon Protocol')
wiki.add_edge('Port (Protokoll)', 'Bootstrap Protocol')
wiki.add_edge('Port (Protokoll)', 'Client')
wiki.add_edge('Port (Protokoll)', 'Common Open Policy Service')
wiki.add_edge('Port (Protokoll)', 'DNS over HTTPS')
wiki.add_edge('Port (Protokoll)', 'DNS over TLS')
wiki.add_edge('Port (Protokoll)', 'Daytime')
wiki.add_edge('Port (Protokoll)', 'Discard')
wiki.add_edge('Port (Protokoll)', 'Dynamic Host Configuration Protocol')
wiki.add_edge('Port (Protokoll)', 'EDonkey2000')
wiki.add_edge('Port (Protokoll)', 'Echo (Netzwerkdienst)')
wiki.add_edge('Port (Protokoll)', 'Extensible Messaging and Presence Protocol')
wiki.add_edge('Port (Protokoll)', 'File Transfer Protocol')
wiki.add_edge('Port (Protokoll)', 'File eXchange Protocol')
wiki.add_edge('Port (Protokoll)', 'Gopher')
wiki.add_edge('Port (Protokoll)', 'Homebanking Computer Interface')
wiki.add_edge('Port (Protokoll)', 'Hypertext Transfer Protocol Secure')
wiki.add_edge('Port (Protokoll)', 'I2P')
wiki.add_edge('Port (Protokoll)', 'IGD-Protokoll')
wiki.add_edge('Port (Protokoll)', 'Ident')
wiki.add_edge('Port (Protokoll)', 'InterAsterisk eXchange')
wiki.add_edge('Port (Protokoll)', 'Internet Assigned Numbers Authority')
wiki.add_edge('Port (Protokoll)', 'Internet Corporation for Assigned Names and Numbers')
wiki.add_edge('Port (Protokoll)', 'Internet Message Access Protocol')
wiki.add_edge('Port (Protokoll)', 'JSON Meta Application Protocol')
wiki.add_edge('Port (Protokoll)', 'JetDirect')
wiki.add_edge('Port (Protokoll)', 'Lightweight Directory Access Protocol')
wiki.add_edge('Port (Protokoll)', 'Liste der standardisierten Ports')
wiki.add_edge('Port (Protokoll)', 'Liste von TCP/IP-basierten Netzwerkdiensten')
wiki.add_edge('Port (Protokoll)', 'MMS-Protokoll')
wiki.add_edge('Port (Protokoll)', 'MQTT')
wiki.add_edge('Port (Protokoll)', 'Multi-purpose Business Security over IP')
wiki.add_edge('Port (Protokoll)', 'NAT Port Mapping Protocol')
wiki.add_edge('Port (Protokoll)', 'NTLM')
wiki.add_edge('Port (Protokoll)', 'Netfilter')
wiki.add_edge('Port (Protokoll)', 'Netflow')
wiki.add_edge('Port (Protokoll)', 'Network File System')
wiki.add_edge('Port (Protokoll)', 'Network News Transfer Protocol')
wiki.add_edge('Port (Protokoll)', 'Network Time Protocol')
wiki.add_edge('Port (Protokoll)', 'Netzwerkadressübersetzung')
wiki.add_edge('Port (Protokoll)', 'Netzwerkprotokoll')
wiki.add_edge('Port (Protokoll)', 'OSI-Modell')
wiki.add_edge('Port (Protokoll)', 'On-Demand Mail Relay')
wiki.add_edge('Port (Protokoll)', 'POP3S')
wiki.add_edge('Port (Protokoll)', 'Pi Digit Generation Protocol')
wiki.add_edge('Port (Protokoll)', 'Pichat')
wiki.add_edge('Port (Protokoll)', 'Port Address Translation')
wiki.add_edge('Port (Protokoll)', 'Portweiterleitung')
wiki.add_edge('Port (Protokoll)', 'Post Office Protocol')
wiki.add_edge('Port (Protokoll)', 'Proxy Auto-Config')
wiki.add_edge('Port (Protokoll)', 'Rapid Spanning Tree Protocol')
wiki.add_edge('Port (Protokoll)', 'Real-Time Streaming Protocol')
wiki.add_edge('Port (Protokoll)', 'Remote Desktop Protocol')
wiki.add_edge('Port (Protokoll)', 'Remote Framebuffer Protocol')
wiki.add_edge('Port (Protokoll)', 'Remote Function Call')
wiki.add_edge('Port (Protokoll)', 'Remote Method Invocation')
wiki.add_edge('Port (Protokoll)', 'Remote Procedure Call')
wiki.add_edge('Port (Protokoll)', 'RetroShare')
wiki.add_edge('Port (Protokoll)', 'Rsync')
wiki.add_edge('Port (Protokoll)', 'SMTPS')
wiki.add_edge('Port (Protokoll)', 'SYN-Cookies')
wiki.add_edge('Port (Protokoll)', 'SafetyNET p')
wiki.add_edge('Port (Protokoll)', 'Secure Shell')
wiki.add_edge('Port (Protokoll)', 'Secure Socket Tunneling Protocol')
wiki.add_edge('Port (Protokoll)', 'Server Message Block')
wiki.add_edge('Port (Protokoll)', 'Session Announcement Protocol')
wiki.add_edge('Port (Protokoll)', 'Session Description Protocol')
wiki.add_edge('Port (Protokoll)', 'Session Traversal Utilities for NAT')
wiki.add_edge('Port (Protokoll)', 'Simple Asynchronous File Transfer')
wiki.add_edge('Port (Protokoll)', 'Simple File Transfer Protocol')
wiki.add_edge('Port (Protokoll)', 'Simple Mail Transfer Protocol')
wiki.add_edge('Port (Protokoll)', 'Simple Network Management Protocol')
wiki.add_edge('Port (Protokoll)', 'Simple Service Discovery Protocol')
wiki.add_edge('Port (Protokoll)', 'Socket (Software)')
wiki.add_edge('Port (Protokoll)', 'TCP Port Service Multiplexer')
wiki.add_edge('Port (Protokoll)', 'Transmission Control Protocol')
wiki.add_edge('Port (Protokoll)', 'Transport Layer Security')
wiki.add_edge('Port (Protokoll)', 'Tunnel (Rechnernetz)')
wiki.add_edge('Port (Protokoll)', 'Universal Plug and Play')
wiki.add_edge('Port (Protokoll)', 'WebDAV')
wiki.add_edge('Port (Protokoll)', 'XDMCP')
wiki.add_edge('Port (Protokoll)', 'Z39.50')
wiki.add_edge('Port (Protokoll)', 'Zephyr (Protokoll)')
wiki.add_edge('Port (UML)', 'Classifier (UML)')
wiki.add_edge('Port (UML)', 'Komponente (UML)')
wiki.add_edge('Port (UML)', 'Komponentendiagramm')
wiki.add_edge('Port (UML)', 'Kompositionsstrukturdiagramm')
wiki.add_edge('Port (UML)', 'Konnektor (UML)')
wiki.add_edge('Port (UML)', 'Merkmal (UML)')
wiki.add_edge('Port Address Translation', 'WebRTC')
wiki.add_edge('Portable Operating System Interface', 'Gnome Virtual File System')
wiki.add_edge('Portweiterleitung', 'IGD-Protokoll')
wiki.add_edge('Portweiterleitung', 'Port Address Translation')
wiki.add_edge('Portweiterleitung', 'SIMON')
wiki.add_edge('Portweiterleitung', 'Tox (Protokoll)')
wiki.add_edge('Portweiterleitung', 'Tunnel (Rechnernetz)')
wiki.add_edge('Post Office Protocol', 'ASCII-Protokoll')
wiki.add_edge('Post Office Protocol', 'Authenticated Post Office Protocol')
wiki.add_edge('Post Office Protocol', 'I2P')
wiki.add_edge('Post Office Protocol', 'Internet Message Access Protocol')
wiki.add_edge('Post Office Protocol', 'Jakarta Mail')
wiki.add_edge('Post Office Protocol', 'Liste der standardisierten Ports')
wiki.add_edge('Post Office Protocol', 'POP3S')
wiki.add_edge('Post Office Protocol', 'STARTTLS')
wiki.add_edge('Post Office Protocol', 'Simple Authentication and Security Layer')
wiki.add_edge('Post Office Protocol', 'Simple Mail Access Protocol')
wiki.add_edge('Post Office Protocol', 'Unix to Unix Copy')
wiki.add_edge('Power over Ethernet', 'HDBaseT')
wiki.add_edge('Power over Ethernet', 'RJ-Steckverbindung')
wiki.add_edge('PowerDesigner', 'Datenbankdesign')
wiki.add_edge('Preboot Execution Environment', 'Network File System')
wiki.add_edge('Preboot Execution Environment', 'Unified Extensible Firmware Interface')
wiki.add_edge('Prinzipien objektorientierten Designs', 'Abstrakte Fabrik')
wiki.add_edge('Prinzipien objektorientierten Designs', 'GRASP')
wiki.add_edge('Prinzipien objektorientierten Designs', 'Kopplung (Softwareentwicklung)')
wiki.add_edge('Prinzipien objektorientierten Designs', 'Modul (Software)')
wiki.add_edge('Prinzipien objektorientierten Designs', 'Refactoring')
wiki.add_edge('Prinzipien objektorientierten Designs', 'Spaghetticode')
wiki.add_edge('Prinzipien objektorientierten Designs', 'VIPER (Entwurfsmuster)')
wiki.add_edge('Privacy Coin', 'Dash (Kryptowährung)')
wiki.add_edge('Privacy Coin', 'MobileCoin')
wiki.add_edge('Privacy Coin', 'Particl (Kryptowährung)')
wiki.add_edge('Privacy Coin', 'Verge (Kryptowährung)')
wiki.add_edge('Private IP-Adresse', 'Carrier-grade NAT')
wiki.add_edge('Private IP-Adresse', 'NAT Port Mapping Protocol')
wiki.add_edge('Private IP-Adresse', 'Port Address Translation')
wiki.add_edge('Private IP-Adresse', 'Portweiterleitung')
wiki.add_edge('Profil (UML)', 'Innovator (Software)')
wiki.add_edge('Profil (UML)', 'Modeling and Analysis of Real Time and Embedded systems')
wiki.add_edge('Profildiagramm', 'Stereotyp (UML)')
wiki.add_edge('Profinet', '100BASE-TX')
wiki.add_edge('Profinet', '10BASE-T1L')
wiki.add_edge('Profinet', 'Advanced Physical Layer')
wiki.add_edge('Profinet', 'EtherNet/IP')
wiki.add_edge('Profinet', 'Media Redundancy Protocol')
wiki.add_edge('Profinet', 'OpenSAFETY')
wiki.add_edge('Profinet', 'Time-Sensitive Networking')
wiki.add_edge('Programmablaufplan', 'Ablaufdiagramm')
wiki.add_edge('Programmablaufplan', 'Datenflussdiagramm')
wiki.add_edge('Programmablaufplan', 'Dia (Software)')
wiki.add_edge('Programmablaufplan', 'Diagrams.net')
wiki.add_edge('Programmablaufplan', 'Kontrollflussgraph')
wiki.add_edge('Programmablaufplan', 'Microsoft Visio')
wiki.add_edge('Programmablaufplan', 'Nassi-Shneiderman-Diagramm')
wiki.add_edge('Programmablaufplan', 'Normierte Programmierung')
wiki.add_edge('Programmierschnittstelle', '464XLAT')
wiki.add_edge('Programmierschnittstelle', 'AMD Mantle')
wiki.add_edge('Programmierschnittstelle', 'ActiveX')
wiki.add_edge('Programmierschnittstelle', 'ActiveX Data Objects')
wiki.add_edge('Programmierschnittstelle', 'Advanced Linux Sound Architecture')
wiki.add_edge('Programmierschnittstelle', 'Apache MINA')
wiki.add_edge('Programmierschnittstelle', 'Application Response Measurement')
wiki.add_edge('Programmierschnittstelle', 'Asynchronous module definition')
wiki.add_edge('Programmierschnittstelle', 'Binary Large Object')
wiki.add_edge('Programmierschnittstelle', 'Binärschnittstelle')
wiki.add_edge('Programmierschnittstelle', 'Bouncy Castle')
wiki.add_edge('Programmierschnittstelle', 'Call Level Interface')
wiki.add_edge('Programmierschnittstelle', 'Carbon (Apple)')
wiki.add_edge('Programmierschnittstelle', 'Cocoa (API)')
wiki.add_edge('Programmierschnittstelle', 'Common ISDN Application Programming Interface')
wiki.add_edge('Programmierschnittstelle', 'Core Animation')
wiki.add_edge('Programmierschnittstelle', 'Core Audio')
wiki.add_edge('Programmierschnittstelle', 'Cross Platform Component Object Model')
wiki.add_edge('Programmierschnittstelle', 'DOS Protected Mode Interface')
wiki.add_edge('Programmierschnittstelle', 'Data Access Object')
wiki.add_edge('Programmierschnittstelle', 'Datenbankabstraktionsschicht')
wiki.add_edge('Programmierschnittstelle', 'Datenbankschnittstelle')
wiki.add_edge('Programmierschnittstelle', 'Diem (Internetwährung)')
wiki.add_edge('Programmierschnittstelle', 'Dienst (Informatik)')
wiki.add_edge('Programmierschnittstelle', 'DirectX')
wiki.add_edge('Programmierschnittstelle', 'DirectX Media Objects')
wiki.add_edge('Programmierschnittstelle', 'DirectX Raytracing')
wiki.add_edge('Programmierschnittstelle', 'EGL (Programmierschnittstelle)')
wiki.add_edge('Programmierschnittstelle', 'Enterprise Architect')
wiki.add_edge('Programmierschnittstelle', 'Ethereum')
wiki.add_edge('Programmierschnittstelle', 'Facebook')
wiki.add_edge('Programmierschnittstelle', 'File Transfer Protocol')
wiki.add_edge('Programmierschnittstelle', 'Fluent Interface')
wiki.add_edge('Programmierschnittstelle', 'Framework')
wiki.add_edge('Programmierschnittstelle', 'GSSAPI')
wiki.add_edge('Programmierschnittstelle', 'Gallium3D')
wiki.add_edge('Programmierschnittstelle', 'Grand Central Dispatch')
wiki.add_edge('Programmierschnittstelle', 'Graphics Device Interface')
wiki.add_edge('Programmierschnittstelle', 'IEC 61970')
wiki.add_edge('Programmierschnittstelle', 'IRIS GL')
wiki.add_edge('Programmierschnittstelle', 'Image and Scanner Interface Specification')
wiki.add_edge('Programmierschnittstelle', 'Immediate Mode (Computergrafik)')
wiki.add_edge('Programmierschnittstelle', 'Indexed Database API')
wiki.add_edge('Programmierschnittstelle', 'Innovator (Software)')
wiki.add_edge('Programmierschnittstelle', 'Jakarta Connectors')
wiki.add_edge('Programmierschnittstelle', 'Jakarta Enterprise Beans')
wiki.add_edge('Programmierschnittstelle', 'Jakarta Mail')
wiki.add_edge('Programmierschnittstelle', 'Jakarta Messaging')
wiki.add_edge('Programmierschnittstelle', 'Jakarta Persistence API')
wiki.add_edge('Programmierschnittstelle', 'Jakarta RESTful Web Services')
wiki.add_edge('Programmierschnittstelle', 'Jakarta Transactions API')
wiki.add_edge('Programmierschnittstelle', 'Jakarta XML RPC')
wiki.add_edge('Programmierschnittstelle', 'Jakarta XML Web Services')
wiki.add_edge('Programmierschnittstelle', 'Java API for XML Processing')
wiki.add_edge('Programmierschnittstelle', 'Java APIs for Integrated Networks')
wiki.add_edge('Programmierschnittstelle', 'Java Accessibility API')
wiki.add_edge('Programmierschnittstelle', 'Java Cryptography Extension')
wiki.add_edge('Programmierschnittstelle', 'Java Data Objects')
wiki.add_edge('Programmierschnittstelle', 'Java Database Connectivity')
wiki.add_edge('Programmierschnittstelle', 'Java Management Extensions')
wiki.add_edge('Programmierschnittstelle', 'Java Naming and Directory Interface')
wiki.add_edge('Programmierschnittstelle', 'Java Secure Socket Extension')
wiki.add_edge('Programmierschnittstelle', 'Java Sound')
wiki.add_edge('Programmierschnittstelle', 'JavaFX')
wiki.add_edge('Programmierschnittstelle', 'JavaHelp System')
wiki.add_edge('Programmierschnittstelle', 'Komponente (Software)')
wiki.add_edge('Programmierschnittstelle', 'LV2')
wiki.add_edge('Programmierschnittstelle', 'LibreSSL')
wiki.add_edge('Programmierschnittstelle', 'Lightning-Netzwerk')
wiki.add_edge('Programmierschnittstelle', 'Linux Audio Developer’s Simple Plugin API')
wiki.add_edge('Programmierschnittstelle', 'MSXML')
wiki.add_edge('Programmierschnittstelle', 'Macintosh-Baukasten')
wiki.add_edge('Programmierschnittstelle', 'Mashup (Internet)')
wiki.add_edge('Programmierschnittstelle', 'Matrix (Kommunikationsprotokoll)')
wiki.add_edge('Programmierschnittstelle', 'Message Oriented Middleware')
wiki.add_edge('Programmierschnittstelle', 'Message Passing Interface')
wiki.add_edge('Programmierschnittstelle', 'Messaging Application Programming Interface')
wiki.add_edge('Programmierschnittstelle', 'Metal (API)')
wiki.add_edge('Programmierschnittstelle', 'Microservices')
wiki.add_edge('Programmierschnittstelle', 'Modelio')
wiki.add_edge('Programmierschnittstelle', 'Modellgetriebene Architektur')
wiki.add_edge('Programmierschnittstelle', 'MoltenVK')
wiki.add_edge('Programmierschnittstelle', 'Multimedia-Framework')
wiki.add_edge('Programmierschnittstelle', 'NAT64')
wiki.add_edge('Programmierschnittstelle', 'Native API')
wiki.add_edge('Programmierschnittstelle', 'NetBIOS')
wiki.add_edge('Programmierschnittstelle', 'Netzwerkprotokoll')
wiki.add_edge('Programmierschnittstelle', 'OLE DB')
wiki.add_edge('Programmierschnittstelle', 'OPENSTEP')
wiki.add_edge('Programmierschnittstelle', 'Oparl')
wiki.add_edge('Programmierschnittstelle', 'Open Banking')
wiki.add_edge('Programmierschnittstelle', 'Open Cirrus')
wiki.add_edge('Programmierschnittstelle', 'Open Database Connectivity')
wiki.add_edge('Programmierschnittstelle', 'Open Graphics Library for Embedded Systems')
wiki.add_edge('Programmierschnittstelle', 'OpenAL')
wiki.add_edge('Programmierschnittstelle', 'OpenAPI')
wiki.add_edge('Programmierschnittstelle', 'OpenCL')
wiki.add_edge('Programmierschnittstelle', 'OpenGL')
wiki.add_edge('Programmierschnittstelle', 'OpenGL Utility Toolkit')
wiki.add_edge('Programmierschnittstelle', 'OpenSocial')
wiki.add_edge('Programmierschnittstelle', 'OpenStep')
wiki.add_edge('Programmierschnittstelle', 'OpenWebRTC')
wiki.add_edge('Programmierschnittstelle', 'Parameter (Informatik)')
wiki.add_edge('Programmierschnittstelle', 'Parlay')
wiki.add_edge('Programmierschnittstelle', 'Phonon (KDE)')
wiki.add_edge('Programmierschnittstelle', 'Pipe (Informatik)')
wiki.add_edge('Programmierschnittstelle', 'Pluggable Authentication Modules')
wiki.add_edge('Programmierschnittstelle', 'Portable Operating System Interface')
wiki.add_edge('Programmierschnittstelle', 'PowerDesigner')
wiki.add_edge('Programmierschnittstelle', 'Production Performance Management Protocol')
wiki.add_edge('Programmierschnittstelle', 'Push-Model')
wiki.add_edge('Programmierschnittstelle', 'QXML DOM')
wiki.add_edge('Programmierschnittstelle', 'Real-Time Java')
wiki.add_edge('Programmierschnittstelle', 'Refactoring')
wiki.add_edge('Programmierschnittstelle', 'Remote Data Objects')
wiki.add_edge('Programmierschnittstelle', 'Retained Mode')
wiki.add_edge('Programmierschnittstelle', 'SIMON')
wiki.add_edge('Programmierschnittstelle', 'SMS-Gateway')
wiki.add_edge('Programmierschnittstelle', 'SOAP')
wiki.add_edge('Programmierschnittstelle', 'Scanner Access Now Easy')
wiki.add_edge('Programmierschnittstelle', 'Schablonenmethode')
wiki.add_edge('Programmierschnittstelle', 'Service Data Objects')
wiki.add_edge('Programmierschnittstelle', 'Service Delivery Platform')
wiki.add_edge('Programmierschnittstelle', 'Simple Mail Transfer Protocol')
wiki.add_edge('Programmierschnittstelle', 'Socket (Software)')
wiki.add_edge('Programmierschnittstelle', 'Stream Control Transmission Protocol')
wiki.add_edge('Programmierschnittstelle', 'Swagger (Software)')
wiki.add_edge('Programmierschnittstelle', 'Telegram')
wiki.add_edge('Programmierschnittstelle', 'Telephony Application Programming Interface')
wiki.add_edge('Programmierschnittstelle', 'Threema')
wiki.add_edge('Programmierschnittstelle', 'Typbibliothek')
wiki.add_edge('Programmierschnittstelle', 'UnAPI')
wiki.add_edge('Programmierschnittstelle', 'Vendor Independent Messaging')
wiki.add_edge('Programmierschnittstelle', 'Video Decode and Presentation API for Unix')
wiki.add_edge('Programmierschnittstelle', 'Video for Windows')
wiki.add_edge('Programmierschnittstelle', 'Virtual Studio Technology')
wiki.add_edge('Programmierschnittstelle', 'Vulkan (API)')
wiki.add_edge('Programmierschnittstelle', 'W3C Geolocation API')
wiki.add_edge('Programmierschnittstelle', 'Web Audio API')
wiki.add_edge('Programmierschnittstelle', 'Web Services Interoperability Technology')
wiki.add_edge('Programmierschnittstelle', 'Web Storage')
wiki.add_edge('Programmierschnittstelle', 'WebCL')
wiki.add_edge('Programmierschnittstelle', 'WebGL')
wiki.add_edge('Programmierschnittstelle', 'WebRTC')
wiki.add_edge('Programmierschnittstelle', 'WebVR')
wiki.add_edge('Programmierschnittstelle', 'Webintegration')
wiki.add_edge('Programmierschnittstelle', 'Webservice')
wiki.add_edge('Programmierschnittstelle', 'WinG')
wiki.add_edge('Programmierschnittstelle', 'Windows Application Programming Interface')
wiki.add_edge('Programmierschnittstelle', 'Windows ML')
wiki.add_edge('Programmierschnittstelle', 'Winsock')
wiki.add_edge('Programmierschnittstelle', 'X-Video Bitstream Acceleration')
wiki.add_edge('Programmierschnittstelle', 'XMLHttpRequest')
wiki.add_edge('Programmierschnittstelle', 'XNA (Microsoft)')
wiki.add_edge('Programmierschnittstelle', 'Zeroconf')
wiki.add_edge('Programmierschnittstelle', 'Zwischenablage')
wiki.add_edge('Programmpaket', 'IP Address Management')
wiki.add_edge('Programmpaket', 'Java Telephony API')
wiki.add_edge('Programmpaket', 'Keeping Emulation Environments Portable')
wiki.add_edge('Proof of Work', 'Chia (Kryptowährung)')
wiki.add_edge('Proof of Work', 'Dash (Kryptowährung)')
wiki.add_edge('Proof of Work', 'Ethereum')
wiki.add_edge('Proof of Work', 'Iota (Kommunikationsprotokoll)')
wiki.add_edge('Proof of Work', 'MobileCoin')
wiki.add_edge('Proof of Work', 'Monero')
wiki.add_edge('Proof of Work', 'Nano (Kryptowährung)')
wiki.add_edge('Proof of Work', 'Nxt (Kryptowährung)')
wiki.add_edge('Proof of Work', 'Steem')
wiki.add_edge('Propel', 'Creole (Framework)')
wiki.add_edge('Propel', 'PHP Data Objects')
wiki.add_edge('Prospero (Protokoll)', 'Asynchronous Reliable Delivery Protocol')
wiki.add_edge('Protocol Data Unit', 'Rail Safe Transport Application')
wiki.add_edge('Protocol Data Unit', 'Transport Packet')
wiki.add_edge('Protocol Independent Multicast', 'Distance Vector Multicast Routing Protocol')
wiki.add_edge('Protocol Independent Multicast', 'IEEE 802.1aq')
wiki.add_edge('Protocol Independent Multicast', 'MOSPF')
wiki.add_edge('Protokoll (IP)', '6in4')
wiki.add_edge('Protokoll (IP)', 'Encapsulating Security Payload')
wiki.add_edge('Protokoll (IP)', 'Generic Routing Encapsulation')
wiki.add_edge('Protokoll (IP)', 'Host Identity Protocol')
wiki.add_edge('Protokoll (IP)', 'ROHC')
wiki.add_edge('Protokoll (IP)', 'Resource Reservation Protocol')
wiki.add_edge('Protokoll (IP)', 'Virtual Router Redundancy Protocol')
wiki.add_edge('Protokollstapel', 'Apache Thrift')
wiki.add_edge('Protokollstapel', 'CardDAV')
wiki.add_edge('Protokollstapel', 'IPsec')
wiki.add_edge('Protokollstapel', 'Internetwork Datagram Protocol')
wiki.add_edge('Protokollstapel', 'KAME-Projekt')
wiki.add_edge('Protokollstapel', 'OpenWebRTC')
wiki.add_edge('Protokollstapel', 'SAE J1708')
wiki.add_edge('Protokollstapel', 'SAE J1939')
wiki.add_edge('Protokollstapel', 'Schichtenarchitektur')
wiki.add_edge('Protokollstapel', 'Secure Reliable Transport')
wiki.add_edge('Protokollstapel', 'Signalling System 7')
wiki.add_edge('Protokollstapel', 'Verhaltensmuster (Software)')
wiki.add_edge('Protokollstapel', 'Vermittlungsschicht')
wiki.add_edge('Protokollstapel', 'WiTECK')
wiki.add_edge('Prototyp (Entwurfsmuster)', 'Abstrakte Fabrik')
wiki.add_edge('Prototyp (Entwurfsmuster)', 'Assoziation (UML)')
wiki.add_edge('Prototyp (Entwurfsmuster)', 'Decorator')
wiki.add_edge('Prototyp (Entwurfsmuster)', 'Klassendiagramm')
wiki.add_edge('Prototyp (Entwurfsmuster)', 'Kompositum (Entwurfsmuster)')
wiki.add_edge('Prototyp (Entwurfsmuster)', 'Singleton (Entwurfsmuster)')
wiki.add_edge('Provider Aggregatable Address Space', 'Provider Independent Address Space')
wiki.add_edge('Pull-Model', 'Java API for XML Processing')
wiki.add_edge('Push-Model', 'Beobachter (Entwurfsmuster)')
wiki.add_edge('Push-Model', 'Common Object Request Broker Architecture')
wiki.add_edge('Push-Model', 'Java API for XML Processing')
wiki.add_edge('Push-Model', 'Pull-Model')
wiki.add_edge('Push-Model', 'Signal (Messenger)')
wiki.add_edge('Push-Model', 'Threema')
wiki.add_edge('Push-Model', 'Universally Unique Identifier')
wiki.add_edge('Q.931', 'H.225.0')
wiki.add_edge('QUIC', 'Explicit Congestion Notification')
wiki.add_edge('QUIC', 'Multipath TCP')
wiki.add_edge('Quantität (Analysemuster)', 'Measurement (Analysemuster)')
wiki.add_edge('Quantität (Analysemuster)', 'Observation (Analysemuster)')
wiki.add_edge('RIPE Network Coordination Centre', 'Border Gateway Protocol')
wiki.add_edge('RIPE Network Coordination Centre', 'Local Internet Registry')
wiki.add_edge('RIPE Network Coordination Centre', 'Provider Independent Address Space')
wiki.add_edge('RIPE Network Coordination Centre', 'Regional Internet Registry')
wiki.add_edge('RIPE Network Coordination Centre', 'Whois')
wiki.add_edge('RJ-Steckverbindung', 'Dante (Netzwerkprotokoll)')
wiki.add_edge('ROHC', 'Real-time Transport Protocol')
wiki.add_edge('Radio Link Control', 'ARQ-Protokoll')
wiki.add_edge('Radio Link Control', 'Packet Data Convergence Protocol')
wiki.add_edge('Rational Rose', 'StarUML')
wiki.add_edge('Rational Rose', 'UMLet')
wiki.add_edge('Real Time Messaging Protocol', 'Secure Reliable Transport')
wiki.add_edge('Real-Time Streaming Protocol', 'Media Resource Control Protocol')
wiki.add_edge('Real-Time Streaming Protocol', 'Real Data Transport')
wiki.add_edge('Real-Time Streaming Protocol', 'RealTime Control Protocol')
wiki.add_edge('Real-Time Streaming Protocol', 'Streaming-Protokoll')
wiki.add_edge('Real-Time Transport Protocol', 'H.323')
wiki.add_edge('Real-Time Transport Protocol', 'Liste der standardisierten Ports')
wiki.add_edge('Real-Time Transport Protocol', 'MIDI')
wiki.add_edge('Real-Time Transport Protocol', 'Media Resource Control Protocol')
wiki.add_edge('Real-Time Transport Protocol', 'Real Data Transport')
wiki.add_edge('Real-Time Transport Protocol', 'Real-Time Streaming Protocol')
wiki.add_edge('Real-Time Transport Protocol', 'Real-time Transport Protocol')
wiki.add_edge('Real-Time Transport Protocol', 'RealTime Control Protocol')
wiki.add_edge('Real-Time Transport Protocol', 'Secure Real-Time Transport Protocol')
wiki.add_edge('Real-Time Transport Protocol', 'Session Description Protocol')
wiki.add_edge('Real-Time Transport Protocol', 'Session Initiation Protocol')
wiki.add_edge('Real-Time Transport Protocol', 'WebRTC')
wiki.add_edge('Real-Time Transport Protocol', 'ZRTP')
wiki.add_edge('Refactoring', 'Big Ball of Mud')
wiki.add_edge('Refactoring', 'Code-Smell')
wiki.add_edge('Refactoring', 'Hibernate (Framework)')
wiki.add_edge('Refactoring', 'Kopplung (Softwareentwicklung)')
wiki.add_edge('Refactoring', 'Modul (Software)')
wiki.add_edge('Refactoring', 'Quelltextklon')
wiki.add_edge('Refactoring', 'ReSharper')
wiki.add_edge('Refactoring', 'Service-Locator')
wiki.add_edge('Refactoring', 'Softwaredesign')
wiki.add_edge('Refactoring', 'Spaghetticode')
wiki.add_edge('Referenzarchitektur', 'International Data Spaces')
wiki.add_edge('Referenzarchitektur', 'Meta-Architektur-Dokument')
wiki.add_edge('Regional Internet Registry', '6rd')
wiki.add_edge('Regional Internet Registry', 'American Registry for Internet Numbers')
wiki.add_edge('Regional Internet Registry', 'Local Internet Registry')
wiki.add_edge('Regional Internet Registry', 'Provider Aggregatable Address Space')
wiki.add_edge('Regional Internet Registry', 'Provider Independent Address Space')
wiki.add_edge('Regional Internet Registry', 'Whois')
wiki.add_edge('Registry (Entwurfsmuster)', 'Singleton (Entwurfsmuster)')
wiki.add_edge('Remote Authentication Dial-In User Service', 'Diameter (Protokoll)')
wiki.add_edge('Remote Authentication Dial-In User Service', 'Extensible Authentication Protocol')
wiki.add_edge('Remote Authentication Dial-In User Service', 'WPA2')
wiki.add_edge('Remote Desktop Protocol', 'Druckprotokoll')
wiki.add_edge('Remote Desktop Protocol', 'Independent Computing Architecture')
wiki.add_edge('Remote Desktop Protocol', 'Liste von TCP/IP-basierten Netzwerkdiensten')
wiki.add_edge('Remote Desktop Protocol', 'Peer Name Resolution Protocol')
wiki.add_edge('Remote Desktop Protocol', 'Remote Framebuffer Protocol')
wiki.add_edge('Remote Desktop Protocol', 'Streaming-Protokoll')
wiki.add_edge('Remote Function Call', 'Dynamic Information and Action Gateway')
wiki.add_edge('Remote Method Invocation', 'Burlap')
wiki.add_edge('Remote Method Invocation', 'Business Delegate')
wiki.add_edge('Remote Method Invocation', 'Hessian (Webprotokoll)')
wiki.add_edge('Remote Method Invocation', 'Jakarta Messaging')
wiki.add_edge('Remote Method Invocation', 'Java Naming and Directory Interface')
wiki.add_edge('Remote Method Invocation', 'Kopplung (Softwareentwicklung)')
wiki.add_edge('Remote Method Invocation', 'Remote Function Call')
wiki.add_edge('Remote Method Invocation', 'SIMON')
wiki.add_edge('Remote Procedure Call', 'Burlap')
wiki.add_edge('Remote Procedure Call', 'Common Object Request Broker Architecture')
wiki.add_edge('Remote Procedure Call', 'Component Object Model')
wiki.add_edge('Remote Procedure Call', 'External Data Representation')
wiki.add_edge('Remote Procedure Call', 'Hessian (Webprotokoll)')
wiki.add_edge('Remote Procedure Call', 'Internet Communications Engine')
wiki.add_edge('Remote Procedure Call', 'Internet Content Adaptation Protocol')
wiki.add_edge('Remote Procedure Call', 'JSON-RPC')
wiki.add_edge('Remote Procedure Call', 'Jakarta XML RPC')
wiki.add_edge('Remote Procedure Call', 'Kopplung (Softwareentwicklung)')
wiki.add_edge('Remote Procedure Call', 'Liste der standardisierten Ports')
wiki.add_edge('Remote Procedure Call', 'Network File System')
wiki.add_edge('Remote Procedure Call', 'OpenSocial')
wiki.add_edge('Remote Procedure Call', 'Pipe (Informatik)')
wiki.add_edge('Remote Procedure Call', 'Profinet')
wiki.add_edge('Remote Procedure Call', 'Protokoll (IP)')
wiki.add_edge('Remote Procedure Call', 'Remote Function Call')
wiki.add_edge('Remote Procedure Call', 'Remote Method Invocation')
wiki.add_edge('Remote Procedure Call', 'SIMON')
wiki.add_edge('Remote Procedure Call', 'XML-RPC')
wiki.add_edge('Remote Shell', 'FISH (Protokoll)')
wiki.add_edge('Remote Shell', 'Liste der standardisierten Ports')
wiki.add_edge('Remote Shell', 'Remote login')
wiki.add_edge('Remote Shell', 'Rsync')
wiki.add_edge('Repository (Entwurfsmuster)', 'ADO.NET')
wiki.add_edge('Representational State Transfer', 'CRUD')
wiki.add_edge('Representational State Transfer', 'Constrained Application Protocol')
wiki.add_edge('Representational State Transfer', 'HTTP-Authentifizierung')
wiki.add_edge('Representational State Transfer', 'Humane Registry')
wiki.add_edge('Representational State Transfer', 'JSON-RPC')
wiki.add_edge('Representational State Transfer', 'Jakarta RESTful Web Services')
wiki.add_edge('Representational State Transfer', 'Jakarta XML RPC')
wiki.add_edge('Representational State Transfer', 'Lightning-Netzwerk')
wiki.add_edge('Representational State Transfer', 'Liste der standardisierten Ports')
wiki.add_edge('Representational State Transfer', 'Mashup (Internet)')
wiki.add_edge('Representational State Transfer', 'Matrix (Kommunikationsprotokoll)')
wiki.add_edge('Representational State Transfer', 'Microservices')
wiki.add_edge('Representational State Transfer', 'Oparl')
wiki.add_edge('Representational State Transfer', 'Open Data Protocol')
wiki.add_edge('Representational State Transfer', 'OpenAPI')
wiki.add_edge('Representational State Transfer', 'Remote Function Call')
wiki.add_edge('Representational State Transfer', 'Remote Procedure Call')
wiki.add_edge('Representational State Transfer', 'SOAP')
wiki.add_edge('Representational State Transfer', 'Serviceorientierte Architektur')
wiki.add_edge('Representational State Transfer', 'Softwarearchitektur')
wiki.add_edge('Representational State Transfer', 'UnAPI')
wiki.add_edge('Representational State Transfer', 'WS-Security')
wiki.add_edge('Representational State Transfer', 'Web Application Description Language')
wiki.add_edge('Representational State Transfer', 'Web Services Description Language')
wiki.add_edge('Representational State Transfer', 'WebHooks')
wiki.add_edge('Representational State Transfer', 'Webservice')
wiki.add_edge('Representational State Transfer', 'XML-RPC')
wiki.add_edge('Resource Reservation Protocol', 'Common Open Policy Service')
wiki.add_edge('Resource Reservation Protocol', 'IntServ')
wiki.add_edge('Resource Reservation Protocol', 'RealTime Control Protocol')
wiki.add_edge('Retained Mode', 'Immediate mode (Computergrafik)')
wiki.add_edge('RetroShare', 'Tox (Protokoll)')
wiki.add_edge('Reverse Address Resolution Protocol', 'IEEE 802.1X')
wiki.add_edge('Ripple (Geldsystem)', 'Dash (Kryptowährung)')
wiki.add_edge('Ripple (Geldsystem)', 'Stellar (Bezahlsystem)')
wiki.add_edge('Robots Exclusion Standard', 'HTTP-Authentifizierung')
wiki.add_edge('Routing Information Protocol', '6LoWPAN')
wiki.add_edge('Routing Information Protocol', 'Bidirectional Forwarding Detection')
wiki.add_edge('Routing Information Protocol', 'Border Gateway Protocol')
wiki.add_edge('Routing Information Protocol', 'Distance Vector Multicast Routing Protocol')
wiki.add_edge('Routing Information Protocol', 'Distanzvektoralgorithmus')
wiki.add_edge('Routing Information Protocol', 'Enhanced Interior Gateway Routing Protocol')
wiki.add_edge('Routing Information Protocol', 'Exterior-Gateway-Protokoll')
wiki.add_edge('Routing Information Protocol', 'IS-IS')
wiki.add_edge('Routing Information Protocol', 'Interior Gateway Protocol')
wiki.add_edge('Routing Information Protocol', 'Interior Gateway Routing Protocol')
wiki.add_edge('Routing Information Protocol', 'Link-State')
wiki.add_edge('Routing Information Protocol', 'NetWare Link-Services Protocol')
wiki.add_edge('Routing Information Protocol', 'Open Shortest Path First')
wiki.add_edge('Routing Information Protocol', 'Protocol Independent Multicast')
wiki.add_edge('Routing Information Protocol', 'Routing Protocol for Low power and Lossy Networks')
wiki.add_edge('Routing Information Protocol', 'Triggered Updates')
wiki.add_edge('Routing Table Maintenance Protocol', 'Datagram Delivery Protocol')
wiki.add_edge('Routing Table Maintenance Protocol', 'EtherTalk Link Access Protocol')
wiki.add_edge('Routing Table Maintenance Protocol', 'LocalTalk')
wiki.add_edge('Routing Table Maintenance Protocol', 'LocalTalk Link Access Protocol')
wiki.add_edge('Routing Table Maintenance Protocol', 'Routing Information Protocol')
wiki.add_edge('Routing Table Maintenance Protocol', 'TokenTalk Link Access Protocol')
wiki.add_edge('Routing Table Maintenance Protocol', 'Transportschicht')
wiki.add_edge('S-HTTP', 'Tunnel (Rechnernetz)')
wiki.add_edge('SAAM', 'Bewertung von Softwarearchitekturen')
wiki.add_edge('SAAM', 'Szenariobasierte Architekturbewertung')
wiki.add_edge('SAE J1939', 'SAE J1708')
wiki.add_edge('SERCOS III', 'OpenSAFETY')
wiki.add_edge('SILC', 'Cocoa (API)')
wiki.add_edge('SIMON', 'Apache MINA')
wiki.add_edge('SMS-Gateway', 'Threema')
wiki.add_edge('SMTP-Auth', 'CRAM-MD5')
wiki.add_edge('SMTP-Auth', 'NTLM')
wiki.add_edge('SMTP-Auth', 'On-Demand Mail Relay')
wiki.add_edge('SMTPS', 'Transportschicht')
wiki.add_edge('SOAP', 'Apache Axis')
wiki.add_edge('SOAP', 'Apache CXF')
wiki.add_edge('SOAP', 'Burlap')
wiki.add_edge('SOAP', 'Common Object Request Broker Architecture')
wiki.add_edge('SOAP', 'Dienstekomposition')
wiki.add_edge('SOAP', 'Direct Internet Message Encapsulation')
wiki.add_edge('SOAP', 'Enterprise Application Integration')
wiki.add_edge('SOAP', 'Etch (Protokoll)')
wiki.add_edge('SOAP', 'File Transfer, Access and Management')
wiki.add_edge('SOAP', 'Framework')
wiki.add_edge('SOAP', 'GroupDAV')
wiki.add_edge('SOAP', 'Hessian (Webprotokoll)')
wiki.add_edge('SOAP', 'IGD-Protokoll')
wiki.add_edge('SOAP', 'Internet Communications Engine')
wiki.add_edge('SOAP', 'Jakarta XML RPC')
wiki.add_edge('SOAP', 'Jakarta XML Web Services')
wiki.add_edge('SOAP', 'Komponente (Software)')
wiki.add_edge('SOAP', 'Komponentenbasierte Entwicklung')
wiki.add_edge('SOAP', 'Kopplung (Softwareentwicklung)')
wiki.add_edge('SOAP', 'Liste der standardisierten Ports')
wiki.add_edge('SOAP', 'Lose Kopplung')
wiki.add_edge('SOAP', 'Mashup (Internet)')
wiki.add_edge('SOAP', 'Message Oriented Middleware')
wiki.add_edge('SOAP', 'OPC Unified Architecture')
wiki.add_edge('SOAP', 'Open Building Information Exchange')
wiki.add_edge('SOAP', 'Remote Procedure Call')
wiki.add_edge('SOAP', 'SOAP Message Transmission Optimization Mechanism')
wiki.add_edge('SOAP', 'SOAP with Attachments')
wiki.add_edge('SOAP', 'Schichtenarchitektur')
wiki.add_edge('SOAP', 'Serviceorientierte Architektur')
wiki.add_edge('SOAP', 'SoapUI')
wiki.add_edge('SOAP', 'Swagger (Software)')
wiki.add_edge('SOAP', 'TR-069')
wiki.add_edge('SOAP', 'Universal Description, Discovery and Integration')
wiki.add_edge('SOAP', 'Universal Plug and Play')
wiki.add_edge('SOAP', 'WS-*')
wiki.add_edge('SOAP', 'WS-Addressing')
wiki.add_edge('SOAP', 'WS-Referral')
wiki.add_edge('SOAP', 'WS-Reliability')
wiki.add_edge('SOAP', 'WS-Reliable Messaging')
wiki.add_edge('SOAP', 'WS-Routing')
wiki.add_edge('SOAP', 'WS-Security')
wiki.add_edge('SOAP', 'WS-Security UsernameToken Profile')
wiki.add_edge('SOAP', 'Web Application Description Language')
wiki.add_edge('SOAP', 'Web Map Tile Service')
wiki.add_edge('SOAP', 'Web Services Description Language')
wiki.add_edge('SOAP', 'Web Services Interoperability Technology')
wiki.add_edge('SOAP', 'WebHooks')
wiki.add_edge('SOAP', 'XFire (Framework)')
wiki.add_edge('SOAP', 'XL (XML-Programmiersprache)')
wiki.add_edge('SOAP', 'XML-RPC')
wiki.add_edge('SOAP Message Transmission Optimization Mechanism', 'Direct Internet Message Encapsulation')
wiki.add_edge('SOAP Message Transmission Optimization Mechanism', 'SOAP with Attachments')
wiki.add_edge('SOAP Message Transmission Optimization Mechanism', 'Web Services Interoperability Technology')
wiki.add_edge('SOAP with Attachments', 'Direct Internet Message Encapsulation')
wiki.add_edge('SOCKS', 'Socket (Software)')
wiki.add_edge('SPDY', 'QUIC')
wiki.add_edge('SPDY', 'TCP Fast Open')
wiki.add_edge('SQLAlchemy', 'Mapper (Softwareentwicklung)')
wiki.add_edge('SSH File Transfer Protocol', 'CAcert')
wiki.add_edge('SSH File Transfer Protocol', 'Connect Direct')
wiki.add_edge('SSH File Transfer Protocol', 'Gnome Virtual File System')
wiki.add_edge('SSH File Transfer Protocol', 'Liste der standardisierten Ports')
wiki.add_edge('SSH File Transfer Protocol', 'Liste von TCP/IP-basierten Netzwerkdiensten')
wiki.add_edge('SSH File Transfer Protocol', 'Secure Copy')
wiki.add_edge('SSH File Transfer Protocol', 'Secure Shell')
wiki.add_edge('SSH File Transfer Protocol', 'Simple File Transfer Protocol')
wiki.add_edge('SSH File Transfer Protocol', 'WebDAV')
wiki.add_edge('STARTTLS', 'Authenticated Post Office Protocol')
wiki.add_edge('STARTTLS', 'Deep Packet Inspection')
wiki.add_edge('STARTTLS', 'Domain Name System Security Extensions')
wiki.add_edge('STARTTLS', 'Network News Transfer Protocol')
wiki.add_edge('STARTTLS', 'POP3S')
wiki.add_edge('STARTTLS', 'SMTP-Auth')
wiki.add_edge('STARTTLS', 'SMTPS')
wiki.add_edge('SYN-Cookies', 'TCP Fast Open')
wiki.add_edge('Scanner Access Now Easy', 'Liste der standardisierten Ports')
wiki.add_edge('Scanner Access Now Easy', 'Video4Linux')
wiki.add_edge('Schablonenmethode', 'Strategie (Entwurfsmuster)')
wiki.add_edge('Schablonenmethode', 'Zuständigkeitskette')
wiki.add_edge('Schema (Informatik)', 'ADO.NET')
wiki.add_edge('Schema (Informatik)', 'ANSI-SPARC-Architektur')
wiki.add_edge('Schema (Informatik)', 'Data-Dictionary')
wiki.add_edge('Schema (Informatik)', 'Datenbankdesign')
wiki.add_edge('Schema (Informatik)', 'Global-as-View')
wiki.add_edge('Schema (Informatik)', 'Innovator (Software)')
wiki.add_edge('Schema (Informatik)', 'Jakarta Enterprise Beans')
wiki.add_edge('Schema (Informatik)', 'Local-as-View')
wiki.add_edge('Schema (Informatik)', 'Normalisierung (Datenbank)')
wiki.add_edge('Schema (Informatik)', 'Object-relational impedance mismatch')
wiki.add_edge('Schema (Informatik)', 'Schematransformation und -integration')
wiki.add_edge('Schema (Informatik)', 'Sternschema')
wiki.add_edge('Schema (Informatik)', 'XL (XML-Programmiersprache)')
wiki.add_edge('Schematransformation und -integration', 'Dataspaces')
wiki.add_edge('Schichtenarchitektur', 'Anti-Pattern')
wiki.add_edge('Schichtenarchitektur', 'Anwendungsfassade')
wiki.add_edge('Schichtenarchitektur', 'Anwendungsserver')
wiki.add_edge('Schichtenarchitektur', 'Application Response Measurement')
wiki.add_edge('Schichtenarchitektur', 'Architekturmuster')
wiki.add_edge('Schichtenarchitektur', 'CANaerospace')
wiki.add_edge('Schichtenarchitektur', 'Datenbankabstraktionsschicht')
wiki.add_edge('Schichtenarchitektur', 'Datenbankschnittstelle')
wiki.add_edge('Schichtenarchitektur', 'Delegation (Softwareentwicklung)')
wiki.add_edge('Schichtenarchitektur', 'Geschäftslogik')
wiki.add_edge('Schichtenarchitektur', 'Java Authentication and Authorization Service')
wiki.add_edge('Schichtenarchitektur', 'Komponente (Software)')
wiki.add_edge('Schichtenarchitektur', 'Kopplung (Softwareentwicklung)')
wiki.add_edge('Schichtenarchitektur', 'Logical Link Control')
wiki.add_edge('Schichtenarchitektur', 'Lose Kopplung')
wiki.add_edge('Schichtenarchitektur', 'Model View Controller')
wiki.add_edge('Schichtenarchitektur', 'Model View ViewModel')
wiki.add_edge('Schichtenarchitektur', 'Portable Operating System Interface')
wiki.add_edge('Schichtenarchitektur', 'Registry (Entwurfsmuster)')
wiki.add_edge('Schichtenarchitektur', 'Remote Method Invocation')
wiki.add_edge('Schichtenarchitektur', 'Service-Locator')
wiki.add_edge('Schichtenarchitektur', 'Serviceorientierte Architektur')
wiki.add_edge('Schichtenarchitektur', 'Smart UI')
wiki.add_edge('Schichtenarchitektur', 'Softwarearchitektur')
wiki.add_edge('Schichtenarchitektur', 'Swing (Java)')
wiki.add_edge('Schichtenarchitektur', 'Verteilte Anwendung')
wiki.add_edge('Schneeflockenschema', 'Aggregation (Informatik)')
wiki.add_edge('Schneeflockenschema', 'Galaxy-Schema')
wiki.add_edge('Schneeflockenschema', 'Slowly Changing Dimensions')
wiki.add_edge('Schnittstelle (UML)', 'Abhängigkeitsbeziehung')
wiki.add_edge('Schnittstelle (UML)', 'Attribut (UML)')
wiki.add_edge('Schnittstelle (UML)', 'Classifier (UML)')
wiki.add_edge('Schnittstelle (UML)', 'Generalisierung (UML)')
wiki.add_edge('Schnittstelle (UML)', 'Klassendiagramm')
wiki.add_edge('Schnittstelle (UML)', 'Komponente (UML)')
wiki.add_edge('Schnittstelle (UML)', 'Komponentendiagramm')
wiki.add_edge('Schnittstelle (UML)', 'Kompositionsstrukturdiagramm')
wiki.add_edge('Schnittstelle (UML)', 'Konnektor (UML)')
wiki.add_edge('Schnittstelle (UML)', 'Operation (UML)')
wiki.add_edge('Schnittstelle (UML)', 'Port (UML)')
wiki.add_edge('Schnittstelle (UML)', 'Signalempfänger (UML)')
wiki.add_edge('Schwache Entität', 'IDEF1X')
wiki.add_edge('Secure Copy', 'Certificate Management Protocol')
wiki.add_edge('Secure Copy', 'Connect Direct')
wiki.add_edge('Secure Real-Time Transport Protocol', 'WebRTC')
wiki.add_edge('Secure Real-Time Transport Protocol', 'Wire (Messenger)')
wiki.add_edge('Secure Real-Time Transport Protocol', 'ZRTP')
wiki.add_edge('Secure Shell', 'ARP-Spoofing')
wiki.add_edge('Secure Shell', 'FISH (Protokoll)')
wiki.add_edge('Secure Shell', 'Fast and Secure Protocol')
wiki.add_edge('Secure Shell', 'GSSAPI')
wiki.add_edge('Secure Shell', 'Liste der standardisierten Ports')
wiki.add_edge('Secure Shell', 'Liste von TCP/IP-basierten Netzwerkdiensten')
wiki.add_edge('Secure Shell', 'Nagle-Algorithmus')
wiki.add_edge('Secure Shell', 'Open Platform Communications')
wiki.add_edge('Secure Shell', 'Pipe (Informatik)')
wiki.add_edge('Secure Shell', 'Portweiterleitung')
wiki.add_edge('Secure Shell', 'Remote Shell')
wiki.add_edge('Secure Shell', 'Remote login')
wiki.add_edge('Secure Shell', 'Rsync')
wiki.add_edge('Secure Shell', 'SOCKS')
wiki.add_edge('Secure Shell', 'Scanner Access Now Easy')
wiki.add_edge('Secure Shell', 'Secure Copy')
wiki.add_edge('Secure Shell', 'Simple Network Management Protocol')
wiki.add_edge('Secure Shell', 'Stream Control Transmission Protocol')
wiki.add_edge('Secure Shell', 'Telnet')
wiki.add_edge('Secure Shell', 'Tunnel (Rechnernetz)')
wiki.add_edge('Secure Shell', 'Verschlüsselungsprotokoll')
wiki.add_edge('Secure Shell', 'Virtual Network Computing')
wiki.add_edge('Secure Shell', 'ZRTP')
wiki.add_edge('Security Assertion Markup Language', 'Java Authentication and Authorization Service')
wiki.add_edge('Security Assertion Markup Language', 'WS-Security')
wiki.add_edge('SegWit', 'Bitcoin Cash')
wiki.add_edge('SegWit', 'Lightning-Netzwerk')
wiki.add_edge('SegWit', 'Litecoin')
wiki.add_edge('SegWit', 'Particl (Kryptowährung)')
wiki.add_edge('SegWit', 'Proof of Work')
wiki.add_edge('Semantic Web Services', 'WS-Discovery')
wiki.add_edge('Semantic Web Services', 'Web Ontology Language for Web Services')
wiki.add_edge('Semantic Web Services', 'Web Service Modeling Ontology')
wiki.add_edge('Semantisches Datenmodell', 'Adapter (Entwurfsmuster)')
wiki.add_edge('Semantisches Datenmodell', 'Aggregation (Informatik)')
wiki.add_edge('Semantisches Datenmodell', 'Datenmodell')
wiki.add_edge('Semantisches Datenmodell', 'Datenmodellierung')
wiki.add_edge('Semantisches Datenmodell', 'Entität (Informatik)')
wiki.add_edge('Semantisches Datenmodell', 'Martin-Notation')
wiki.add_edge('Semantisches Datenmodell', 'Schema (Informatik)')
wiki.add_edge('Sensor Media Access Control', 'Berkeley Media Access Control')
wiki.add_edge('Sensor Media Access Control', 'Timeout Media Access Control')
wiki.add_edge('Sensor Media Access Control', 'Zebra Media Access Control')
wiki.add_edge('Sequenced Packet Exchange', 'Internetwork Datagram Protocol')
wiki.add_edge('Sequenced Packet Exchange', 'NetBEUI')
wiki.add_edge('Sequenced Packet Exchange', 'NetWare Core Protocol')
wiki.add_edge('Sequenzdiagramm', '4+1 Sichtenmodell')
wiki.add_edge('Sequenzdiagramm', 'Ereignisgesteuerte Architektur')
wiki.add_edge('Sequenzdiagramm', 'Funktionsbaum')
wiki.add_edge('Sequenzdiagramm', 'Innovator (Software)')
wiki.add_edge('Sequenzdiagramm', 'Interaktionsübersichtsdiagramm')
wiki.add_edge('Sequenzdiagramm', 'Kommunikationsdiagramm (UML)')
wiki.add_edge('Sequenzdiagramm', 'Proactor')
wiki.add_edge('Sequenzdiagramm', 'UMLet')
wiki.add_edge('Sequenzdiagramm', 'Violet (UML-Werkzeug)')
wiki.add_edge('Sequenznummer', 'ARQ-Protokoll')
wiki.add_edge('Sequenznummer', 'BattleEye RCon Protocol')
wiki.add_edge('Sequenznummer', 'SACK')
wiki.add_edge('Sequenznummer', 'SYN-Cookies')
wiki.add_edge('Sequenznummer', 'TCP Sequence Prediction')
wiki.add_edge('Sercos III', 'SERCOS III')
wiki.add_edge('Serial Line Internet Protocol', 'CSLIP')
wiki.add_edge('Serial Line Internet Protocol', 'Data Link Control')
wiki.add_edge('Serial Line Internet Protocol', 'IP-Spoofing')
wiki.add_edge('Serial Line Internet Protocol', 'Layer 2 Forwarding')
wiki.add_edge('Serial Line Internet Protocol', 'VTun')
wiki.add_edge('Server Message Block', 'Druckprotokoll')
wiki.add_edge('Server Message Block', 'Gnome Virtual File System')
wiki.add_edge('Server Message Block', 'Link-local Multicast Name Resolution')
wiki.add_edge('Server Message Block', 'NetBIOS')
wiki.add_edge('Server Message Block', 'NetBIOS over TCP/IP')
wiki.add_edge('Server Message Block', 'Network File System')
wiki.add_edge('Server Message Block', 'Pipe (Informatik)')
wiki.add_edge('Server Name Indication', 'DNS over HTTPS')
wiki.add_edge('Server Name Indication', 'DNS over TLS')
wiki.add_edge('Server Name Indication', 'Domain Name System Security Extensions')
wiki.add_edge('Server Name Indication', 'Signal (Messenger)')
wiki.add_edge('Serverseitige Anwendung', 'ActiveX')
wiki.add_edge('Serverseitige Anwendung', 'BattleEye RCon Protocol')
wiki.add_edge('Service Advertising Protocol', 'Service Location Protocol')
wiki.add_edge('Service Component Architecture', 'Apache Tuscany')
wiki.add_edge('Service Component Architecture', 'Service Data Objects')
wiki.add_edge('Service Data Objects', 'Apache Tuscany')
wiki.add_edge('Service Data Objects', 'EclipseLink')
wiki.add_edge('Service Data Objects', 'Jakarta Persistence API')
wiki.add_edge('Service Data Unit', 'Datagramm')
wiki.add_edge('Service Data Unit', 'Dienst (Informatik)')
wiki.add_edge('Service Data Unit', 'Interface Control Information')
wiki.add_edge('Service Data Unit', 'Protocol Data Unit')
wiki.add_edge('Service Delivery Platform', 'Parlay')
wiki.add_edge('Service Delivery Platform', 'Service Logic Execution Environment')
wiki.add_edge('Service Location Protocol', 'Simple Service Discovery Protocol')
wiki.add_edge('Service Location Protocol', 'Streaming-Protokoll')
wiki.add_edge('Service Location Protocol', 'WS-Discovery')
wiki.add_edge('Service Logic Execution Environment', 'Java APIs for Integrated Networks')
wiki.add_edge('Serviceorientierte Architektur', 'Advanced Message Queuing Protocol')
wiki.add_edge('Serviceorientierte Architektur', 'Apache Axis')
wiki.add_edge('Serviceorientierte Architektur', 'Apache CXF')
wiki.add_edge('Serviceorientierte Architektur', 'Apache ServiceMix')
wiki.add_edge('Serviceorientierte Architektur', 'Apache Tuscany')
wiki.add_edge('Serviceorientierte Architektur', 'Autonomic Computing')
wiki.add_edge('Serviceorientierte Architektur', 'Cloud-native Computing')
wiki.add_edge('Serviceorientierte Architektur', 'Dienst (Informatik)')
wiki.add_edge('Serviceorientierte Architektur', 'Dienstekomposition')
wiki.add_edge('Serviceorientierte Architektur', 'Enterprise Application Integration')
wiki.add_edge('Serviceorientierte Architektur', 'Ereignisgesteuerte Architektur')
wiki.add_edge('Serviceorientierte Architektur', 'Etch (Protokoll)')
wiki.add_edge('Serviceorientierte Architektur', 'Innovator (Software)')
wiki.add_edge('Serviceorientierte Architektur', 'Jakarta Persistence API')
wiki.add_edge('Serviceorientierte Architektur', 'Java Business Integration')
wiki.add_edge('Serviceorientierte Architektur', 'Lose Kopplung')
wiki.add_edge('Serviceorientierte Architektur', 'Message Oriented Middleware')
wiki.add_edge('Serviceorientierte Architektur', 'Meta-Architektur-Dokument')
wiki.add_edge('Serviceorientierte Architektur', 'Microservices')
wiki.add_edge('Serviceorientierte Architektur', 'OPC Unified Architecture')
wiki.add_edge('Serviceorientierte Architektur', 'Remote Function Call')
wiki.add_edge('Serviceorientierte Architektur', 'Saga (Entwurfsmuster)')
wiki.add_edge('Serviceorientierte Architektur', 'Service Component Architecture')
wiki.add_edge('Serviceorientierte Architektur', 'Service Data Objects')
wiki.add_edge('Serviceorientierte Architektur', 'Services Computing')
wiki.add_edge('Serviceorientierte Architektur', 'SoaML')
wiki.add_edge('Serviceorientierte Architektur', 'SoapUI')
wiki.add_edge('Serviceorientierte Architektur', 'Softwarearchitekt')
wiki.add_edge('Serviceorientierte Architektur', 'Stereotyp (UML)')
wiki.add_edge('Serviceorientierte Architektur', 'TR-069')
wiki.add_edge('Serviceorientierte Architektur', 'Universal Description, Discovery and Integration')
wiki.add_edge('Serviceorientierte Architektur', 'Verteilte Anwendung')
wiki.add_edge('Serviceorientierte Architektur', 'WS-Agreement')
wiki.add_edge('Serviceorientierte Architektur', 'WS-Business Process Execution Language')
wiki.add_edge('Serviceorientierte Architektur', 'WS-Distributed Management')
wiki.add_edge('Serviceorientierte Architektur', 'XML-RPC')
wiki.add_edge('Session Announcement Protocol', 'Streaming-Protokoll')
wiki.add_edge('Session Description Protocol', 'Media Resource Control Protocol')
wiki.add_edge('Session Description Protocol', 'Session Announcement Protocol')
wiki.add_edge('Session Description Protocol', 'Streaming-Protokoll')
wiki.add_edge('Session Initiation Protocol', 'Common ISDN Application Programming Interface')
wiki.add_edge('Session Initiation Protocol', 'DiffServ')
wiki.add_edge('Session Initiation Protocol', 'H.323')
wiki.add_edge('Session Initiation Protocol', 'ISDN over IP')
wiki.add_edge('Session Initiation Protocol', 'InterAsterisk eXchange')
wiki.add_edge('Session Initiation Protocol', 'Interactive Connectivity Establishment')
wiki.add_edge('Session Initiation Protocol', 'Media Gateway Control Protocol (Protokoll)')
wiki.add_edge('Session Initiation Protocol', 'Media Resource Control Protocol')
wiki.add_edge('Session Initiation Protocol', 'Megaco')
wiki.add_edge('Session Initiation Protocol', 'Message Oriented Middleware')
wiki.add_edge('Session Initiation Protocol', 'Secure Real-Time Transport Protocol')
wiki.add_edge('Session Initiation Protocol', 'Session Announcement Protocol')
wiki.add_edge('Session Initiation Protocol', 'Session Description Protocol')
wiki.add_edge('Session Initiation Protocol', 'Session Traversal Utilities for NAT')
wiki.add_edge('Session Initiation Protocol', 'Skinny Client Control Protocol')
wiki.add_edge('Session Initiation Protocol', 'Skype')
wiki.add_edge('Session Initiation Protocol', 'Telepathy')
wiki.add_edge('Session Initiation Protocol', 'XIMSS')
wiki.add_edge('Session Initiation Protocol', 'ZRTP')
wiki.add_edge('Session Traversal Utilities for NAT', 'Threema')
wiki.add_edge('Session Traversal Utilities for NAT', 'WebRTC')
wiki.add_edge('Short Message Peer to Peer', 'SMS-Gateway')
wiki.add_edge('Sicherung (Entwurfsmuster)', 'Bulkhead')
wiki.add_edge('Sicherung (Entwurfsmuster)', 'Operations-Datenbank')
wiki.add_edge('Sicht (Datenbank)', 'Datenbankdesign')
wiki.add_edge('Sicht (Datenbank)', 'Microservices')
wiki.add_edge('Signal (Messenger)', 'MobileCoin')
wiki.add_edge('Signal (Messenger)', 'Privacy Coin')
wiki.add_edge('Signal-Protokoll', 'Signal (Messenger)')
wiki.add_edge('Signal-Protokoll', 'Silent Circle Instant Messaging Protocol')
wiki.add_edge('Signal-Protokoll', 'WhatsApp')
wiki.add_edge('Signal-Protokoll', 'Wire (Messenger)')
wiki.add_edge('Signalling System 7', 'Signalling Connection Control Part')
wiki.add_edge('Signalling System 7', 'Telegram')
wiki.add_edge('Signalling System 7', 'WhatsApp')
wiki.add_edge('Simple API for XML', 'Apache Axis')
wiki.add_edge('Simple API for XML', 'Jakarta XML Binding')
wiki.add_edge('Simple API for XML', 'Java API for XML Processing')
wiki.add_edge('Simple API for XML', 'MSXML')
wiki.add_edge('Simple API for XML', 'SOAP')
wiki.add_edge('Simple API for XML', 'Streaming API for XML')
wiki.add_edge('Simple Authentication and Security Layer', 'Authenticated Post Office Protocol')
wiki.add_edge('Simple Authentication and Security Layer', 'Blocks Extensible Exchange Protocol')
wiki.add_edge('Simple Authentication and Security Layer', 'CRAM-MD5')
wiki.add_edge('Simple Authentication and Security Layer', 'SMTP-Auth')
wiki.add_edge('Simple Authentication and Security Layer', 'SPICE (Protokoll)')
wiki.add_edge('Simple Common Gateway Interface', 'Anwendungsserver')
wiki.add_edge('Simple Mail Transfer Protocol', 'ETRN')
wiki.add_edge('Simple Mail Transfer Protocol', 'Extensible Messaging and Presence Protocol')
wiki.add_edge('Simple Mail Transfer Protocol', 'Extensible Provisioning Protocol')
wiki.add_edge('Simple Mail Transfer Protocol', 'File Transfer, Access and Management')
wiki.add_edge('Simple Mail Transfer Protocol', 'Ident')
wiki.add_edge('Simple Mail Transfer Protocol', 'Internet Message Access Protocol')
wiki.add_edge('Simple Mail Transfer Protocol', 'JSON Meta Application Protocol')
wiki.add_edge('Simple Mail Transfer Protocol', 'Jakarta Mail')
wiki.add_edge('Simple Mail Transfer Protocol', 'Keepalive')
wiki.add_edge('Simple Mail Transfer Protocol', 'Liste der standardisierten Ports')
wiki.add_edge('Simple Mail Transfer Protocol', 'Liste von TCP/IP-basierten Netzwerkdiensten')
wiki.add_edge('Simple Mail Transfer Protocol', 'Message Oriented Middleware')
wiki.add_edge('Simple Mail Transfer Protocol', 'Netzwerkprotokoll')
wiki.add_edge('Simple Mail Transfer Protocol', 'On-Demand Mail Relay')
wiki.add_edge('Simple Mail Transfer Protocol', 'Post Office Protocol')
wiki.add_edge('Simple Mail Transfer Protocol', 'Protocol for SYnchronous Conferencing')
wiki.add_edge('Simple Mail Transfer Protocol', 'Resource Reservation Protocol')
wiki.add_edge('Simple Mail Transfer Protocol', 'Ripple (Geldsystem)')
wiki.add_edge('Simple Mail Transfer Protocol', 'SMTP-Auth')
wiki.add_edge('Simple Mail Transfer Protocol', 'SMTPS')
wiki.add_edge('Simple Mail Transfer Protocol', 'STARTTLS')
wiki.add_edge('Simple Mail Transfer Protocol', 'Simple Authentication and Security Layer')
wiki.add_edge('Simple Mail Transfer Protocol', 'Simple Mail Access Protocol')
wiki.add_edge('Simple Mail Transfer Protocol', 'Telnet')
wiki.add_edge('Simple Mail Transfer Protocol', 'Transport Layer Security')
wiki.add_edge('Simple Mail Transfer Protocol', 'Unix to Unix Copy')
wiki.add_edge('Simple Mail Transfer Protocol', 'Webservice')
wiki.add_edge('Simple Network Management Protocol', 'ARP-Spoofing')
wiki.add_edge('Simple Network Management Protocol', 'Agent Extensibility Protocol')
wiki.add_edge('Simple Network Management Protocol', 'Border Gateway Protocol')
wiki.add_edge('Simple Network Management Protocol', 'Druckprotokoll')
wiki.add_edge('Simple Network Management Protocol', 'IP-Spoofing')
wiki.add_edge('Simple Network Management Protocol', 'Java Management Extensions')
wiki.add_edge('Simple Network Management Protocol', 'JetDirect')
wiki.add_edge('Simple Network Management Protocol', 'Netflow')
wiki.add_edge('Simple Network Management Protocol', 'Profinet')
wiki.add_edge('Simple Network Management Protocol', 'Protocol Data Unit')
wiki.add_edge('Simple Network Management Protocol', 'RMON')
wiki.add_edge('Singleton (Entwurfsmuster)', 'Initialization-on-demand holder idiom')
wiki.add_edge('Singleton (Entwurfsmuster)', 'Liste von Singleton-Implementierungen')
wiki.add_edge('Singleton (Entwurfsmuster)', 'Verzögerte Initialisierung')
wiki.add_edge('Skype', 'Deep Packet Inspection')
wiki.add_edge('Skype', 'Gadu-Gadu')
wiki.add_edge('Skype', 'H.323')
wiki.add_edge('Skype', 'Line (Instant Messaging)')
wiki.add_edge('Skype', 'Matrix (Kommunikationsprotokoll)')
wiki.add_edge('Skype', 'Session Traversal Utilities for NAT')
wiki.add_edge('Skype', 'Signal-Protokoll')
wiki.add_edge('Skype', 'Standalone')
wiki.add_edge('Skype', 'TeamSpeak')
wiki.add_edge('Skype', 'Telepathy')
wiki.add_edge('Skype', 'Tox (Protokoll)')
wiki.add_edge('Skype', 'Viber')
wiki.add_edge('Skype', 'Video4Linux')
wiki.add_edge('Skype', 'WebRTC')
wiki.add_edge('Skype', 'Wire (Messenger)')
wiki.add_edge('Slowly Changing Dimensions', 'Temporale Datenhaltung')
wiki.add_edge('Smart UI', 'Model View ViewModel')
wiki.add_edge('Smart UI', 'Spaghetticode')
wiki.add_edge('SoaML', 'Modelio')
wiki.add_edge('SoapUI', 'Swing (Java)')
wiki.add_edge('Socket (Software)', 'Dynamic Data Exchange')
wiki.add_edge('Socket (Software)', 'Jakarta Messaging')
wiki.add_edge('Socket (Software)', 'MMS-Protokoll')
wiki.add_edge('Socket (Software)', 'Noise Protocol Framework')
wiki.add_edge('Socket (Software)', 'On-Demand Mail Relay')
wiki.add_edge('Socket (Software)', 'Pipe (Informatik)')
wiki.add_edge('Socket (Software)', 'Sockets Direct Protocol')
wiki.add_edge('Socket (Software)', 'TIPC')
wiki.add_edge('Socket (Software)', 'Winsock')
wiki.add_edge('Softwarearchitekt', 'Bewertung von Softwarearchitekturen')
wiki.add_edge('Softwarearchitekt', 'C4 model')
wiki.add_edge('Softwarearchitekt', 'Innovator (Software)')
wiki.add_edge('Softwarearchitektur', '4+1 Sichtenmodell')
wiki.add_edge('Softwarearchitektur', 'Anti-Pattern')
wiki.add_edge('Softwarearchitektur', 'Architecture Description Language')
wiki.add_edge('Softwarearchitektur', 'Architekturmuster')
wiki.add_edge('Softwarearchitektur', 'Autonomic Computing')
wiki.add_edge('Softwarearchitektur', 'Bewertung von Softwarearchitekturen')
wiki.add_edge('Softwarearchitektur', 'Big Ball of Mud')
wiki.add_edge('Softwarearchitektur', 'C4 model')
wiki.add_edge('Softwarearchitektur', 'Dienst (Informatik)')
wiki.add_edge('Softwarearchitektur', 'Document Message')
wiki.add_edge('Softwarearchitektur', 'Ereignisgesteuerte Architektur')
wiki.add_edge('Softwarearchitektur', 'Extension Interface')
wiki.add_edge('Softwarearchitektur', 'Fundamental Modeling Concepts')
wiki.add_edge('Softwarearchitektur', 'IEEE 1471')
wiki.add_edge('Softwarearchitektur', 'International Data Spaces')
wiki.add_edge('Softwarearchitektur', 'Jakarta Connectors')
wiki.add_edge('Softwarearchitektur', 'Komponente (Software)')
wiki.add_edge('Softwarearchitektur', 'Kopplung (Softwareentwicklung)')
wiki.add_edge('Softwarearchitektur', 'Lose Kopplung')
wiki.add_edge('Softwarearchitektur', 'Message (Entwurfsmuster)')
wiki.add_edge('Softwarearchitektur', 'Modifizierbarkeit')
wiki.add_edge('Softwarearchitektur', 'Modul (Software)')
wiki.add_edge('Softwarearchitektur', 'Mustersprache')
wiki.add_edge('Softwarearchitektur', 'Parameter (Informatik)')
wiki.add_edge('Softwarearchitektur', 'Prinzipien objektorientierten Designs')
wiki.add_edge('Softwarearchitektur', 'Refactoring')
wiki.add_edge('Softwarearchitektur', 'Referenzarchitektur')
wiki.add_edge('Softwarearchitektur', 'SAAM')
wiki.add_edge('Softwarearchitektur', 'Serviceorientierte Architektur')
wiki.add_edge('Softwarearchitektur', 'Softwarearchitekt')
wiki.add_edge('Softwarearchitektur', 'Softwaredesign')
wiki.add_edge('Softwarearchitektur', 'Szenariobasierte Architekturbewertung')
wiki.add_edge('Softwarearchitektur', 'Unified Modeling Language')
wiki.add_edge('Softwarearchitektur', 'VIPER (Entwurfsmuster)')
wiki.add_edge('Softwaredesign', 'Modifizierbarkeit')
wiki.add_edge('Softwaredesign', 'Modul (Software)')
wiki.add_edge('Softwaredesign', 'Softwarearchitekt')
wiki.add_edge('Source-specific Multicast', 'Any-source Multicast')
wiki.add_edge('Spaghetticode', 'Big Ball of Mud')
wiki.add_edge('Spanning Tree Protocol', 'IEEE 802')
wiki.add_edge('Spanning Tree Protocol', 'IEEE 802.1Q')
wiki.add_edge('Spanning Tree Protocol', 'IEEE 802.1aq')
wiki.add_edge('Spanning Tree Protocol', 'PAgP')
wiki.add_edge('Spanning Tree Protocol', 'Rapid Spanning Tree Protocol')
wiki.add_edge('Spanning Tree Protocol', 'Split Multi-Link Trunking')
wiki.add_edge('Spanning Tree Protocol', 'Virtual Local Area Network')
wiki.add_edge('Stablecoin', 'Stellar (Bezahlsystem)')
wiki.add_edge('Stablecoin', 'Tether')
wiki.add_edge('Stellvertreter (Entwurfsmuster)', 'Adapter (Entwurfsmuster)')
wiki.add_edge('Stellvertreter (Entwurfsmuster)', 'Assoziation (UML)')
wiki.add_edge('Stellvertreter (Entwurfsmuster)', 'Delegation (Softwareentwicklung)')
wiki.add_edge('Stellvertreter (Entwurfsmuster)', 'Future (Programmierung)')
wiki.add_edge('Stellvertreter (Entwurfsmuster)', 'Memento (Entwurfsmuster)')
wiki.add_edge('Stellvertreter (Entwurfsmuster)', 'Remote Method Invocation')
wiki.add_edge('Stellvertreter (Entwurfsmuster)', 'Verzögerte Initialisierung')
wiki.add_edge('Stereotyp (UML)', 'Knoten (UML)')
wiki.add_edge('Stereotyp (UML)', 'Operation (UML)')
wiki.add_edge('Stereotyp (UML)', 'Profil (UML)')
wiki.add_edge('Stereotyp (UML)', 'SoaML')
wiki.add_edge('Sternschema', 'Aggregation (Informatik)')
wiki.add_edge('Sternschema', 'Data Vault')
wiki.add_edge('Sternschema', 'Galaxy-Schema')
wiki.add_edge('Sternschema', 'Schneeflockenschema')
wiki.add_edge('Sternschema', 'Slowly Changing Dimensions')
wiki.add_edge('Sternschema', 'Temporale Datenhaltung')
wiki.add_edge('Strategie (Entwurfsmuster)', 'Abstrakte Fabrik')
wiki.add_edge('Strategie (Entwurfsmuster)', 'Adapter (Entwurfsmuster)')
wiki.add_edge('Strategie (Entwurfsmuster)', 'Zustand (Entwurfsmuster)')
wiki.add_edge('Stream Control Transmission Protocol', 'Datagramm')
wiki.add_edge('Stream Control Transmission Protocol', 'Diameter (Protokoll)')
wiki.add_edge('Stream Control Transmission Protocol', 'Extensible Provisioning Protocol')
wiki.add_edge('Stream Control Transmission Protocol', 'Internet Protocol Flow Information Export')
wiki.add_edge('Stream Control Transmission Protocol', 'Megaco')
wiki.add_edge('Stream Control Transmission Protocol', 'Protokoll (IP)')
wiki.add_edge('Stream Control Transmission Protocol', 'SYN-Cookies')
wiki.add_edge('Stream Control Transmission Protocol', 'Signalling System 7')
wiki.add_edge('Stream Control Transmission Protocol', 'Streaming-Protokoll')
wiki.add_edge('Stream Control Transmission Protocol', 'Transportschicht')
wiki.add_edge('Streaming API for XML', 'Java API for XML Processing')
wiki.add_edge('Streaming-Protokoll', 'Digital Audio Access Protocol')
wiki.add_edge('Streaming-Protokoll', 'MMS-Protokoll')
wiki.add_edge('Streaming-Protokoll', 'Real Time Messaging Protocol')
wiki.add_edge('Streaming-Protokoll', 'Secure Reliable Transport')
wiki.add_edge('Streaming-Protokoll', 'WebRTC')
wiki.add_edge('Structured-Entity-Relationship-Modell', 'Innovator (Software)')
wiki.add_edge('Structured-Entity-Relationship-Modell', 'Min-Max-Notation')
wiki.add_edge('Structured-Entity-Relationship-Modell', 'Normalisierung (Datenbank)')
wiki.add_edge('Strukturmuster', 'Adapter (Entwurfsmuster)')
wiki.add_edge('Strukturmuster', 'Brücke (Entwurfsmuster)')
wiki.add_edge('Strukturmuster', 'Decorator')
wiki.add_edge('Strukturmuster', 'Entität (Informatik)')
wiki.add_edge('Strukturmuster', 'Fassade (Entwurfsmuster)')
wiki.add_edge('Strukturmuster', 'Fliegengewicht (Entwurfsmuster)')
wiki.add_edge('Strukturmuster', 'Kompositum (Entwurfsmuster)')
wiki.add_edge('Strukturmuster', 'Nebenläufigkeit (Entwurfsmuster)')
wiki.add_edge('Strukturmuster', 'Pipe (Informatik)')
wiki.add_edge('Strukturmuster', 'Stellvertreter (Entwurfsmuster)')
wiki.add_edge('Strukturmuster', 'Verhaltensmuster (Software)')
wiki.add_edge('Subnetz', 'Address Resolution Protocol')
wiki.add_edge('Subnetz', 'Classless Inter-Domain Routing')
wiki.add_edge('Subnetz', 'Common Address Redundancy Protocol')
wiki.add_edge('Subnetz', 'Internet Assigned Numbers Authority')
wiki.add_edge('Subnetz', 'Liste von IPv6-Tunnelbrokern')
wiki.add_edge('Subnetz', 'Mobile IP')
wiki.add_edge('Subnetz', 'Multiprotocol over ATM')
wiki.add_edge('Subnetz', 'Netzklasse')
wiki.add_edge('Subnetz', 'Netzmaske')
wiki.add_edge('Subnetz', 'Netzwerkadressübersetzung')
wiki.add_edge('Subnetz', 'Peg DHCP')
wiki.add_edge('Subnetz', 'Private IP-Adresse')
wiki.add_edge('Subnetz', 'Rogue DHCP')
wiki.add_edge('Subnetz', 'Routing Information Protocol')
wiki.add_edge('Subnetz', 'Variable Length Subnet Mask')
wiki.add_edge('Subnetz', 'Virtual Local Area Network')
wiki.add_edge('Subnetz', 'XDMCP')
wiki.add_edge('Swing (Java)', 'Abstract Window Toolkit')
wiki.add_edge('Swing (Java)', 'Jakarta Server Faces')
wiki.add_edge('Swing (Java)', 'Java Accessibility API')
wiki.add_edge('Swing (Java)', 'Java Foundation Classes')
wiki.add_edge('Swing (Java)', 'JavaFX')
wiki.add_edge('Synchronous Data Link Control', 'Binary Synchronous Communication')
wiki.add_edge('Synchronous Data Link Control', 'Data Link Control')
wiki.add_edge('Synchronous Data Link Control', 'Digital Data Communications Message Protocol')
wiki.add_edge('Synchronous Data Link Control', 'Liste der standardisierten Ports')
wiki.add_edge('Systems Modeling Language', 'Aktivitätsdiagramm')
wiki.add_edge('Systems Modeling Language', 'Anwendungsfalldiagramm')
wiki.add_edge('Systems Modeling Language', 'Enterprise Architect')
wiki.add_edge('Systems Modeling Language', 'Innovator (Software)')
wiki.add_edge('Systems Modeling Language', 'Modelio')
wiki.add_edge('Systems Modeling Language', 'Paketdiagramm')
wiki.add_edge('Systems Modeling Language', 'Papyrus (UML)')
wiki.add_edge('Systems Modeling Language', 'Rational Rhapsody')
wiki.add_edge('Systems Modeling Language', 'Sequenzdiagramm')
wiki.add_edge('Systems Modeling Language', 'TOPCASED')
wiki.add_edge('Systems Modeling Language', 'Visual Paradigm')
wiki.add_edge('Systems Modeling Language', 'Zustandsdiagramm (UML)')
wiki.add_edge('Szenariobasierte Architekturbewertung', 'Bewertung von Softwarearchitekturen')
wiki.add_edge('Szenariobasierte Architekturbewertung', 'IEEE 1471')
wiki.add_edge('T-Stück', 'EAD-Kabel')
wiki.add_edge('T.120', 'Streaming-Protokoll')
wiki.add_edge('TASE.2', 'IEC 60870')
wiki.add_edge('TCP Receive Window', 'Parameter (Informatik)')
wiki.add_edge('TCP Receive Window', 'SACK')
wiki.add_edge('TELNET', 'Pichat')
wiki.add_edge('TOGAF', 'ArchiMate')
wiki.add_edge('TR-069', 'TR-143')
wiki.add_edge('TR-069', 'TR-369')
wiki.add_edge('TWAIN', 'Image and Scanner Interface Specification')
wiki.add_edge('TWAIN', 'Scanner Access Now Easy')
wiki.add_edge('TeamSpeak', 'Mumble')
wiki.add_edge('TeamSpeak', 'Xfire')
wiki.add_edge('Telegram', 'Signal (Messenger)')
wiki.add_edge('Telegram', 'Viber')
wiki.add_edge('Telephone Number Mapping', 'Client')
wiki.add_edge('Telephone Number Mapping', 'RIPE Network Coordination Centre')
wiki.add_edge('Telephone Number Mapping', 'Session Initiation Protocol')
wiki.add_edge('Telnet', 'ASCII-Protokoll')
wiki.add_edge('Telnet', 'Daytime')
wiki.add_edge('Telnet', 'I2P')
wiki.add_edge('Telnet', 'JetDirect')
wiki.add_edge('Telnet', 'Liste der standardisierten Ports')
wiki.add_edge('Telnet', 'Liste von TCP/IP-basierten Netzwerkdiensten')
wiki.add_edge('Telnet', 'Nagle-Algorithmus')
wiki.add_edge('Telnet', 'Post Office Protocol')
wiki.add_edge('Telnet', 'Protocol for SYnchronous Conferencing')
wiki.add_edge('Telnet', 'Remote Shell')
wiki.add_edge('Telnet', 'Remote login')
wiki.add_edge('Telnet', 'TELNET')
wiki.add_edge('Telnet', 'TeamSpeak')
wiki.add_edge('Telnet', 'Tn3270')
wiki.add_edge('Telnet', 'Tunnel (Rechnernetz)')
wiki.add_edge('Telnet', 'Virtual Terminal')
wiki.add_edge('Temporale Datenhaltung', 'Data Vault')
wiki.add_edge('Teredo', 'DirectAccess')
wiki.add_edge('Teredo', 'ISATAP')
wiki.add_edge('Teredo', 'Peer Name Resolution Protocol')
wiki.add_edge('Tether', 'Stellar (Bezahlsystem)')
wiki.add_edge('Tether', 'Tron (Kryptowährung)')
wiki.add_edge('Threema', 'Signal (Messenger)')
wiki.add_edge('Threema', 'Surespot')
wiki.add_edge('Time-Sensitive Networking', 'Advanced Physical Layer')
wiki.add_edge('Time-Sensitive Networking', 'Parallel Redundancy Protocol')
wiki.add_edge('Timeout Media Access Control', 'Berkeley Media Access Control')
wiki.add_edge('Timeout Media Access Control', 'Zebra Media Access Control')
wiki.add_edge('Together (Software)', 'StarUML')
wiki.add_edge('Token Bus', 'Certificate Management Protocol')
wiki.add_edge('Token Bus', 'Data Link Control')
wiki.add_edge('Token Bus', 'EtherCAT')
wiki.add_edge('Token Bus', 'ICMPv6')
wiki.add_edge('Token Bus', 'IEEE 802')
wiki.add_edge('Token Bus', 'Internet Group Management Protocol')
wiki.add_edge('Token Bus', 'Internet Printing Protocol')
wiki.add_edge('Token Bus', 'NetBIOS over TCP/IP')
wiki.add_edge('Token Bus', 'PROFINET')
wiki.add_edge('Token Bus', 'Pfsync')
wiki.add_edge('Token Bus', 'SERCOS III')
wiki.add_edge('Token Bus', 'Secure Socket Tunneling Protocol')
wiki.add_edge('Token Bus', 'Server Message Block')
wiki.add_edge('Token Bus', 'Token Ring')
wiki.add_edge('Token Bus', 'Virtual Router Redundancy Protocol')
wiki.add_edge('Token Ring', 'Apple Filing Protocol')
wiki.add_edge('Token Ring', 'AppleTalk')
wiki.add_edge('Token Ring', 'Certificate Management Protocol')
wiki.add_edge('Token Ring', 'Data Link Control')
wiki.add_edge('Token Ring', 'Datagram Delivery Protocol')
wiki.add_edge('Token Ring', 'EchoRing')
wiki.add_edge('Token Ring', 'EtherTalk Link Access Protocol')
wiki.add_edge('Token Ring', 'Fiber Distributed Data Interface')
wiki.add_edge('Token Ring', 'Hot Standby Router Protocol')
wiki.add_edge('Token Ring', 'IEEE 802')
wiki.add_edge('Token Ring', 'IEEE 802.12')
wiki.add_edge('Token Ring', 'IEEE 802.1Q')
wiki.add_edge('Token Ring', 'Inter-Switch Link Protocol')
wiki.add_edge('Token Ring', 'Internet Group Management Protocol')
wiki.add_edge('Token Ring', 'Internet Printing Protocol')
wiki.add_edge('Token Ring', 'Internetwork Packet Exchange')
wiki.add_edge('Token Ring', 'Internetwork Packet eXchange Fragmentation')
wiki.add_edge('Token Ring', 'JetDirect')
wiki.add_edge('Token Ring', 'LocalTalk')
wiki.add_edge('Token Ring', 'LocalTalk Link Access Protocol')
wiki.add_edge('Token Ring', 'NetBIOS')
wiki.add_edge('Token Ring', 'NetBIOS over TCP/IP')
wiki.add_edge('Token Ring', 'NetWare Core Protocol')
wiki.add_edge('Token Ring', 'Network File System')
wiki.add_edge('Token Ring', 'Pfsync')
wiki.add_edge('Token Ring', 'RJ-Steckverbindung')
wiki.add_edge('Token Ring', 'Resource Reservation Protocol')
wiki.add_edge('Token Ring', 'Routing Table Maintenance Protocol')
wiki.add_edge('Token Ring', 'Secure Socket Tunneling Protocol')
wiki.add_edge('Token Ring', 'Sequenced Packet Exchange')
wiki.add_edge('Token Ring', 'Server Message Block')
wiki.add_edge('Token Ring', 'TokenTalk Link Access Protocol')
wiki.add_edge('Token Ring', 'Virtual Router Redundancy Protocol')
wiki.add_edge('Token Ring', 'Zone Information Protocol')
wiki.add_edge('TopLink', 'Jakarta Transactions API')
wiki.add_edge('Tox (Protokoll)', 'Briar (Instant Messenger)')
wiki.add_edge('Transferobjekt', 'Data Access Object')
wiki.add_edge('Transferobjekt', 'Jakarta Enterprise Beans')
wiki.add_edge('Transferobjekt', 'Klassendiagramm')
wiki.add_edge('Transferobjekt', 'Value Object')
wiki.add_edge('Transmission Control Protocol', '464XLAT')
wiki.add_edge('Transmission Control Protocol', 'ARQ-Protokoll')
wiki.add_edge('Transmission Control Protocol', 'ATA over Ethernet')
wiki.add_edge('Transmission Control Protocol', 'Advanced Physical Layer')
wiki.add_edge('Transmission Control Protocol', 'Agent Extensibility Protocol')
wiki.add_edge('Transmission Control Protocol', 'AppleTalk')
wiki.add_edge('Transmission Control Protocol', 'Asynchronous Reliable Delivery Protocol')
wiki.add_edge('Transmission Control Protocol', 'Berkeley Printing System')
wiki.add_edge('Transmission Control Protocol', 'Bitcoin')
wiki.add_edge('Transmission Control Protocol', 'Blocks Extensible Exchange Protocol')
wiki.add_edge('Transmission Control Protocol', 'Border Gateway Protocol')
wiki.add_edge('Transmission Control Protocol', 'CSLIP')
wiki.add_edge('Transmission Control Protocol', 'Certificate Management Protocol')
wiki.add_edge('Transmission Control Protocol', 'Client')
wiki.add_edge('Transmission Control Protocol', 'Common Open Policy Service')
wiki.add_edge('Transmission Control Protocol', 'Crypto IP Encapsulation')
wiki.add_edge('Transmission Control Protocol', 'DICT (Protokoll)')
wiki.add_edge('Transmission Control Protocol', 'DNS over TLS')
wiki.add_edge('Transmission Control Protocol', 'Datagram Congestion Control Protocol')
wiki.add_edge('Transmission Control Protocol', 'Datagramm')
wiki.add_edge('Transmission Control Protocol', 'Daytime')
wiki.add_edge('Transmission Control Protocol', 'Diameter (Protokoll)')
wiki.add_edge('Transmission Control Protocol', 'Digital Audio Access Protocol')
wiki.add_edge('Transmission Control Protocol', 'Discard')
wiki.add_edge('Transmission Control Protocol', 'Echo (Netzwerkdienst)')
wiki.add_edge('Transmission Control Protocol', 'EtherNet/IP')
wiki.add_edge('Transmission Control Protocol', 'Ethernet')
wiki.add_edge('Transmission Control Protocol', 'Ethernet Powerlink')
wiki.add_edge('Transmission Control Protocol', 'Explicit Congestion Notification')
wiki.add_edge('Transmission Control Protocol', 'Extensible Provisioning Protocol')
wiki.add_edge('Transmission Control Protocol', 'Fast and Secure Protocol')
wiki.add_edge('Transmission Control Protocol', 'FastCGI')
wiki.add_edge('Transmission Control Protocol', 'File Transfer Protocol')
wiki.add_edge('Transmission Control Protocol', 'Finger (Internetprotokoll)')
wiki.add_edge('Transmission Control Protocol', 'Gopher')
wiki.add_edge('Transmission Control Protocol', 'Hessian (Webprotokoll)')
wiki.add_edge('Transmission Control Protocol', 'Host Identity Protocol')
wiki.add_edge('Transmission Control Protocol', 'Hyper Text Caching Protocol')
wiki.add_edge('Transmission Control Protocol', 'HyperSCSI')
wiki.add_edge('Transmission Control Protocol', 'Hypertext Transfer Protocol Secure')
wiki.add_edge('Transmission Control Protocol', 'I2P')
wiki.add_edge('Transmission Control Protocol', 'ICMP-Tunnel')
wiki.add_edge('Transmission Control Protocol', 'IP-Fragmentierung')
wiki.add_edge('Transmission Control Protocol', 'IP-Paket')
wiki.add_edge('Transmission Control Protocol', 'IP-Spoofing')
wiki.add_edge('Transmission Control Protocol', 'ISCSI')
wiki.add_edge('Transmission Control Protocol', 'Ident')
wiki.add_edge('Transmission Control Protocol', 'Internet Assigned Numbers Authority')
wiki.add_edge('Transmission Control Protocol', 'Internet Printing Protocol')
wiki.add_edge('Transmission Control Protocol', 'Internet Protocol Flow Information Export')
wiki.add_edge('Transmission Control Protocol', 'Internet Relay Chat')
wiki.add_edge('Transmission Control Protocol', 'Internetwork Packet Exchange')
wiki.add_edge('Transmission Control Protocol', 'Internetwork Packet eXchange Fragmentation')
wiki.add_edge('Transmission Control Protocol', 'JSON Meta Application Protocol')
wiki.add_edge('Transmission Control Protocol', 'Jakarta Messaging')
wiki.add_edge('Transmission Control Protocol', 'JetDirect')
wiki.add_edge('Transmission Control Protocol', 'Jumbo Frames')
wiki.add_edge('Transmission Control Protocol', 'Keepalive')
wiki.add_edge('Transmission Control Protocol', 'Label Distribution Protocol')
wiki.add_edge('Transmission Control Protocol', 'Lightning-Netzwerk')
wiki.add_edge('Transmission Control Protocol', 'Lightweight Directory Access Protocol')
wiki.add_edge('Transmission Control Protocol', 'Link-local Multicast Name Resolution')
wiki.add_edge('Transmission Control Protocol', 'Liste der standardisierten Ports')
wiki.add_edge('Transmission Control Protocol', 'Liste von TCP/IP-basierten Netzwerkdiensten')
wiki.add_edge('Transmission Control Protocol', 'MMS-Protokoll')
wiki.add_edge('Transmission Control Protocol', 'MQTT')
wiki.add_edge('Transmission Control Protocol', 'Message Passing Interface')
wiki.add_edge('Transmission Control Protocol', 'NAT Port Mapping Protocol')
wiki.add_edge('Transmission Control Protocol', 'Nagle-Algorithmus')
wiki.add_edge('Transmission Control Protocol', 'Neighbor Discovery Protocol')
wiki.add_edge('Transmission Control Protocol', 'NetBIOS')
wiki.add_edge('Transmission Control Protocol', 'NetBIOS over TCP/IP')
wiki.add_edge('Transmission Control Protocol', 'Netfilter')
wiki.add_edge('Transmission Control Protocol', 'Netflow')
wiki.add_edge('Transmission Control Protocol', 'Network File System')
wiki.add_edge('Transmission Control Protocol', 'Network Time Protocol')
wiki.add_edge('Transmission Control Protocol', 'Netzwerkprotokoll')
wiki.add_edge('Transmission Control Protocol', 'OSCAR (Protokoll)')
wiki.add_edge('Transmission Control Protocol', 'OSI-Modell')
wiki.add_edge('Transmission Control Protocol', 'On-Demand Mail Relay')
wiki.add_edge('Transmission Control Protocol', 'POP3S')
wiki.add_edge('Transmission Control Protocol', 'Pi Digit Generation Protocol')
wiki.add_edge('Transmission Control Protocol', 'Pipe (Informatik)')
wiki.add_edge('Transmission Control Protocol', 'Point-to-Point Protocol')
wiki.add_edge('Transmission Control Protocol', 'Positive Acknowledgement with Retransmission')
wiki.add_edge('Transmission Control Protocol', 'Protocol for SYnchronous Conferencing')
wiki.add_edge('Transmission Control Protocol', 'Protokoll (IP)')
wiki.add_edge('Transmission Control Protocol', 'Protokollstapel')
wiki.add_edge('Transmission Control Protocol', 'QUIC')
wiki.add_edge('Transmission Control Protocol', 'ROHC')
wiki.add_edge('Transmission Control Protocol', 'Rail Safe Transport Application')
wiki.add_edge('Transmission Control Protocol', 'Real-Time Streaming Protocol')
wiki.add_edge('Transmission Control Protocol', 'Remote Desktop Protocol')
wiki.add_edge('Transmission Control Protocol', 'Remote Procedure Call')
wiki.add_edge('Transmission Control Protocol', 'Remote login')
wiki.add_edge('Transmission Control Protocol', 'Resource Reservation Protocol')
wiki.add_edge('Transmission Control Protocol', 'Reverse Connection')
wiki.add_edge('Transmission Control Protocol', 'Rsync')
wiki.add_edge('Transmission Control Protocol', 'SACK')
wiki.add_edge('Transmission Control Protocol', 'SIMON')
wiki.add_edge('Transmission Control Protocol', 'SMTPS')
wiki.add_edge('Transmission Control Protocol', 'SOAP')
wiki.add_edge('Transmission Control Protocol', 'SOCKS')
wiki.add_edge('Transmission Control Protocol', 'SPDY')
wiki.add_edge('Transmission Control Protocol', 'SYN-Cookies')
wiki.add_edge('Transmission Control Protocol', 'Secure Shell')
wiki.add_edge('Transmission Control Protocol', 'Secure Socket Tunneling Protocol')
wiki.add_edge('Transmission Control Protocol', 'Sequenced Packet Exchange')
wiki.add_edge('Transmission Control Protocol', 'Sequenznummer')
wiki.add_edge('Transmission Control Protocol', 'Serial Line Internet Protocol')
wiki.add_edge('Transmission Control Protocol', 'Server Message Block')
wiki.add_edge('Transmission Control Protocol', 'Serviceorientierte Architektur')
wiki.add_edge('Transmission Control Protocol', 'Simple File Transfer Protocol')
wiki.add_edge('Transmission Control Protocol', 'Skype')
wiki.add_edge('Transmission Control Protocol', 'Socket (Software)')
wiki.add_edge('Transmission Control Protocol', 'Sockets Direct Protocol')
wiki.add_edge('Transmission Control Protocol', 'Stream Control Transmission Protocol')
wiki.add_edge('Transmission Control Protocol', 'Streaming-Protokoll')
wiki.add_edge('Transmission Control Protocol', 'TCP Global Synchronization')
wiki.add_edge('Transmission Control Protocol', 'TCP Port Service Multiplexer')
wiki.add_edge('Transmission Control Protocol', 'TP/IX')
wiki.add_edge('Transmission Control Protocol', 'TR-143')
wiki.add_edge('Transmission Control Protocol', 'Telnet')
wiki.add_edge('Transmission Control Protocol', 'Train Real Time Data Protocol')
wiki.add_edge('Transmission Control Protocol', 'Transmission Control Protocol/Internet Protocol')
wiki.add_edge('Transmission Control Protocol', 'Transport Layer Security')
wiki.add_edge('Transmission Control Protocol', 'Transport Packet')
wiki.add_edge('Transmission Control Protocol', 'Trivial File Transfer Protocol')
wiki.add_edge('Transmission Control Protocol', 'Tunnel (Rechnernetz)')
wiki.add_edge('Transmission Control Protocol', 'Universal Measurement and Calibration Protocol')
wiki.add_edge('Transmission Control Protocol', 'Universal Plug and Play')
wiki.add_edge('Transmission Control Protocol', 'Virtual Local Area Network')
wiki.add_edge('Transmission Control Protocol', 'WPA2')
wiki.add_edge('Transmission Control Protocol', 'WebSocket')
wiki.add_edge('Transmission Control Protocol', 'Weihenstephaner Standards')
wiki.add_edge('Transmission Control Protocol', 'Whois')
wiki.add_edge('Transmission Control Protocol', 'Winsock')
wiki.add_edge('Transmission Control Protocol', 'Zeroconf')
wiki.add_edge('Transmission Control Protocol/Internet Protocol', 'DoIP')
wiki.add_edge('Transmission Control Protocol/Internet Protocol', 'IEC 61850')
wiki.add_edge('Transmission Control Protocol/Internet Protocol', 'Kinetics Internet Protocol')
wiki.add_edge('Transmission Control Protocol/Internet Protocol', 'NetBEUI')
wiki.add_edge('Transmission Control Protocol/Internet Protocol', 'Real Time Messaging Protocol')
wiki.add_edge('Transmission Control Protocol/Internet Protocol', 'SIMON')
wiki.add_edge('Transport Layer Security', 'ARP-Spoofing')
wiki.add_edge('Transport Layer Security', 'Apache MINA')
wiki.add_edge('Transport Layer Security', 'Blocks Extensible Exchange Protocol')
wiki.add_edge('Transport Layer Security', 'CAcert')
wiki.add_edge('Transport Layer Security', 'CRAM-MD5')
wiki.add_edge('Transport Layer Security', 'Connect Direct')
wiki.add_edge('Transport Layer Security', 'DNS over TLS')
wiki.add_edge('Transport Layer Security', 'DNSCurve')
wiki.add_edge('Transport Layer Security', 'Datenbank-Muster')
wiki.add_edge('Transport Layer Security', 'Diameter (Protokoll)')
wiki.add_edge('Transport Layer Security', 'Domain Name System Security Extensions')
wiki.add_edge('Transport Layer Security', 'Durchsatzratenbegrenzung')
wiki.add_edge('Transport Layer Security', 'Electronic Banking Internet Communication Standard')
wiki.add_edge('Transport Layer Security', 'Ethernet')
wiki.add_edge('Transport Layer Security', 'Extensible Authentication Protocol')
wiki.add_edge('Transport Layer Security', 'Extensible Messaging and Presence Protocol')
wiki.add_edge('Transport Layer Security', 'FREAK')
wiki.add_edge('Transport Layer Security', 'Fiber Distributed Data Interface')
wiki.add_edge('Transport Layer Security', 'File Transfer Protocol')
wiki.add_edge('Transport Layer Security', 'Gadu-Gadu')
wiki.add_edge('Transport Layer Security', 'Geminispace')
wiki.add_edge('Transport Layer Security', 'GnuTLS')
wiki.add_edge('Transport Layer Security', 'HTTPS Everywhere')
wiki.add_edge('Transport Layer Security', 'Heartbleed')
wiki.add_edge('Transport Layer Security', 'Hoccer')
wiki.add_edge('Transport Layer Security', 'Homebanking Computer Interface')
wiki.add_edge('Transport Layer Security', 'Hypertext Transfer Protocol Secure')
wiki.add_edge('Transport Layer Security', 'IEC 62351')
wiki.add_edge('Transport Layer Security', 'IEEE 802.1X')
wiki.add_edge('Transport Layer Security', 'IPsec')
wiki.add_edge('Transport Layer Security', 'Internet Message Access Protocol')
wiki.add_edge('Transport Layer Security', 'Internet Printing Protocol')
wiki.add_edge('Transport Layer Security', 'Internet Relay Chat')
wiki.add_edge('Transport Layer Security', 'Java Secure Socket Extension')
wiki.add_edge('Transport Layer Security', 'LibreSSL')
wiki.add_edge('Transport Layer Security', 'Lightweight Directory Access Protocol')
wiki.add_edge('Transport Layer Security', 'Liste der standardisierten Ports')
wiki.add_edge('Transport Layer Security', 'Liste von TCP/IP-basierten Netzwerkdiensten')
wiki.add_edge('Transport Layer Security', 'MQTT')
wiki.add_edge('Transport Layer Security', 'Mbed TLS')
wiki.add_edge('Transport Layer Security', 'Multi-purpose Business Security over IP')
wiki.add_edge('Transport Layer Security', 'Network File System')
wiki.add_edge('Transport Layer Security', 'Network News Transfer Protocol')
wiki.add_edge('Transport Layer Security', 'Network Time Protocol')
wiki.add_edge('Transport Layer Security', 'Noise Protocol Framework')
wiki.add_edge('Transport Layer Security', 'OSI-Modell')
wiki.add_edge('Transport Layer Security', 'Online Certificate Status Protocol stapling')
wiki.add_edge('Transport Layer Security', 'OpenSSL')
wiki.add_edge('Transport Layer Security', 'POP3S')
wiki.add_edge('Transport Layer Security', 'Point-to-Point Tunneling Protocol')
wiki.add_edge('Transport Layer Security', 'Post Office Protocol')
wiki.add_edge('Transport Layer Security', 'QUIC')
wiki.add_edge('Transport Layer Security', 'Rail Safe Transport Application')
wiki.add_edge('Transport Layer Security', 'Real Time Messaging Protocol')
wiki.add_edge('Transport Layer Security', 'Remote Desktop Protocol')
wiki.add_edge('Transport Layer Security', 'Representational State Transfer')
wiki.add_edge('Transport Layer Security', 'S-HTTP')
wiki.add_edge('Transport Layer Security', 'SMTP-Auth')
wiki.add_edge('Transport Layer Security', 'SMTPS')
wiki.add_edge('Transport Layer Security', 'SPDY')
wiki.add_edge('Transport Layer Security', 'SPICE (Protokoll)')
wiki.add_edge('Transport Layer Security', 'STARTTLS')
wiki.add_edge('Transport Layer Security', 'Secure Socket Tunneling Protocol')
wiki.add_edge('Transport Layer Security', 'Server Name Indication')
wiki.add_edge('Transport Layer Security', 'Serviceorientierte Architektur')
wiki.add_edge('Transport Layer Security', 'Session Initiation Protocol')
wiki.add_edge('Transport Layer Security', 'Signal (Messenger)')
wiki.add_edge('Transport Layer Security', 'Simple Authentication and Security Layer')
wiki.add_edge('Transport Layer Security', 'Syslog')
wiki.add_edge('Transport Layer Security', 'Threema')
wiki.add_edge('Transport Layer Security', 'Tn3270')
wiki.add_edge('Transport Layer Security', 'Token Bus')
wiki.add_edge('Transport Layer Security', 'Token Ring')
wiki.add_edge('Transport Layer Security', 'Transportschicht')
wiki.add_edge('Transport Layer Security', 'Tunnel (Rechnernetz)')
wiki.add_edge('Transport Layer Security', 'VTun')
wiki.add_edge('Transport Layer Security', 'Verschlüsselungsprotokoll')
wiki.add_edge('Transport Layer Security', 'Virtual Network Computing')
wiki.add_edge('Transport Layer Security', 'WS-SecureConversation')
wiki.add_edge('Transport Layer Security', 'WS-Security UsernameToken Profile')
wiki.add_edge('Transport Layer Security', 'Wireless Transport Layer Security')
wiki.add_edge('Transportschicht', 'Durchsatzratenbegrenzung')
wiki.add_edge('Transportschicht', 'Host Identity Protocol')
wiki.add_edge('Trivial File Transfer Protocol', 'ARQ-Protokoll')
wiki.add_edge('Trivial File Transfer Protocol', 'Bootstrap Protocol')
wiki.add_edge('Trivial File Transfer Protocol', 'EtherCAT')
wiki.add_edge('Trivial File Transfer Protocol', 'Liste der standardisierten Ports')
wiki.add_edge('Trivial File Transfer Protocol', 'Liste von TCP/IP-basierten Netzwerkdiensten')
wiki.add_edge('Trivial File Transfer Protocol', 'Network File System')
wiki.add_edge('Trivial File Transfer Protocol', 'Preboot Execution Environment')
wiki.add_edge('Trivial File Transfer Protocol', 'Reverse Address Resolution Protocol')
wiki.add_edge('Trivial File Transfer Protocol', 'Simple File Transfer Protocol')
wiki.add_edge('Trivial File Transfer Protocol', 'Unified Extensible Firmware Interface')
wiki.add_edge('Tunnel (Rechnernetz)', '4in6')
wiki.add_edge('Tunnel (Rechnernetz)', '6Bone')
wiki.add_edge('Tunnel (Rechnernetz)', 'CorNet')
wiki.add_edge('Tunnel (Rechnernetz)', 'Crypto IP Encapsulation')
wiki.add_edge('Tunnel (Rechnernetz)', 'Frame Relay')
wiki.add_edge('Tunnel (Rechnernetz)', 'GPRS Tunneling Protocol')
wiki.add_edge('Tunnel (Rechnernetz)', 'Generic Routing Encapsulation')
wiki.add_edge('Tunnel (Rechnernetz)', 'ICMP-Tunnel')
wiki.add_edge('Tunnel (Rechnernetz)', 'Kinetics Internet Protocol')
wiki.add_edge('Tunnel (Rechnernetz)', 'Layer 2 Tunneling Protocol')
wiki.add_edge('Tunnel (Rechnernetz)', 'Netfilter')
wiki.add_edge('Tunnel (Rechnernetz)', 'Real Time Messaging Protocol')
wiki.add_edge('Tunnel (Rechnernetz)', 'Remote Authentication Dial-In User Service')
wiki.add_edge('Tunnel (Rechnernetz)', 'RetroShare')
wiki.add_edge('Tunnel (Rechnernetz)', 'VTun')
wiki.add_edge('Typbibliothek', 'Component Object Model')
wiki.add_edge('UML-Werkzeug', 'ArgoUML')
wiki.add_edge('UML-Werkzeug', 'Artefakt (UML)')
wiki.add_edge('UML-Werkzeug', 'Classifier (UML)')
wiki.add_edge('UML-Werkzeug', 'Dia (Software)')
wiki.add_edge('UML-Werkzeug', 'Enterprise Architect')
wiki.add_edge('UML-Werkzeug', 'Fujaba')
wiki.add_edge('UML-Werkzeug', 'Gaphor')
wiki.add_edge('UML-Werkzeug', 'Innovator (Software)')
wiki.add_edge('UML-Werkzeug', 'Liste von Datenmodellierungswerkzeugen')
wiki.add_edge('UML-Werkzeug', 'MOF QVT')
wiki.add_edge('UML-Werkzeug', 'Microsoft Visio')
wiki.add_edge('UML-Werkzeug', 'Modelio')
wiki.add_edge('UML-Werkzeug', 'ObjectiF')
wiki.add_edge('UML-Werkzeug', 'Papyrus (UML)')
wiki.add_edge('UML-Werkzeug', 'Poseidon for UML')
wiki.add_edge('UML-Werkzeug', 'PowerDesigner')
wiki.add_edge('UML-Werkzeug', 'Rational Rose')
wiki.add_edge('UML-Werkzeug', 'Sequenzdiagramm')
wiki.add_edge('UML-Werkzeug', 'StarUML')
wiki.add_edge('UML-Werkzeug', 'TOPCASED')
wiki.add_edge('UML-Werkzeug', 'Together (Software)')
wiki.add_edge('UML-Werkzeug', 'UMLet')
wiki.add_edge('UML-Werkzeug', 'Umbrello')
wiki.add_edge('UML-Werkzeug', 'Violet (UML-Werkzeug)')
wiki.add_edge('UML-Werkzeug', 'Visual Paradigm')
wiki.add_edge('UML-Werkzeug', 'Zeitverlaufsdiagramm')
wiki.add_edge('Unified Diagnostic Services', 'DoIP')
wiki.add_edge('Unified Diagnostic Services', 'KWP2000')
wiki.add_edge('Unified Modeling Language', 'Abhängigkeitsbeziehung')
wiki.add_edge('Unified Modeling Language', 'Accounting Transaction')
wiki.add_edge('Unified Modeling Language', 'Aggregation (Informatik)')
wiki.add_edge('Unified Modeling Language', 'Akteur (UML)')
wiki.add_edge('Unified Modeling Language', 'Aktion (UML)')
wiki.add_edge('Unified Modeling Language', 'Aktivität (UML)')
wiki.add_edge('Unified Modeling Language', 'Aktivitätsdiagramm')
wiki.add_edge('Unified Modeling Language', 'Aktivitätsparameterknoten')
wiki.add_edge('Unified Modeling Language', 'Anwendungsfall (UML)')
wiki.add_edge('Unified Modeling Language', 'Anwendungsfalldiagramm')
wiki.add_edge('Unified Modeling Language', 'ArchiMate')
wiki.add_edge('Unified Modeling Language', 'Architecture Description Language')
wiki.add_edge('Unified Modeling Language', 'Artefakt (UML)')
wiki.add_edge('Unified Modeling Language', 'Assoziation (UML)')
wiki.add_edge('Unified Modeling Language', 'Assoziationsklasse')
wiki.add_edge('Unified Modeling Language', 'Attribut (UML)')
wiki.add_edge('Unified Modeling Language', 'Ausprägungsspezifikation (UML)')
wiki.add_edge('Unified Modeling Language', 'C4 model')
wiki.add_edge('Unified Modeling Language', 'Classifier (UML)')
wiki.add_edge('Unified Modeling Language', 'Cross-Cutting Concern')
wiki.add_edge('Unified Modeling Language', 'Datenfluss')
wiki.add_edge('Unified Modeling Language', 'Datenflussdiagramm')
wiki.add_edge('Unified Modeling Language', 'Dia (Software)')
wiki.add_edge('Unified Modeling Language', 'Diagrams.net')
wiki.add_edge('Unified Modeling Language', 'Entität (Informatik)')
wiki.add_edge('Unified Modeling Language', 'Ereignis (UML)')
wiki.add_edge('Unified Modeling Language', 'Fundamental Modeling Concepts')
wiki.add_edge('Unified Modeling Language', 'Funktionsbaum')
wiki.add_edge('Unified Modeling Language', 'Generalisierung (UML)')
wiki.add_edge('Unified Modeling Language', 'IDEF')
wiki.add_edge('Unified Modeling Language', 'Informationseinheit (UML)')
wiki.add_edge('Unified Modeling Language', 'Informationsfluss (UML)')
wiki.add_edge('Unified Modeling Language', 'Innovator (Software)')
wiki.add_edge('Unified Modeling Language', 'Interaktion (UML)')
wiki.add_edge('Unified Modeling Language', 'Interaktionsübersichtsdiagramm')
wiki.add_edge('Unified Modeling Language', 'Klasse (UML)')
wiki.add_edge('Unified Modeling Language', 'Klassendiagramm')
wiki.add_edge('Unified Modeling Language', 'Knoten (UML)')
wiki.add_edge('Unified Modeling Language', 'Kommentar (UML)')
wiki.add_edge('Unified Modeling Language', 'Kommunikationsdiagramm (UML)')
wiki.add_edge('Unified Modeling Language', 'Komponente (UML)')
wiki.add_edge('Unified Modeling Language', 'Komponentendiagramm')
wiki.add_edge('Unified Modeling Language', 'Kompositionsstrukturdiagramm')
wiki.add_edge('Unified Modeling Language', 'Konnektor (UML)')
wiki.add_edge('Unified Modeling Language', 'Kontrollfluss')
wiki.add_edge('Unified Modeling Language', 'Kontrollfluss (UML)')
wiki.add_edge('Unified Modeling Language', 'Kontrollknoten (UML)')
wiki.add_edge('Unified Modeling Language', 'MOF QVT')
wiki.add_edge('Unified Modeling Language', 'Merkmal (UML)')
wiki.add_edge('Unified Modeling Language', 'Modeling and Analysis of Real Time and Embedded systems')
wiki.add_edge('Unified Modeling Language', 'Modelio')
wiki.add_edge('Unified Modeling Language', 'Modell (UML)')
wiki.add_edge('Unified Modeling Language', 'Modellgetriebene Architektur')
wiki.add_edge('Unified Modeling Language', 'Multiplizität (UML)')
wiki.add_edge('Unified Modeling Language', 'Namensraum (UML)')
wiki.add_edge('Unified Modeling Language', 'Normalisierung (Datenbank)')
wiki.add_edge('Unified Modeling Language', 'Object Constraint Language')
wiki.add_edge('Unified Modeling Language', 'ObjectiF')
wiki.add_edge('Unified Modeling Language', 'Objektdiagramm')
wiki.add_edge('Unified Modeling Language', 'Objektfluss (UML)')
wiki.add_edge('Unified Modeling Language', 'Objektknoten (UML)')
wiki.add_edge('Unified Modeling Language', 'Operation (UML)')
wiki.add_edge('Unified Modeling Language', 'Paket (UML)')
wiki.add_edge('Unified Modeling Language', 'Paketdiagramm')
wiki.add_edge('Unified Modeling Language', 'Paketimport (UML)')
wiki.add_edge('Unified Modeling Language', 'Paketverschmelzung')
wiki.add_edge('Unified Modeling Language', 'Pin (UML)')
wiki.add_edge('Unified Modeling Language', 'Port (UML)')
wiki.add_edge('Unified Modeling Language', 'Profil (UML)')
wiki.add_edge('Unified Modeling Language', 'Profildiagramm')
wiki.add_edge('Unified Modeling Language', 'Programmablaufplan')
wiki.add_edge('Unified Modeling Language', 'Pufferknoten (UML)')
wiki.add_edge('Unified Modeling Language', 'Rational Rhapsody')
wiki.add_edge('Unified Modeling Language', 'Rational Rose')
wiki.add_edge('Unified Modeling Language', 'Schnittstelle (UML)')
wiki.add_edge('Unified Modeling Language', 'Semantisches Datenmodell')
wiki.add_edge('Unified Modeling Language', 'Sequenzdiagramm')
wiki.add_edge('Unified Modeling Language', 'Signal (UML)')
wiki.add_edge('Unified Modeling Language', 'Signalempfänger (UML)')
wiki.add_edge('Unified Modeling Language', 'SoaML')
wiki.add_edge('Unified Modeling Language', 'Softwarearchitekt')
wiki.add_edge('Unified Modeling Language', 'Softwaredesign')
wiki.add_edge('Unified Modeling Language', 'StarUML')
wiki.add_edge('Unified Modeling Language', 'Stereotyp (UML)')
wiki.add_edge('Unified Modeling Language', 'Systems Modeling Language')
wiki.add_edge('Unified Modeling Language', 'TOPCASED')
wiki.add_edge('Unified Modeling Language', 'Together (Software)')
wiki.add_edge('Unified Modeling Language', 'UML-Werkzeug')
wiki.add_edge('Unified Modeling Language', 'UMLet')
wiki.add_edge('Unified Modeling Language', 'Umbrello')
wiki.add_edge('Unified Modeling Language', 'Verhalten (UML)')
wiki.add_edge('Unified Modeling Language', 'Verteilungsdiagramm')
wiki.add_edge('Unified Modeling Language', 'YEd')
wiki.add_edge('Unified Modeling Language', 'Zeitverlaufsdiagramm')
wiki.add_edge('Unified Modeling Language', 'Zustandsdiagramm (UML)')
wiki.add_edge('Universal Description, Discovery and Integration', 'Dienstekomposition')
wiki.add_edge('Universal Plug and Play', 'IGD-Protokoll')
wiki.add_edge('Universal Plug and Play', 'NAT Port Mapping Protocol')
wiki.add_edge('Universal Plug and Play', 'RetroShare')
wiki.add_edge('Universal Plug and Play', 'Service Location Protocol')
wiki.add_edge('Universal Plug and Play', 'Simple Service Discovery Protocol')
wiki.add_edge('Universal Plug and Play', 'Windows Live Messenger')
wiki.add_edge('Universally Unique Identifier', 'Innovator (Software)')
wiki.add_edge('Universally Unique Identifier', 'Train Real Time Data Protocol')
wiki.add_edge('User Datagram Protocol', '464XLAT')
wiki.add_edge('User Datagram Protocol', '6LoWPAN')
wiki.add_edge('User Datagram Protocol', 'ATA over Ethernet')
wiki.add_edge('User Datagram Protocol', 'Advanced Physical Layer')
wiki.add_edge('User Datagram Protocol', 'AppleTalk')
wiki.add_edge('User Datagram Protocol', 'Asynchronous Reliable Delivery Protocol')
wiki.add_edge('User Datagram Protocol', 'B.A.T.M.A.N.')
wiki.add_edge('User Datagram Protocol', 'BattleEye RCon Protocol')
wiki.add_edge('User Datagram Protocol', 'BitTorrent')
wiki.add_edge('User Datagram Protocol', 'Bitcoin')
wiki.add_edge('User Datagram Protocol', 'Bootstrap Protocol')
wiki.add_edge('User Datagram Protocol', 'CSLIP')
wiki.add_edge('User Datagram Protocol', 'Crypto IP Encapsulation')
wiki.add_edge('User Datagram Protocol', 'DNS over HTTPS')
wiki.add_edge('User Datagram Protocol', 'DNS over TLS')
wiki.add_edge('User Datagram Protocol', 'Datagram Congestion Control Protocol')
wiki.add_edge('User Datagram Protocol', 'Datagramm')
wiki.add_edge('User Datagram Protocol', 'Daytime')
wiki.add_edge('User Datagram Protocol', 'Discard')
wiki.add_edge('User Datagram Protocol', 'Domain Name System Security Extensions')
wiki.add_edge('User Datagram Protocol', 'Dynamic Host Configuration Protocol')
wiki.add_edge('User Datagram Protocol', 'Echo (Netzwerkdienst)')
wiki.add_edge('User Datagram Protocol', 'EtherNet/IP')
wiki.add_edge('User Datagram Protocol', 'Ethernet')
wiki.add_edge('User Datagram Protocol', 'Ethernet Powerlink')
wiki.add_edge('User Datagram Protocol', 'Explicit Congestion Notification')
wiki.add_edge('User Datagram Protocol', 'Extensible Messaging and Presence Protocol')
wiki.add_edge('User Datagram Protocol', 'Fast and Secure Protocol')
wiki.add_edge('User Datagram Protocol', 'File Service Protocol')
wiki.add_edge('User Datagram Protocol', 'GigE Vision')
wiki.add_edge('User Datagram Protocol', 'Host Identity Protocol')
wiki.add_edge('User Datagram Protocol', 'Hyper Text Caching Protocol')
wiki.add_edge('User Datagram Protocol', 'HyperSCSI')
wiki.add_edge('User Datagram Protocol', 'ILMI')
wiki.add_edge('User Datagram Protocol', 'IP-Fragmentierung')
wiki.add_edge('User Datagram Protocol', 'IP-Spoofing')
wiki.add_edge('User Datagram Protocol', 'IPsec')
wiki.add_edge('User Datagram Protocol', 'InterAsterisk eXchange')
wiki.add_edge('User Datagram Protocol', 'Internet Cache Protocol')
wiki.add_edge('User Datagram Protocol', 'Internet Control Message Protocol')
wiki.add_edge('User Datagram Protocol', 'Internet Protocol Flow Information Export')
wiki.add_edge('User Datagram Protocol', 'Internetwork Packet Exchange')
wiki.add_edge('User Datagram Protocol', 'Internetwork Packet eXchange Fragmentation')
wiki.add_edge('User Datagram Protocol', 'Label Distribution Protocol')
wiki.add_edge('User Datagram Protocol', 'Layer 2 Tunneling Protocol')
wiki.add_edge('User Datagram Protocol', 'Lightweight Directory Access Protocol')
wiki.add_edge('User Datagram Protocol', 'Link-local Multicast Name Resolution')
wiki.add_edge('User Datagram Protocol', 'Liste der standardisierten Ports')
wiki.add_edge('User Datagram Protocol', 'Liste von TCP/IP-basierten Netzwerkdiensten')
wiki.add_edge('User Datagram Protocol', 'MMS-Protokoll')
wiki.add_edge('User Datagram Protocol', 'Media Gateway Control Protocol (Protokoll)')
wiki.add_edge('User Datagram Protocol', 'Multicast Backbone')
wiki.add_edge('User Datagram Protocol', 'NAT Port Mapping Protocol')
wiki.add_edge('User Datagram Protocol', 'NetBIOS')
wiki.add_edge('User Datagram Protocol', 'NetBIOS over TCP/IP')
wiki.add_edge('User Datagram Protocol', 'Netfilter')
wiki.add_edge('User Datagram Protocol', 'Netflow')
wiki.add_edge('User Datagram Protocol', 'Network File System')
wiki.add_edge('User Datagram Protocol', 'Network Time Protocol')
wiki.add_edge('User Datagram Protocol', 'Netzwerkprotokoll')
wiki.add_edge('User Datagram Protocol', 'OSCAR (Protokoll)')
wiki.add_edge('User Datagram Protocol', 'OSI-Modell')
wiki.add_edge('User Datagram Protocol', 'Pi Digit Generation Protocol')
wiki.add_edge('User Datagram Protocol', 'Port (Protokoll)')
wiki.add_edge('User Datagram Protocol', 'Preboot Execution Environment')
wiki.add_edge('User Datagram Protocol', 'Profinet')
wiki.add_edge('User Datagram Protocol', 'Prospero (Protokoll)')
wiki.add_edge('User Datagram Protocol', 'Protocol for SYnchronous Conferencing')
wiki.add_edge('User Datagram Protocol', 'Protokoll (IP)')
wiki.add_edge('User Datagram Protocol', 'QUIC')
wiki.add_edge('User Datagram Protocol', 'ROHC')
wiki.add_edge('User Datagram Protocol', 'Rail Safe Transport Application')
wiki.add_edge('User Datagram Protocol', 'Real-Time Streaming Protocol')
wiki.add_edge('User Datagram Protocol', 'Real-Time Transport Protocol')
wiki.add_edge('User Datagram Protocol', 'RealTime Control Protocol')
wiki.add_edge('User Datagram Protocol', 'Remote Desktop Protocol')
wiki.add_edge('User Datagram Protocol', 'Remote Procedure Call')
wiki.add_edge('User Datagram Protocol', 'Resource Reservation Protocol')
wiki.add_edge('User Datagram Protocol', 'Routing Information Protocol')
wiki.add_edge('User Datagram Protocol', 'SIMON')
wiki.add_edge('User Datagram Protocol', 'SOCKS')
wiki.add_edge('User Datagram Protocol', 'Secure Shell')
wiki.add_edge('User Datagram Protocol', 'Serial Line Internet Protocol')
wiki.add_edge('User Datagram Protocol', 'Server Message Block')
wiki.add_edge('User Datagram Protocol', 'Session Initiation Protocol')
wiki.add_edge('User Datagram Protocol', 'Session Traversal Utilities for NAT')
wiki.add_edge('User Datagram Protocol', 'Simple Network Management Protocol')
wiki.add_edge('User Datagram Protocol', 'Simple Service Discovery Protocol')
wiki.add_edge('User Datagram Protocol', 'Socket (Software)')
wiki.add_edge('User Datagram Protocol', 'Stream Control Transmission Protocol')
wiki.add_edge('User Datagram Protocol', 'Streaming-Protokoll')
wiki.add_edge('User Datagram Protocol', 'Syslog')
wiki.add_edge('User Datagram Protocol', 'TP/IX')
wiki.add_edge('User Datagram Protocol', 'Teredo')
wiki.add_edge('User Datagram Protocol', 'Train Real Time Data Protocol')
wiki.add_edge('User Datagram Protocol', 'Transmission Control Protocol')
wiki.add_edge('User Datagram Protocol', 'Transmission Control Protocol/Internet Protocol')
wiki.add_edge('User Datagram Protocol', 'Trivial File Transfer Protocol')
wiki.add_edge('User Datagram Protocol', 'Tunnel (Rechnernetz)')
wiki.add_edge('User Datagram Protocol', 'Universal Measurement and Calibration Protocol')
wiki.add_edge('User Datagram Protocol', 'Universal Plug and Play')
wiki.add_edge('User Datagram Protocol', 'Virtual Extensible LAN')
wiki.add_edge('User Datagram Protocol', 'Virtual Local Area Network')
wiki.add_edge('User Datagram Protocol', 'XDMCP')
wiki.add_edge('User Datagram Protocol', 'Zephyr (Protokoll)')
wiki.add_edge('User Datagram Protocol', 'Zeroconf')
wiki.add_edge('VTAM', 'Advanced Program to Program Communications')
wiki.add_edge('Value Object', 'Money (Analysemuster)')
wiki.add_edge('Variable Length Subnet Mask', 'Enhanced Interior Gateway Routing Protocol')
wiki.add_edge('Variable Length Subnet Mask', 'Interior Gateway Routing Protocol')
wiki.add_edge('Variable Length Subnet Mask', 'Routing Information Protocol')
wiki.add_edge('Verhalten (UML)', 'Abhängigkeitsbeziehung')
wiki.add_edge('Verhalten (UML)', 'Aktivitätsparameterknoten')
wiki.add_edge('Verhalten (UML)', 'Classifier (UML)')
wiki.add_edge('Verhalten (UML)', 'Ereignis (UML)')
wiki.add_edge('Verhalten (UML)', 'Generalisierung (UML)')
wiki.add_edge('Verhalten (UML)', 'Interaktion (UML)')
wiki.add_edge('Verhalten (UML)', 'Merkmal (UML)')
wiki.add_edge('Verhalten (UML)', 'Objektfluss (UML)')
wiki.add_edge('Verhalten (UML)', 'Objektknoten (UML)')
wiki.add_edge('Verhalten (UML)', 'Port (UML)')
wiki.add_edge('Verhalten (UML)', 'Sequenzdiagramm')
wiki.add_edge('Verhaltensmuster (Software)', 'Adapter (Entwurfsmuster)')
wiki.add_edge('Verhaltensmuster (Software)', 'Aggregation (Informatik)')
wiki.add_edge('Verhaltensmuster (Software)', 'Beobachter (Entwurfsmuster)')
wiki.add_edge('Verhaltensmuster (Software)', 'Besucher (Entwurfsmuster)')
wiki.add_edge('Verhaltensmuster (Software)', 'Interceptor (Entwurfsmuster)')
wiki.add_edge('Verhaltensmuster (Software)', 'Interpreter (Entwurfsmuster)')
wiki.add_edge('Verhaltensmuster (Software)', 'Iterator (Entwurfsmuster)')
wiki.add_edge('Verhaltensmuster (Software)', 'Kommando (Entwurfsmuster)')
wiki.add_edge('Verhaltensmuster (Software)', 'MapReduce')
wiki.add_edge('Verhaltensmuster (Software)', 'Memento (Entwurfsmuster)')
wiki.add_edge('Verhaltensmuster (Software)', 'Nebenläufigkeit (Entwurfsmuster)')
wiki.add_edge('Verhaltensmuster (Software)', 'Nullobjekt (Entwurfsmuster)')
wiki.add_edge('Verhaltensmuster (Software)', 'Plugin (Entwurfsmuster)')
wiki.add_edge('Verhaltensmuster (Software)', 'Schablonenmethode')
wiki.add_edge('Verhaltensmuster (Software)', 'Sicherung (Entwurfsmuster)')
wiki.add_edge('Verhaltensmuster (Software)', 'Strategie (Entwurfsmuster)')
wiki.add_edge('Verhaltensmuster (Software)', 'Unit of Work')
wiki.add_edge('Verhaltensmuster (Software)', 'Vermittler (Entwurfsmuster)')
wiki.add_edge('Verhaltensmuster (Software)', 'Zustand (Entwurfsmuster)')
wiki.add_edge('Verhaltensmuster (Software)', 'Zuständigkeitskette')
wiki.add_edge('Vermittler (Entwurfsmuster)', 'Beobachter (Entwurfsmuster)')
wiki.add_edge('Vermittler (Entwurfsmuster)', 'Common Object Request Broker Architecture')
wiki.add_edge('Vermittler (Entwurfsmuster)', 'Mapper (Softwareentwicklung)')
wiki.add_edge('Vermittlungsschicht', 'Internetwork Datagram Protocol')
wiki.add_edge('Vermittlungsschicht', 'Jumbo Frames')
wiki.add_edge('Verschlüsselungsprotokoll', 'LibreSSL')
wiki.add_edge('Verschlüsselungsprotokoll', 'Noise Protocol Framework')
wiki.add_edge('Verschlüsselungsprotokoll', 'WPA2')
wiki.add_edge('Verschlüsselungsprotokoll', 'Wireless Transport Layer Security')
wiki.add_edge('Verteilungsdiagramm', '4+1 Sichtenmodell')
wiki.add_edge('Verteilungsdiagramm', 'Artefakt (UML)')
wiki.add_edge('Verteilungsdiagramm', 'Ausprägungsspezifikation (UML)')
wiki.add_edge('Verteilungsdiagramm', 'Innovator (Software)')
wiki.add_edge('Verteilungsdiagramm', 'Knoten (UML)')
wiki.add_edge('Verteilungsdiagramm', 'Komponente (UML)')
wiki.add_edge('Verteilungsdiagramm', 'UMLet')
wiki.add_edge('Video Decode and Presentation API for Unix', 'X-Video Bitstream Acceleration')
wiki.add_edge('Video for Windows', 'Multimedia-Framework')
wiki.add_edge('Video for Windows', 'Video4Linux')
wiki.add_edge('Video4Linux', 'Phonon (KDE)')
wiki.add_edge('Virtual Local Area Network', 'IEEE 802.1X')
wiki.add_edge('Virtual Local Area Network', 'Inter-Switch Link Protocol')
wiki.add_edge('Virtual Local Area Network', 'Link Aggregation')
wiki.add_edge('Virtual Local Area Network', 'Sequenced Packet Exchange')
wiki.add_edge('Virtual Local Area Network', 'Time-Sensitive Networking')
wiki.add_edge('Virtual Local Area Network', 'Virtual Extensible LAN')
wiki.add_edge('Virtual Network Computing', 'Liste von TCP/IP-basierten Netzwerkdiensten')
wiki.add_edge('Virtual Network Computing', 'Remote Desktop Protocol')
wiki.add_edge('Virtual Network Computing', 'Remote Framebuffer Protocol')
wiki.add_edge('Virtual Network Computing', 'SPICE (Protokoll)')
wiki.add_edge('Virtual Studio Technology', 'Audio Random Access')
wiki.add_edge('Virtual Studio Technology', 'Audio Stream Input/Output')
wiki.add_edge('Virtual Studio Technology', 'Audio Unit')
wiki.add_edge('Virtual Studio Technology', 'DSSI')
wiki.add_edge('Virtual Studio Technology', 'LV2')
wiki.add_edge('Virtual Studio Technology', 'Linux Audio Developer’s Simple Plugin API')
wiki.add_edge('Virtual Terminal', 'Job Transfer and Manipulation')
wiki.add_edge('Vulkan (API)', 'AMD Mantle')
wiki.add_edge('Vulkan (API)', 'DirectX Raytracing')
wiki.add_edge('Vulkan (API)', 'Metal (API)')
wiki.add_edge('Vulkan (API)', 'MoltenVK')
wiki.add_edge('Vulkan (API)', 'OpenCL')
wiki.add_edge('WHOIS', 'Registration Data Access Protocol')
wiki.add_edge('WPA3', 'Verschlüsselungsprotokoll')
wiki.add_edge('WPA3', 'WPA2')
wiki.add_edge('WS-*', 'Devices Profile for Web Services')
wiki.add_edge('WS-*', 'Jakarta XML Web Services')
wiki.add_edge('WS-*', 'WS-Addressing')
wiki.add_edge('WS-*', 'WS-Agreement')
wiki.add_edge('WS-*', 'WS-BPEL4People')
wiki.add_edge('WS-*', 'WS-Choreography Description Language')
wiki.add_edge('WS-*', 'WS-Choreography Interface')
wiki.add_edge('WS-*', 'WS-Coordination')
wiki.add_edge('WS-*', 'WS-Discovery')
wiki.add_edge('WS-*', 'WS-Distributed Management')
wiki.add_edge('WS-*', 'WS-Enumeration')
wiki.add_edge('WS-*', 'WS-Flow Language')
wiki.add_edge('WS-*', 'WS-HumanTask')
wiki.add_edge('WS-*', 'WS-Interoperability')
wiki.add_edge('WS-*', 'WS-Policy')
wiki.add_edge('WS-*', 'WS-Referral')
wiki.add_edge('WS-*', 'WS-Reliable Messaging')
wiki.add_edge('WS-*', 'WS-Routing')
wiki.add_edge('WS-*', 'WS-SecureConversation')
wiki.add_edge('WS-*', 'WS-Security UsernameToken Profile')
wiki.add_edge('WS-*', 'WS-SecurityPolicy')
wiki.add_edge('WS-*', 'WS-Transaction')
wiki.add_edge('WS-*', 'WS-Trust')
wiki.add_edge('WS-*', 'Web Services Interoperability Technology')
wiki.add_edge('WS-Addressing', 'Devices Profile for Web Services')
wiki.add_edge('WS-Addressing', 'Jakarta XML Web Services')
wiki.add_edge('WS-Addressing', 'WS-Agreement')
wiki.add_edge('WS-Addressing', 'WS-Enumeration')
wiki.add_edge('WS-Addressing', 'WS-Referral')
wiki.add_edge('WS-Addressing', 'WS-Routing')
wiki.add_edge('WS-Addressing', 'Web Services Interoperability Technology')
wiki.add_edge('WS-AtomicTransaction', 'Jakarta XML Web Services')
wiki.add_edge('WS-AtomicTransaction', 'Web Services Interoperability Technology')
wiki.add_edge('WS-BPEL4People', 'WS-HumanTask')
wiki.add_edge('WS-Business Process Execution Language', 'Dienstekomposition')
wiki.add_edge('WS-Business Process Execution Language', 'Mashup (Internet)')
wiki.add_edge('WS-Business Process Execution Language', 'Modelio')
wiki.add_edge('WS-Business Process Execution Language', 'WS-*')
wiki.add_edge('WS-Business Process Execution Language', 'WS-Addressing')
wiki.add_edge('WS-Business Process Execution Language', 'WS-BPEL4People')
wiki.add_edge('WS-Business Process Execution Language', 'WS-Choreography Description Language')
wiki.add_edge('WS-Business Process Execution Language', 'WS-Flow Language')
wiki.add_edge('WS-Business Process Execution Language', 'WS-HumanTask')
wiki.add_edge('WS-Business Process Execution Language', 'WS-Policy')
wiki.add_edge('WS-Business Process Execution Language', 'WS-Security')
wiki.add_edge('WS-Business Process Execution Language', 'XL (XML-Programmiersprache)')
wiki.add_edge('WS-Choreography Description Language', 'Dienstekomposition')
wiki.add_edge('WS-Choreography Interface', 'Dienstekomposition')
wiki.add_edge('WS-Choreography Interface', 'WS-Choreography Description Language')
wiki.add_edge('WS-Discovery', 'Devices Profile for Web Services')
wiki.add_edge('WS-Discovery', 'WS-Addressing')
wiki.add_edge('WS-Discovery', 'WS-Reliable Messaging')
wiki.add_edge('WS-Discovery', 'WS-Transaction')
wiki.add_edge('WS-Policy', 'Devices Profile for Web Services')
wiki.add_edge('WS-Policy', 'Jakarta XML Web Services')
wiki.add_edge('WS-Policy', 'WS-Enumeration')
wiki.add_edge('WS-Policy', 'WS-SecurityPolicy')
wiki.add_edge('WS-Policy', 'Web Services Interoperability Technology')
wiki.add_edge('WS-Referral', 'WS-Routing')
wiki.add_edge('WS-Reliability', 'WS-*')
wiki.add_edge('WS-Reliability', 'WS-Referral')
wiki.add_edge('WS-Reliability', 'WS-Reliable Messaging')
wiki.add_edge('WS-Reliability', 'WS-Routing')
wiki.add_edge('WS-Reliable Messaging', 'Jakarta XML Web Services')
wiki.add_edge('WS-Reliable Messaging', 'WS-Policy')
wiki.add_edge('WS-Reliable Messaging', 'WS-Referral')
wiki.add_edge('WS-Reliable Messaging', 'WS-Routing')
wiki.add_edge('WS-Reliable Messaging', 'Web Services Interoperability Technology')
wiki.add_edge('WS-SecureConversation', 'WS-Enumeration')
wiki.add_edge('WS-SecureConversation', 'WS-SecurityPolicy')
wiki.add_edge('WS-SecureConversation', 'WS-Trust')
wiki.add_edge('WS-Security', 'Apache Axis')
wiki.add_edge('WS-Security', 'Apache CXF')
wiki.add_edge('WS-Security', 'Devices Profile for Web Services')
wiki.add_edge('WS-Security', 'Jakarta XML Web Services')
wiki.add_edge('WS-Security', 'WS-*')
wiki.add_edge('WS-Security', 'WS-Discovery')
wiki.add_edge('WS-Security', 'WS-Enumeration')
wiki.add_edge('WS-Security', 'WS-SecureConversation')
wiki.add_edge('WS-Security', 'WS-Security UsernameToken Profile')
wiki.add_edge('WS-Security', 'WS-SecurityPolicy')
wiki.add_edge('WS-Security', 'WS-Trust')
wiki.add_edge('WS-Security', 'Web Services Interoperability Technology')
wiki.add_edge('WS-SecurityPolicy', 'WS-Enumeration')
wiki.add_edge('WS-SecurityPolicy', 'WS-Trust')
wiki.add_edge('WeChat', 'Line (Instant Messaging)')
wiki.add_edge('Web Audio API', 'WebVR')
wiki.add_edge('Web Feature Service', 'Web Processing Service')
wiki.add_edge('Web Map Service', 'Geodienst')
wiki.add_edge('Web Map Service', 'Web Feature Service')
wiki.add_edge('Web Map Service', 'Web Map Tile Service')
wiki.add_edge('Web Map Service', 'Web Processing Service')
wiki.add_edge('Web Ontology Language for Web Services', 'Web Service Modeling Ontology')
wiki.add_edge('Web Proxy Autodiscovery Protocol', 'Proxy Auto-Config')
wiki.add_edge('Web Proxy Autodiscovery Protocol', 'Service Location Protocol')
wiki.add_edge('Web Services Description Language', 'Apache Axis')
wiki.add_edge('Web Services Description Language', 'Apache CXF')
wiki.add_edge('Web Services Description Language', 'Dienstekomposition')
wiki.add_edge('Web Services Description Language', 'Innovator (Software)')
wiki.add_edge('Web Services Description Language', 'Java Business Integration')
wiki.add_edge('Web Services Description Language', 'Message Oriented Middleware')
wiki.add_edge('Web Services Description Language', 'Open Platform Communications')
wiki.add_edge('Web Services Description Language', 'Universal Description, Discovery and Integration')
wiki.add_edge('Web Services Description Language', 'WS-*')
wiki.add_edge('Web Services Description Language', 'WS-Business Process Execution Language')
wiki.add_edge('Web Services Description Language', 'WS-Policy')
wiki.add_edge('Web Services Description Language', 'Web Application Description Language')
wiki.add_edge('Web Services Description Language', 'XL (XML-Programmiersprache)')
wiki.add_edge('Web Services Interoperability Technology', 'Jakarta XML Binding')
wiki.add_edge('Web Services Interoperability Technology', 'Jakarta XML Web Services')
wiki.add_edge('Web Services Interoperability Technology', 'Java Secure Socket Extension')
wiki.add_edge('Web Services Interoperability Technology', 'WS-Coordination')
wiki.add_edge('Web Services Interoperability Technology', 'WS-SecureConversation')
wiki.add_edge('Web Services Interoperability Technology', 'WS-SecurityPolicy')
wiki.add_edge('Web Services Interoperability Technology', 'WS-Trust')
wiki.add_edge('Web Storage', 'Indexed Database API')
wiki.add_edge('Web Worker', 'Service Worker')
wiki.add_edge('WebDAV', 'CalDAV')
wiki.add_edge('WebDAV', 'CardDAV')
wiki.add_edge('WebDAV', 'Gnome Virtual File System')
wiki.add_edge('WebDAV', 'GroupDAV')
wiki.add_edge('WebDAV', 'Liste der standardisierten Ports')
wiki.add_edge('WebDAV', 'Representational State Transfer')
wiki.add_edge('WebDAV', 'Secure Copy')
wiki.add_edge('WebDAV', 'Secure Shell')
wiki.add_edge('WebDAV', 'Threema')
wiki.add_edge('WebDAV', 'Webservice')
wiki.add_edge('WebGL', 'WebCL')
wiki.add_edge('WebGL', 'WebVR')
wiki.add_edge('WebHooks', 'IFTTT')
wiki.add_edge('WebRTC', 'Matrix (Kommunikationsprotokoll)')
wiki.add_edge('WebRTC', 'OpenWebRTC')
wiki.add_edge('WebRTC', 'Threema')
wiki.add_edge('WebRTC', 'Web Audio API')
wiki.add_edge('WebRTC', 'Wire (Messenger)')
wiki.add_edge('WebSocket', 'JSON Meta Application Protocol')
wiki.add_edge('WebSocket', 'TR-369')
wiki.add_edge('WebSocket', 'WhatsApp')
wiki.add_edge('Webservice', 'Anwendungsserver')
wiki.add_edge('Webservice', 'Apache Axis')
wiki.add_edge('Webservice', 'Apache CXF')
wiki.add_edge('Webservice', 'Common Object Request Broker Architecture')
wiki.add_edge('Webservice', 'Dienstekomposition')
wiki.add_edge('Webservice', 'Enterprise Application Integration')
wiki.add_edge('Webservice', 'Entwurfsmuster')
wiki.add_edge('Webservice', 'Fassade (Entwurfsmuster)')
wiki.add_edge('Webservice', 'Geodienst')
wiki.add_edge('Webservice', 'Hessian (Webprotokoll)')
wiki.add_edge('Webservice', 'Humane Registry')
wiki.add_edge('Webservice', 'IFTTT')
wiki.add_edge('Webservice', 'Jakarta RESTful Web Services')
wiki.add_edge('Webservice', 'Jakarta XML Web Services')
wiki.add_edge('Webservice', 'Message Oriented Middleware')
wiki.add_edge('Webservice', 'Modellgetriebene Architektur')
wiki.add_edge('Webservice', 'Money (Analysemuster)')
wiki.add_edge('Webservice', 'Open Data Protocol')
wiki.add_edge('Webservice', 'Open Platform Communications')
wiki.add_edge('Webservice', 'OpenAPI')
wiki.add_edge('Webservice', 'Operations-Datenbank')
wiki.add_edge('Webservice', 'Remote Method Invocation')
wiki.add_edge('Webservice', 'Remote Procedure Call')
wiki.add_edge('Webservice', 'SOAP')
wiki.add_edge('Webservice', 'SOAP Message Transmission Optimization Mechanism')
wiki.add_edge('Webservice', 'Schichtenarchitektur')
wiki.add_edge('Webservice', 'Security Assertion Markup Language')
wiki.add_edge('Webservice', 'Semantic Web Services')
wiki.add_edge('Webservice', 'Service Component Architecture')
wiki.add_edge('Webservice', 'Serviceorientierte Architektur')
wiki.add_edge('Webservice', 'Services Computing')
wiki.add_edge('Webservice', 'Sicherung (Entwurfsmuster)')
wiki.add_edge('Webservice', 'SoapUI')
wiki.add_edge('Webservice', 'Software-Ökosystem')
wiki.add_edge('Webservice', 'Swagger (Software)')
wiki.add_edge('Webservice', 'TCP Fast Open')
wiki.add_edge('Webservice', 'UnAPI')
wiki.add_edge('Webservice', 'Universal Description, Discovery and Integration')
wiki.add_edge('Webservice', 'WS-*')
wiki.add_edge('Webservice', 'WS-Addressing')
wiki.add_edge('Webservice', 'WS-Business Process Execution Language')
wiki.add_edge('Webservice', 'WS-Choreography Interface')
wiki.add_edge('Webservice', 'WS-Discovery')
wiki.add_edge('Webservice', 'WS-Distributed Management')
wiki.add_edge('Webservice', 'WS-Interoperability')
wiki.add_edge('Webservice', 'WS-Security')
wiki.add_edge('Webservice', 'WS-Security UsernameToken Profile')
wiki.add_edge('Webservice', 'WS-SecurityPolicy')
wiki.add_edge('Webservice', 'WS-Transaction')
wiki.add_edge('Webservice', 'WS-Trust')
wiki.add_edge('Webservice', 'Web Map Service')
wiki.add_edge('Webservice', 'Web Map Tile Service')
wiki.add_edge('Webservice', 'Web Ontology Language for Web Services')
wiki.add_edge('Webservice', 'Web Services Description Language')
wiki.add_edge('Webservice', 'Web Services Interoperability Technology')
wiki.add_edge('Webservice', 'XL (XML-Programmiersprache)')
wiki.add_edge('Webservice', 'Zustandsdiagramm (UML)')
wiki.add_edge('WhatsApp', 'Briar (Instant Messenger)')
wiki.add_edge('WhatsApp', 'Hoccer')
wiki.add_edge('WhatsApp', 'Line (Instant Messaging)')
wiki.add_edge('WhatsApp', 'Matrix (Kommunikationsprotokoll)')
wiki.add_edge('WhatsApp', 'Noise Protocol Framework')
wiki.add_edge('WhatsApp', 'Signal (Messenger)')
wiki.add_edge('WhatsApp', 'Surespot')
wiki.add_edge('WhatsApp', 'Telegram')
wiki.add_edge('WhatsApp', 'Threema')
wiki.add_edge('WhatsApp', 'Viber')
wiki.add_edge('Whatsapp', 'WhatsApp')
wiki.add_edge('Whois', 'Liste der standardisierten Ports')
wiki.add_edge('Whois', 'Liste von TCP/IP-basierten Netzwerkdiensten')
wiki.add_edge('Whois', 'Network Information Center')
wiki.add_edge('Whois', 'Socket (Software)')
wiki.add_edge('Whois', 'Telnet')
wiki.add_edge('Whois', 'WHOIS')
wiki.add_edge('WinG', 'GDI+')
wiki.add_edge('WinG', 'Graphics Device Interface')
wiki.add_edge('Windows Application Programming Interface', 'DirectX')
wiki.add_edge('Windows Application Programming Interface', 'Graphics Device Interface')
wiki.add_edge('Windows Application Programming Interface', 'HX DOS Extender')
wiki.add_edge('Windows Application Programming Interface', 'Native API')
wiki.add_edge('Windows Application Programming Interface', 'OpenGL')
wiki.add_edge('Windows Application Programming Interface', 'Video for Windows')
wiki.add_edge('Windows Live Messenger', 'Skype')
wiki.add_edge('Windows Live Messenger', 'Telepathy')
wiki.add_edge('Windows Live Messenger', 'Yahoo Messenger')
wiki.add_edge('Wire (Messenger)', 'Messaging Layer Security')
wiki.add_edge('Wired Equivalent Privacy', 'Skype')
wiki.add_edge('Wired Equivalent Privacy', 'WPA2')
wiki.add_edge('Wired Equivalent Privacy', 'WPA3')
wiki.add_edge('X Window System', 'EGL (Programmierschnittstelle)')
wiki.add_edge('X Window System', 'Liste der standardisierten Ports')
wiki.add_edge('X Window System', 'Liste von TCP/IP-basierten Netzwerkdiensten')
wiki.add_edge('X Window System', 'OpenGL')
wiki.add_edge('X Window System', 'OpenGL Utility Toolkit')
wiki.add_edge('X Window System', 'Remote Desktop Protocol')
wiki.add_edge('X Window System', 'Remote Framebuffer Protocol')
wiki.add_edge('X Window System', 'Virtual Network Computing')
wiki.add_edge('X Window System', 'Vulkan (API)')
wiki.add_edge('X.25', 'AppleTalk')
wiki.add_edge('X.25', 'Data Link Control')
wiki.add_edge('X.25', 'Frame Relay')
wiki.add_edge('X.25', 'High-Level Data Link Control')
wiki.add_edge('X.25', 'Link Access Procedure Balanced')
wiki.add_edge('X.25', 'Netzwerkprotokoll')
wiki.add_edge('X.25', 'OSI-Modell')
wiki.add_edge('X.25', 'Transmission Control Protocol/Internet Protocol')
wiki.add_edge('X.25', 'Tunnel (Rechnernetz)')
wiki.add_edge('X.25', 'Unix to Unix Copy')
wiki.add_edge('X.25', 'X.75')
wiki.add_edge('X.500', 'Universally Unique Identifier')
wiki.add_edge('X10 (Protokoll)', 'CEBus')
wiki.add_edge('XFire (Framework)', 'Apache Axis')
wiki.add_edge('XFire (Framework)', 'Streaming API for XML')
wiki.add_edge('XFire (Framework)', 'Xfire')
wiki.add_edge('XML-RPC', 'Blocks Extensible Exchange Protocol')
wiki.add_edge('XML-RPC', 'Burlap')
wiki.add_edge('XML-RPC', 'GroupDAV')
wiki.add_edge('XML-RPC', 'Hessian (Webprotokoll)')
wiki.add_edge('XML-RPC', 'JSON-RPC')
wiki.add_edge('XML-RPC', 'Jakarta XML RPC')
wiki.add_edge('XMLHttpRequest', 'ActiveX')
wiki.add_edge('XMLHttpRequest', 'Webservice')
wiki.add_edge('Yahoo Messenger', 'OSCAR (Protokoll)')
wiki.add_edge('Yahoo Messenger', 'Skype')
wiki.add_edge('Z-Wave', 'Thread (Netzwerkprotokoll)')
wiki.add_edge('Zcash', 'Liste von Kryptowährungen')
wiki.add_edge('Zcash', 'Privacy Coin')
wiki.add_edge('Zeitverlaufsdiagramm', 'StarUML')
wiki.add_edge('ZeroNet', 'InterPlanetary File System')
wiki.add_edge('Zeroconf', '0.0.0.0')
wiki.add_edge('Zeroconf', 'Address Resolution Protocol')
wiki.add_edge('Zeroconf', 'AppleTalk')
wiki.add_edge('Zeroconf', 'BitTorrent')
wiki.add_edge('Zeroconf', 'Bonjour (Apple)')
wiki.add_edge('Zeroconf', 'Dienst (Informatik)')
wiki.add_edge('Zeroconf', 'Digital Audio Access Protocol')
wiki.add_edge('Zeroconf', 'Ethernet')
wiki.add_edge('Zeroconf', 'Fiber Distributed Data Interface')
wiki.add_edge('Zeroconf', 'Gnome Virtual File System')
wiki.add_edge('Zeroconf', 'Internet Printing Protocol')
wiki.add_edge('Zeroconf', 'JetDirect')
wiki.add_edge('Zeroconf', 'Link-local Multicast Name Resolution')
wiki.add_edge('Zeroconf', 'NAT Port Mapping Protocol')
wiki.add_edge('Zeroconf', 'NetBIOS')
wiki.add_edge('Zeroconf', 'Netzmaske')
wiki.add_edge('Zeroconf', 'Private IP-Adresse')
wiki.add_edge('Zeroconf', 'Service Location Protocol')
wiki.add_edge('Zeroconf', 'Simple Service Discovery Protocol')
wiki.add_edge('Zeroconf', 'Streaming-Protokoll')
wiki.add_edge('Zeroconf', 'Token Ring')
wiki.add_edge('Zeroconf', 'Universal Plug and Play')
wiki.add_edge('ZigBee', '6LoWPAN')
wiki.add_edge('ZigBee', 'Highway Addressable Remote Transducer')
wiki.add_edge('ZigBee', 'MQTT')
wiki.add_edge('ZigBee', 'Routing Protocol for Low power and Lossy Networks')
wiki.add_edge('ZigBee', 'Z-Wave')
wiki.add_edge('ZigBee', 'Zigbee')
wiki.add_edge('Zigbee', 'Thread (Netzwerkprotokoll)')
wiki.add_edge('Zone Information Protocol', 'Datagram Delivery Protocol')
wiki.add_edge('Zone Information Protocol', 'EtherTalk Link Access Protocol')
wiki.add_edge('Zone Information Protocol', 'LocalTalk')
wiki.add_edge('Zone Information Protocol', 'LocalTalk Link Access Protocol')
wiki.add_edge('Zone Information Protocol', 'Routing Table Maintenance Protocol')
wiki.add_edge('Zone Information Protocol', 'TokenTalk Link Access Protocol')
wiki.add_edge('Zustandsbehaftung', '464XLAT')
wiki.add_edge('Zustandsbehaftung', 'Diameter (Protokoll)')
wiki.add_edge('Zustandsbehaftung', 'JSON Meta Application Protocol')
wiki.add_edge('Zustandsdiagramm (UML)', 'Classifier (UML)')
wiki.add_edge('Zustandsdiagramm (UML)', 'Innovator (Software)')
wiki.add_edge('Zustandsdiagramm (UML)', 'Interaktion (UML)')
wiki.add_edge('Zustandsdiagramm (UML)', 'Merkmal (UML)')
wiki.add_edge('Zustandsdiagramm (UML)', 'Sequenzdiagramm')
wiki.add_edge('Zustandsdiagramm (UML)', 'UMLet')
wiki.add_edge('Zustandsdiagramm (UML)', 'Verhalten (UML)')
wiki.add_edge('Zustandsdiagramm (UML)', 'Violet (UML-Werkzeug)')
wiki.add_edge('Zustandslosigkeit', '464XLAT')
wiki.add_edge('Zustandslosigkeit', 'Diameter (Protokoll)')
wiki.add_edge('Zustandslosigkeit', 'Distributed Application Runtime')
wiki.add_edge('Zustandslosigkeit', 'Network File System')
wiki.add_edge('Zustandslosigkeit', 'Remote Framebuffer Protocol')
wiki.add_edge('Zustandslosigkeit', 'Representational State Transfer')
wiki.add_edge('Zustandslosigkeit', 'Zustandsbehaftung')
wiki.add_edge('Zuständigkeitskette', 'Kopplung (Softwareentwicklung)')
wiki.add_edge('Zwischenablage', 'Dynamic Data Exchange')
wiki.add_edge('Zwischenablage', 'Framework')
wiki.add_edge('Zwischenablage', 'UMLet')
wiki.add_edge('Zwischenablage', 'X Window System')

wiki_pos = {
    'Document Object Model': (3915.0457, 4130.632),
    'Programmierschnittstelle': (7587.9976, -11276.758),
    'Objektmodell': (11312.479, -1943.6477),
    'Hypertext Transfer Protocol': (758.3048, 1516.5432),
    'XMLHttpRequest': (4133.229, 3637.2913),
    'Simple API for XML': (3634.6606, 5307.47),
    'Domain Name System': (-4670.1, -6903.281),
    'Internetprotokollfamilie': (-2158.5735, 3130.4988),
    'Datagram Transport Layer Security': (-3022.8115, -2758.7588),
    'Internet Protocol': (-8980.173, -1229.614),
    'IP-Adresse': (-9462.476, -12625.0),
    'IPv4': (-13043.15, -5305.854),
    'IPv6': (-11618.364, -8136.1836),
    'User Datagram Protocol': (-6806.134, -3418.2366),
    'Port (Protokoll)': (-1488.7637, -3221.103),
    'Transmission Control Protocol': (-1855.7738, 5453.415),
    'Simple Mail Transfer Protocol': (388.16568, -516.06714),
    'Dynamic Host Configuration Protocol': (-4385.5005, -2186.545),
    'DHCPv6': (-1569.5028, -955.57074),
    'Subnetz': (-10389.32, -4261.156),
    'Telephone Number Mapping': (-2414.1072, -5445.76),
    'X.25': (-5190.174, 4085.0725),
    'Client': (1262.0851, -4767.976),
    'Transport Layer Security': (1313.1759, -1802.6613),
    'Internet Assigned Numbers Authority': (-6093.0093, -5000.122),
    'Internet Corporation for Assigned Names and Numbers': (-6350.4585, -6601.828),
    'Zeroconf': (-5421.8525, -1766.8241),
    'Bonjour (Apple)': (-4273.3154, -1271.2012),
    'Bitcoin': (-1463.8701, -14971.155),
    'Kryptowährung': (-1774.9542, -17933.473),
    'OpenNIC': (-3905.618, -5413.5415),
    'Lightweight Directory Access Protocol': (1985.6465, 249.51521),
    'File Transfer Protocol': (-635.71765, 1972.1193),
    'Netzwerkprotokoll': (-2350.1492, 14735.203),
    'Datenaustausch': (1677.541, 2123.0356),
    'OSI-Modell': (-2787.7007, 9743.062),
    'File Exchange Protocol': (3561.7117, -6853.7207),
    'AS3': (922.4306, 2775.6182),
    'SSH File Transfer Protocol': (-299.36108, 448.76532),
    'Trivial File Transfer Protocol': (-1932.8024, 950.8148),
    'Zustandslosigkeit': (-837.7025, 147.92755),
    'Hypertext Transfer Protocol Secure': (472.05948, -1362.1368),
    'WebDAV': (29.692707, 595.1111),
    'Common Gateway Interface': (5313.017, -1570.6974),
    'Keepalive': (-5780.7314, 33.403854),
    'Internet Media Type': (1939.9888, -888.0716),
    'SPDY': (-4221.128, 6.2195225),
    'QUIC': (-4778.4067, -259.06964),
    'Facebook': (-145.07379, -7092.8687),
    'Representational State Transfer': (2722.168, 2526.4102),
    'Webservice': (4520.0146, 4927.7974),
    'CRUD': (7725.255, -1560.3638),
    'SOAP': (2658.9272, 5274.7354),
    'S-HTTP': (-6557.0845, 387.6275),
    'HTTPS Everywhere': (799.4331, -1496.3024),
    'CAcert': (-29.409424, 49.071526),
    'OpenSSL': (-117.09566, -1845.1239),
    'Server Name Indication': (-4357.772, -8605.905),
    'Heartbleed': (-322.65784, -1715.0905),
    'Internet Relay Chat': (-445.466, -4998.4404),
    'Direct Client-to-Client': (-327.54587, -4871.1987),
    'Off-the-Record Messaging': (-637.74054, -6341.5557),
    'Extensible Messaging and Presence Protocol': (-517.8236, -5820.475),
    'SILC': (5039.688, -5956.081),
    'IP-Paket': (-9414.543, -2216.405),
    'Netzklasse': (-10487.431, -3918.0835),
    'Netzmaske': (-10773.624, -3930.0598),
    'Ethernet': (-12559.066, 11338.47),
    'Address Resolution Protocol': (-7161.1367, -1045.1804),
    'Netzwerkadressübersetzung': (-5905.1973, -3736.1743),
    'Classless Inter-Domain Routing': (-10976.831, -3630.4875),
    'Private IP-Adresse': (-7333.982, -5007.7764),
    'Carrier-grade NAT': (-8157.8315, -5991.416),
    '6to4': (-9257.19, -4755.546),
    'Secure Shell': (-508.13937, 146.75923),
    'Bootstrap Protocol': (-2065.1492, 657.12427),
    'Multicast Backbone': (-11429.554, 1666.9504),
    'Asia-Pacific Network Information Centre': (-7797.72, -7111.8),
    'RIPE Network Coordination Centre': (-7090.3403, -6539.016),
    'Regional Internet Registry': (-7712.217, -6849.1973),
    'American Registry for Internet Numbers': (-8160.45, -6183.291),
    'Local Internet Registry': (-7384.9204, -6844.148),
    'Whois': (-5627.126, -6109.1562),
    'Point-to-Point Protocol': (-5808.091, 1952.1937),
    'Internet Control Message Protocol': (-9387.651, 1323.1105),
    'Mobile IP': (-6889.446, -429.8534),
    'Protokollstapel': (-3138.3616, 1262.3618),
    'OPENSTEP': (5117.3945, -6043.578),
    'OpenStep': (5149.854, -6019.9478),
    'Cocoa (API)': (5097.6787, -5975.6763),
    'Carbon (Apple)': (5155.698, -6068.5264),
    'Schichtenarchitektur': (6423.403, 2450.9648),
    'Logical Link Control': (-9121.954, 6936.7783),
    'Media Access Control': (-9588.486, 7273.5034),
    'High-Level Data Link Control': (-5281.7056, 3700.8564),
    'Synchronous Data Link Control': (-4992.8555, 3752.332),
    'Digital Data Communications Message Protocol': (-4987.77, 3823.694),
    'Radio Link Control': (-4309.0127, 1867.145),
    'Packet Data Convergence Protocol': (-4375.358, 1825.9875),
    'Reverse Address Resolution Protocol': (-2380.6584, 512.37726),
    'Spanning Tree Protocol': (-11680.537, 6052.511),
    'Token Bus': (-7927.272, 4142.2363),
    'Token Ring': (-8718.097, 5385.5503),
    'Fiber Distributed Data Interface': (-7813.118, 5352.322),
    'IP-Fragmentierung': (-9950.403, 312.11108),
    'Connectionless Network Protocol': (-9651.328, 355.63284),
    'IPsec': (-5758.221, -194.63773),
    'Service Data Unit': (-3111.2346, 3222.6838),
    'Stream Control Transmission Protocol': (-3021.2908, -691.6227),
    'Datagram Congestion Control Protocol': (-4515.792, -591.94684),
    'Remote Procedure Call': (2292.1648, 1622.135),
    'IP-Header': (-9980.799, -291.0807),
    'IEEE 802.1Q': (-11556.97, 5661.151),
    'EtherType': (-10929.062, 3419.9197),
    'AppleTalk': (-6353.31, 4989.3447),
    'Internetwork Packet Exchange': (-9295.811, 3318.61),
    'Interface Control Information': (-3091.5176, 3263.2002),
    'Telnet': (-468.01895, -386.3306),
    'Tn3270': (-373.6717, -508.77765),
    'Remote Shell': (-474.83014, -121.28735),
    'TWAIN': (2942.943, -3091.6282),
    'Image and Scanner Interface Specification': (2968.9148, -3084.3975),
    'Scanner Access Now Easy': (2941.407, -3060.2002),
    'Zwischenablage': (5588.358, -2450.4543),
    'X Window System': (2450.8691, -3168.09),
    'Framework': (6073.295, -2839.0637),
    'World IPv6 Day und World IPv6 Launch Day': (-3588.6758, -10057.041),
    'Serial Line Internet Protocol': (-5124.8794, 1687.1198),
    'Point-to-Point Tunneling Protocol': (-6276.579, 970.2942),
    'Vermittlungsschicht': (-10349.401, 2846.2097),
    'Appletalk': (-11829.3545, -1478.7667),
    'Internet Protocol Control Protocol': (-9011.006, 2868.2776),
    '0.0.0.0': (-2854.5583, -1116.2048),
    'CSLIP': (-4681.4536, 1731.1259),
    'Apple Filing Protocol': (-6662.8286, 6116.056),
    'Zone Information Protocol': (-7203.2695, 6552.8643),
    'Routing Table Maintenance Protocol': (-7306.3633, 6280.057),
    'Datagram Delivery Protocol': (-7309.1665, 6431.359),
    'EtherTalk Link Access Protocol': (-7250.69, 6653.8223),
    'IEEE 802': (-11224.544, 6996.8994),
    'Echtzeit-Ethernet': (-7351.6895, 9460.571),
    'IEEE 802.12': (-11128.651, 7623.726),
    'Industrial Ethernet': (-7201.2793, 9321.302),
    'ALOHA': (-9825.612, 8611.93),
    'Full-duplex Ethernet': (-12925.306, 8761.768),
    'Fibre Channel over Ethernet': (-14274.378, 4902.499),
    'Virtual Local Area Network': (-11111.854, 5638.696),
    'Class of Service': (-8860.785, 384.31778),
    'Jumbo Frames': (-12368.878, 4686.24),
    'PROFINET': (-6884.7856, 9354.906),
    'ATA over Ethernet': (-14394.685, 4887.058),
    'EtherCAT': (-5956.8286, 8423.968),
    'Ethernet Powerlink': (-7444.965, 9155.005),
    'Generic Object Oriented Substation Events': (-6804.952, 3295.694),
    'LLDP': (-8203.747, 8812.401),
    'Sercos III': (-7184.062, 9423.1045),
    'Ethernet-Anschlussdose': (-12832.172, 8821.365),
    '10BASE5': (-12821.804, 8921.44),
    'Attachment Unit Interface': (-12862.325, 8896.601),
    'BroadR-Reach': (-12904.67, 8804.726),
    'IEEE 802.3ae': (-11563.382, 7495.25),
    'IEEE 802.3ak': (-11576.757, 7521.45),
    'IEEE 802.3an': (-13598.47, 9537.001),
    'Power over Ethernet': (-10412.904, 10106.788),
    'Autonegotiation': (-12972.331, 8812.754),
    'RJ-Steckverbindung': (-10450.119, 10094.465),
    'Entwurfsmuster': (11915.931, 11678.021),
    'Entwurfsmuster (Buch)': (10045.036, 5404.84),
    'Softwarearchitektur': (9637.029, 1686.1354),
    'Erzeugungsmuster': (10799.015, 6054.2544),
    'Strukturmuster': (9819.525, 5066.878),
    'Verhaltensmuster (Software)': (10052.711, 6376.671),
    'Nachrichtenübermittlungsmuster': (10083.908, 2088.8574),
    'Idiom (Softwaretechnik)': (7552.1157, 1193.5187),
    'Mustersprache': (9009.393, 1257.5917),
    'Architekturmuster': (7510.368, 1175.7667),
    'Erbauer (Entwurfsmuster)': (10854.375, 5758.757),
    'Fabrikmethode': (10577.203, 5648.878),
    'Multiton': (10669.146, 6066.051),
    'Prototyp (Entwurfsmuster)': (11061.276, 5088.6016),
    'Container (Entwurfsmuster)': (9449.038, 4985.2183),
    'Decorator': (10339.671, 5091.9297),
    'Fassade (Entwurfsmuster)': (9018.222, 4172.0073),
    'Fliegengewicht (Entwurfsmuster)': (10420.754, 5408.781),
    'Kompositum (Entwurfsmuster)': (10459.205, 5211.9746),
    'Stellvertreter (Entwurfsmuster)': (10163.111, 5017.974),
    'Interceptor (Entwurfsmuster)': (10060.951, 7473.6714),
    'Interpreter (Entwurfsmuster)': (10707.731, 5510.856),
    'Iterator (Entwurfsmuster)': (10641.019, 5423.5786),
    'Kommando (Entwurfsmuster)': (10678.246, 4700.351),
    'Memento (Entwurfsmuster)': (9991.097, 5814.173),
    'Nullobjekt (Entwurfsmuster)': (10701.146, 5976.813),
    'Schablonenmethode': (10276.224, 5580.811),
    'Strategie (Entwurfsmuster)': (10264.722, 5479.975),
    'Vermittler (Entwurfsmuster)': (6368.883, 917.9337),
    'Zustand (Entwurfsmuster)': (10078.373, 5820.848),
    'Zuständigkeitskette': (10251.423, 5259.281),
    'MapReduce': (10453.439, 7827.9214),
    'Transferobjekt': (11265.499, 2756.1265),
    'Active Record': (8902.27, -625.65106),
    'Unit of Work': (10577.011, 8107.5557),
    'Command-Query-Responsibility-Segregation': (8927.633, 1731.0482),
    'Message (Entwurfsmuster)': (10063.116, 2107.823),
    'Document Message': (10047.448, 2135.2566),
    'Extension Interface': (12103.424, 747.11084),
    'Fluent Interface': (11143.6455, 5952.2646),
    'Inversion of Control': (10109.712, 2763.572),
    'Repository (Entwurfsmuster)': (8355.279, -3102.1423),
    'Value Object': (11365.502, 3507.2412),
    'Registry (Entwurfsmuster)': (10648.399, 5898.2515),
    'Model View Controller': (7293.8105, 649.0358),
    'Analysemuster': (13053.5, 3649.606),
    'Anti-Pattern': (8743.088, 2996.6196),
    'Serviceorientierte Architektur': (6495.288, 3398.711),
    'Smart UI': (7524.8184, 1800.7994),
    'Plugin (Entwurfsmuster)': (10126.938, 7164.979),
    'Prinzipien objektorientierten Designs': (8684.272, 2488.2434),
    'Referenzarchitektur': (6439.731, 1238.9174),
    'Serverseitige Anwendung': (758.29346, -474.85303),
    'Real-Time Transport Protocol': (-2742.2913, -4016.9297),
    'Transmission Control Protocol/Internet Protocol': (-5296.1436, 5491.5674),
    'Sequenced Packet Exchange': (-9520.711, 4701.804),
    'TCP Receive Window': (-3061.8284, 1807.9716),
    'NetBEUI': (-8228.371, 4628.8677),
    'BACnet': (-7612.215, 803.39624),
    'Post Office Protocol': (27.008327, -133.87332),
    'Internet Message Access Protocol': (178.9015, -236.55473),
    'BitTorrent': (-2305.09, -8879.397),
    'Liste der standardisierten Ports': (-1508.0261, -767.9035),
    'MIDI': (4507.0654, -6973.365),
    'Network Time Protocol': (-178.5739, -713.83795),
    'Web Proxy Autodiscovery Protocol': (-4656.2285, -1867.3427),
    'Preboot Execution Environment': (-2163.733, 906.04297),
    'IEEE 802.1X': (-2384.2673, 210.50038),
    'Peg DHCP': (-10992.6875, -4460.011),
    'Session Initiation Protocol': (-2350.4731, -5329.707),
    'Simple Mail Access Protocol': (178.28902, -145.93495),
    'Push-Model': (2163.7563, -725.2815),
    'GSSAPI': (-142.02502, -43.90339),
    'STARTTLS': (130.10307, 65.18056),
    'Binärschnittstelle': (4063.9922, 203.4484),
    'Windows Application Programming Interface': (5034.2305, -7034.902),
    'Parameter (Informatik)': (5479.8896, 1772.7175),
    'ActiveX': (3778.9668, 272.56088),
    'Typbibliothek': (3878.3455, 90.04082),
    'Refactoring': (8521.715, 2524.9143),
    'Portable Operating System Interface': (-776.34344, 1373.8839),
    'ASCII-Protokoll': (-143.52458, -197.8777),
    'Simple Authentication and Security Layer': (498.34802, -271.58295),
    'Authenticated Post Office Protocol': (47.820435, -10.838194),
    'POP3S': (0.023978835, 4.530252),
    'Unified Modeling Language': (20195.863, 3528.5754),
    'Systems Modeling Language': (17085.814, -1039.5352),
    'UML-Werkzeug': (16485.143, -1214.3687),
    'Anwendungsfalldiagramm': (17487.889, -167.90294),
    'Aktivitätsdiagramm': (18110.957, -613.06104),
    'Verteilungsdiagramm': (17157.56, 611.61847),
    'Profildiagramm': (17240.018, 1347.9307),
    'Klasse (UML)': (17856.814, 1926.4154),
    'Assoziation (UML)': (16088.207, 2368.5476),
    'Multiplizität (UML)': (17435.256, 1296.0377),
    'Anwendungsfall (UML)': (18082.688, 987.4365),
    'Aktivität (UML)': (18660.06, -122.93306),
    'Zustandsdiagramm (UML)': (17814.93, -140.43456),
    'Object Constraint Language': (15642.903, 323.3391),
    'Datenfluss': (15026.793, 303.111),
    'Kontrollfluss': (15251.68, -445.2609),
    'Pin (UML)': (18784.707, -167.65892),
    'Aktion (UML)': (18635.92, 74.74195),
    'Verhalten (UML)': (18696.758, 411.00653),
    'Objektknoten (UML)': (18948.352, -105.05368),
    'Kontrollknoten (UML)': (18922.043, -173.99767),
    'Objektfluss (UML)': (18814.346, -62.040024),
    'Kontrollfluss (UML)': (18851.94, -168.37895),
    'Interaktion (UML)': (18082.725, -29.825668),
    'Ereignis (UML)': (18055.287, -315.25195),
    'Akteur (UML)': (18053.797, 915.8926),
    'Informationseinheit (UML)': (18262.076, 1891.7198),
    'Informationsfluss (UML)': (18294.5, 1924.978),
    'Ausprägungsspezifikation (UML)': (17230.732, 643.22986),
    'Schnittstelle (UML)': (17970.572, 1569.8381),
    'Port (UML)': (18186.19, 1450.0981),
    'Sequenzdiagramm': (17533.098, -605.9734),
    'Kommunikationsdiagramm (UML)': (17511.906, -38.615147),
    'Zeitverlaufsdiagramm': (17229.715, -205.18668),
    'Kommentar (UML)': (17191.479, 1331.2343),
    'Namensraum (UML)': (17850.904, 1013.1876),
    'Paket (UML)': (17708.988, 863.4106),
    'Attribut (UML)': (17435.273, 1421.4624),
    'Paketimport (UML)': (17734.504, 732.404),
    'Paketverschmelzung': (17764.71, 756.06836),
    'Generalisierung (UML)': (17933.336, 1067.8705),
    'Assoziationsklasse': (14524.249, 146.41815),
    'Klassendiagramm': (16259.204, 1007.8688),
    'Objektdiagramm': (17040.236, 147.77148),
    'Paketdiagramm': (17727.154, 661.8315),
    'Komponente (UML)': (18013.775, 1348.2291),
    'Konnektor (UML)': (17839.148, 1636.1547),
    'Komponentendiagramm': (17913.3, 1432.4779),
    'Classifier (UML)': (18279.02, 1053.9717),
    'Kompositionsstrukturdiagramm': (17809.682, 1356.5676),
    'Modell (UML)': (17834.135, 789.85736),
    'Artefakt (UML)': (17181.564, 1000.131),
    'Stereotyp (UML)': (17225.648, 1310.0355),
    'Profil (UML)': (17674.63, 751.5289),
    'Knoten (UML)': (17377.82, 809.4647),
    'Operation (UML)': (17649.29, 1418.6721),
    'Interaktionsübersichtsdiagramm': (17727.373, -273.0473),
    'MOF QVT': (15330.285, 64.69195),
    'Modeling and Analysis of Real Time and Embedded systems': (16892.793, -1160.1855),
    'Fundamental Modeling Concepts': (12190.198, 762.9302),
    'IDEF': (12840.737, -350.68015),
    'BNC-Steckverbinder': (-12754.124, 8873.782),
    '10BASE2': (-12803.039, 8865.715),
    'Secure Copy': (-325.04254, 594.62244),
    'Portweiterleitung': (-4476.965, -4288.6396),
    'Virtual Network Computing': (190.60687, -1000.43005),
    'SOCKS': (-2267.8142, -1557.0743),
    'Tunnel (Rechnernetz)': (-6792.179, 409.77222),
    'Simple Network Management Protocol': (-3604.2542, 2474.5474),
    'Server Message Block': (-6860.4097, 4174.4536),
    'NetBIOS': (-7414.078, 4147.2124),
    'NetBIOS over TCP/IP': (-7646.5503, 4562.9956),
    'Network File System': (-1777.067, 1365.7848),
    'Link-local Multicast Name Resolution': (-7116.2, 3736.157),
    'Variable Length Subnet Mask': (-11945.069, -2128.849),
    'Port Address Translation': (-4354.9976, -5109.2837),
    'Datagramm': (-6824.8896, 2390.4778),
    'Component Object Model': (3897.0513, 147.92075),
    'Object Linking and Embedding': (4149.1704, 243.36864),
    'DirectX': (4896.0557, -7177.67),
    'Common Object Request Broker Architecture': (4127.008, 1113.8326),
    'Dienst (Informatik)': (6604.1675, 1506.4984),
    'Z39.50': (-1067.7853, 10567.197),
    'Enterprise Application Integration': (10383.307, 3961.9514),
    'TELNET': (-453.86963, -683.6919),
    'Internetwork Datagram Protocol': (-10366.843, 2779.0073),
    'NetWare Core Protocol': (-9246.287, 4728.015),
    'Service Advertising Protocol': (-5795.374, -1168.0483),
    'Routing Information Protocol': (-12341.237, -1011.67596),
    'Internetwork Packet eXchange Fragmentation': (-9020.075, 4566.093),
    'Link-State': (-12144.518, -831.67505),
    'NetWare Link-Services Protocol': (-12040.896, -978.98254),
    'Wired Equivalent Privacy': (-619.4175, -2876.5967),
    'WPA3': (-587.2332, -2790.2893),
    'Controller Area Network': (-5023.252, 9844.527),
    'ISO 15765-2': (-5057.254, 9990.367),
    'CANopen': (-5887.218, 9169.63),
    'ISOBUS': (-3661.7979, 7242.654),
    'CANaerospace': (-4708.5938, 10269.7295),
    'ARINC 825': (-4736.338, 10309.051),
    'Unified Diagnostic Services': (-4993.7275, 9975.397),
    'KWP2000': (-5020.0244, 10012.879),
    'FlexRay': (-5222.3135, 9975.416),
    'Border Gateway Protocol': (-11473.243, -2325.1135),
    'Exterior-Gateway-Protokoll': (-11736.161, -1079.4493),
    'Pfadvektorprotokoll': (-12407.407, -1480.5079),
    'Interior Gateway Protocol': (-11743.499, -1147.7623),
    'IS-IS': (-11799.345, -773.61414),
    'Enhanced Interior Gateway Routing Protocol': (-11886.251, -1496.576),
    'Interior Gateway Routing Protocol': (-12143.851, -1462.8887),
    'Distanzvektoralgorithmus': (-12353.861, -1421.3348),
    'Whatsapp': (-985.2004, -7531.9883),
    'Homebanking Computer Interface': (552.95416, 2807.8457),
    'Electronic Banking Internet Communication Standard': (583.96454, 2850.4954),
    'Daytime': (-639.52625, -509.56342),
    'Komponente (Software)': (4971.2803, 397.1615),
    'ActiveX Data Objects': (8207.737, -3346.5388),
    'Killbit': (3790.9666, 216.67255),
    'Open Database Connectivity': (8344.245, -3421.234),
    'Datenbankschnittstelle': (8246.787, -3607.5637),
    'Call Level Interface': (7998.7285, -2028.0083),
    'Data Access Objects': (8280.352, -3279.169),
    'OLE DB': (8323.704, -3260.4092),
    'Nagle-Algorithmus': (-415.8883, 336.23917),
    'Java Database Connectivity': (7876.6597, -2078.1199),
    'Java Community Process': (5181.537, -166.11218),
    'Perl-DBI': (8121.2314, -3568.6895),
    'XML-RPC': (2491.4756, 1920.8116),
    'Web Services Description Language': (5019.002, 6345.488),
    'Web Map Service': (1909.7715, 9415.015),
    'Universal Description, Discovery and Integration': (5664.697, 6653.2817),
    'Remote Method Invocation': (3708.6687, 1933.326),
    'Security Assertion Markup Language': (6456.273, 5856.3564),
    'WS-Business Process Execution Language': (5606.816, 6985.05),
    'Geodienst': (1930.4558, 9450.535),
    'Semantic Web Services': (4446.668, 8494.541),
    'SoapUI': (5848.911, -417.43814),
    'Entity-Relationship-Modell': (13721.719, -339.54434),
    'Semantisches Datenmodell': (12603.154, -409.7844),
    'Datenmodellierung': (12776.817, -477.26758),
    'Datenmodell': (12594.48, -777.2351),
    'Entität (Informatik)': (13138.984, -270.86203),
    'Schwache Entität': (13057.687, -485.95828),
    'Structured-Entity-Relationship-Modell': (13197.582, -581.45917),
    'Kardinalität (Datenbankmodellierung)': (12862.832, -668.85376),
    'Datenbankmodell': (12951.259, -1106.5237),
    'IDEF1X': (13022.232, -478.72324),
    'Martin-Notation': (12900.196, -599.38477),
    'Min-Max-Notation': (13150.674, -574.9627),
    'Liste von Datenmodellierungswerkzeugen': (14873.887, -1707.5356),
    'WS-Security': (4932.47, 7457.637),
    'Lose Kopplung': (7535.9077, 2585.367),
    'Apache CXF': (4510.9043, 6722.654),
    'XFire (Framework)': (3624.6538, 5065.6636),
    'Apache Axis': (4047.0916, 6284.839),
    'WS-Reliability': (4768.07, 8322.054),
    'WS-*': (5099.8555, 8234.026),
    'TR-069': (981.70654, 308.40738),
    'Hessian (Webprotokoll)': (2714.7856, 1878.1136),
    'Burlap': (2648.733, 1956.7596),
    'SOAP Message Transmission Optimization Mechanism': (4047.121, 7215.6836),
    'SOAP with Attachments': (4003.3828, 7181.886),
    'Komponentenbasierte Entwicklung': (10511.068, 58.65492),
    'Internet Communications Engine': (3821.3342, 1237.7996),
    'Neighbor Discovery Protocol': (-10730.443, 799.5691),
    'ARP-Cache': (-3528.907, -23.7953),
    'Programmablaufplan': (15277.842, -980.8257),
    'ICMPv6': (-10739.91, 969.5307),
    'ARP-Spoofing': (-3495.0251, -1.2702141),
    'FastCGI': (5348.548, -1613.7759),
    'Simple Common Gateway Interface': (5404.9478, -1593.9448),
    'Macintosh-Baukasten': (5198.929, -6047.165),
    'OpenGL': (5126.871, -7537.264),
    'Vulkan (API)': (5185.6694, -7874.6157),
    'Java Native Interface': (3535.0007, 212.67505),
    'Brücke (Entwurfsmuster)': (8824.527, 4617.8057),
    'Pull-Model': (3709.1033, 4778.342),
    'X.500': (2484.2473, -123.695496),
    'Schema (Informatik)': (12086.124, -966.1114),
    'Filter (Software)': (3130.2097, 693.7214),
    'Pluggable Authentication Modules': (7886.8433, 3679.0032),
    'Normalisierung (Datenbank)': (12292.774, -631.61304),
    'Kerberos (Protokoll)': (178.33365, -46.246304),
    'LocalTalk': (-7049.8403, 6441.5347),
    'JetDirect': (-6970.4824, 5361.195),
    'Internet Printing Protocol': (-4456.3037, 2893.0632),
    'Socket (Software)': (-2543.3877, -1265.6323),
    'LocalTalk Link Access Protocol': (-7368.7876, 6597.5713),
    'TokenTalk Link Access Protocol': (-7160.2227, 6730.1875),
    'Kinetics Internet Protocol': (-6007.1143, 5905.114),
    'Fast Ethernet': (-7695.0527, 8022.4927),
    'OPC Unified Architecture': (220.02562, 4942.925),
    'DoIP': (-4944.169, 9912.342),
    'Internet Protocol Flow Information Export': (-4318.197, 880.167),
    'Multi-purpose Business Security over IP': (-193.35968, 1563.582),
    'Network News Transfer Protocol': (-6.181427, -377.48035),
    'Remote Desktop Protocol': (-382.044, -1137.0533),
    'Open Shortest Path First': (-11866.693, -1099.7223),
    'Internet Group Management Protocol': (-11191.777, 1545.0337),
    'Layer 2 Tunneling Protocol': (-5883.565, 777.9164),
    'Internet Protocol over Avian Carriers': (-11055.71, -4463.764),
    'Zustandsbehaftung': (-1022.30005, 27.719337),
    'Teredo': (-9317.832, -4858.7236),
    '6Bone': (-11505.155, 1641.9628),
    'NAT64': (-8119.694, -5656.2954),
    'DiffServ': (-8582.4795, -2448.741),
    'Explicit Congestion Notification': (-8109.776, -2051.392),
    'Protokoll (IP)': (-10348.1455, -233.79582),
    '6in4': (-10418.868, -2245.5437),
    '6over4': (-12955.838, -6526.059),
    'ISATAP': (-9413.492, -4810.439),
    'AYIYA': (-10239.111, -3666.0679),
    'Generic Routing Encapsulation': (-8340.784, 1264.5791),
    'KAME-Projekt': (-5888.004, -367.00113),
    'Resource Reservation Protocol': (-8357.58, -2672.3103),
    'Label Distribution Protocol': (-8505.862, -7688.44),
    'Protocol Independent Multicast': (-12204.751, 2010.4271),
    'Netflow': (-4396.8833, 895.3658),
    'Liste von IPv6-Tunnelbrokern': (-10226.002, -3636.0056),
    'Peer Name Resolution Protocol': (-8867.378, -4317.532),
    'Anomalie (Informatik)': (11966.908, -581.3425),
    'Synthesealgorithmus': (12472.768, -607.48065),
    'Denormalisierung': (11917.176, -582.4211),
    'Sternschema': (11995.586, -547.0682),
    'Ablaufdiagramm': (15321.396, -1199.4932),
    'Nassi-Shneiderman-Diagramm': (15201.838, -1121.6176),
    'Jackson-Diagramm': (15248.429, -1168.2009),
    'Bildschirmschoner': (758.3049, -14044.155),
    'Unix to Unix Copy': (-200.12256, 172.0456),
    'ICMP-Tunnel': (-10529.762, 1120.6223),
    'Unified Extensible Firmware Interface': (-2088.7297, 942.37177),
    'Liste von TCP/IP-basierten Netzwerkdiensten': (-127.625404, -449.54486),
    'Messaging Application Programming Interface': (758.30475, 742.1259),
    'Vendor Independent Messaging': (788.3134, 747.3253),
    'File eXchange Protocol': (3587.9424, -6870.726),
    'WinG': (5145.8276, -7221.7104),
    'AMD Mantle': (5240.3203, -7806.703),
    'Graphics Device Interface': (5125.801, -7182.4155),
    'Winsock': (-1740.3329, -1804.4575),
    'OpenAL': (4848.7744, -7606.507),
    'OpenCL': (5015.3184, -7859.938),
    'OpenML': (5013.154, -7370.7812),
    'TP/IX': (-14316.011, -9148.177),
    'IRIS GL': (5921.1484, -9289.721),
    'OpenGL Utility Toolkit': (2816.434, -3832.291),
    'Open Graphics Library for Embedded Systems': (4719.6816, -8167.661),
    'Java 3D': (5500.1104, -1277.0133),
    'IEEE 802.1aq': (-12341.392, 3375.052),
    'Web Application Description Language': (3924.3538, 5885.425),
    'Binärstandard': (4005.8538, 166.91823),
    'Netfilter': (-8070.0938, 1242.2479),
    'Universal Plug and Play': (-4459.9395, -3763.888),
    'IGD-Protokoll': (-4512.7085, -4138.863),
    'IP-Spoofing': (-4773.942, 538.2143),
    'Sequenznummer': (-4300.484, 630.56946),
    'Embedded SQL': (8426.868, -3085.8806),
    'ADO.NET': (8380.065, -3126.443),
    'Swing (Java)': (5769.2383, -396.80362),
    'Java Foundation Classes': (5505.529, -1224.0801),
    'Java Accessibility API': (5585.926, -1010.6483),
    'Abstract Window Toolkit': (6004.0254, 103.65641),
    'JavaFX': (5994.811, 160.1917),
    'Time Token Rotation Protocol': (-9624.369, 7455.911),
    'ANSI-SPARC-Architektur': (12125.063, -770.53845),
    'Data-Dictionary': (13725.524, -776.6206),
    'Web Feature Service': (1867.6317, 9428.19),
    'Web Map Tile Service': (1956.5122, 9403.314),
    'Code-Smell': (8538.254, 3078.087),
    'Softwaredesign': (12169.229, 717.07465),
    'Modul (Software)': (8580.133, 2034.1743),
    'Kopplung (Softwareentwicklung)': (8839.347, 2595.066),
    'OpenDBX': (8130.0815, -3681.1453),
    'Borland Database Engine': (8354.66, -3531.12),
    'EDonkey2000': (-1532.5663, -7859.5503),
    'FastTrack': (-1492.9675, -7802.0547),
    'Protocol Data Unit': (-3165.5557, 3206.8345),
    'Agent Extensibility Protocol': (-3284.291, 2810.9875),
    'SMTPS': (-86.65525, 56.86817),
    'Remote login': (-410.22595, -156.89046),
    'SAAM': (10698.419, 149.34773),
    'Szenariobasierte Architekturbewertung': (10723.092, 126.56812),
    'Network Information Center': (-3991.5237, -5414.0244),
    'Provider Aggregatable Address Space': (-7506.5073, -6861.507),
    'Provider Independent Address Space': (-7554.624, -6827.6387),
    'IEEE 1471': (12340.735, 163.45432),
    'Softwarearchitekt': (13235.7705, 468.29245),
    'Modifizierbarkeit': (12048.305, 982.98865),
    'Architecture Description Language': (19856.746, 8225.191),
    '4+1 Sichtenmodell': (17344.1, 146.7688),
    'Bewertung von Softwarearchitekturen': (10664.241, 141.51028),
    'Modularität': (9782.168, -215.45761),
    'Streaming API for XML': (3681.648, 5064.5503),
    'FTZ 1 TR 6': (-2795.8179, -2193.7961),
    'Digital Subscriber System No. 1': (-2785.9487, -2156.0251),
    'Link Access Procedure for the D-Channel': (-5037.0776, 3465.6572),
    'IP-Multicast': (50.312195, 862.0283),
    'Distance Vector Multicast Routing Protocol': (-12129.063, 2014.503),
    'MOSPF': (-12136.166, 1956.9711),
    'Schneeflockenschema': (11936.421, -450.93295),
    'Aggregation (Informatik)': (11186.042, 1041.938),
    'Slowly Changing Dimensions': (12003.397, -489.63052),
    'Galaxy-Schema': (11925.089, -489.49854),
    'Bidirectional Forwarding Detection': (-11694.574, -1026.6855),
    'Microsoft Point-To-Point Encryption Protocol': (-6272.6807, 1110.7081),
    'IEEE 802.1p': (-10159.068, 1387.2483),
    'IEEE 802.1ah-2008': (-11319.577, 3704.3857),
    'Rapid Spanning Tree Protocol': (-11343.427, 6474.0386),
    '10BASE-T': (-12937.194, 8848.33),
    'Gigabit Ethernet': (-12408.768, 4831.3003),
    'Link Aggregation': (-11993.033, 6073.885),
    '10-Gigabit-Ethernet': (-11546.787, 7468.044),
    'Energy Efficient Ethernet': (-10518.098, 10075.2705),
    'Distributed Queue Dual Bus': (-11613.474, 7920.5776),
    'ZigBee': (-9033.646, 801.32007),
    'Resilient Packet Ring': (-10707.486, 13272.608),
    'Gopher': (-1516.5074, -4080.3423),
    'Geminispace': (1854.9124, -963.2122),
    'Merkmal (UML)': (18333.578, 1301.8771),
    'Windows Live Messenger': (-598.59607, -6452.261),
    'Yahoo Messenger': (-665.6126, -6418.965),
    'Skype': (-797.1081, -6602.1147),
    'Hot Standby Router Protocol': (-8060.287, 2621.3435),
    'Advanced Peer-to-Peer Networking': (-5499.85, 3118.1252),
    'Common Address Redundancy Protocol': (-8291.922, 2482.6504),
    'Virtual Router Redundancy Protocol': (-8232.774, 2690.1506),
    'CalDAV': (250.65794, 461.73035),
    'CardDAV': (187.31357, 408.68817),
    'GroupDAV': (696.2771, 742.51056),
    'Common ISDN Application Programming Interface': (-2664.27, -2204.2297),
    'H.323': (-2423.8455, -4295.3477),
    'Telephony Application Programming Interface': (-2406.8923, -2185.0708),
    'Dateikonverter': (5395.526, -1743.7684),
    'Beobachter (Entwurfsmuster)': (6335.913, 807.7531),
    'H.-Standards': (-2516.4795, -4328.7524),
    'InterAsterisk eXchange': (-1842.3114, -4978.373),
    'Q.931': (-2800.8557, -2228.9065),
    'H.225.0': (-2784.5903, -2336.28),
    'Triggered Updates': (-12286.065, -1456.0372),
    'Geschäftslogik': (8337.272, 1588.0724),
    'Model View ViewModel': (7395.1953, 1610.1133),
    'WebSocket': (764.7064, 84.84168),
    'Remote Presentation Model': (7394.001, 1212.89),
    'Data Context Interaction': (7384.1445, 786.96326),
    'SMTP-Auth': (376.05872, -60.348854),
    'CRAM-MD5': (359.33276, -143.5775),
    'NTLM': (304.656, 143.27777),
    'Remote Function Call': (3564.004, 2347.3765),
    'JSON-RPC': (2349.2087, 1923.5151),
    'Virtual Studio Technology': (4315.047, -7126.064),
    'Audio Unit': (4490.201, -7030.9033),
    'Audio Random Access': (4454.8726, -7021.454),
    'DSSI': (4277.825, -7173.192),
    'Linux Audio Developer’s Simple Plugin API': (4292.0146, -7199.936),
    'LV2': (4257.241, -7197.539),
    'Audio Stream Input/Output': (4514.64, -7064.681),
    'Service Location Protocol': (-4677.859, -1775.0519),
    'WS-Discovery': (4508.413, 8329.361),
    'Top-Down- und Bottom-Up-Design': (15129.782, -1109.2999),
    'SERCOS III': (-7215.742, 9441.562),
    'Java Naming and Directory Interface': (3284.842, 1353.236),
    'SIMON': (3121.1577, 1411.7179),
    'ISDN over IP': (-3029.109, -6440.7056),
    'Session Description Protocol': (-2886.7407, -3406.7126),
    'Secure Real-Time Transport Protocol': (-1684.2722, -6235.013),
    'Interactive Connectivity Establishment': (-341.97357, -7778.0376),
    'Adapter (Entwurfsmuster)': (9080.052, 4569.2974),
    'Delegation (Softwareentwicklung)': (9472.399, 4460.729),
    'Domain Name System Security Extensions': (-4541.4893, -8291.524),
    'DNS over TLS': (-4569.2334, -8722.733),
    'DNS over HTTPS': (-4511.6895, -8580.95),
    'DNSCurve': (-4597.5283, -8552.492),
    'External Data Representation': (184.71805, 1921.0157),
    'Link Access Procedure Balanced': (-5157.3687, 3753.8213),
    'Frame Relay': (-7283.671, 1407.3491),
    'Funktionsbaum': (16162.731, -972.83673),
    'Balanced hybrid': (-12379.362, -1301.7303),
    'Diffusing Update Algorithm': (-11054.343, -1103.3318),
    'Signal-Protokoll': (-961.1145, -7023.686),
    'Session Traversal Utilities for NAT': (-381.08148, -7748.2544),
    'Standalone': (911.6915, -9774.477),
    'Telepathy': (-437.87772, -6325.779),
    'Link Aggregation Control Protocol': (-11944.026, 6103.433),
    'Inter-Switch Link Protocol': (-12270.953, 6740.68),
    'T-Stück': (-12785.036, 8939.445),
    'Jabber (Ethernet)': (-12946.033, 8249.751),
    'EAD-Kabel': (-12754.247, 8906.554),
    'AirPrint': (-4294.9077, -717.9589),
    'Transportschicht': (-3002.8074, 122.731285),
    'SYN-Cookies': (-4081.4504, 149.99022),
    'Signalling System 7': (-1177.4703, -7575.628),
    'VTun': (-5309.623, 1696.9006),
    'Crypto IP Encapsulation': (-6557.9663, 908.8398),
    'Wrapper-Klasse': (9079.97, 4748.287),
    'Kernel-Modul': (-1314.2642, -1521.7705),
    'Internet Security Association and Key Management Protocol': (-5921.672, -377.55746),
    'ARQ-Protokoll': (-4183.5225, 1866.5931),
    'SACK': (-3717.8035, 1693.9332),
    'Datenflussdiagramm': (15428.595, -432.80518),
    'Pufferknoten (UML)': (18725.887, -279.88544),
    'File Transfer, Access and Management': (598.7368, 2904.719),
    'AS2': (922.8608, 2739.4055),
    'Banking Communication Standard': (543.67365, 2853.0542),
    'Dia (Software)': (15419.271, -1439.311),
    'Microsoft Visio': (15371.834, -1294.7367),
    'Jakarta Enterprise Beans': (10649.443, 3023.0898),
    'Singleton (Entwurfsmuster)': (10696.333, 5917.0513),
    'IPSec': (-3514.909, -3978.965),
    'Netzwerkdatenbankmodell': (-856.61755, -5244.2876),
    'Independent Computing Architecture': (-3945.9995, 2828.6846),
    'Anwendungsserver': (5562.4727, -1629.0137),
    'Message Oriented Middleware': (4743.17, 1401.3248),
    'Rsync': (-384.89084, -34.02764),
    'Dynamic Data Exchange': (3754.3374, -465.47702),
    'Netzwerk-DDE': (3688.3376, -455.3897),
    'Simple Service Discovery Protocol': (-4695.409, -3154.4937),
    'NAT Port Mapping Protocol': (-4570.8955, -4053.215),
    'Remote Authentication Dial-In User Service': (-1860.3458, -91.25623),
    'Diameter (Protokoll)': (-1139.3838, 36.1036),
    'Profinet': (-6771.106, 9297.607),
    'Layer 2 Forwarding': (-5442.5396, 1558.7148),
    'Protocol for SYnchronous Conferencing': (-135.825, -4250.903),
    'Besucher (Entwurfsmuster)': (10773.876, 5394.9927),
    'Liste von Singleton-Implementierungen': (10567.904, 6123.2646),
    'Pipe (Informatik)': (2642.468, 601.0354),
    'Abstrakte Fabrik': (10223.505, 4592.4653),
    'VTAM': (-5474.1025, 3123.297),
    'Verschlüsselungsprotokoll': (-539.9885, -2729.2285),
    'Extensible Authentication Protocol': (-2090.3022, -27.556694),
    'FREAK': (-7.599822, -1921.412),
    'GnuTLS': (338.64395, -1960.9006),
    'LibreSSL': (-252.52898, -2148.2234),
    'Core Image': (4558.854, -7015.268),
    'Core Animation': (4588.8096, -7023.021),
    'Undo': (9937.081, 5775.526),
    'Open Platform Communications': (685.68896, 4221.5444),
    'Port Based Network Access Control': (-2275.6238, 132.63998),
    'Optimized Link State Routing': (-10878.961, -1052.179),
    'Ad-hoc On-demand Distance Vector': (-10903.001, -993.20825),
    'B.A.T.M.A.N.': (-7256.516, -2189.5269),
    'Business Delegate': (7799.7573, 2833.0476),
    'Native API': (5081.17, -7044.604),
    'Video for Windows': (4335.192, -3529.465),
    'HX DOS Extender': (4199.793, -4766.6504),
    'Jakarta Messaging': (3481.26, 1304.4282),
    'Remote Framebuffer Protocol': (37.367184, -1037.2574),
    'Kontrollflussgraph': (15248.959, -584.464),
    'Schematransformation und -integration': (6428.9185, 1157.3673),
    'Object-relational impedance mismatch': (8592.68, -2490.8066),
    'Berkeley Printing System': (-4014.2212, 2885.901),
    'Advanced Linux Sound Architecture': (9436.664, -14301.933),
    'Session Announcement Protocol': (-2832.94, -3249.997),
    'WS-Flow Language': (14942.735, 275.1888),
    'WS-Choreography Interface': (5730.5444, 6778.818),
    'WS-Choreography Description Language': (5707.621, 6806.669),
    'Dienstekomposition': (5719.9985, 6730.2876),
    'Modellgetriebene Architektur': (14681.913, -298.20413),
    'ObjectiF': (21664.975, -2590.42),
    'Real-Time Streaming Protocol': (-2991.7793, -3785.3516),
    'RealTime Control Protocol': (-3227.2205, -3890.1853),
    'IntServ': (-8596.273, -2559.259),
    'Common Open Policy Service': (-8164.205, -3026.2944),
    'Future (Programmierung)': (14863.472, 510.84933),
    'Time-Sensitive Networking': (-8185.8813, 8906.412),
    'Media Redundancy Protocol': (-7652.447, 9521.48),
    '10BASE-T1L': (-6975.853, 9210.061),
    '100BASE-TX': (-12884.897, 8831.269),
    'TeamSpeak': (751.85175, -2047.8994),
    'Xfire': (1373.3839, -788.02734),
    'Mumble': (710.5431, -2162.1611),
    'Binary Large Object': (9491.251, -360.80246),
    'CorNet': (-7589.57, 308.44727),
    'Gadu-Gadu': (-58.955666, -6785.0596),
    'GDI+': (5165.359, -7197.297),
    'Java Media Framework': (5299.9424, -1898.212),
    'Prospero (Protokoll)': (-492.65274, 5559.6636),
    'Asynchronous Reliable Delivery Protocol': (-526.84076, 5562.388),
    '6LoWPAN': (-11121.722, -735.8951),
    'Routing Protocol for Low power and Lossy Networks': (-11641.948, -663.06024),
    'Z-Wave': (-9103.389, 739.124),
    'ISCSI': (-14281.876, 4827.0273),
    'Fibre Channel over IP': (-14334.516, 4886.08),
    'HyperSCSI': (-14390.087, 4817.7295),
    'Advanced Program to Program Communications': (-5446.785, 3131.8303),
    'Core Audio': (4540.571, -7043.266),
    'Megaco': (-2401.526, -4440.548),
    'Media Gateway Control Protocol (Protokoll)': (-2409.9094, -4505.7637),
    'Network Driver Interface Specification': (-5869.604, 2518.7378),
    'Open Datalink Interface': (-5896.667, 2545.8542),
    'Picture Transfer Protocol over IP': (5.845499, 11119.919),
    'YEd': (15343.723, -1257.236),
    'T.120': (-3640.652, -1878.5144),
    'Finger (Internetprotokoll)': (84.48277, -3540.5134),
    'Ident': (77.645546, -3591.83),
    'Streaming-Protokoll': (-3687.6812, -1880.489),
    'Real Time Messaging Protocol': (-1815.9547, 300.57257),
    'Secure Reliable Transport': (-1818.7623, 205.45854),
    'WebRTC': (-1632.2397, -6328.154),
    'Digital Audio Access Protocol': (-4111.005, -1367.4309),
    'MMS-Protokoll': (-3029.4692, -1672.6638),
    'WPA2': (-625.9701, -2773.0469),
    'Advanced Message Queuing Protocol': (1585.5431, 40.041645),
    'SoaML': (14498.525, 1071.945),
    'Verteilte Anwendung': (6331.788, 2841.5872),
    'Service Component Architecture': (7781.485, -2687.4966),
    'Service Data Objects': (7750.97, -2666.1582),
    'Ereignisgesteuerte Architektur': (7393.5957, 2218.1428),
    'Jakarta Server Faces': (5581.6655, -448.57126),
    'Entity Framework': (8110.2334, -2378.7473),
    'RMON': (-3538.6162, 2612.3545),
    'Message Passing Interface': (7283.267, -14459.269),
    'Signalling Connection Control Part': (-1232.9724, -7635.4717),
    'WhatsApp': (-941.25214, -7527.241),
    'Telegram': (-1148.5199, -8568.518),
    'Robots Exclusion Standard': (426.64755, 717.3611),
    'HTTP-Authentifizierung': (429.42087, 688.7782),
    'Java Data Objects': (7889.849, -2537.5125),
    'DataNucleus': (7861.2056, -2567.8828),
    'Jakarta Persistence API': (7816.281, -2568.7231),
    'Hibernate (Framework)': (7943.9316, -2371.0618),
    'Skinny Client Control Protocol': (-2420.6934, -4570.1064),
    'Umbrello': (15459.436, -1506.4623),
    'Zigbee': (-9125.949, 771.6509),
    'Web Processing Service': (1884.8951, 9461.227),
    'XL (XML-Programmiersprache)': (5426.275, 6360.481),
    'X.75': (-4850.2163, 3105.5183),
    'Rational Rose': (16971.979, -169.067),
    'DOS Protected Mode Interface': (4190.9326, -4741.972),
    'QuickDraw': (5199.3867, -6083.2734),
    'Together (Software)': (17132.105, -219.28943),
    'ILMI': (-10015.391, -377.20142),
    'I2P': (-924.47424, -5260.842),
    'RetroShare': (-3760.1423, -4446.298),
    'Cross-Cutting Concern': (8778.326, 2100.5095),
    'EclipseLink': (7635.427, -2673.5999),
    'Etherchannel': (-11986.997, 6130.1357),
    'PAgP': (-12038.436, 6133.5415),
    'Local Mail Transfer Protocol': (1281.4558, 17227.816),
    'GPRS Tunneling Protocol': (-7475.0786, 109.60428),
    'GRASP': (9146.194, 3352.2556),
    'Forwarding Equivalence Class': (-8500.467, -7722.058),
    'Cisco Discovery Protocol': (-8584.978, 8241.202),
    'Dotted decimal notation': (-12050.922, -4749.1133),
    'Signalempfänger (UML)': (18417.291, 1705.6373),
    'Java Business Integration': (5887.209, 3171.9846),
    'Signal (UML)': (18280.05, 37.89379),
    'ADOdb': (8136.2173, -3839.1274),
    'Datenbankabstraktionsschicht': (8095.7485, -3670.8484),
    'WS-Reliable Messaging': (4642.8965, 8258.128),
    'WS-AtomicTransaction': (4379.6104, 7620.7324),
    'WS-Addressing': (4652.438, 8109.985),
    'WS-Policy': (4531.8286, 7844.2485),
    'Web Services Interoperability Technology': (4356.992, 7709.176),
    'EAPOL': (-2162.254, 31.395302),
    'Global-as-View': (11248.801, -964.18555),
    'Local-as-View': (11224.003, -950.9576),
    'Sicht (Datenbank)': (11180.359, -924.84174),
    'OSCAR (Protokoll)': (-649.98755, -6196.5854),
    'Abhängigkeitsbeziehung': (17649.307, 1182.4261),
    'Data Access Object': (8176.492, -2676.2766),
    'SafetyNET p': (-7392.002, 9763.293),
    'EtherNet/IP': (-7063.5215, 9446.212),
    'Druckprotokoll': (-3982.5874, 2854.708),
    'ETRN': (509.6075, -114.56658),
    'MSXML': (3638.3857, 5249.5986),
    'Data Source Name': (8096.2793, -3806.5708),
    'Pipes und Filter': (3153.5366, 663.86365),
    'Model View Presenter': (7420.9287, 1380.7484),
    'Presentation-Abstraction-Control': (7394.8945, 887.37177),
    'Aktivitätsparameterknoten': (18956.666, 28.097792),
    'AppleShare': (-6862.962, 6207.395),
    'Echo (Netzwerkdienst)': (-746.51385, -531.4072),
    'Discard': (-785.897, -461.71063),
    'Dynamic Trunking Protocol': (-12245.877, 6535.808),
    'KParts': (4358.5635, 236.32239),
    'Bonobo (GNOME)': (4322.584, 246.54562),
    '3964R': (-4377.8447, 4291.8794),
    'RK512': (-4387.872, 4265.3755),
    'Java Authentication and Authorization Service': (7784.0034, 4415.8545),
    'Jakarta Connectors': (10407.43, 3899.2078),
    'Universally Unique Identifier': (2525.4197, -146.34218),
    'WS-BPEL4People': (5622.6816, 7170.482),
    'Mashup (Internet)': (4828.962, 2948.1824),
    'WS-HumanTask': (5621.6523, 7139.311),
    'Modelio': (14588.561, 792.69666),
    'Jakarta XML Binding': (4112.573, 6644.233),
    'Windows Server Update Services': (1384.624, -10994.276),
    'Syslog': (-2437.0972, -596.28357),
    'Programmpaket': (3530.115, -2014.7213),
    'Silent Circle Instant Messaging Protocol': (-899.6513, -6869.862),
    'IEC 61850': (-4887.6333, 5900.724),
    'Manufacturing Messaging Specification': (-4874.3057, 5949.4595),
    'TASE.2': (-4842.44, 5948.6953),
    'IEC 60870': (-4850.6206, 5921.701),
    'Java Plug-in': (3569.1292, 220.44032),
    'DNP3': (-4694.6616, 6060.5063),
    'Simple File Transfer Protocol': (-783.6808, 887.65344),
    'File Service Protocol': (-243.18578, 2388.2917),
    'Encryption Control Protocol': (-4931.258, 4730.4775),
    'Virtual Terminal': (400.28763, 2885.5186),
    'Background File Transfer Program': (-490.9421, 3109.4734),
    'Multipath TCP': (-4877.2383, -253.87311),
    'Binary Synchronous Communication': (-5042.6445, 3751.3267),
    'Job Transfer and Manipulation': (428.294, 2901.9385),
    'SQLAlchemy': (8183.6343, -1388.5686),
    'Jakarta XML RPC': (3289.6165, 3600.8574),
    'Jakarta XML Web Services': (4423.1396, 7569.5996),
    'Jakarta RESTful Web Services': (3382.9072, 3548.396),
    'DirectX Media Objects': (4731.121, -2993.3896),
    'Application Response Measurement': (6081.077, 2405.3555),
    'TCP Sequence Prediction': (-4581.772, 571.6069),
    'TOC (Protokoll)': (-566.1938, -6118.3794),
    'ReSharper': (8492.696, 2989.7827),
    'Video4Linux': (4187.7627, -3493.7966),
    'Konsolenanwendung': (421.71085, -1882.1526),
    'XDMCP': (-10222.116, -6746.538),
    'Constrained Application Protocol': (-807.9001, 542.2947),
    'Open Data Protocol': (7451.0957, -1355.845),
    'OpenAPI': (10267.902, -724.10516),
    'Autonomic Computing': (7450.715, 3640.4773),
    'Normierte Programmierung': (5804.8315, 1593.8379),
    'Spaghetticode': (7691.829, 2107.5496),
    'X10 (Protokoll)': (-2245.1487, 6346.926),
    'CEBus': (-2247.0908, 6320.708),
    'Universal Measurement and Calibration Protocol': (-5247.1245, 10022.921),
    'AS1': (950.69904, 2784.535),
    'TOGAF': (12635.23, 126.64202),
    'On-Demand Mail Relay': (464.81207, -91.662056),
    'Java Sound': (5345.0205, -1870.8279),
    'JavaHelp System': (5469.7134, -1159.7026),
    'Short Message Peer to Peer': (-180.23018, -7840.151),
    'Papyrus (UML)': (16976.176, -1235.3317),
    'TOPCASED': (17007.062, -1263.5338),
    'Innovator (Software)': (14300.27, -463.3516),
    'Rational Rhapsody': (16970.877, -1143.4333),
    'Visual Paradigm': (16421.137, -1795.0009),
    'Sockets Direct Protocol': (-2448.4067, -1037.2883),
    'Encapsulating Security Payload': (-5963.897, -387.69128),
    'Host Identity Protocol': (-3121.1357, 142.48186),
    'ROHC': (-4476.978, 1738.2216),
    'Service Delivery Platform': (4967.0464, -3663.7563),
    'Parlay': (4940.4243, -3646.841),
    'Service Logic Execution Environment': (4977.859, -3635.797),
    'Temporale Datenhaltung': (12035.029, -505.872),
    'Java API for XML Processing': (3728.2383, 4864.8877),
    'SecureIM': (-637.33795, -6288.918),
    'Quantität (Analysemuster)': (13106.746, 3836.3066),
    'Measurement (Analysemuster)': (13070.955, 3827.8208),
    'Observation (Analysemuster)': (13054.488, 3856.3433),
    'Accounting Transaction': (16522.21, 4031.96),
    'XNA (Microsoft)': (5804.0024, -7721.3345),
    'Organization Hierarchy': (12658.3955, 3354.7087),
    'Organization Structure': (12695.206, 3352.04),
    'Accountability': (12685.79, 3380.112),
    'Event Sourcing': (10762.521, 1842.4135),
    'Range (Analysemuster)': (13043.757, 3817.708),
    'Anwendungsfassade': (12861.839, 3977.8481),
    'Event (Analysemuster)': (10796.331, 1871.6921),
    'Fixed Property': (14083.356, 3700.723),
    'Money (Analysemuster)': (11365.563, 3553.2976),
    'Item-Item Description': (13716.054, 2667.874),
    'Koordinator (Analysemuster)': (15854.296, 3112.0586),
    'Internet Content Adaptation Protocol': (1445.0461, 1014.48706),
    'Zusicherung (Softwaretechnik)': (17277.002, 1529.4249),
    'Simple Asynchronous File Transfer': (-2297.0627, -3207.8186),
    'Cable-Sharing': (-12944.364, 8885.765),
    'Java Management Extensions': (5236.1943, 2359.0632),
    'Deep Packet Inspection': (751.4774, 523.86646),
    'Phonon (KDE)': (4212.833, -3464.0054),
    'WHOIS': (-7789.792, -7173.529),
    'Webintegration': (10411.209, 3992.339),
    'ArgoUML': (14771.197, -1956.9673),
    'Enterprise Architect': (16821.207, -1243.7804),
    'Fujaba': (16764.07, -425.6931),
    'Poseidon for UML': (14742.163, -1965.6383),
    'PowerDesigner': (12904.896, -1355.4702),
    'Gaphor': (17688.182, 147.47754),
    'StarUML': (17170.021, -170.8671),
    'UMLet': (16639.527, -144.1088),
    'Violet (UML-Werkzeug)': (17473.32, -337.15762),
    'Multiprotocol over ATM': (-10922.456, -5482.5146),
    'Proxy Auto-Config': (-4649.6978, -1911.1593),
    'Propel': (8193.128, -3950.3567),
    'PHP Data Objects': (8170.362, -3925.9258),
    'Softwareerosion': (8610.798, 3060.3145),
    'Extensible Provisioning Protocol': (-1348.5396, -2093.9038),
    'Blocks Extensible Exchange Protocol': (934.3384, -283.09735),
    'ZRTP': (-1939.5463, -5761.909),
    'Highway Addressable Remote Transducer': (-6984.568, 8902.568),
    'WiTECK': (-6962.7217, 8866.994),
    '1000BASE-T': (-12870.236, 8764.582),
    'Ethernet APL': (-7024.4453, 9244.904),
    'Cisco Group Management Protocol': (-10419.346, 1559.9159),
    'GMRP': (-10376.334, 1537.8207),
    'Internet Cache Protocol': (-2899.596, -7982.4927),
    'Hyper Text Caching Protocol': (-2914.325, -7953.808),
    'Dynamic Information and Action Gateway': (3537.4663, 2401.441),
    'Web Ontology Language for Web Services': (4424.229, 8528.259),
    'Web Service Modeling Ontology': (4452.0435, 8528.6045),
    'MySQLi': (8143.0117, -3912.378),
    'IEC 61970': (-4930.669, 5905.593),
    'MQTT': (1145.7275, -68.96614),
    'Datasource': (7149.0815, -2548.3992),
    'Java Transaction Service': (7173.636, -2531.076),
    'ADOdb Lite': (8162.9263, -3886.728),
    'Zachman Framework': (11363.884, -1920.5493),
    'Zephyr (Protokoll)': (-1570.0874, -5710.179),
    'Service-Locator': (8182.504, 2474.8008),
    'Gottobjekt': (8598.704, 3120.701),
    'WS-Coordination': (4267.826, 7868.3647),
    'WS-SecureConversation': (4696.7866, 7736.362),
    'WS-SecurityPolicy': (4627.8813, 7795.1714),
    'WS-Trust': (4654.115, 7649.979),
    'Data Link Switching': (-7433.49, 4277.432),
    'Media Resource Control Protocol': (-2871.043, -3693.092),
    'Gallium3D': (4755.3125, -8143.0576),
    'WebGL': (4681.6167, -8194.806),
    'Meta-Architektur-Dokument': (6477.007, 1265.329),
    'Java APIs for Integrated Networks': (4957.182, -3615.5752),
    'Cellular-IP': (-23455.336, 3096.7927),
    'Boot Service Discovery Protocol': (-2327.0894, 1325.2517),
    'Real-time Transport Protocol': (-4410.3135, 1586.9789),
    'Wireless Transport Layer Security': (-357.15988, -2668.2212),
    'Positive Acknowledgement with Retransmission': (-861.32086, 3361.4526),
    'Cross Platform Component Object Model': (4018.3936, 819.5069),
    'FISH (Protokoll)': (-489.95386, -65.157455),
    'Remote Data Objects': (8315.303, -3321.8174),
    'SAE J1939': (-3652.181, 7214.9727),
    'SAE J1708': (-3640.905, 7179.4785),
    'Fachkomponente': (16875.48, 1035.533),
    'SMS-Gateway': (-207.91583, -7835.897),
    'Open Building Information Exchange': (1428.3984, 10135.802),
    'WS-Distributed Management': (5668.658, 8325.864),
    'Sensor Media Access Control': (-9748.931, 8591.155),
    'Timeout Media Access Control': (-9753.316, 8625.527),
    'Berkeley Media Access Control': (-9783.404, 8612.802),
    'Zebra Media Access Control': (-9719.169, 8615.12),
    'Nebenläufigkeit (Entwurfsmuster)': (10617.991, 6198.147),
    'Transparent Network Substrate': (-3868.3513, 20376.547),
    'Link Layer Topology Discovery': (-14433.03, -10884.816),
    'Hyper Text Coffee Pot Control Protocol': (-11055.079, -4553.033),
    'Apache Tuscany': (7822.691, -2692.217),
    'Front Controller': (7476.3335, 988.43536),
    'Canvas (HTML-Element)': (4350.3545, -8003.845),
    'Internet Protocol over Fibre Channel': (-14319.476, 4948.443),
    'Certificate Management Protocol': (-762.019, 506.04788),
    'Certificate Request Message Format': (789.94904, -1862.467),
    'Tox (Protokoll)': (-3876.1838, -4580.6777),
    'TIPC': (-1348.6853, -1519.1624),
    'Data Link Control': (-5211.1323, 3837.8408),
    'TopLink': (7384.965, -2569.6187),
    'Apache OpenJPA': (7623.021, -2509.1252),
    'Datenbankdesign': (12788.886, -1274.3992),
    'Semistrukturierte Daten': (13584.077, -873.1948),
    'Creole (Framework)': (8223.341, -3969.887),
    'Devices Profile for Web Services': (4526.4434, 8150.7495),
    'Direct Internet Message Encapsulation': (3990.5183, 7210.046),
    'OpenSocial': (807.21106, -7159.0),
    'UnAPI': (2740.9233, 3230.5325),
    'Pichat': (-455.49063, -713.42316),
    'IEEE 802.1q': (-12234.687, 6498.8657),
    'Split Multi-Link Trunking': (-11933.713, 6132.3174),
    'Gnome Virtual File System': (-846.34784, 1338.7024),
    'Reverse Connection': (-491.55823, 2792.3508),
    'XIMSS': (-2385.6216, -5850.118),
    'Multimedia-Framework': (4703.204, -3038.2725),
    'Pi Digit Generation Protocol': (-11043.053, -4514.4805),
    'WebCL': (4732.266, -8220.53),
    'Transport Packet': (-3172.2292, 3240.1477),
    'Data Vault': (12089.444, -514.5526),
    'ArchiMate': (12589.751, 134.3057),
    'WS-Agreement': (5060.1777, 8548.34),
    'WS-Transaction': (4542.2134, 8420.724),
    'WS-Enumeration': (4698.419, 7875.785),
    'WS-Referral': (4733.7036, 8360.263),
    'WS-Routing': (4716.4404, 8312.974),
    'WS-Security UsernameToken Profile': (5272.1953, 7790.252),
    'WS-Interoperability': (5242.962, 8926.249),
    'Babel (Protokoll)': (-10937.456, -1070.2303),
    'Jakarta Transactions API': (7201.907, -2511.0735),
    'ESTRI Gateway': (955.7367, 2755.2542),
    'Rogue DHCP': (-9982.295, -4752.373),
    'Mapper (Softwareentwicklung)': (6736.9106, 157.1535),
    'Markierungsschnittstelle': (8522.571, 3187.5076),
    'Proactor': (17868.8, -1406.8793),
    'Keeping Emulation Environments Portable': (3552.5732, -2045.7456),
    'Grand Central Dispatch': (5022.044, -7773.898),
    'Big Ball of Mud': (8056.882, 2325.3457),
    'MyBatis': (8030.8887, -2614.8643),
    'Jakarta Mail': (90.49731, -330.01007),
    'Apache Cayenne': (7042.669, -2177.5884),
    'Apache ServiceMix': (5947.188, 3185.7734),
    'Java Secure Socket Extension': (3497.4548, 8432.218),
    'Fast and Secure Protocol': (-378.3935, 1014.2772),
    'Real-Time Java': (5134.525, -304.08926),
    'Open Cirrus': (6533.5312, 1463.7164),
    'GigE Vision': (-15932.0, -6559.5195),
    'Video Decode and Presentation API for Unix': (4240.8564, -3237.6504),
    'X-Video Bitstream Acceleration': (4241.377, -3212.175),
    'SPICE (Protokoll)': (352.21054, -941.3991),
    'Java Telephony API': (3557.4678, -1979.9651),
    'Packet over SONET/SDH': (-5504.5454, 3715.4758),
    'Java Cryptography Extension': (823.38385, -1915.1299),
    'Bouncy Castle': (794.74426, -1892.5553),
    'Connect Direct': (-274.66202, 539.04865),
    'Web Storage': (3884.5605, -6926.23),
    'Indexed Database API': (3888.937, -6957.905),
    'Apache MINA': (4765.6035, 2109.5293),
    'Quelltextklon': (8558.6045, 2794.756),
    'HDBaseT': (-10413.917, 10144.654),
    'Teilnetzbetrieb': (-5186.9116, 9957.72),
    'TCP Global Synchronization': (-8173.8755, -2107.8435),
    'W3C Geolocation API': (3900.2434, -17838.117),
    'Dataspaces': (6411.5757, 1174.1267),
    'International Data Spaces': (6388.802, 1202.0435),
    'Lookeen': (5573.0117, -1744.2701),
    'Software-Ökosystem': (6530.1465, 1607.1925),
    'Viber': (-1121.4862, -8497.806),
    'WebHooks': (2538.944, 291.53873),
    'IFTTT': (2514.332, 245.9616),
    'Diem (Internetwährung)': (-1111.6006, -17010.795),
    'SegWit': (-1641.1471, -16878.27),
    'Lightning-Netzwerk': (-1571.6395, -16633.99),
    'Bitcoin Cash': (-1543.5146, -16924.396),
    'Bitcoin SV': (-1564.5137, -16984.71),
    'Litecoin': (-1769.239, -17015.168),
    'Zcash': (-1831.334, -16780.496),
    'Dogecoin': (-1541.6488, -17294.72),
    'Threema': (-294.60522, -7811.9795),
    'Bitstamp': (-1417.67, -16942.838),
    'Liste von Kryptowährungen': (-1589.9299, -17138.217),
    'TCP Port Service Multiplexer': (-586.6281, 4945.8267),
    'Time Stamp Protocol': (750.3369, -1902.1028),
    'Source-specific Multicast': (-11848.829, 743.86786),
    'Virtual Extensible LAN': (-10349.786, 6116.579),
    'Exchange ActiveSync': (218.18077, 390.61108),
    'OpenSAFETY': (-7182.928, 9479.772),
    'Tripel-Graph-Grammatik': (16702.56, 216.07921),
    'Weihenstephaner Standards': (149.70169, 5037.4604),
    'KakaoTalk': (-630.60504, -6962.0864),
    'WeChat': (-619.34973, -6989.399),
    'Line (Instant Messaging)': (-647.86926, -6983.6216),
    'Etch (Protokoll)': (3855.3704, 1271.4603),
    'Signal (Messenger)': (-1201.8365, -8660.778),
    '464XLAT': (-5336.879, -3477.7268),
    'Delay Tolerant Networking': (-10626.147, -800.87744),
    'Pfsync': (-8546.804, 3701.719),
    'Doppelt überprüfte Sperrung': (10614.141, 6129.9),
    'Sicherung (Entwurfsmuster)': (8600.241, 4681.1436),
    'OpenWebRTC': (-1774.1171, -6268.3906),
    'Apache Thrift': (2840.8806, -6010.609),
    'TCP Fast Open': (-4111.08, 97.75061),
    'Gesetz von Conway': (9526.509, -226.29999),
    '4in6': (-10211.031, -3535.52),
    'Model 2': (7845.8633, -1772.6743),
    'Parallel Redundancy Protocol': (-7661.6606, 9491.723),
    'CyberGhost VPN': (-3492.0957, -4008.24),
    'Asynchronous module definition': (10057.928, 2907.1016),
    'AmiTCP': (-2475.0486, 1926.6829),
    'Real Data Transport': (-2892.742, -3915.9385),
    'Train Real Time Data Protocol': (82.68451, 855.7239),
    'Locator/Identifier Separation Protocol': (-14463.778, -4336.7485),
    'Initialization-on-demand holder idiom': (10647.002, 6111.0063),
    'Datenbank-Muster': (3984.2024, -1908.5706),
    'Dark Pattern': (3603.8267, -3557.2393),
    '6rd': (-9079.185, -5553.739),
    'Secure Socket Tunneling Protocol': (-5856.092, 1228.988),
    'EGL (Programmierschnittstelle)': (2663.4514, -3564.7483),
    'Proof of Work': (-1447.2839, -17338.672),
    'Ripple (Geldsystem)': (-1387.628, -16850.064),
    'Stablecoin': (-1246.3982, -16775.586),
    'Ethereum': (-1196.1125, -17331.5),
    'Nano (Kryptowährung)': (-1489.8505, -17647.13),
    'Monero': (-1842.2003, -16865.982),
    'DirectAccess': (-9222.28, -4867.5835),
    'Nxt (Kryptowährung)': (-2037.3243, -17382.707),
    'NEM (Kryptowährung)': (-2108.728, -17415.86),
    'Surespot': (-482.09137, -8090.2363),
    'Privacy Coin': (-1861.8391, -16697.334),
    'MobileCoin': (-1757.9048, -16681.389),
    'Briar (Instant Messenger)': (-3937.865, -4586.4727),
    'Mbed TLS': (438.63864, -2048.5254),
    'Any-source Multicast': (-11804.439, 790.40405),
    'Hoccer': (-813.768, -7639.4346),
    'IP500 Alliance': (-7984.3564, 623.8402),
    'Services Computing': (5540.7905, 4708.7964),
    'IEC 62351': (-4717.9214, 6038.478),
    'Microservices': (9477.341, -225.58264),
    'Humane Registry': (9824.363, -608.5255),
    'Swagger (Software)': (9876.055, -627.9871),
    'Operations-Datenbank': (8566.033, 4663.7407),
    'Self-contained Systems': (8935.369, 25.52045),
    'Objektpool': (11344.222, 7454.311),
    'HFTP': (-93.14584, 2142.3164),
    'Online Certificate Status Protocol stapling': (-211.28287, -1780.825),
    'Wire (Messenger)': (-1549.6339, -6414.4736),
    'Perl Web Server Gateway Interface': (5299.437, -1603.9261),
    'Ethereum Classic': (-1032.9473, -17307.555),
    'Dante (Netzwerkprotokoll)': (-10485.05, 10090.359),
    'Tether': (-1271.9573, -16695.945),
    'Solana (Blockchain)': (-1245.3171, -17520.453),
    'Shiba Inu (Kryptowährung)': (-1538.7181, -17359.723),
    'Chainlink': (-1330.3724, -17821.344),
    'Stellar (Bezahlsystem)': (-1314.8213, -16759.586),
    'Theta (Kryptowährung)': (-1252.1908, -17645.008),
    'Tron (Kryptowährung)': (-1175.5676, -16041.028),
    'EOS (Kryptowährung)': (-1059.5278, -17514.734),
    'Iota (Kommunikationsprotokoll)': (-1469.025, -17467.287),
    'NEO (Kryptowährung)': (-1175.8926, -17869.633),
    'Dash (Kryptowährung)': (-1875.2896, -17171.795),
    'Waves (Kryptowährung)': (-2014.8693, -17325.291),
    'Petro (Kryptowährung)': (-2150.0786, -17408.07),
    'Verge (Kryptowährung)': (-2128.5703, -16148.905),
    'Chia (Kryptowährung)': (-1580.1385, -17598.805),
    'Bitconnect': (-2065.2053, -18468.186),
    'OneCoin': (-1781.4204, -18441.947),
    'Web Worker': (3914.163, -7022.596),
    'Service Worker': (3897.877, -6990.2563),
    'Web Audio API': (4339.8667, -8059.113),
    'WebVR': (4386.7305, -8075.101),
    'Registration Data Access Protocol': (-7797.795, -7204.218),
    'AS4 (Schnittstelle)': (985.91705, 2687.4653),
    'Bulkhead': (8632.674, 4649.6772),
    'Noise Protocol Framework': (-627.8393, -7390.4956),
    'Durchsatzratenbegrenzung': (769.1187, 571.2337),
    'Rail Safe Transport Application': (-3205.4111, 3166.4392),
    'ZeroNet': (-2356.8806, -9217.778),
    'InterPlanetary File System': (-2365.8813, -9270.338),
    'Multi-Path Routing': (-4918.493, -235.05313),
    'Cardano (Blockchainnetzwerk)': (-1795.509, -17325.697),
    'Windows ML': (6030.199, -14087.554),
    'Matrix (Kommunikationsprotokoll)': (-1037.6562, -6370.2915),
    'QXML DOM': (3799.253, 4067.9133),
    'Saga (Entwurfsmuster)': (9015.243, 323.09753),
    'Dezentralisierte Autonome Organisation': (-1792.7556, -17454.861),
    'VIPER (Entwurfsmuster)': (8276.906, 1717.9799),
    'Production Performance Management Protocol': (1531.7437, 10.721465),
    'DirectX Raytracing': (5289.9473, -7951.3545),
    'IP Address Management': (3380.1904, -2089.2407),
    'Messaging Layer Security': (-1517.6438, -6456.987),
    'JSON Meta Application Protocol': (184.6666, 318.60532),
    'Steem': (-1106.3027, -17498.047),
    'BattleEye RCon Protocol': (-3762.7185, 212.96162),
    'Umati': (322.30103, 4797.232),
    'Distributed Application Runtime': (7324.463, 2166.2312),
    'DICT (Protokoll)': (758.3041, 18015.838),
    'Advanced Physical Layer': (-7015.732, 9204.713),
    'Immediate Mode (Computergrafik)': (11241.023, -2232.3713),
    'Retained Mode': (11186.018, -2264.0298),
    'Immediate mode (Computergrafik)': (11228.002, -2279.1006),
    'Particl (Kryptowährung)': (-1756.1067, -16764.389),
    'Interaktionsdesignmuster': (8987.08, 1236.3915),
    'TR-369': (945.1646, 185.4345),
    'TR-143': (974.1202, 378.4385),
    'Thread (Netzwerkprotokoll)': (-9133.915, 742.21594),
    'Verzögerte Initialisierung': (10588.807, 6028.3276),
    'GNU Taler': (-2424.378, -19447.223),
    'Metal (API)': (5130.073, -7938.88),
    'MoltenVK': (5173.074, -7958.278),
    'C4 model': (13132.466, 422.79224),
    'EchoRing': (-6953.5967, 9377.303),
    'Diagrams.net': (15401.898, -1213.065),
    'Open Banking': (1608.3029, 2181.6663),
    'Cloud-native Computing': (9334.588, -124.567276),
    'Oparl': (2246.7456, 2747.6843)
}

# Community Beispiele
tiny_community = nx.Graph()
tiny_community.add_edge(1, 2)
tiny_community.add_edge(1, 5)
tiny_community.add_edge(2, 3)
tiny_community.add_edge(2, 5)
tiny_community.add_edge(3, 4)
tiny_community.add_edge(4, 5)
tiny_community.add_edge(4, 6)

small_communities = nx.Graph()
small_communities.add_edge(1, 2)
small_communities.add_edge(1, 3)
small_communities.add_edge(1, 4)
small_communities.add_edge(2, 3)
small_communities.add_edge(2, 4)
#small_communities.add_edge(2, 5)
small_communities.add_edge(3, 4)
small_communities.add_edge(3, 15)
small_communities.add_edge(4, 7)
small_communities.add_edge(5, 6)
small_communities.add_edge(5, 7)
small_communities.add_edge(6, 7)
#small_communities.add_edge(7, 8)
small_communities.add_edge(8, 9)
small_communities.add_edge(8, 11)
small_communities.add_edge(8, 13)
small_communities.add_edge(8, 15)
small_communities.add_edge(9, 10)
small_communities.add_edge(10, 11)
small_communities.add_edge(11, 12)
small_communities.add_edge(12, 13)
small_communities.add_edge(13, 14)
small_communities.add_edge(14, 15)
